/* $Id: 3dbutton.C,v 1.1 90/06/17 18:44:39 gjb Exp $
 *
 *  3dbutton.C:								*
 *									*
 *  "Three-D" counterparts for the standard InterViews button classes.  *
 *									*
 *  By Greg Brail. Copyright 1990 Brown University.			*/

#include "3dbutton.h"
#include <InterViews/painter.h>
#include <InterViews/shape.h>
#include <InterViews/canvas.h>
#include <InterViews/font.h>
#include <InterViews/color.h>
#include <InterViews/pattern.h>

const int pad = 3;			// Space around text
const int border = 4;			// 3D border

ThreeDPushButton::ThreeDPushButton(const char* name, ButtonState* s, int v)
: (name, s, (void*)v)

{}

ThreeDPushButton::ThreeDPushButton(const char* name, ButtonState* s, void* v)
: (name, s, v)

{}

void ThreeDPushButton::Reconfig()

{
  SetClassName("ThreeDPushButton");
  shape->width = output->GetFont()->Width(text) + (2 * pad) + (2 * border);
  shape->height = output->GetFont()->Height() + (2 * pad) + (2 * border);
  shape->Rigid(2 * pad, 0, 2 * pad, 0);
}

void ThreeDPushButton::Refresh()

{
  Color* origFg = output->GetFgColor();
  Color* origBg = output->GetBgColor();
  Color* dimmed = new Color("grey40");
  if (!dimmed->Valid())
    dimmed = black;
  Coord xPath[7];
  Coord yPath[7];
  output->ClearRect(canvas, 0, 0, xmax, ymax);
  if (chosen || hit)
    {
      output->SetColors(white, origBg);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      output->SetColors(dimmed, origBg);
      output->FillRect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origFg, origBg);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origBg, dimmed);
      output->Text(canvas, text, pad + border, pad + border);
    }
  else
    {
      output->SetColors(white, origBg);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(white, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      if (!enabled)
	output->SetColors(dimmed, origBg);
      else
	output->SetColors(origFg, origBg);
      output->Text(canvas, text, pad + border, pad + border);
    }
  output->SetColors(origFg, origBg);
  output->Rect(canvas, 0, 0, xmax, ymax);
}

ThreeDRadioButton::ThreeDRadioButton(const char* name, ButtonState* s, int v)
: (name, s, (void*)v)

{}

ThreeDRadioButton::ThreeDRadioButton(const char* name, ButtonState* s, void* v)
: (name, s, v)

{}

void ThreeDRadioButton::Reconfig()

{
  SetClassName("ThreeDRadioButton");
  shape->height = output->GetFont()->Height();
  shape->width = output->GetFont()->Width(text);
  shape->width += shape->height + pad;
  shape->Rigid(2 * pad, 0, 2 * pad, pad);
}

void ThreeDRadioButton::Redraw(Coord x1, Coord y1, Coord x2, Coord y2)

{
  output->ClearRect(canvas, x1, y1, x2, y2);
  output->Text(canvas, text, 0, 0);
  Refresh();
}

void ThreeDRadioButton::Refresh()

{
  Color* origFg = output->GetFgColor();
  Color* origBg = output->GetBgColor();
  Color* dimmed = new Color("grey40");
  if (!dimmed->Valid())
    dimmed = black;
  Coord xPath[7];
  Coord yPath[7];
  Coord leftX = xmax - shape->height;
  output->Clip(canvas, leftX, 0, xmax, ymax);
  if (chosen || hit)
    {
      output->SetColors(white, origBg);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = xmax;			yPath[1] = 0;
      xPath[2] = xmax;			yPath[2] = ymax;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax - border;		yPath[4] = border;
      xPath[5] = leftX + border;	yPath[5] = border;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = leftX + border;	yPath[1] = border;
      xPath[2] = leftX + border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax; 			yPath[4] = ymax;
      xPath[5] = leftX;	       		yPath[5] = ymax;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, leftX, ymax, leftX + border, ymax - border);
      output->SetColors(dimmed, origBg);
      output->FillRect(canvas, leftX + border, border, xmax - border, ymax - border);
      output->SetColors(origFg, origBg);
    }
  else
    {
      output->SetColors(white, origBg);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = leftX + border;	yPath[1] = border;
      xPath[2] = leftX + border;	yPath[2] = ymax - border;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax; 			yPath[4] = ymax;
      xPath[5] = leftX;	       		yPath[5] = ymax;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, leftX, ymax, leftX + border, ymax - border);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = xmax;			yPath[1] = 0;
      xPath[2] = xmax;			yPath[2] = ymax;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax - border;		yPath[4] = border;
      xPath[5] = leftX + border;	yPath[5] = border;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      output->SetColors(origFg, origBg);
      output->ClearRect(canvas, leftX + border, border, xmax - border, ymax - border);
      output->Rect(canvas, leftX + border, border, xmax - border, ymax - border);
    }
  output->NoClip();
  output->Rect(canvas, leftX, 0, xmax, ymax);
}  

ThreeDCheckBox::ThreeDCheckBox(const char* name, ButtonState* s, int on, int off)
: (name, s, (void*)on, (void*)off)

{}

ThreeDCheckBox::ThreeDCheckBox(const char* name, ButtonState* s, void* on, void* off)
: (name, s, on, off)

{}

void ThreeDCheckBox::Reconfig()

{
  SetClassName("ThreeDCheckBox");
  shape->height = output->GetFont()->Height();
  shape->width = output->GetFont()->Width(text);
  shape->width += shape->height + pad;
  shape->Rigid(2 * pad, 0, 2 * pad, pad);
}

void ThreeDCheckBox::Redraw(Coord x1, Coord y1, Coord x2, Coord y2)

{
  output->ClearRect(canvas, x1, y1, x2, y2);
  output->Text(canvas, text, 0, 0);
  Refresh();
}

void ThreeDCheckBox::Refresh()

{
  Color* origFg = output->GetFgColor();
  Color* origBg = output->GetBgColor();
  Color* dimmed = new Color("grey40");
  if (!dimmed->Valid())
    dimmed = black;
  Coord xPath[7];
  Coord yPath[7];
  Coord leftX = xmax - shape->height;
  output->Clip(canvas, leftX, 0, xmax, ymax);
  if (chosen || hit)
    {
      output->SetColors(white, origBg);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = xmax;			yPath[1] = 0;
      xPath[2] = xmax;			yPath[2] = ymax;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax - border;		yPath[4] = border;
      xPath[5] = leftX + border;	yPath[5] = border;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = leftX + border;	yPath[1] = border;
      xPath[2] = leftX + border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax; 			yPath[4] = ymax;
      xPath[5] = leftX;	       		yPath[5] = ymax;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, leftX, ymax, leftX + border, ymax - border);
      output->SetColors(dimmed, origBg);
      output->FillRect(canvas, leftX + border, border, xmax - border, ymax - border);
      output->SetColors(origFg, origBg);
    }
  else
    {
      output->SetColors(white, origBg);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = leftX + border;	yPath[1] = border;
      xPath[2] = leftX + border;	yPath[2] = ymax - border;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax; 			yPath[4] = ymax;
      xPath[5] = leftX;	       		yPath[5] = ymax;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, leftX, ymax, leftX + border, ymax - border);
      xPath[0] = leftX;			yPath[0] = 0;
      xPath[1] = xmax;			yPath[1] = 0;
      xPath[2] = xmax;			yPath[2] = ymax;
      xPath[3] = xmax - border; 	yPath[3] = ymax - border;
      xPath[4] = xmax - border;		yPath[4] = border;
      xPath[5] = leftX + border;	yPath[5] = border;
      xPath[6] = leftX;			yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      output->SetColors(origFg, origBg);
      output->ClearRect(canvas, leftX + border, border, xmax - border, ymax - border);
      output->Rect(canvas, leftX + border, border, xmax - border, ymax - border);
    }
  output->NoClip();
  output->Rect(canvas, leftX, 0, xmax, ymax);
}  

