/*  $Id: 3dmenuitem.C,v 1.1 90/06/17 18:44:46 gjb Exp $
 *	
 *  3DMenuItem.C:							*
 *									*
 *  By Greg Brail. Copyright 1990 Brown University.			*/

#include <InterViews/message.h>
#include "3dmenuitem.h"
#include <string.h>
#include <InterViews/glue.h>
#include <InterViews/interactor.h>
#include <InterViews/resource.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <InterViews/painter.h>
#include <InterViews/color.h>
#include <InterViews/canvas.h>
#include <InterViews/shape.h>
#include <InterViews/font.h>
#include <InterViews/world.h>
#include <InterViews/banner.h>
#include <InterViews/frame.h>
#include <InterViews/pattern.h>

/* Constants */

const int padding = 3;			// Around text
const int border = 4;			// "3D" border

ThreeDMenuItem::ThreeDMenuItem(const char* name, char* key)
: (nil)

{
  text = new char[strlen(name) + 2];
  for (int inc = 0; inc < strlen(name) + 2; inc++)
    text[inc] = '\0';
  text = strcpy(text, name);
  enabled = true;
  highlighted = false;
  dimmed = nil;
  whatKey = key;
}

ThreeDMenuItem::~ThreeDMenuItem()

{
  Unref(dimmed);
}

void ThreeDMenuItem::Reconfig()

{
  SetClassName("ThreeDMenuItem");
  Font* theFont = output->GetFont();
  shape->width = theFont->Width(text) + (2 * padding) + (2 * border);
  if (whatKey != "")
    shape->width += theFont->Width(whatKey) + (2 * padding);
  shape->height = theFont->Height() + (2 * padding) + (2 * border);
  shape->hshrink = (2 * padding);
  shape->hstretch = hfil;
  shape->vshrink = (2 * padding);
  shape->vstretch = vfil;
  dimmed = new Color("grey80");
  if (!dimmed->Valid())
    dimmed = black;
}

void ThreeDMenuItem::Redraw(Coord l, Coord b, Coord r, Coord t)

{
  Color* origFg = output->GetFgColor();
  Color* origBg = output->GetBgColor();
  Color* dimBg = new Color("grey40");
  if (!dimBg->Valid())
    dimBg = black;
  output->Clip(canvas, l, b, r, t);
  output->ClearRect(canvas, l, b, r, t);
  Coord xPath[7];
  Coord yPath[7];
  if (highlighted)
    {
      output->SetColors(white, origBg);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      output->SetColors(dimBg, origBg);
      output->FillRect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origFg, origBg);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origBg, dimBg);
      output->Text(canvas, text, padding + border, padding + border);
      if (whatKey != "")
	{
	  int keyW = output->GetFont()->Width(whatKey);
	  output->Text(canvas, whatKey, xmax - padding - keyW - border, 
		       padding + border);
	}       
    }
  else
    {
      output->SetColors(white, origBg);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(white, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      if (!enabled)
	output->SetColors(dimmed, origBg);
      else
	output->SetColors(origFg, origBg);
      output->Text(canvas, text, padding + border, padding + border);
      if (whatKey != "")
	{
	  int keyW = output->GetFont()->Width(whatKey);
	  output->Text(canvas, whatKey, xmax - padding - keyW - border, 
		       padding + border);
	}       
    }
  output->NoClip();
  output->SetColors(origFg, origBg);
  output->Rect(canvas, l, b, r, t);
}

void ThreeDMenuItem::Draw()

{
  Redraw(0, 0, xmax, ymax);
}

void ThreeDMenuItem::Highlight(boolean high)

{
  if (high != highlighted)
    {
      highlighted = high;
      Draw();
    }
}

void ThreeDMenuItem::Enable()

{
  enabled = true;
}

void ThreeDMenuItem::Disable()

{
  enabled = false;
}

