static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/DiamondBeveller.cc,v $ $Revision: 1.3 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//
// Revision History:
//   $Log: DiamondBeveller.cc,v $
// Revision 1.3  1991/10/11  15:03:15  pbrown
// Added const to parameters to satisy requirements
// of new IV 3.0 release.
//
// Revision 1.2  1991/08/27  20:31:10  morrison
// Brought up to date for the code inspection.
//
// Revision 1.1  1991/07/15  19:50:28  morrison
// Initial revision
//
//
// Description:
//   This routine is used to draw a motif-like radio button.
//   It does so by drawing a diamond shape on a canvas using a medium color.
//   It then adds a light colored peak ('A') and a dark colored 'V' to the 
//   diamond to give is a pseudo-3D effect (a la motif radio button).
//   This makes it look like a small diamond that is 'raised up' from the 
//   surface of the screen.
//
// Hints:
//   By reversing the light and dark colors you can get the effect that 
//   the diamond is 'pressed in'.

#include <Iv/DiamondBeveller.h>

#include <InterViews/canvas.h>
#include <InterViews/color.h>
#include <InterViews/margin.h>

#include <math.h>

void diamond(
    Canvas* canvas, const Color* light, const Color* medium, const Color* dark, Coord thickness,
    Coord left, Coord bottom, Coord right, Coord top)
{
    // determine the center of the diamond
    Coord center_x = (left + right) / 2;
    Coord center_y = (bottom + top) / 2;

    // determine other parameters   (are these used?)
    float slope = (top - bottom) / (right - left);
    float delta_x = thickness / sqrt(slope * slope + 1);
    float delta_y = slope * delta_x;

    // draw the exterior diamond shape using the medium color
    //     /\
    //    /  \
    //   /    \
    //  /      \
    //  \      /
    //   \    /
    //    \  /
    //     \/
    canvas->new_path();
    canvas->move_to(left, center_y);
    canvas->line_to(center_x, top);
    canvas->line_to(right, center_y);
    canvas->line_to(center_x, bottom);
    canvas->close_path();
    canvas->fill(medium);

    // overlay the bottom half of the diamond outline in 'shadow' (dark) color
    //  __    __
    //  \ \  / /
    //   \ \/ /
    //    \  /
    //     \/
    canvas->new_path();
    canvas->move_to(left, center_y);
    canvas->line_to(left + thickness, center_y);
    canvas->line_to(center_x, bottom + thickness);
    canvas->line_to(right - thickness, center_y);
    canvas->line_to(right, center_y);
    canvas->line_to(center_x, bottom);
    canvas->close_path();
    canvas->fill(dark);

    // overlay the top half of the diamond outline in 'light' color
    //     /\
    //    /  \
    //   / /\ \
    //  /_/  \_\
    canvas->new_path();
    canvas->move_to(left, center_y);
    canvas->line_to(left + thickness, center_y);
    canvas->line_to(center_x, top - thickness);
    canvas->line_to(right - thickness, center_y);
    canvas->line_to(right, center_y);
    canvas->line_to(center_x, top);
    canvas->close_path();
    canvas->fill(light);
}
