static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/GenericButton.cc,v $ $Revision: 1.5 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//
// Revision History:
//   $Log: GenericButton.cc,v $
// Revision 1.5  1991/10/31  23:23:56  pbrown
// Resolved action items for inspection #202
//
// Revision 1.4  1991/10/30  17:02:50  pbrown
// Added constructor and destructor logging.
//
// Revision 1.3  1991/10/17  19:40:38  pbrown
// Pre-inspection cleanup.
//
// Revision 1.2  1991/10/11  16:02:03  pbrown
// Remove debugging writes and add more documentation.
//
// Revision 1.1  1991/10/11  15:03:53  pbrown
// Initial revision
//
//
// Description:
//   Implements virtual functions with push button behaviour.

#include <Iv/GenericButton.h>
#include <Rtap/ThisProcess.h>

#include <IV-look/telltale.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <InterViews/listener.h>

#include <stream.h>

// Function:
//   Construct an instance.  Create a listener and set it listening.  If the
//   telltale exists initialize it to be enabled and not yet chosen.  If
//   the action exists attach it.

GenericButton::GenericButton(Action* anAction, Telltale* aTelltale) :
    MonoGlyph(nil), PointerHandler(), _enabled(true), _chosen(false)
{
    ThisProcess::constructorActivated("GenericButton(Action*,Telltale*)",
				       (int) this);
    _action = anAction;
    Resource::ref(_action);
    _telltale = aTelltale;
    Resource::ref(_telltale);
    if (_telltale != nil) {
	_telltale->enable(enabled());
	_telltale->choose(chosen());
    }

    // create a Listener to sense X-events and tell it to accept
    // motion and mouse-button events and pass them to this instance

    _listener = new Listener(_telltale, this);
    _listener->button(true, Event::any);
    _listener->motion(true);
    body(_listener);
}

// Function:
//   Destruct the instance.  Unref the action and the telltale.

GenericButton::~GenericButton()
{
    ThisProcess::destructorActivated("~GenericButton()",
				       (int) this);
    Resource::unref(_action);
    Resource::unref(_telltale);
}

// Function:
//   If button is enabled support drag function by highlighting the
//   telltale only when the cursor is over the button.

void GenericButton::drag(Event& anEvent)
{
    if (enabled() && telltale() != nil) {
	telltale()->highlight(listener()->picks(anEvent.pointer_x(),
						anEvent.pointer_y()));
    }
}

// Function:
//   If button is enabled support release function by making sure
//   the telltale is no longer highlighted.

void GenericButton::release(Event& anEvent)
{
    if (enabled() && telltale() != nil) {
	telltale()->highlight(false);
    }
    PointerHandler::release(anEvent);
}

// Function:
//   If button is enabled, an action exists and the cursor is over the
//   button handle the commit function by executing the action.

void GenericButton::commit(Event& anEvent)
{
    if (enabled() && action() != nil &&
	listener()->picks(anEvent.pointer_x(), anEvent.pointer_y())) {
	action()->execute();
    }
}

// Function:
//   Choose or unchoose the button as desired and relay any change
//   to the telltale.

void GenericButton::choose(boolean state)
{
    _chosen = state;
    if ((telltale() != nil) && (telltale()->chosen() != state)) {
	telltale()->choose(state);
    }
}

// Function:
//   Enable or disable the button as desired and relay any change
//   to the telltale.

void GenericButton::enable(boolean state)
{
    _enabled = state;
    if ((telltale() != nil) && (telltale()->enabled() != state)) {
	telltale()->enable(state);
    }
}
