#ifndef GenericButton_h
#define GenericButton_h

static char _rcs_id_GenericButton_h[] = "DATAP $Source: /usr/datap/include/Iv/RCS/GenericButton.h,v $ $Revision: 1.4 $";

// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project: 987050 C 440
// Library: Iv
// File: GenericButton.cc


/*
 * Revision History:
 *   $Log: GenericButton.h,v $
 * Revision 1.4  1991/10/31  23:20:56  pbrown
 * Resolved action items for inspection #202
 *
 * Revision 1.3  1991/10/17  19:40:07  pbrown
 * Pre-inspection cleanup.
 *
 * Revision 1.2  1991/10/11  16:07:52  pbrown
 * Added more ducumentation.
 *
 * Revision 1.1  1991/10/11  14:40:33  pbrown
 * Initial revision
 *
 */

// Section: Description
//   This class is modelled on the InterViews Button class.  It
//   is intended as a replacement for that class.  It supports
//   extra functionality including button choosing and enabling.
//   It is designed to be used as a virtual base class for buttons
//   with even more features such as toggle buttons and radio buttons.
//
//   The telltale is a glyph-based object that can have multiple looks
//   depending upon its state.  The button doesn't care about the look
//   it simply manipulates the state of the telltale to match the calls
//   to button methods.  It can enable or disable it, choose or unchoose
//   it, and highLight or unhighlight it.  The telltale chooses the
//   appropriate look based on the combination of these booleans.
//
//   Initiation of actions and on-screen behaviour is inhibited if the
//   button is disabled.

#include <InterViews/ptrhandler.h>
#include <InterViews/monoglyph.h>

class ivAction;
class ivListener;
class ivTelltale;

class GenericButton : public MonoGlyph, public PointerHandler
{
public:
    GenericButton(Action* anAction, Telltale* aTelltale);
    virtual ~GenericButton();

    virtual void drag(Event& anEvent);
    virtual void release(Event& anEvent);
    virtual void commit(Event& anEvent);
    virtual void choose(boolean state);
    virtual void enable(boolean state);
    boolean chosen() const;
    boolean enabled() const;

protected:
    Action* action() const;
    Listener* listener() const;
    Telltale* telltale() const;

private:
    Action* _action;
    Listener* _listener;
    Telltale* _telltale;
    boolean _enabled;
    boolean _chosen;
};

// Function:
//   Indicate whether button has been chosen or not.

inline boolean GenericButton::chosen() const
{ return _chosen; }

// Function:
//   Indicate whether button has been enabled or not.

inline boolean GenericButton::enabled() const
{ return _enabled; }

// Protected Function:
//   Return pointer to attached action (or nil if it doesn't exist).

inline Action* GenericButton::action() const
{ return _action; }

// Protected Function:
//   Return pointer to attached listener (which is not nil).

inline Listener* GenericButton::listener() const
{ return _listener; }

// Protected Function:
//   Return pointer to attached telltale (or nil if it doesn't exist).

inline Telltale* GenericButton::telltale() const
{ return _telltale; }

#endif /* GenericButton_h */
