#ifndef Iedit_h
#define Iedit_h

/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "Iedit-core.h"
#include "Iediteditor.h"
#include <InterViews/window.h>

extern boolean ForceReadOnly;
extern boolean iconic;
extern boolean IsWrap;
extern boolean IsIndent;
extern char* HelpFile;
extern class Iedit* KillThis;
extern int tabsize;

class Dialog;

void HelpWindow(void);

class Iedit : public Iedit_core {
public:
	Iedit(const char*);
	Iedit(const char*, const char* startdir);
	~Iedit();

	/* virtual */ void Open();
	/* virtual */ void Open(const char *);
	/* virtual */ void Close();
	/* virtual */ void New();
	/* virtual */ void Save();
	/* virtual */ boolean Save(const char*);
	/* virtual */ void Quit();
	/* virtual */ void Cut();
	/* virtual */ void Copy();
	/* virtual */ void Paste();
	/* virtual */ void Clear();
	/* virtual */ void SelectAll();
	/* virtual */ void Search();
	/* virtual */ void MatchBracket();
	/* virtual */ void GotoLine();
	/* virtual */ void GoToSelection();
	/* virtual */ void Version();
	/* virtual */ void Include();
	/* virtual */ void _Save();
	/* virtual */ void _Load();
	/* virtual */ void HelpMe();
	/* virtual */ void MapWindow();
public:
	/* virtual */ void Handle(Event &);
	void	UpdateStatus(boolean modified);
	void	Report(const char* title, const char* subtitle);
	static	void HelpWindow(Iedit*);

private:
	void	Init(const char* startdir);
	void	InsertChar(char);
	boolean PopUpDialog(Dialog*);
	void	InsertDialog(Dialog*);
	void	InsertDialog(Dialog*, Alignment);
	void	RemoveDialog(Dialog*);
	void	Keyevent(Event&);
	void	SpecialKeys(Event&);
	void	SelectWord();
	void	SelectLine();
	void	KillRestOfLine();
	void	KillToBeginningOfLine();
	void	GotoLineOffset(int);
	int	Selection();
	boolean	ConfirmSave();
	void	Include(const char* fname);
	void	Message(const char* mesg);
	char	Confirm(const char* title, const char* subtitle);
	boolean	ReadOnlyReport();
	void	SetWrap();
	void	SetIndent();
	void	SetWrap(boolean);
	void	SetIndent(boolean);
	void	changeFont(const char* name);
	void TabSize_4();
	void TabSize_8();
	void _YouChooseTabSize();
	void _CourierMedium12();
	void _CourierBold12();
	void _CourierMedium14();
	void _CourierBold14();
	void _CourierMedium18();
	void _CourierBold18();
private:
	class	FileManager*	_mgr;
	class	FileChooser*	fileDialog;
	boolean	dialogActive;
	class	SearchDialog*	searchDialog;
	class	GotoDialog*	gotoDialog;
	class	GotoDialog*	tabDialog;
	class	Banner* messageLine;
	class	Banner* position;
	char	state[120], pos[32];
	boolean quote;
	boolean	doubleClick;
	long	lastClick;
	static	int	views;
public:
	static int Views();
	ManagedWindow	*window;

};

inline int Iedit::Views(){ return views; }

class IeditWindow : public ApplicationWindow {
public:
	IeditWindow(Iedit*);
	~IeditWindow();
	virtual void configure();
};

#endif
