#ifndef _SelStrEditor_h
#define _SelStrEditor_h

/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <InterViews/streditor.h>
#include <InterViews/texteditor.h>

static const char* _SEDone = "\r";

class ButtonState;
class Xselection;

class SelStrEditor : public StringEditor {
public:
	SelStrEditor(
		ButtonState*, const char* sample, const char* done = _SEDone
	);
	SelStrEditor(
		const char* name, ButtonState*, const char*, const char* = _SEDone
	);
	~SelStrEditor();
	virtual boolean HandleChar(char c);
	virtual void Handle(Event&);
	virtual void Reconfig();

public:
	virtual void Copy();
	virtual void Cut();
	virtual void Paste();
private:
	Xselection* _clipboard;
};

class SelEditor : public TextEditor {
public:
	SelEditor(int rows, int cols, int tabsize, int highlight);
	SelEditor(const char* name, int rows, int cols, int tabsize, int highlight);
	~SelEditor();
	virtual void Reconfig();

	void	InsertText(const char* string, int count);
	void	DeleteText(int count);
	void	DeleteSelection();
	void	Edit(TextBuffer*, int index = 0);
public:
	void Copy();
	void Cut();
	void Paste(const char* buf = nil);
	const char* CutBuffer();
	
	boolean	Dirty();
	void	Dirty(boolean);
	boolean	ReadOnly();
	void	ReadOnly(boolean);
	void	TabSize(int ts);
	int	TabSize();
	void	Wrap(boolean);
	boolean Wrap();
	void	Indent(boolean);
	boolean Indent();
	void	SetFont(const char* font);
protected:
	virtual void DoWrap(int);
	virtual int DoIndent(const char);
private:
	Xselection* _clipboard;
	boolean	dirty;
	boolean	wrap, indent;
	int	tabsize_;
	boolean readonly;
};

inline boolean SelEditor::Dirty() { return dirty; }
inline void SelEditor::Dirty(boolean d){ dirty = d; }
inline boolean SelEditor::ReadOnly() { return readonly; }
inline void SelEditor::ReadOnly(boolean r){ readonly = r; }
inline boolean SelEditor::Wrap() { return wrap; }
inline void SelEditor::Wrap(boolean d){ wrap = d; }
inline boolean SelEditor::Indent() { return indent; }
inline void SelEditor::Indent(boolean d){ indent = d; }
inline int SelEditor::TabSize(){ return tabsize_; }
#endif
