/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <osfcn.h>
#include <signal.h>
#include <InterViews/world.h>
#include <InterViews/event.h>
#include <Dispatch/rpcstream.h>
#include <IV-X11/worldrep.h>

#include "Iedit.h"
#include "xselection.h"

static PropertyData properties[] = {
#include "Iedit-props"
	{"*syncScroll", "on"},
	{"*position*font","*-courier-bold-r-*-120-*" },
	{"*messageLine*font", "*-helvetica-bold-r-*-120-*" },
	{"*Dialog*Message*font", "*-helvetica-bold-r-*-140-*" },
	{"*FileChooser*font", "*-helvetica-bold-r-*-120-*" },
	{"*StringChooser*font", "*-helvetica-bold-r-*-140-*" },
	{"*StringEditor*font", "*-helvetica-bold-r-*-140-*" },
	{"*SelStrEditor*font", "*-helvetica-bold-r-*-140-*" },
	{"*readonly","off"},
	{"*wrap","off"},
	{"*indent","off"},
	{"*autowrap",".tex .help"},
	{"*autoindent",".c .h .tex .help"},
	{"*tab","8"},
	{"*helpfile","/usr/local/lib/X11/Iedit.help"},
	{"*registry", ""},
	{"*service","off"},
	{"*verbose","off"},
	{ nil }
};

static OptionDesc options[] = {
	{"-r", "*readonly", OptionValueImplicit, "on"},
	{"-readonly", "*readonly", OptionValueImplicit, "on"},
	{"-helpfile", "*helpfile", OptionValueNext},
	{"-font","*editor*font", OptionValueNext},
	{"-fn","*editor*font", OptionValueNext},
	{"-tab","*tab", OptionValueNext},
	{"-wrap","*wrap",OptionValueImplicit, "on"},
	{"-indent","*indent",OptionValueImplicit, "on"},
	{"-starthelp","*starthelp", OptionValueImplicit, "on"},
	{"-registry","*registry", OptionValueNext},
 	{ nil }
};

char* progname = nil;

#if 0
#include <InterViews/reqerr.h>
class ErrorHandle : public ReqErr {
public:
	ErrorHandle();
	virtual void Error();
};

void  quit(int);

static int fatalHandler(XDisplay* ){
	perror(progname);
	quit(0);
	return 0;
}

ErrorHandle :: ErrorHandle() : ReqErr() {
	Install();
	XSetIOErrorHandler(fatalHandler);
}

void ErrorHandle :: Error(){
	cerr << "Serieal Number: " << msgid << endl
	     << "Error   Code:   " << code << endl
	     << "Request Code:   " << request << endl
	     << "Resource Id:    " << id << endl 
	     << message << endl;
	quit(0);
}

ErrorHandle errHandle; /* Install the handle */
#endif

char* HelpFile;
char HostName[128];
boolean verbose = true;

void help(const char* argv0){
	cerr << "Usage:\n	" << argv0 << " [-options...] [files...]\n\n"
	     << "where -options can be any options specified in World(3I) plus:\n"
	     << "	-tab n			tab size of the editor window\n"
	     << "	-wrap			wrap on the edge of the window\n"
	     << "	-indent			indent each line\n"
	     << "	-fn/-font		font chosen for the editor\n"
	     << "	-r/-readonly		view the contents only\n"
	     << "	-helpfile		path name of the help file\n"
	     << "	-starthelp		start up with the help window\n"
	     << "	-registry		file name of the service registry\n"
	     << "	-help			print this message\n";
	exit(1);
}

#ifdef SERVICE
void createService();
void deleteService();
void client(int argc, char** argv);
class Service* service = nil;
char *registry = nil;
char *dypName = nil;
#endif

void createIedit(const char* file){
	Iedit* iedit;
	char *path = strdup(file);
	char *tail = strrchr(path,'/');
	if(tail){
		*++tail = '\0';
		iedit = new Iedit("iedit",path);
	} else 
   		iedit = new Iedit("iedit");
	iedit->Open(file);
	iedit->MapWindow();
	delete path;
}


void _on_exit(){
#ifdef SERVICE
	deleteService();
#endif
}

void  quit(int){
	_on_exit();
	exit(1);
}

int main (int argc, char** argv) {
	gethostname(HostName, 127);
	World* w = new World("Iedit", argc, argv, options, properties);
	if(argc > 1 && *argv[1] == '-') help(argv[0]);
	progname = argv[0];
	verbose = w->property_is_on("verbose");
	ForceReadOnly = w->property_is_on("readonly");
	HelpFile = strdup(w->property_value("helpfile"));
	iconic = w->property_is_on("iconic");
	tabsize = atoi(w->property_value("tab"));
	IsWrap = w->property_is_on("wrap");
	IsIndent = w->property_is_on("indent");
	signal(SIGINT, quit);
	signal(SIGQUIT, quit);
	signal(SIGTERM, quit);
	dypName = w->property_value("display") != nil ?
			strdup(w->property_value("display")) : nil;
	if(!dypName) dypName = getenv("DISPLAY");
#ifdef SERVICE
	registry = strdup(w->property_value("registry"));
	if(w->property_is_on("service")){
		createService();
		if(!service) client(argc, argv);
	}
#endif
	argv++;
	if(argc == 1){
		if(w->property_is_on("starthelp")){
			Iedit::HelpWindow(nil);
		}else {
    			Iedit* iedit = new Iedit("iedit");
			iedit->MapWindow();
		}
	} else while(--argc >0){
		createIedit(*argv++);
	}
	iconic = false;
	Xselection clipManager(nil);
	Event e;
	KillThis = nil;
	int abnormal = 0;
	do {
		if(KillThis){
			delete KillThis;
			KillThis = nil;
		}
		w->read(e);
		clipManager.Handle(e);
		e.handle();
		if(e.type() == Event::other_event){
			if(abnormal++ > 100){ /* get around the problem that client
						 does not exit after server exit or
						 window killed */
				XNoOp(w->rep()->display());
				abnormal = 0;
			}
		} else abnormal = 0;
	} while(Iedit::Views()>0);
#ifdef SERVICE
	deleteService();
#endif
	delete w;
	return 0;
}

#ifdef SERVICE

//////////////////////////////////////////////////////////////////////

#include <Dispatch/rpchdr.h>
#include <Dispatch/rpcreader.h>
#include <Dispatch/rpcwriter.h>
#include <Dispatch/rpcservice.h>
#include <Dispatch/rpcregistry.h>

class Reader : public RpcReader {
public:
    Reader(int fd, RpcService* service);
    virtual ~Reader();
protected:
    virtual void connectionClosed(int);
protected:
    RpcService* _service;

    enum {
	IEDIT,
	NFCNS
    };
    static void receive_IEDIT(RpcReader*, RpcHdr&, rpcstream&);
};

Reader::Reader(int fd, RpcService* service) : RpcReader(
    fd, NFCNS, /* binary = */true
),
    _service(service)
{
    _function[IEDIT] = &Reader::receive_IEDIT;
}

Reader::~Reader() {}

void Reader::connectionClosed(int fd) {
    delete this;
    close(fd);
}

void Reader::receive_IEDIT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
/* EDIT pwd user file */
    char* file = new char[hdr.ndata()+2];
    char* dir = new char[hdr.ndata()+2];
    char* user = new char[32];
    client >> dir;
    client >> user;
    client >> file;
    chdir(dir);
    if(strchr(file,'/')) createIedit(file);
    else {
    	Iedit* edit = new Iedit("iedit",dir);
    	edit->Open(file);
  	edit->MapWindow();
    }
    delete file; delete dir; delete user;
}

//////////////////////////////////////////////////////////////////////

class Service : public RpcService {
public:
    Service(const char* path);
    virtual ~Service();
protected:
    virtual void createReader(int fd);
};

Service::Service(const char* path) : RpcService(path) {}

Service::~Service() {}

void Service::createReader(int fd) {
    new Reader(fd, this);
}

char* serviceName(){
	char serName[128];
	char* d = strchr(dypName,':');
	char* h;
	if(strncmp(dypName,"unix",4) == 0) h = HostName;
	else { h = ""; d = dypName;}
	if(registry && *registry) {
		sprintf(serName,"%s.%s%s.%s",registry, h, d, cuserid(nil));
	} else {
		sprintf(serName,"/tmp/Iedit.Server.%s.%s%s.%s", 
			HostName, h,d, cuserid(nil));
	}
	return(strdup(serName));
}

void createService(){
	int port; char* host = nil;
	char* sname = serviceName();
	if(RpcRegistry::find(sname,host, port)){
		if(verbose){
			fprintf(stderr,"\nIedit Service already exists at %s on %s\n",
				sname,host);
			fprintf(stderr,"Service not created. Try client mode\n");
		}
		return;
	}
	unlink(sname);
	service = new Service(sname);
	if(verbose) fprintf(stderr,"\nService registed at %s\n",sname);
}

void deleteService(){ if ( service ) delete service; }

//////////////////////////////////////////////////////////////////////

class Writer : public RpcWriter {
public:
	Writer(const char* path, boolean binary);
	~Writer();

	void sendiedit(char*);
	void flush();
	int good();
protected:
	enum {
	IEDIT
	};
};

Writer::Writer(const char* path, boolean binary) : RpcWriter(
	path, /* fatal = */true, binary
) {}

Writer::~Writer() {}

static char pwd[256] = "./";

void Writer::sendiedit(char* iedit) {
/* EDIT pwd user file */
	RpcHdr hdr(IEDIT);
	server() << hdr << getwd(pwd) << cuserid(nil) << iedit;
}

void Writer::flush() {
	server().flush();
}

int Writer::good() {
	return server().good();
}

void client(int argc, char** argv){
	Writer writer(serviceName(), false);
	if(!writer.good()) {
		unlink(serviceName());
		cerr << "Re-Create Service :" << serviceName() << endl;
		createService();
		return;
	}
	if(argc <= 1) {
		writer.sendiedit("");
		writer.flush();
	}
	while(argc-- > 1 && writer.good()){
		writer.sendiedit(*++argv);
		writer.flush();
	}
	if(!writer.good()) { perror("Iedit Service"); return;}
	exit(0);
}

#endif
