// glyph_no_update.cc
// 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <InterViews/glyph.h>
#include <InterViews/action.h>
#include <InterViews/box.h>
#include <InterViews/session.h>
#include <InterViews/patch.h>
#include <InterViews/color.h>
#include <InterViews/brush.h>
#include <InterViews/canvas.h>
#include <InterViews/background.h>

#include <IV-look/kit.h>
#include <IV-look/button.h>
#include <InterViews/style.h>
#include <InterViews/window.h>


class BCube : public Glyph {
public:
    BCube(Coord side, Color*, Brush*);
    virtual ~BCube();

    virtual void request(Requisition&) const;
    virtual void allocate(Canvas*, const Allocation&, Extension&);
    virtual void draw(Canvas*, const Allocation&) const;
    void move();
private:
    Coord side_;
    const Color* color_;
    const Brush* brush_;
    Coord x_,y_;
};



void BCube::move()
{
  fprintf(stderr,"move called\n");
  x_ += 5;
  y_ += 5;
}


BCube::BCube(Coord s, Color* c, Brush* b)
{
    side_ = s;
    Resource::ref(c);
    color_ = c;
    Resource::ref(b);
    brush_ = b;
    x_ = y_ = 0.0;
}

BCube::~BCube() {
    Resource::unref(color_);
    Resource::unref(brush_);
}

void BCube::request(Requisition& req) const {
    Coord diameter = side_ * (1 + sqrt(2));
    Requirement rx(diameter, 0, 0, 0.5);
    Requirement ry(diameter, 0, 0, 0.5);
    req.require(Dimension_X, rx);
    req.require(Dimension_Y, ry);
}

void BCube::allocate(Canvas* c, const Allocation& a, Extension& ext) {
    ext.extend(a);
}

void BCube::draw(Canvas* c, const Allocation& a) const {

  fprintf(stderr,"BCube::draw called\n");
  
  Coord s = side_;
  Coord x = a.left() + s/2 + x_;
  Coord y = a.bottom() + s/2 + y_;
  const Coord diag = s / sqrt(2);  
  
  fprintf(stderr,"x_ and y_ are %f %f\n",x_,y_);
  fprintf(stderr,"x and y are %f %f\n",x,y);
  


  c->new_path();
  c->move_to(x,y);
  c->line_to(x,y+s);
  c->line_to(x+s,y+s);
  c->line_to(x+s,y);
  c->close_path();
  c->stroke(color_,brush_);
  
  c->new_path();
  c->move_to(x+s,y);
  c->line_to(x+s+diag,y+diag);
  c->line_to(x+s+diag,y+s+diag);
  c->line_to(x+s,y+s);
  c->close_path();
  c->stroke(color_,brush_);
  
  c->new_path();
  c->move_to(x,y+s);
  c->line_to(x+diag,y+s+diag);
  c->line_to(x+s+diag,y+s+diag);
  c->line_to(x+s,y+s);
  c->close_path();
  c->stroke(color_,brush_);

  c->redraw(x,y,x+s+diag,y+s+diag);
}


declare(ActionCallback,BCube);
implement(ActionCallback,BCube);

Glyph* makething(Coord s, Color* c, Brush* b, Style* style)
{
  LRBox *lrbox;

  Kit* kit = Kit::instance();
  BCube *temp = new BCube(s,c,b);
  return new LRBox(temp,
		   kit->simple_push_button("move",style,
					   new ActionCallback(BCube)(temp,&BCube::move)));
}


int main(int argc, char** argv)
{
  Session* session = new Session("BCube", argc, argv);
  Style* style = session->style();
  Kit* kit = Kit::instance();

  Glyph *q = 
    makething(100, 
	      style->foreground(),     
	      new Brush(1.0),
	      style);
  session->run_window
    (new ApplicationWindow(kit->flat_frame(q,style)));
}

/*

BBoard-ID: 2491
BB-Posted: Sat, 09 May 92 02:51:24 +0100
Reply-To: comp.windows.interviews
From: pscheng@rice.edu (Perry Cheng)
Newsgroups: comp.windows.interviews
Subject: can't get glyph to update itself
Keywords: help
Message-ID: <1992May8.002153.6346@rice.edu>
Date: 8 May 92 00:21:53 GMT
Sender: news@rice.edu (News)
Original-Reply-To: pscheng@rice.edu (Perry Cheng)
Organization: Rice University
Lines: 169
Originator: pscheng@cs.rice.edu


	I am trying to learn InterViews and i can't seem to get my glyphs to redraw.
	I have written a program with a window with a glyph(BCube) of a picture
		of a cube and a button that moves the glyph.  When the
		button is pressed, the BCube::draw is called twice, once before and
		once after the routine that the button was supposed to call, 
		namely BCube::move.
	The internal variables of my class BCube are updated but I can get the
		picture on the screen to update only by iconifying and then de-iconifying
		the window.  
	Can someone tell me what the problem is???
	It might be that my fundamental design is flawed, because I do not
		quite understand the class hierarchy of IV!"!

	The following is the code:

----------------------------cut here------------------------------------------------

----------------------------cut here------------------------------------------------

thanks in advance

pscheng@owlnet.rice.edu

*/
