//   update_an_area.cc
//
#include <IV-look/kit.h>
#include <InterViews/background.h>
#include <InterViews/patch.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/window.h>

class App {
public:
    Glyph* make(WidgetKit&, const LayoutKit&);
    void next();
    void prev();
private:
    Deck* deck_;
    Patch* patch_;

    void flip(GlyphIndex);
};

declareActionCallback(App)
implementActionCallback(App)

Glyph* App::make(WidgetKit& kit, const LayoutKit& layout) {
    deck_ = layout.deck();
    deck_->append(kit.label("Hi mom!"));
    deck_->append(kit.label("Big Bird"));
    deck_->append(kit.label("Oscar"));
    deck_->flip_to(0);
    patch_ = new Patch(deck_);
    return patch_;
}

void App::next() {
    GlyphIndex cur = deck_->card() + 1;
    flip(cur == deck_->count() ? 0 : cur);
}

void App::prev() {
    GlyphIndex cur = deck_->card() - 1;
    flip(cur == -1 ? deck_->count() - 1 : cur);
}

void App::flip(GlyphIndex cur) {
    deck_->flip_to(cur);
    patch_->redraw();
    patch_->reallocate();
    patch_->redraw();
}

int main(int argc, char** argv) {
    Session* session = new Session("Himom", argc, argv);
    WidgetKit& kit = *WidgetKit::instance();
    const LayoutKit& layout = *LayoutKit::instance();
    App* a = new App;
    session->run_window(
	new ApplicationWindow(
	    kit.inset_frame(
		layout.margin(
		    layout.vbox(
			a->make(kit, layout),
			layout.vglue(5.0),
			layout.hbox(
			    kit.push_button(
				"Next",
				new ActionCallback(App)(a, &App::next)
			    ),
			    layout.hglue(10.0),
			    kit.push_button(
				"Previous",
				new ActionCallback(App)(a, &App::prev)
			    )
			)
		    ),
		    10.0
		)
	    )
	)
    );
}


/* 

BBoard-ID: 2495
BB-Posted: Sat, 09 May 92 08:37:39 +0100
Reply-To: comp.windows.interviews
From: linton@marktwain.rad.sgi.com (Mark Linton)
Newsgroups: comp.windows.interviews
Subject: Re: Refresh the screen
Keywords: refresh
Message-ID: <khhlhu0@fido.asd.sgi.com>
Date: 8 May 92 18:22:27 GMT
References: <3205@babcock.cerc.wvu.wvnet.edu>
Sender: news@fido.asd.sgi.com (Usenet News Admin)
Original-Reply-To: linton@marktwain.rad.sgi.com (Mark Linton)
Organization: sgi
Lines: 95

In article <3205@babcock.cerc.wvu.wvnet.edu>, many@cathedral.cerc.wvu.wvnet.edu (Mary Meng Li) writes:
|> I wrote a program which used draw(Canvas*, Allocation&) to draw some 
|> geometry figure.  I need to write some characters to the same canvas
|> when certain event is happen (for example, press the button).  I put
|> canvas->character(....) in the event handle routine.  The problem is
|> that the characters are not show until I refresh the screen.  

You should only put Canvas operations, such as character, in a draw operation.
If the area is not being redrawn, then you probably are not damaging it
in response to the event.

Below is an example (3.1-based) program that shows the use of a patch
to update an area.  The redraw/resize/redraw sequence of calls is that
the object could shrink or grow.  If your object is static in size, then
you can use a single redraw.

*/
