static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/RadioButton.cc,v $ $Revision: 1.4 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//   InterViews 3.0
//
// Revision History:
//   $Log: RadioButton.cc,v $
// Revision 1.4  1991/10/31  23:23:56  pbrown
// Resolved action items for inspection #202
//
// Revision 1.3  1991/10/17  19:42:46  pbrown
// Pre-inspection cleanup.
//
// Revision 1.2  1991/10/11  18:46:09  dmorriso
// Supports new IV 3.0 release
//
// Revision 1.1  1991/07/15  19:50:28  morrison
// Initial revision
//
// Description:
//   A radio button is meant to be placed in a RadioButtonSet.  This class
//   provides routines to choose a button when it is pressed (mouse hit)
//   and to inform the button set so that the other buttons can be unchosen.

#include <Iv/RadioButton.h>
#include <Iv/RadioButtonSet.h>
#include <Rtap/ThisProcess.h>

#include <IV-look/telltale.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <InterViews/listener.h>
#include <InterViews/sensor.h>
#include <InterViews/world.h>

#include <stream.h>

// Function:
//   This constructor creates a ToggleButton that has a reference to a
//   RadioButtonSet.  It also adds this button into the button set
//   passed by the caller (if it exists).  No ref() is done upon the
//   button set to avoid circular references.

RadioButton::RadioButton(Action* anAction, Telltale* telltale,
    RadioButtonSet* buttonSet) : ToggleButton(anAction, nil, telltale)
{ 
    ThisProcess::
        constructorActivated("RadioButton(Action*,Telltale*,RadioButtonSet*)",
			     (int) this);
    _buttonSet = buttonSet;
    if (_buttonSet != nil) {
        _buttonSet->addButton(this);
    }
}

// Function:
//   Destruct the instance.

RadioButton::~RadioButton()
{
    ThisProcess::destructorActivated("~RadioButton()", (int) this);
}

// Function:
//   If button is enabled and not yet chosen support drag function by
//   highlighting the telltale only when the cursor is over the button.

void RadioButton::drag(Event& anEvent)
{
    if (enabled() && !chosen() && telltale() != nil) {
	telltale()->highlight(listener()->picks(anEvent.pointer_x(),
						anEvent.pointer_y()));
    }
}

// Function:
//   If button is enabled and not yet chosen support release function
//   by making sure the telltale is no longer highlighted.

void RadioButton::release(Event& anEvent)
{
    if (enabled() && !chosen() && telltale() != nil) {
	telltale()->highlight(false);
    }
    PointerHandler::release(anEvent);
}

// Function:
//   If button is enabled, not yet chosen and the cursor is over the
//   button handle the commit function by choosing the button and if
//   in addition the action exists executing it.

void RadioButton::commit(Event& anEvent)
{
    if (enabled()) {
	if (listener()->picks(anEvent.pointer_x(), anEvent.pointer_y())) {
	    if (!chosen()) {
		choose(true);
		if (action() != nil) {
		    action()->execute();
		}
	    }
	}
    }
}

// Function:
//   Choose or unchoose the button as desired and relay any change
//   to the telltale.  If the button changes to chosen then inform
//   the button set. 

void RadioButton::choose(boolean state)
{
    if (state && !chosen()) {
	ToggleButton::choose(state);
	informButtonSet();
    } else {
	ToggleButton::choose(state);
    }
}

// Private Function:
//   Tell the button set that this button is the 'chosen' one.

void RadioButton::informButtonSet()
{
    if (_buttonSet != nil) {
        _buttonSet->chooseButton(this);
    }
}
