static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/RadioButtonBox.cc,v $ $Revision: 1.6 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//   InterViews 3.0
//
// Revision History:
//   $Log: RadioButtonBox.cc,v $
// Revision 1.6  1991/10/31  23:23:56  pbrown
// Resolved action items for inspection #202
//
// Revision 1.5  1991/10/17  19:45:05  pbrown
// Pre-inspection cleanup.
//
// Revision 1.4  1991/10/11  18:48:08  dmorriso
// Supports new IV 3.0 release
// Now use 'choose' instead of 'select' to specify which one of the radio
// buttons is 'pressed'.
//
// Revision 1.3  1991/09/13  21:37:36  pbrown
// Fixed BevelStyle for selected button
//
// Revision 1.2  1991/09/11  20:34:38  dmorriso
// Restored 'addButton' to accept an 'inputGlyph'.
//
// Revision 1.1  1991/07/15  19:50:28  morrison
// Initial revision
//
// Description:
//   This class provides a container for a set of radio buttons.
//   By convention, only one radio button can be selected at any one time.
//   Accessor functions allow you to find out which button is currently
//   selected and also permit the addition of additional radio buttons
//   to the group.  The look and feel of the buttons is chosen by the Kit.
//
// Hints:
//   When the RadioButtonSet is created the chosen button is set to zero.
//   Unless it is changed by software (initial button chosen using
//   chooseButtonIndex()) or by the user's selection of a button (mouse hit)
//   this will indicate that none of the buttons in the set have been selected.

#include <Iv/RadioButtonBox.h>
#include <Iv/RadioButton.h>
#include <Iv/UIKit.h>
#include <Rtap/ThisProcess.h>

#include <InterViews/box.h>
#include <InterViews/center.h>
#include <InterViews/fixedspan.h>
#include <InterViews/margin.h>
#include <InterViews/style.h>

#include <stream.h>

// Function:
//   Construct the instance and reference the style and various
//   dependent instances.

RadioButtonBox::RadioButtonBox(Style* style, int defaultNbrOfButtons) : MonoGlyph()
{
    // Construct a RadioButtonSet to link all the buttons that will be created
    // and make the body of this glyph a box to put the button glyphs in.

    ThisProcess::constructorActivated("RadioButtonBox(Style*,int)",
				      (int) this);
    Resource::ref(_style = style);
    Resource::ref(_buttonSet = new RadioButtonSet(defaultNbrOfButtons));
    body(_box = new TBBox(defaultNbrOfButtons));
}

// Function:
//   Destruct this instance and unref any dependents so they are blown away
//   if not used elsewhere.

RadioButtonBox::~RadioButtonBox()
{
    ThisProcess::destructorActivated("~RadioButtonBox()", (int) this);
    Resource::unref(_style);
    Resource::unref(_buttonSet);
}

// Function:
//   Use a Kit to create a new RadioButton (using the text passed in as the label) 
//   and append it to the box already created.  If there is an input glyph, 
//   e.g. an analog input slider, then place it to the right of the button.

void RadioButtonBox::addButton(const char* text, Glyph* inputGlyph, 
			       Action* action)
{
    Glyph* choice = UIKit::instance()->radio_button(text, _style, action, _buttonSet);

    // handle the inputGlyph (if one exists)

    LRBox* inputGlyphBox;
    if (inputGlyph == nil) {
	inputGlyphBox = new LRBox(inputGlyph);
    } else {
	inputGlyphBox = new LRBox(new FixedSpan(inputGlyph, Dimension_X, 200.0));
    }
    
    // append the new RadioButton to the _box
    
    _box->append(new VMargin(new LRBox(new VCenter(choice, 0.0), 
				       new VCenter(inputGlyphBox, 0.0)),
			     2.0));
}
