#ifndef RadioButtonBox_h
#define RadioButtonBox_h

static char _rcs_id_RadioButtonBox_h[] = "DATAP $Source: /usr/datap/include/Iv/RCS/RadioButtonBox.h,v $ $Revision: 1.4 $";

// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project: 987050 C 440
// Library: Iv
// File: RadioButtonBox.cc

/*
 * Revision History:
 *   $Log: RadioButtonBox.h,v $
 * Revision 1.4  1991/10/31  23:20:57  pbrown
 * Resolved action items for inspection #202
 *
 * Revision 1.3  1991/10/17  19:44:04  pbrown
 * Pre-inspection cleanup.
 *
 * Revision 1.2  1991/10/11  15:19:12  dmorriso
 * Minor modifications to comply with changes between IV 3.0 beta and IV 3.0
 *
 * Revision 1.1  1991/07/15  20:09:30  morrison
 * Initial revision
 *
 */

// Section: Description
//   This class provides the container for a number of radio buttons.
//   It is also responsible for the 'look' of the radio buttons as a 
//   group (provides boxes and glue).  The button box is actually
//   defined as a monoglyph, you can place it into your window as you 
//   would any other glyph (be sure to allow enough room for the 
//   diamonds, labels, and glue that make up the members of this box).

// Section: Usage Hints
//   Use the constructor to set up the style and the default number of
//   buttons.  It will be most efficient if you know ahead of time how 
//   many buttons are in the group and you specify it in the call to the
//   constructor.  Once the box has been created you use the method
//   addButton() to place the desired text labels (and an input glyph
//   if desired) in the box and associate to a button an action that is
//   to be invoked when the button is chosen.

#include <InterViews/monoglyph.h>
#include <InterViews/action.h>
#include <InterViews/coord.h>

#include <Iv/RadioButtonSet.h>

class ivStyle;

class RadioButtonBox : public MonoGlyph
{
public:
    RadioButtonBox(Style* style, int defaultNbrOfButtons = 4);
    virtual ~RadioButtonBox();
    void addButton(const char* text, Glyph* inputGlyph, Action* action);
    int chosenButtonIndex() const;
    boolean buttonEnabled(int button) const;
    void chooseButtonIndex(int button);
    void enableButton(int button, boolean state);
    void enableAllButtons(boolean state);

private:
    Glyph* _box;
    Style* _style;
    RadioButtonSet* _buttonSet;
};

// Function:
//   Returns the index (in the order of addition) of the button
//   that is currently selected.  Zero indicates that no button has
//   been selected.

inline int RadioButtonBox::chosenButtonIndex() const
{ return (_buttonSet->chosenButtonIndex()); }

// Function:
//   This function returns the state of the button by indicating
//   whether it is enabled or not.  The button is described by its
//   index.

inline boolean RadioButtonBox::buttonEnabled(int button) const
{ return (_buttonSet->buttonEnabled(button)); }


// Function:
//   Make an initial choice for the chosen button from among the buttons
//   that are in the botton box.  Zero indicates that no button should be
//   marked chosen.

inline void RadioButtonBox::chooseButtonIndex(int button)
{ _buttonSet->chooseButton(button); }

// Function:
//   This function provides the ability to enable (state == true) or
//   disable (state == false) a radio button by index.  If
//   the "chosen" button is disabled it will stay chosen.

inline void RadioButtonBox::enableButton(int button, boolean state)
{ _buttonSet->enableButton(button, state); }

// Function:
//   This function provides the ability to enable (state == true) or
//   disable (state == false) all radio buttons in the box.  The
//   chosen button will remain chosen.

inline void RadioButtonBox::enableAllButtons(boolean state)
{ _buttonSet->enableAllButtons(state); }

#endif /* RadioButtonBox_h */
