#ifndef RadioButtonSet_h
#define RadioButtonSet_h

static char _rcs_id_RadioButtonSet_h[] = "DATAP $Source: /usr/datap/include/Iv/RCS/RadioButtonSet.h,v $ $Revision: 1.4 $";

// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project: 987050 C 440
// Library: Iv
// File: RadioButtonSet.cc

/*
 * Revision History:
 *   $Log: RadioButtonSet.h,v $
 * Revision 1.4  1991/10/31  23:20:57  pbrown
 * Resolved action items for inspection #202
 *
 * Revision 1.3  1991/10/17  19:43:13  pbrown
 * Pre-inspection cleanup.
 *
 * Revision 1.2  1991/10/11  15:20:16  dmorriso
 * Minor changes necessary to keep pace with InterViews. (3.0 beta -> 3.0)
 *
 * Revision 1.1  1991/07/15  20:09:30  morrison
 * Initial revision
 *
 */

// Section: Description
//   This class provides the linkage mechanism for a group of radio 
//   buttons.  When a new radio button is chosen it informs the button
//   set which remembers which button is chosen and unchooses all the rest.

// Section: Usage Hints
//   A RadioButtonSet should be instantiated before any of the
//   RadioButtons are created so that the button set may be passed as a
//   parameter to the RadioButton constructor.  To choose the initial button
//   the choose(boolean) method on the desired button can be used.
//   Alternately the chooseButton(int) will perform the same function.  The
//   methods chosenButtonIndex() and chosenButton() are used to determine
//   which button has been chosen after the user has fiddled around with them.
 
#include <InterViews/resource.h>

#include <Iv/RadioButton.h>

class RadioButtonSet : public Resource
{
friend class RadioButton;

public:
    RadioButtonSet(int defaultNbrOfButtons = 4);
    virtual ~RadioButtonSet();
    int chosenButtonIndex() const;
    RadioButton* chosenButton() const;
    boolean buttonEnabled(int button) const;
    void display() const;
    void chooseButton(int button);
    void enableButton(int button, boolean state);
    void enableAllButtons(boolean state);

private:
    int _totalNbrOfButtons;
    int _currentNbrOfButtons;
    int _chosenButton;
    RadioButton** _buttons;

    void addButton(RadioButton* button);
    void chooseButton(const RadioButton* button);
};

// Function:
//   Return index of chosen button, based upon original order
//   of insertion (or zero if no button has yet been chosen).

inline int RadioButtonSet::chosenButtonIndex() const
{ return _chosenButton; }

// Function:
//   Return pointer to chosen button (or nil if no button
//   has yet been chosen).

inline RadioButton* RadioButtonSet::chosenButton() const
{ return _buttons[chosenButtonIndex()]; }

#endif /* RadioButtonSet_h */
