static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/ToggleButton.cc,v $ $Revision: 1.4 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//
// Revision History:
//   $Log: ToggleButton.cc,v $
// Revision 1.4  1991/10/31  23:23:56  pbrown
// Resolved action items for inspection #202
//
// Revision 1.3  1991/10/17  19:42:04  pbrown
// Pre-inspection cleanup.
//
// Revision 1.2  1991/10/11  18:38:18  pbrown
// Inherit from GenericButton and support new
// IV 3.0 release.
//
// Revision 1.1  1991/07/15  19:50:28  morrison
// Initial revision
//
//
// Description:
//   Implements a toggle button that has a state (chosen or unchosen) 
//   and two actions (select action and deselect action).

#include <Iv/ToggleButton.h>
#include <Rtap/ThisProcess.h>

#include <IV-look/telltale.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <InterViews/listener.h>
#include <InterViews/resource.h>

#include <stream.h>

// Function:
//   Creates a new button with a telltale and both a 'select action' and 
//   a 'deselectAction'.

ToggleButton::ToggleButton(Action* selectAction, Action* deselectAction,
    Telltale* telltale) :
    GenericButton(selectAction, telltale), _deselectAction(deselectAction)
{
    ThisProcess::constructorActivated("ToggleButton(Action*, Action*, Telltale*)",
				      (int) this);
    Resource::ref(_deselectAction);
}

// Function:
//   Destruct the instance and unref the deselect action.

ToggleButton::~ToggleButton()
{
    ThisProcess::destructorActivated("~ToggleButton()",
				     (int) this);
    Resource::unref(_deselectAction);
}

// Function:
//   If button is enabled and the cursor is over the button handle
//   the commit function by toggling the chosen flag and executing the
//   appropriate action (select or deselect) if it exists.

void ToggleButton::commit(Event& anEvent)
{
    if (enabled()) {
	if (listener()->picks(anEvent.pointer_x(), anEvent.pointer_y())) {
	    choose(!chosen());
	    if (chosen()) {
		if (action() != nil) {
		    action()->execute();
		}
	    } else {
		if (deselectAction() != nil) {
		    deselectAction()->execute();
		}
	    }
	}
    }
}
