#ifndef ToggleButton_h
#define ToggleButton_h

static char _rcs_id_ToggleButton_h[] = "DATAP $Source: /usr/datap/include/Iv/RCS/ToggleButton.h,v $ $Revision: 1.4 $";

// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project: 987050 C 440
// Library: Iv
// File: ToggleButton.cc

/*
 * Revision History:
 *   $Log: ToggleButton.h,v $
 * Revision 1.4  1991/10/31  23:20:57  pbrown
 * Resolved action items for inspection #202
 *
 * Revision 1.3  1991/10/17  19:41:39  pbrown
 * Pre-inspection cleanup.
 *
 * Revision 1.2  1991/10/11  14:52:56  pbrown
 * Modified to inherit from GenericButton rather than
 * IV Button beacuse of changes in new IV 3.0 release.
 *
 * Revision 1.1  1991/07/15  20:09:30  morrison
 * Initial revision
 *
 */

// Section: Description
//   This class provides an button with two (2) states and two (2)
//   corresponding actions.  The state may be changed from chosen to
//   unchosen or vice versa by software (use of chosen(boolean)) without
//   triggering an action.  If the state is changed by a mouse button hit
//   an action is triggered.  The select action is used if the button
//   becomes chosen and the deselect action is used if the button becomes
//   unchosen.
//
//   Initiation of actions and on-screen behaviour is inhibited if the
//   button is disabled.

#include <Iv/GenericButton.h>

class ToggleButton: public GenericButton
{
public:
    ToggleButton(Action* selectAction, Action* deselectAction,
        Telltale* telltale);
    ~ToggleButton();
    virtual void commit(Event& anEvent);

protected:
    Action* deselectAction() const;

private:
    Action* _deselectAction;

};

// Protected Function:
//   Return pointer to attached deselect action (or nil if it
//   doesn't exist).

inline Action* ToggleButton::deselectAction() const
{ return _deselectAction; }

#endif /* ToggleButton_h */
