static char _rcs_id[] = "DATAP $Source: /usr/datap/src/Iv/RCS/UIKit.cc,v $ $Revision: 1.1 $";

//
// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project:
//   Iris 77000
//
// Configuration:
//
// Revision History:
//   $Log: UIKit.cc,v $
// Revision 1.1  1991/10/11  15:04:19  pbrown
// Initial revision
//
//
// Description:
//   Implements an InterViews Kit button-handling replacement.

#include <Iv/UIKit.h>
#include <Iv/GenericButton.h>
#include <Iv/ToggleButton.h>
#include <Iv/RadioButton.h>
#include <Iv/DiamondBeveller.h>
#include <IV-look/choice.h>
#include <InterViews/center.h>
#include <InterViews/box.h>
#include <InterViews/font.h>
#include <InterViews/glue.h>
#include <InterViews/margin.h>
#include <InterViews/style.h>

UIKit* UIKit::_instance = nil;
 
UIKit::UIKit()
{ }

 UIKit::~UIKit()
{ 
     _instance = nil;
}

UIKit* UIKit::instance()
{
     if (_instance == nil) {
	 _instance = make_kit();
     }
     return _instance;
}

UIKit* UIKit::make_kit()
{
     return new UIKit;
}

GenericButton* UIKit::push_button(Glyph* g, Style* s, Action* a) const
{
    Coord margin = 3.0;
    Coord in_margin = margin + frame_thickness(s);
    return new GenericButton
	(a,
	 new ChoiceItem(Kit::instance()->outset_frame(new Margin(g, margin), s),
			Kit::instance()->inset_frame(new Margin(g, in_margin),s)));
}

GenericButton* UIKit::toggle_button(Glyph* g, Style* s,
				    Action* selectAction, Action* deselectAction) const
{
    Coord margin = 3.0;
    Coord in_margin = margin + frame_thickness(s);
    Glyph* normal = Kit::instance()->outset_frame(new Margin(g, margin), s);
    Glyph* pressed = Kit::instance()->inset_frame(new Margin(g, in_margin),s);
    Glyph* disabled = Kit::instance()->flat_frame(new Margin(g, margin),s);

    ChoiceItem* choice = new ChoiceItem();

    for (unsigned int i = 0; i < choice->numlooks(); i++) {
	if (!choice->contains_looks(i, ChoiceItem::is_enabled)) {
	    choice->look(i, disabled);
	} else {
	    if ((choice->contains_looks(i, ChoiceItem::is_pressed)) ==
		(choice->contains_looks(i, ChoiceItem::is_chosen))) {
		choice->look(i, normal);
	    } else {
		choice->look(i, pressed);
	    }
	}
    }
    return new ToggleButton(selectAction, deselectAction, choice);
}

GenericButton* UIKit::radio_button(Glyph* g, Style* s, Action* a, RadioButtonSet* set) const
{
    Coord margin = 3.0;
    Coord in_margin = margin + frame_thickness(s);
    Glyph* normal = Kit::instance()->outset_frame(new Margin(g, margin), s);
    Glyph* pressed = Kit::instance()->inset_frame(new Margin(g, in_margin),s);
    Glyph* disabled = Kit::instance()->flat_frame(new Margin(g, margin),s);

    ChoiceItem* choice = new ChoiceItem();

    for (unsigned int i = 0; i < choice->numlooks(); i++) {
	if (!choice->contains_looks(i, ChoiceItem::is_enabled)) {
	    choice->look(i, disabled);
	} else {
	    if ((choice->contains_looks(i, ChoiceItem::is_pressed)) ==
		(choice->contains_looks(i, ChoiceItem::is_chosen))) {
		choice->look(i, normal);
	    } else {
		choice->look(i, pressed);
	    }
	}
    }
    return new RadioButton(a, choice, set);
}

GenericButton* UIKit::radio_button(const String& str, Style* s,
				   Action* a, RadioButtonSet* set) const
{
    Coord margin = 3.0;
    Coord in_margin = margin + frame_thickness(s);

    Coord size = (((s != nil) && (s->font() != nil))? (0.8 * s->font()->ascent()) : 20);

    Glyph* square = new VCenter(new LRBox(new HGlue(size, 0, 0),
					  new VGlue(size, 0, 0)),
				0.0);
    Glyph* label = new Margin(Kit::instance()->label(str, s), margin);   

    Glyph* normal = new LRBox(
			      new HGlue(5, 0, 0),
			      new Margin(outset_diamond(square, s), margin),
			      label,
			      new HGlue(5, fil, 0));

    Glyph* pressed = new LRBox(
			      new HGlue(5, 0, 0),
			      new Margin(inset_diamond(square, s), margin),
			      label,
			      new HGlue(5, fil, 0));

    Glyph* disabled = new LRBox(
			      new HGlue(5, 0, 0),
			      new Margin(flat_diamond(square, s), margin),
			      label,
			      new HGlue(5, fil, 0));

    ChoiceItem* choice = new ChoiceItem();

    for (unsigned int i = 0; i < choice->numlooks(); i++) {
	if (!choice->contains_looks(i, ChoiceItem::is_enabled)) {
	    choice->look(i, disabled);
	} else {
	    if ((choice->contains_looks(i, ChoiceItem::is_pressed)) ==
		(choice->contains_looks(i, ChoiceItem::is_chosen))) {
		choice->look(i, normal);
	    } else {
		choice->look(i, pressed);
	    }
	}
    }
    return new RadioButton(a, choice, set);
}

Glyph* UIKit::inset_diamond(Glyph* g, Style* s) const
{
    return new Bevel(g, diamond, s->dark(), s-> dull(), s ->light(), frame_thickness(s));
}

Glyph* UIKit::outset_diamond(Glyph* g, Style* s) const
{
    return new Bevel(g, diamond, s->light(), s-> flat(), s ->dark(), frame_thickness(s));
}

Glyph* UIKit::flat_diamond(Glyph* g, Style* s) const
{
    return new Bevel(g, diamond, s->flat(), s-> flat(), s ->flat(), frame_thickness(s));
}

Coord UIKit::frame_thickness(Style* s)
{
    Coord c;
    if (!s->find_attribute("frameThickness", c)) {
	c = 2.0;
    }
    return c;
}
