#ifndef UIKit_h
#define UIKit_h

static char _rcs_id_UIKit_h[] = "DATAP $Source: /usr/datap/include/Iv/RCS/UIKit.h,v $ $Revision: 1.1 $";

// Copyright (c) DATAP Systems, 1991. All rights reserved.
// License material - Property of DATAP Systems.
//
// Project: 987050 C 440
// Library: Iv
// File: UIKit.cc

/*
 * Revision History:
 *   $Log: UIKit.h,v $
 * Revision 1.1  1991/10/11  14:54:06  pbrown
 * Initial revision
 *
 */

// Section: Description
//   This class provides a version of the InterViews Kit to deal
//   with Iris buttons.

#include <InterViews/enter-scope.h>
#include <InterViews/coord.h>
#include <IV-look/kit.h>
#include <OS/string.h>

class Action;
class GenericButton;
class RadioButtonSet;
class Glyph;
class Style;
class String;

class UIKit {
public:
    UIKit();
    static UIKit* instance();
    ~UIKit();

    /* buttons */
    GenericButton* simple_push_button(const char*, Style*, Action*) const;
    GenericButton* simple_push_button(const String&, Style*, Action*) const;
    virtual GenericButton* push_button(Glyph*, Style*, Action*) const;

    GenericButton* simple_toggle_button(const char* str, Style* s, Action* a) const;
    GenericButton* simple_toggle_button(const String& str, Style* s, Action* a) const;
    GenericButton* simple_toggle_button(Glyph* g, Style* s, Action* a) const;
    GenericButton* simple_toggle_button(const char* str, Style* s,
					Action* selectAction, Action* deselectAction) const;
    GenericButton* simple_toggle_button(const String& str, Style* s,
					Action* selectAction, Action* deselectAction) const;
    virtual GenericButton* toggle_button(Glyph* g, Style* s,
					 Action* selectAction, Action* deselectAction) const;

    GenericButton* radio_button(const char* str, Style* s, Action* a,
				       RadioButtonSet* set) const;
    virtual GenericButton* radio_button(const String& str, Style* s, Action* a,
				       RadioButtonSet* set) const;
    virtual GenericButton* radio_button(Glyph* g, Style* s, Action* a,
					RadioButtonSet* set) const;

private:
    static UIKit* _instance;
    static UIKit* make_kit();
    static Coord frame_thickness(Style* s);
    static Glyph* inset_diamond(Glyph* g, Style* s) const;
    static Glyph* outset_diamond(Glyph* g, Style* s) const;
    static Glyph* flat_diamond(Glyph* g, Style* s) const;
};

/* push buttons */

inline GenericButton* UIKit::simple_push_button(
    const char* str, Style* s, Action* a) const
{ return push_button(Kit::instance()->label(str, s), s, a); }

inline GenericButton* UIKit::simple_push_button(
    const String& str, Style* s, Action* a) const
{ return push_button(Kit::instance()->label(str, s), s, a); }

/* toggle buttons */

inline GenericButton* UIKit::simple_toggle_button(
    const char* str, Style* s, Action* a) const
{ return toggle_button(Kit::instance()->label(str, s), s, a, a); }

inline GenericButton* UIKit::simple_toggle_button(
    const String& str, Style* s, Action* a) const
{ return toggle_button(Kit::instance()->label(str, s), s, a, a); }

inline GenericButton* UIKit::simple_toggle_button(
    Glyph* g, Style* s, Action* a) const
{ return toggle_button(g, s, a, a); }

inline GenericButton* UIKit::simple_toggle_button(
    const char* str, Style* s, Action* selectAction, Action* deselectAction) const
{ return toggle_button(Kit::instance()->label(str, s), s, selectAction, deselectAction); }

inline GenericButton* UIKit::simple_toggle_button(
    const String& str, Style* s, Action* selectAction, Action* deselectAction) const
{ return toggle_button(Kit::instance()->label(str, s), s, selectAction, deselectAction); }

/* radio buttons */

inline GenericButton* UIKit::radio_button(
    const char* str, Style* s, Action* a, RadioButtonSet* set) const
{
    String aString(str);
    return radio_button(aString, s, a, set);
}

#endif
