/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef TextView_h
#define TextView_h

#include <InterViews/handler.h>
#include <InterViews/monoglyph.h>

#include <stdio.h>

class Color;
class Composition;
class DocDeck;
class FontTag;
class FontTag_List;
class HyperViewer;
class Listener;
class ParaInfo_List;
class ParaTag;
class ParaTag_List;
class Patch;
class TextInfo_List;

//
// *** text information list item
//
class TextInfo
   {
   public:
      unsigned char  _code;
      unsigned char  _tocindex;
   };

//
// *** text information list item
//
class ParaInfo
   {
   public:
      long first;
      long last;
      long tag;
   };
 


class TextView : public MonoGlyph, public Handler 
   {
   public:
      TextView (HyperViewer*,char*,char*,int,Coord,Coord); 
					// constructor
      ~TextView ();			// destructor 
      virtual void view_page(int);	// changes currently view page
      virtual boolean event(Event&);	// catches mouse events
      virtual void printout();		// generates PostScript
      void         jump_to(int);	// jump to page with given offset 
      int	   last_page();		// jump to page with given offset 
      int	   Count();		// returns number of pages
      FontTag*     GetFont(const char*);	
 					// return font index if defined
      const char*  MMLFile();		// returns filename

   protected:
      ParaInfo_List* _parainfo;		// paragraph tag list
      ParaTag_List*  _paralist;		// paragraph tag list
      FontTag_List* _fontlist;		// font tag list
      virtual void   marginize();
					// set margins on each paragrap
      virtual void   wordhit(GlyphIndex);
					// processes mouse click for hypertext
      virtual void   read(FILE*);	// reads a MML file
      boolean        parse_command(char*, FILE*);
					// parses an MML command in <>
      virtual void   make_defaults();	// creates default tags
      virtual void   define(const char*, FILE*);
					// defines a tag
      virtual void   fontstyle(const char*);
					// changes font style (e.g. bold)
      virtual void   appendchar(const char);
					// adds a character to list

   private:
      HyperViewer*   _parent;		// Controlling helpview instance
      ParaInfo       _curpar;		// Current paragraph being defined
      int            _newpartag;	// Tag index of next paragraph
      char*          _home;		// directory containing document
      char*          _file;		// current document file
      const Color*   _fg;		// foreground text color
      const Color*   _bg;		// background text color
      TextInfo_List* _text;		// text information list
      Patch*	     _patch;		// main patch
      Listener*      _listener;		// event listener
      Composition*   _lines;		// lines of character glyphs
      Composition*   _characters;	// character glyphs
      DocDeck* 	     _pages;		// stack of pages
      ParaTag*       _partag;		// current paragraph tag
      FontTag*       _fonttag;		// current font tag
      boolean        _ishyper;		// flag for defining hypertext
      int	     _tocindex;		// Table of contents index
   };

//
//  *** Node in font list
//
class FontItem
   {
   public:
      FontTag*   fonttag;        //  View of MML file
   };

//
//  *** Node in paragraph list
//
class ParaItem
   {
   public:
      ParaTag*   paratag;        //  View of MML file
   };

#endif
