/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* caselessCompare.c: SGI provides a useful pair of functions,
 *
 *         strcasecmp       strncasecmp
 *
 * which aren't part of the ANSI standard (ANS X3.159-1989).  this small
 * module emulates the behavior of the SGI functions.
 * the functions do exactly the same thing as the standard functions
 * strcmp and strncmp except that in the new functions, the case of the 
 * characters in the strings is ignored.  (that's accomplished here by
 * simply down-casing both strings before comparison.)
 *
 * This code was graciously donated by Paul Shannon of the NRAO.
 * Thank you Paul (pshannon@iapetus.cv.nrao.edu). 
 */
/*---------------------------------------------------------------------------*/
static char* SCCSid = "@(#)caselessCompare.c	1.2 1/22/93 20:02:24";
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <malloc.h>
#include "caselessCompare.h"
/*---------------------------------------------------------------------------*/
int strcasecmp (const char *s1, const char* s2)
{
  char* s1Lowered, *s2Lowered;
  int i, result;

  s1Lowered = (char*)malloc (strlen (s1) + 1);
  s2Lowered = (char*)malloc (strlen (s2) + 1);

  for (i=0; i < strlen (s1); i++)
     s1Lowered [i] = tolower (s1 [i]);
  s1Lowered [i] = 0;

  for (i=0; i < strlen (s2); i++)
    s2Lowered [i] = tolower (s2 [i]);
  s2Lowered [i] = 0;

  result = strcmp (s1Lowered, s2Lowered);

  free (s1Lowered);
  free (s2Lowered);
  
  return result;

} /* strcasecmp */
/*---------------------------------------------------------------------------*/
int strncasecmp (const char *s1, const char* s2, int n)
{
  char* s1Lowered, *s2Lowered;
  int i, result;

  s1Lowered = (char*)malloc (strlen (s1) + 1);
  s2Lowered = (char*)malloc (strlen (s2) + 1);

  for (i=0; i < strlen (s1); i++)
     s1Lowered [i] = tolower (s1 [i]);
  s1Lowered [i] = 0;

  for (i=0; i < strlen (s2); i++)
    s2Lowered [i] = tolower (s2 [i]);
  s2Lowered [i] = 0;

  result = strncmp (s1Lowered, s2Lowered, n);

  free (s1Lowered);
  free (s2Lowered);
  
  return result;

} /* strcasecmp */
/*---------------------------------------------------------------------------*/
#ifdef EMBEDDED_TEST
/*---------------------------------------------------------------------------*/
int main (int argc, char **argv)
{

  if (argc != 4) {
     fprintf (stderr, "usage:   %s string1 string2 n\n", argv [0]);
     fprintf (stderr, "example  %s abcdefg ABCDEFh 4\n", argv [0]);
     exit (1);
     }

  printf ("strcasecmp (%s, %s) = %d\n", argv [1], argv [2], 
          strcasecmp (argv [1], argv [2]));
  printf ("strncasecmp (%s, %s) = %d\n", argv [1], argv [2], 
          strncasecmp (argv [1], argv [2], atoi (argv [3])));

  return 0;

} /* main */
/*---------------------------------------------------------------------------*/
#endif

