/*
 * Copyright (c) 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
#include <InterViews/enter-scope.h>
#include <stdio.h>
#include <string.h>

class MetricUnit {
public:
    const char* _name;
    float _conversion;
};
 
MetricUnit metric_unit[] = {
    {"pt", 1.0 },
    {"pts", 1.0 },
    {"point", 1.0 },
    {"points", 1.0 },
    {"in", 72.0 },
    {"ins", 72.0 },
    {"inch", 72.0 },
    {"inches", 72.0 },
    {"cm", 72.0 / 2.54 },
    {"mm", 72.0 / 25.4 },
    {"pc", 72.0 / 12 },
    {"pica", 72.0 / 12 },
    {nil, 0 }
};


float convert_metric (const char* value) 
    {
    if (value == nil || strlen(value) == 0)
        return 0;

    float f;
    char unit[100];
    /* copy value into local buffer to workaround silly sscanf */
    char buf[100];

    strncpy(buf, value, sizeof(buf));
    buf[sizeof(buf) - 1] = '\0';

    if (sscanf(buf, "%g %s", &f, unit) == 2) 
        {
        int i = 0;
        while (metric_unit[i]._name != nil) 
           {
           if (strcmp(metric_unit[i]._name, unit) == 0)
              {
              return f * metric_unit[i]._conversion;
              }
           ++i;
           }
        return 0;
        } 
    else if (sscanf(buf, "%g", &f) == 1) 
        return f;
    else 
        return 0;
    }
 
