
/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <InterViews/session.h>
#include <InterViews/style.h>
#include <string.h>
#include <stdlib.h>

#include "HyperView.h"

static PropertyData props[] = {
    { "*name",       "Hypertext Help" },
    { "*book",       "." },
    { "*pagewidth",  "6.5in" },
    { "*pageheight", "6.5in" },
    { "*geometry",   "+300+300" },
    { "*iconName",   "XHyper" },
    { nil }
};
 
static OptionDesc options[] = {
    { "-pagewidth",  "*pagewidth",  OptionValueNext, "on" },
    { "-pageheight", "*pageheight", OptionValueNext, "on" },
    { "-book", "*book", OptionValueNext, "on" },
    { nil }
};

/*-------------------------------------------------------------------------
  Jim Hemmer
  Hughes STX Corporation
  4/92

  This program demonstrates a hypertext implementation used for an On-Line
  Help system.  The format for the help information is similar to  the
  Maker Markup Language used in the FrameMaker desktop publisher.  
  
  Usage:
	XHyper	[tag] [-pagewidth pw] [-pageheight ph]

  where the tag defines the hypertext tag for the initial display.
  Pagewidth is the width of the page (ex. 6.5in, 600pt, etc.).
  Pageheight is the height of the page (ex. 6.5in, 600pt, etc.).
 *-------------------------------------------------------------------------*/

main(int argc, char** argv)
   {
   char	       *tag;

   //
   //  *** check if tag argument is present
   //
   if ((argc > 1) && (argv[1][0] != '-'))
      tag = argv[1];


   if (getenv("HYPERDOC") == nil)
      putenv("HYPERDOC=.");

   //
   //  *** create InterViews session
   //
   Session*  session = new Session("ON-LINE HELP", argc, argv, options, props);
   session->run_window(new HyperViewer(tag));
   }


