/* $Id: checkitem.C,v 1.2 90/06/17 18:45:14 gjb Exp $
 *  These are the implementations for CheckItems and CheckStates.		*
 *									*
 *  By Greg Brail. Copyright 1990 Brown University.			*/

#include <InterViews/message.h>
#include "checkitem.h"
#include <InterViews/interactor.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <InterViews/painter.h>
#include <InterViews/color.h>
#include <InterViews/canvas.h>
#include <InterViews/shape.h>
#include <InterViews/font.h>
#include <InterViews/world.h>
#include <InterViews/bitmap.h>
#include <InterViews/glue.h>
#include <InterViews/subject.h>
#include <InterViews/banner.h>
#include <InterViews/frame.h>

#define padding 3		/* What the fuck? */

const int border = 4;

CheckState::CheckState(boolean initVal)		// initial value
     : ()

{
  value = initVal;
}

void CheckState::SetValue(boolean newVal)

{
  value = newVal;
  Notify();
}

CheckItem::CheckItem(const char* name,			// name of item
		     Command menuFunction,		// function to call
		     void* what,			// object to use
		     char* key,				// Optional command 
		     CheckState* s)			// State to attach to
     : (name, menuFunction, what, key)

{
  checkIcon = nil;
  state = s;
  if (state == nil)
    value = false;
  else
    {
      value = state->GetValue();
      state->Attach(this);
    }
  checkIcon = new Bitmap((void*)check_bits, check_width, check_height);
}

CheckItem::~CheckItem()

{
  if (state != nil)
    state->Detach(this);		// hik : prevents seg. fault
  delete checkIcon;
}

void CheckItem::Do()

{
  if (state != nil)
    {
      if (value)
	value = false;
      else
	value = true;
    }
  if (whatFunction != nil)
    (*whatFunction)(whatObject);
}

void CheckItem::Update()

{
  boolean newVal = state->GetValue();
  if (newVal != value)
    value = newVal;
}

void CheckItem::Reconfig()

{
  SetClassName("CheckItem");
  CommandItem::Reconfig();
  Font* theFont = output->GetFont();
  shape->width = theFont->Width(text) + (3 * padding) + checkIcon->Width() + (2 * border);
  if (whatKey != "")
    shape->width += theFont->Width(whatKey) + (2 * padding);
  shape->height = max(theFont->Height(), checkIcon->Height()) + (2 * padding) + (2 * border);
  shape->hshrink = (2 * padding);
  shape->hstretch = hfil;
  shape->vshrink = (2 * padding);
  shape->vstretch = vfil;
}

void CheckItem::Redraw(Coord l, Coord b, Coord r, Coord t)

{
  Color* origFg = output->GetFgColor();
  Color* origBg = output->GetBgColor();
  Color* dimBg = new Color("grey40");
  if (!dimBg->Valid())
    dimBg = black;
  output->Clip(canvas, l, b, r, t);
  output->ClearRect(canvas, l, b, r, t);
  Coord xPath[7];
  Coord yPath[7];
  if (highlighted)
    {
      output->SetColors(white, origBg);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(white, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      output->SetColors(dimBg, origBg);
      output->FillRect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origFg, origBg);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(origBg, dimBg);
      output->Text(canvas, text, padding + border, padding + border);
      if (whatKey != "")
	{
	  int keyW = output->GetFont()->Width(whatKey);
	  output->Text(canvas, whatKey, 
		       xmax - padding - keyW - border - checkIcon->Width(),
		       padding + border);
	}       
    }
  else
    {
      output->SetColors(white, origBg);
      xPath[0] = 0; 		yPath[0] = 0;
      xPath[1] = border; 	yPath[1] = border;
      xPath[2] = border; 	yPath[2] = ymax - border;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax; 		yPath[4] = ymax;
      xPath[5] = 0;	       	yPath[5] = ymax;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->SetColors(black, origBg);
      output->Line(canvas, 0, ymax, border, ymax - border);
      xPath[0] = 0;		yPath[0] = 0;
      xPath[1] = xmax;		yPath[1] = 0;
      xPath[2] = xmax;		yPath[2] = ymax;
      xPath[3] = xmax - border; yPath[3] = ymax - border;
      xPath[4] = xmax - border;	yPath[4] = border;
      xPath[5] = border;	yPath[5] = border;
      xPath[6] = 0;		yPath[6] = 0;
      output->FillPolygon(canvas, xPath, yPath, 7);
      output->Rect(canvas, border, border, xmax - border, ymax - border);
      output->SetColors(white, origBg);
      output->Line(canvas, xmax, 0, xmax - border, border);
      if (!enabled)
	output->SetColors(dimmed, origBg);
      else
	output->SetColors(origFg, origBg);
      output->Text(canvas, text, padding + border, padding + border);
      if (whatKey != "")
	{
	  int keyW = output->GetFont()->Width(whatKey);
	  output->Text(canvas, whatKey, 
		       xmax - padding - keyW - border - checkIcon->Width(), 
		       padding + border);
	}       
    }
  output->NoClip();
  output->SetColors(origFg, origBg);
  if (value)
    {
      if (highlighted)
	{
	  Color* oldFg = output->GetFgColor();
	  Color* newBg = new Color("grey40");
	  if (!newBg->Valid())
	    newBg = black;
	  output->SetColors(output->GetBgColor(), newBg);
	  output->Stencil(canvas, xmax - checkIcon->Width() - padding - border, 
			  padding + border, checkIcon);
	  output->SetColors(oldFg, output->GetFgColor());
	}
      else
	output->Stencil(canvas, xmax - checkIcon->Width() - padding - border, 
			padding + border, checkIcon);
    }
  output->Rect(canvas, l, b, r, t);
}
