
#include <InterViews/action.h>
#include <InterViews/background.h>
#include <InterViews/bitmap.h>
#include <InterViews/border.h>
#include <InterViews/box.h>
#include <InterViews/center.h>
#include <InterViews/color.h>
#include <InterViews/glue.h>
#include <InterViews/margin.h>
#include <InterViews/session.h>
#include <InterViews/stencil.h>
#include <InterViews/style.h>
#include <InterViews/window.h>

#include <IV-look/bevel.h>
#include <IV-look/button.h>
#include <IV-look/choice.h>
#include <IV-look/kit.h>
#include <IV-look/menu.h>

#include "./GenericButton.h"
#include "./RadioButton.h"
#include "./RadioButtonBox.h"
#include "./RadioButtonSet.h"
#include "./ToggleButton.h"
#include "./UIKit.h"

#include <iostream.h>

class testClass {
public:
    testClass();
    ~testClass();
    void saveButton(GenericButton* g);
    void saveBox(RadioButtonBox* r);
    void toggleUp();
    void toggleDown();
    void disableToggle();
    void disableBox();
private:
    GenericButton* _button;
    boolean _buttonEnable;
    boolean _boxEnable;
    RadioButtonBox* _box;
};

declare(ActionCallback,testClass)
implement(ActionCallback,testClass)


testClass::testClass() {
    _buttonEnable = true;
    _boxEnable = true;
}

testClass::~testClass() {

}

void testClass::saveButton(GenericButton* g) {
    _button = g;
}

void testClass::saveBox(RadioButtonBox* r) {
    _box = r;
}

void testClass::toggleDown() {
//    cout << "Toggle Button is in!" << endl;
}

void testClass::toggleUp() {
//    cout << "Toggle Button is out!" << endl;
}

void testClass::disableToggle() {
    _buttonEnable = !_buttonEnable;
    _button->enable(_buttonEnable);
}

void testClass::disableBox() {
    _boxEnable = !_boxEnable;
    _box->enableAllButtons(_boxEnable);
}


int main(int argc, char** argv) {
    Session* session = new Session("demo", argc, argv);
    Style* style = session->style();
    Kit* kit = Kit::instance();
    UIKit* uiKit = UIKit::instance();

    Glyph* vspace = new VGlue(20.0);
    Glyph* hspace = new HGlue(10.0);

    testClass* tester = new testClass();

    Menu* fileMenu = kit->pulldown(style);
    fileMenu->add_item(
        kit->simple_menu_item("Disable Toggle", style), 
        new ActionCallback(testClass) (tester,&testClass::disableToggle),
    );
    fileMenu->add_item(kit->menu_item_separator(style));
    fileMenu->add_item(kit->simple_menu_item("Quit", style), kit->quit());

    Menu* menuBar = kit->menubar(style);
    menuBar->add_item(kit->simple_menubar_item("File", style), fileMenu);

    RadioButtonBox* rBox = new RadioButtonBox(style);
    rBox->addButton("1200", nil, nil);
    rBox->addButton("2400", nil, nil);
    rBox->addButton("4800", nil, nil);
    rBox->addButton("9600", nil, nil);
    rBox->chooseButtonIndex(3);
    tester->saveBox(rBox);

    rBox->enableButton(2, false);

    // read the bitmap data file into a new bitmap object
    Bitmap* color1Bitmap = Bitmap::open("control.image");

    if (color1Bitmap == nil) {
	cout << " File: control.image   --> problems" << endl;
    }

    GenericButton* toggle = uiKit->toggle_button(
        new Stencil(color1Bitmap, style->foreground()),
        style, 
        new ActionCallback(testClass) (tester,&testClass::toggleDown),
        new ActionCallback(testClass) (tester, &testClass::toggleUp)
    );
    tester->saveButton(toggle);

    GenericButton* disabler = uiKit->simple_push_button(
        "Flatten Them!",
        style,
        new ActionCallback(testClass) (tester, &testClass::disableBox)
    );

    Glyph* dialog = new Background(
	new TBBox(
	    menuBar,
	    new Margin(kit->label("Radio Buttons", style), 15.0),
            new LRBox(hspace, rBox, hspace),
	    new Margin(kit->label("Toggle Button", style), 15.0),
            new LRBox(hspace, toggle, hspace),
	    new Margin(kit->label("Disable Radio Buttons", style), 15.0),
            new LRBox(hspace, disabler, hspace),
            vspace
	),
	style->flat()
    );

    Window* w = new ApplicationWindow(dialog);

    session->run_window(w);

    cout << "Button Status Report" << endl;
    cout << "\nchosen radio button index: " << rBox->chosenButtonIndex()
         << "\n" << endl;

    cout << "radio button 1: " 
         << (rBox->buttonEnabled(1) ? "enabled" : "disabled")
         << endl;

    cout << "radio button 2: " 
         << (rBox->buttonEnabled(2) ? "enabled" : "disabled")
         << endl;

    cout << "radio button 3: " 
         << (rBox->buttonEnabled(3) ? "enabled" : "disabled")
         << endl;

    cout << "radio button 4: " 
         << (rBox->buttonEnabled(4) ? "enabled" : "disabled")
         << "\n" << endl;

    cout << "toggle button: " 
         << (toggle->enabled() ? "enabled" : "disabled")
         << endl;

    cout << "toggle button state: " 
	 << (toggle->chosen() ? "true" : "false") 
         << "\n" << endl;

    delete tester;
    delete w;
    delete session;
    return 0;
}
