/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Main class for drawing editor, derived from Editor class.
 */

#ifndef dwged_h
#define dwged_h

#include <Unidraw/editor.h>

class Box;
class BrushVar;
class ColorVar;
class Command;
class CompNameVar;
class ControlState;
class EditorInfo;
class FontVar;
class GraphicComp;
class GravityVar;
class MagnifVar;
class ModifStatusVar;
class PatternVar;
class PulldownMenu;
class Tray;

class DrawingEditor : public Editor {
public:
    DrawingEditor(GraphicComp* = nil);
    virtual ~DrawingEditor();

    GraphicComp* GetGraphicComp();

    virtual Component* GetComponent();
    virtual Viewer* GetViewer(int = 0);
    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();
    virtual StateVar* GetState(const char*);

    virtual void SetComponent(Component*);
    virtual void SetViewer(Viewer*, int = 0);
    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);
private:
    Interactor* Interior();
    void InitEditorInfo();
    void InitStateVars();
    void InitViewer();

    Interactor* Commands();
    Interactor* InitCommands();
    Interactor* ReadCommands();

    Interactor* Tools();
    Interactor* InitTools();
    Interactor* ReadTools();

    void Include(Command*, PulldownMenu*, const char*, const char*);
    void Separator(PulldownMenu*, const char* = nil);
    void Include(Tool*, Box*, const char*);

    PulldownMenu* FileMenu();
    PulldownMenu* EditMenu();
    PulldownMenu* StructureMenu();
    PulldownMenu* FontMenu();
    PulldownMenu* BrushMenu();
    PulldownMenu* PatternMenu();
    PulldownMenu* ColorMenu(const char* name, const char* attrib);
    PulldownMenu* AlignMenu();
    PulldownMenu* ViewMenu();
    PulldownMenu* ReadMenu();
private:
    GraphicComp* _comp;
    KeyMap* _keymap;
    ControlState* _curCtrl;
    Viewer* _viewer;
    Selection* _selection;
    Tray* _tray;

    EditorInfo* _edInfo;
    boolean _initEdInfo;

    CompNameVar* _name;
    ModifStatusVar* _modifStatus;
    GravityVar* _gravity;
    MagnifVar* _magnif;
    FontVar* _font;
    BrushVar* _brush;
    PatternVar* _pattern;
    ColorVar* _color;
};

inline GraphicComp* DrawingEditor::GetGraphicComp () { return _comp; }

#endif
