// Interviews 3.0 test program
#include <IV-look/button.h>
#include <IV-look/kit.h>
#include <IV-look/menu.h>

#include <InterViews/background.h>
#include <InterViews/label.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>
#include <InterViews/glue.h>
#include <InterViews/box.h>
#include <InterViews/margin.h>
#include <InterViews/center.h>
#include <InterViews/display.h>
#include <InterViews/action.h>

#include <OS/list.h>
#include <OS/string.h>

#include "t2.h"

static PropertyData props[] =
  {
      {"*quitbutton","Quit"},
      {"*font", "9x15"},
      {"*openlook","on"},
      {"*background","blue"},
      {"*foreground","white"},
      {nil}
  } ;

static OptionDesc options[] =
  {
    {"button=","*quitbutton",OptionValueAfter},
    {"-openlook","*openlook",OptionValueImplicit,"on"},
    {nil}
  } ;

Glyph* vspace = new VGlue(18.0) ;
Glyph* hspace = new HGlue(36.0) ;

declare(ActionCallback,TestWindow) ;
implement(ActionCallback,TestWindow) ;
declare(ActionCallback,MyWindow) ;
implement(ActionCallback,MyWindow) ;
declareList(MyWindowList,MyWindow*) ;
implementList(MyWindowList,MyWindow*) ;

// -------------------------------------------------------------------
void DoBeep::execute()
  {
    Display* d = session->default_display() ;
    d->ring_bell(0) ;
  }

// -------------------------------------------------------------------
TestWindow::TestWindow(Session* s)
  {
    session = s ;
    style = session->style();
    kit = Kit::instance() ;
    myWindowCount = 0 ;
    myWinList = new MyWindowList ;
    Create() ;
    w->map() ;
  }

TestWindow::~TestWindow()
  {
    w->unmap() ;
    delete myWinList ;
  }

Glyph* TestWindow::MakeLRBox(Glyph* g)
  {
    return new LRBox(
      hspace,
      g,
      hspace
    ) ;
  }

Glyph* TestWindow::bevels()
  {
    return 
      new LRBox(
	hspace,
        new TBBox(
          MakeLRBox(kit->inset_frame(kit->label("Inset_frame",style),style)),
	  vspace,
          MakeLRBox(kit->outset_frame(kit->label("Outset_frame",style),style)),
	  vspace,
	  MakeLRBox(kit->flat_frame(kit->label("Flat_frame",style),style))
        ),
      hspace
    ) ;
  }
 
Glyph* TestWindow::buttons()
  {
     String quit_label ;
     style->find_attribute("quitbutton",quit_label) ;

     Action* myCallback = new ActionCallback(TestWindow)
       (this,&TestWindow::CreateMyWindow) ;

     // Callback<TestWindow>* myCallback = new Callback(this,&CreateMyWindow) ;

     Action* quitCall = new ActionCallback(TestWindow)
       (this,&TestWindow::Quit) ;

     return new LRBox(
       hspace,
       kit->simple_push_button("Beep",style,new DoBeep(session)),
       hspace,
       kit->simple_push_button("My Window",style,myCallback),
       hspace,
       kit->simple_push_button(quit_label,style,quitCall),
       hspace
     ) ;
  }

Glyph* TestWindow::menu()
  {
    Menu* menu = kit->menubar(style) ;

    Menu* p1 = kit->pulldown(style) ;
    Action* p1_action = new ActionCallback(TestWindow)
      (this,&TestWindow::Menu_Open) ;
    p1->add_item(kit->menu_item(kit->label("Open...",style),style),p1_action) ;

    Telltale* f_menu = kit->menubar_item(kit->label("File",style),style) ;
    menu->add_item(f_menu,p1) ;

    Menu* p2 = kit->pulldown(style) ;
    Action* p2_action = new ActionCallback(TestWindow)
      (this,&TestWindow::Quit) ;
    p2->add_item(kit->menu_item(kit->label("Quit",style),style),p2_action) ;

    Telltale* f_menu2 = kit->menubar_item(kit->label("Other",style),style) ;
    menu->add_item(f_menu2,p2) ;

    return menu ;
  }

void TestWindow::Menu_Open()
  {

  }

void TestWindow::Create()
  {
    Glyph* inside = 
      new Background(
	new TBBox(
          menu(),
	  vspace,
	  bevels(),
	  vspace,
          buttons(),
          vspace
        ),
       style->flat()
     ) ;

    w = new ApplicationWindow(inside) ;
  }

void TestWindow::Quit()
  {
    if (myWindowCount > 0)
      {    
	Display* d = session->default_display() ;
        d->ring_bell(0) ;
      }
    else
      kit->quit()->execute() ;
  }

void TestWindow::CreateMyWindow()
  {     
     Action* quitCallback = new ActionCallback(TestWindow)
       (this,&TestWindow::QuitMyWindow) ;

     MyWindow* myWindow = new MyWindow(session,quitCallback) ;
     myWindowCount++ ;
     myWinList->append(myWindow) ;
  }

void TestWindow::QuitMyWindow()
  {
    myWindowCount-- ;
  }

// --------------------------------------------------------------------
MyWindow::MyWindow(Session* s, Action* q) : session(s), quitAction(q)
  {
    style = session->style();
    kit = Kit::instance() ; 
    win = new TopLevelWindow(Build()) ;
    win->map() ;  
  }

MyWindow::~MyWindow()
  {
    delete win ;
  }

Glyph* MyWindow::Build()
  {
    Action* closeAction = new ActionCallback(MyWindow)(this,&MyWindow::Remove) ;

    return new Background(
      new TBBox(
	vspace,
        new LRBox(
          hspace,
          kit->outset_frame(kit->label("This is a MyWindow",style),style),
          hspace
        ),
        vspace,
        new LRBox(
          hspace,
          kit->simple_push_button("Close",style,closeAction),
          hspace		  
        ),
        vspace
      ),
      style->flat()
    ) ;
  }

void MyWindow::Remove()
  {
    quitAction->execute() ;    
    win->unmap() ;
  }

int main(int argc, char** argv) 
  {
    Session* session = new Session("Test", argc, argv,options,props);

    TestWindow* t = new TestWindow(session) ;
    
    session->run() ;

    delete t ;
    delete session ;
    return 0 ;
  }
