/*  class ClassGraph: member functions.		     
Copyright 1991, Brown University, Providence, RI

Permission to use and modify this software and its documentation for any
purpose other than its incorporation into a commercial product is hereby
granted without fee.  Permission to copy and distribute this software and its
documentation for non-commercial use only is also granted without fee,
provided, however, that the above copyright notice appear in all copies, that
both that copyright notice and this permission notice appear in supporting
documentation, that the name of Brown University not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission, and that Brown University be notified by the distributor when
such distributions are done outside of his or her organization. Brown
University makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied warranty.
Brown University requests notification of any modifications to this software
or its documentation.

Please send the name, organization, and address (postal and electronic) of
recipients of software outside of the receiving organization to:

	Software Librarian
	Computer Science Department, Box 1910
	Brown University
	Providence, RI 02912
		or
	brusd@cs.brown.edu

All electronic notifications will be acknowledged.
*/
 

#include <stdio.h>
#include <stream.h>
#include <fstream.h>
#include <stdlib.h>
#include <sysent.h>
#include <vfork.h>
#include <string.h>
#include <math.h>
#include "classgraph.h"
#include <InterViews/geometry.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/label.h>
#include <IV-look/button.h>
#include <IV-look/scrollbar.h>
#include <InterViews/rule.h>
#include <IV-look/menu.h>
#include <InterViews/transformer.h>
#include <InterViews/printer.h>
#include <InterViews/color.h>
#include <InterViews/world.h>
#include <InterViews/background.h>
#include <InterViews/listener.h>
#include <InterViews/sensor.h>
#include <InterViews/window.h>
#include <InterViews/fixedspan.h>
#include <InterViews/bitmap.h>
#include <IV-look/mover.h>
#include <IV-look/bevel.h>
#include <InterViews/action.h>
declare(ActionCallback,ClassGraph);
#include <InterViews/Bitmaps/enlarge.bm>
#include <InterViews/Bitmaps/reducer.bm>
#include "glyphport.h"
#include "hiergraph.h"
#include "classnode.h"
#include "menulabel.h"

#include "classbuffer.h"
#include "classinfo.h"
#include "iclass.h"
#include "globals.h"

extern "C" char* strdup(const char*);

#define CLOSE_LBL "^W" 
#define CLOSE_CODE '\027'

#define PRINT_LBL "^P" 
#define PRINT_CODE '\020'

#define QUIT_LBL "^Q" 
#define QUIT_CODE '\021'

#define TOP_LBL "^T" 
#define TOP_CODE '\024'

#define FULL_LBL "^F" 
#define FULL_CODE '\006'

static const int moverHeight = 18;
static const Coord pageWidth = 8 * 72;
static const Coord pageHeight = 10.5 * 72;
static const double scaleStep = 1.41421356;	// Square root of 2

ClassGraph::ClassGraph(ClassBuffer* buf, IClass* win) : Patch(nil)

{
  theBuf = buf;
  mainWindow = win;
  theState = 0;
  curScale = 1.0;
  currentPick = nil;
  horiz = false;
  theCommand = new ActionCallback(ClassGraph)(this, ClassGraph::ShowClass);
  MakeDisplay();
  contents = new GlyphPort(typeDisplay);
  Pulldown* file = new Pulldown();
  file->add(new MenuLabel("Close Window", CLOSE_LBL),
	    new ActionCallback(ClassGraph)(this, ClassGraph::CloseCmd));
  file->add(new MenuLabel("Print Window...", PRINT_LBL),
	    new ActionCallback(ClassGraph)(this, ClassGraph::PrintCmd));
  file->add(new MenuLabel("Remove Window", QUIT_LBL),
	    new ActionCallback(ClassGraph)(this, ClassGraph::QuitCmd));
  Pulldown* display = new Pulldown();
  display->add(new MenuLabel("Make Root Node", TOP_LBL),
	       new ActionCallback(ClassGraph)(this, ClassGraph::TopCmd));
  display->add(new MenuLabel("Show Full Graph", FULL_LBL),
	       new ActionCallback(ClassGraph)(this, ClassGraph::FullCmd));
  display->add(new MenuLabel("Lay Out Horizontally"),
	       new ActionCallback(ClassGraph)(this, ClassGraph::HorizCmd));
  displayMenu = display;
  horizIndex = 2;
  ivMenuBar* theBar = new ivMenuBar();
  theBar->add(new MenuLabel("File"), file);
  theBar->add(new MenuLabel("Display"), display);
  Color* fg = World::current()->foreground();
  Color* bg = World::current()->background();
  Bitmap* enlargeBits = new Bitmap(enlarger_plain_bits, enlarger_plain_width,
				   enlarger_plain_height);
  Bitmap* reduceBits = new Bitmap(reducer_plain_bits, reducer_plain_width,
				  reducer_plain_height);
  Glyph* interior = new Background(
	new LRBox(
	new TBBox(
	  new LRBox(
		    theBar,
		    new ivHGlue(8 * inch, fil, 7 * inch),
		    new Bevel(
		      new LRBox(
				new IconButton(new ActionCallback(ClassGraph)
					       (this, ClassGraph::ZoomInCmd),
					       enlargeBits),
				new ivHGlue(2, 0, 0),
				new IconButton(new ActionCallback(ClassGraph)
					       (this, ClassGraph::ZoomOutCmd),
					       reduceBits)
				),
			      new Inset(bg)
				),
		    ),
		  contents,
	  new FixedSpan(new HScrollBar(contents, bg), Dimension_Y, moverHeight)
		  ),
	  new FixedSpan(new VScrollBar(contents, bg), Dimension_X, moverHeight)
		  ),
		  bg);
  listener = new Listener(interior, this);
  listener->sensor()->motion(false);
  listener->sensor()->key(true);
  listener->sensor()->button(false);
  body(listener);
  World::current()->SetHint("TransformFonts:cache");
  myWindow = new TopLevelWindow(this);
  myWindow->name("Class Hierarchy");
  myWindow->icon_name("Class Hierarchy");
  myWindow->map();
}

void ClassGraph::MakeDisplay()

{
  allocNodes = 64;
  allocRoots = 16;
  numNodes = 0;
  numRoots = 0;
  theNodes = (HierNode**)malloc(sizeof(HierNode*) * allocNodes);
  theRoots = (HierNode**)malloc(sizeof(HierNode*) * allocRoots);
  int curClass;
  char* curClassName;
  ClassInfo* curClassInfo;
  for (curClass = 0; curClass < theBuf->NumClasses(); curClass++)
    {
      curClassName = strnew(theBuf->Class(curClass));
      curClassInfo = theBuf->Info(curClassName);
      theNodes[numNodes] =  new ClassNode(curClassName, &theState, &currentPick,
					  numNodes, theCommand);
      numNodes++;
      if (numNodes >= allocNodes)
	{
	  allocNodes += 64;
	  theNodes = (HierNode**)realloc((char*)theNodes, 
					 sizeof(HierNode*) * allocNodes);
	}
    }
  int curParent;
  int curChild;
  char* parentName;
  char* childName;
  int i;
  for (int inc = 0; inc < numNodes; inc++)
    {
      curClassInfo = theBuf->Info(theNodes[inc]->getName());
      for (curParent = 0; curParent < curClassInfo->NumParents(); curParent++)
	{
	  parentName = strnew(curClassInfo->Parent(curParent)->Name());
	  for (i = 0; (i < numNodes) && 
	       (strcmp(parentName, theNodes[i]->getName()) != 0); i++);
	  if (i < numNodes)
	    theNodes[inc]->AddPredecessor(theNodes[i]);
	}
      for (curChild = 0; curChild < curClassInfo->NumChildren(); curChild++)
	{
	  childName = strdup(curClassInfo->Child(curChild)->Name());
	  for (i = 0; (i < numNodes) && 
	       (strcmp(childName, theNodes[i]->getName()) != 0); i++);
	  if (i < numNodes)
	    theNodes[inc]->AddSuccessor(theNodes[i]);
	}
      if (theNodes[inc]->GetPredecessors()->Size() == 0)
	{
	  theRoots[numRoots] = theNodes[inc];
	  numRoots++;
	  if (numRoots >= allocRoots)
	    {
	      allocRoots += 16;
	      theRoots = (HierNode**)realloc((char*)theRoots, 
					     sizeof(HierNode*) * allocRoots);
	    }
	}
    }
  theNodes[0]->SetPick(theNodes[0]->telltale());
  typeDisplay = new HierGraph(&theState, numNodes, theNodes, 
			      numRoots, theRoots, horiz); 
}

void ClassGraph::event(Event& theEvent)

{
  if (theEvent.type() == Event::key)
    {
      char keyString[2];
      theEvent.mapkey(keyString, 1);
      Command(theEvent.keystring[0]);
    }
}

boolean ClassGraph::Command (char c)

{
  boolean exec = true;
  switch (c)
    {
    case CLOSE_CODE:		CloseCmd(); break;
    case PRINT_CODE: 		PrintCmd(); break;
    case QUIT_CODE:		QuitCmd(); break;
    case TOP_CODE:	        TopCmd(); break;
    case FULL_CODE:		FullCmd(); break;
    default:			exec = false; break;
    }
  return exec;
}
  
void ClassGraph::CloseCmd()

{
  myWindow->iconify();
}

void ClassGraph::PrintCmd()

{
  float xScale = 1.0;		// Figure it out later
  float yScale = 1.0;
  Transformer pTrans;
  char fileName[100];
  char lprArgs[20];
  sprintf(fileName, "/tmp/gclass%i", getpid());
  ofstream out(fileName);
  Printer* printer = new Printer(&out);
  printer->prolog("InterViews Class Browser");
  if ((xScale < 1.0) || (yScale < 1.0))
    {
      if (xScale < yScale)
	pTrans.scale(xScale, xScale);
      else
	pTrans.scale(yScale, yScale);
      printer->transform(pTrans);
    }
  Allocation alloc;
  Allotment width(typeDisplay->left(), 
		  (typeDisplay->right() - typeDisplay->left()), 0.0);
  Allotment height(typeDisplay->bottom(),
		   (typeDisplay->top() - typeDisplay->bottom()), 0.0);
  alloc.allot(Dimension_X, width);
  alloc.allot(Dimension_Y, height);
  typeDisplay->print(printer, alloc);
  printer->epilog();
  delete printer;
  out.close();
  if (getenv("PRINTER") == nil)
    strcpy(lprArgs, "");
  else
    sprintf(lprArgs, "-P%s", getenv("PRINTER"));
  if (vfork() == 0)
    execl("/usr/ucb/lpr", lprArgs, fileName, (char*)nil);
  wait(nil);
  unlink(fileName);
}

void ClassGraph::QuitCmd()

{
  myWindow->unmap();
  delete this;
}

void ClassGraph::TopCmd()

{
  HierNode* curNode = typeDisplay->FindDisp(theState);
  typeDisplay->NewRoot(curNode);
}

void ClassGraph::FullCmd()

{
  typeDisplay->FullView();
}

void ClassGraph::HorizCmd()

{
  delete typeDisplay;
  horiz = true;
  delete theNodes;
  delete theRoots;
  MakeDisplay();
  contents->body(typeDisplay);
  redraw();
  displayMenu->set(horizIndex, new MenuLabel("Lay Out Vertically"),
		   new ActionCallback(ClassGraph)(this, ClassGraph::VertCmd));
}

void ClassGraph::VertCmd()

{
  delete typeDisplay;
  horiz = false;
  delete theNodes;
  delete theRoots;
  MakeDisplay();
  contents->body(typeDisplay);
  redraw();
  displayMenu->set(horizIndex, new MenuLabel("Lay Out Horizontally"),
		   new ActionCallback(ClassGraph)(this, ClassGraph::HorizCmd));
}

void ClassGraph::ZoomInCmd()

{
  curScale *= scaleStep;
  contents->zoom_to(curScale);
}

void ClassGraph::ZoomOutCmd()

{
  curScale /= scaleStep;
  contents->zoom_to(curScale);
}

void ClassGraph::Update()

{
  MakeDisplay();
  contents->body(typeDisplay);
  redraw();
}

void ClassGraph::ShowClass()

{
  mainWindow->UpdateCurClass(theState);
  mainWindow->UpdateClassesBrowser();
  mainWindow->UpdateParentBrowser();
  mainWindow->UpdateChildBrowser();
}


implement(ActionCallback,ClassGraph);

