/*  class ClassGraph:						*
 *								*
 *    The ClassGraph class is a window that draws the graph of	*
 *  the class hierarchy.					*
Copyright 1991, Brown University, Providence, RI

Permission to use and modify this software and its documentation for any
purpose other than its incorporation into a commercial product is hereby
granted without fee.  Permission to copy and distribute this software and its
documentation for non-commercial use only is also granted without fee,
provided, however, that the above copyright notice appear in all copies, that
both that copyright notice and this permission notice appear in supporting
documentation, that the name of Brown University not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission, and that Brown University be notified by the distributor when
such distributions are done outside of his or her organization. Brown
University makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied warranty.
Brown University requests notification of any modifications to this software
or its documentation.

Please send the name, organization, and address (postal and electronic) of
recipients of software outside of the receiving organization to:

	Software Librarian
	Computer Science Department, Box 1910
	Brown University
	Providence, RI 02912
		or
	brusd@cs.brown.edu

All electronic notifications will be acknowledged.
*/

#ifndef CLASSGRAPH_H
#define CLASSGRAPH_H

#include <InterViews/patch.h>
#include <InterViews/handler.h>
#include <stream.h>

class HierGraph;
class ClassBuffer;
class ButtonState;
class IClass;
class Event;
class Action;
class GlyphPort;
class Listener;
class Telltale;
class ManagedWindow;
class Pulldown;
class GlyphPort;
class HierNode;

class ClassGraph : public Patch, public Handler

{
public:
  ClassGraph(ClassBuffer* buf, 
	     IClass* win);	// Constructor
  void MakeDisplay();			// Make the hierarchy display
  virtual void event(Event&);
  virtual void Update();
  void ShowClass();		   	// Show the current class
  void CloseCmd();
  void PrintCmd();
  void QuitCmd();
  void TopCmd();
  void FullCmd();
  void HorizCmd();
  void VertCmd();
  void ZoomInCmd();
  void ZoomOutCmd();
private:
  boolean Command(char);
  HierGraph* typeDisplay;		// The actual display of the types
  ClassBuffer* theBuf;			// The ClassBuffer used here
  int theState;
  Action* theCommand;
  GlyphPort* contents;
  IClass* mainWindow;
  Listener* listener;
  Telltale* currentPick;
  ManagedWindow* myWindow;
  Pulldown* displayMenu;
  int horizIndex;
  HierNode** theNodes;
  HierNode** theRoots;
  int numNodes;
  int numRoots;
  int allocNodes;
  int allocRoots;
  float curScale;
  boolean horiz;
};

#endif




