/*  GlyphPort: A Glyph that can be adjusted like an Adjuster.
 *
 * $Id$
Copyright 199X, Brown University, Providence, RI

Permission to use and modify this software and its documentation for any
purpose other than its incorporation into a commercial product is hereby
granted without fee.  Permission to copy and distribute this software and its
documentation for non-commercial use only is also granted without fee,
provided, however, that the above copyright notice appear in all copies, that
both that copyright notice and this permission notice appear in supporting
documentation, that the name of Brown University not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission, and that Brown University be notified by the distributor when
such distributions are done outside of his or her organization. Brown
University makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied warranty.
Brown University requests notification of any modifications to this software
or its documentation.

Please send the name, organization, and address (postal and electronic) of
recipients of software outside of the receiving organization to:

	Software Librarian
	Computer Science Department, Box 1910
	Brown University
	Providence, RI 02912
		or
	brusd@cs.brown.edu

All electronic notifications will be acknowledged.

 */

#ifndef GLYPHPORT_H
#define GLYPHPORT_H

#include <IV-look/adjustable.h>
#include <InterViews/monoglyph.h>
#include <InterViews/geometry.h>

class Transformer;
class Color;

class GlyphPort : public MonoGlyph, public Adjustable

{
public:
  GlyphPort(Glyph*);
  GlyphPort(Glyph*, Coord, Coord);
  virtual ~GlyphPort();
  virtual void request(Requisition&);
  virtual void allocate(Canvas*, Allocation&, Extension&);
  virtual void draw(Canvas*, Allocation&);
  virtual void redraw();
  virtual void print(Printer*, Allocation&);
  virtual boolean pick(Canvas*, Allocation&, Coord x, Coord y,
		       Hit&, int depth);
  virtual Allotment& allotment(int, DimensionName);
  virtual Coord lower(DimensionName);
  virtual Coord upper(DimensionName);
  virtual Coord length(DimensionName);
  virtual Coord cur_lower(DimensionName);
  virtual Coord cur_upper(DimensionName);
  virtual Coord cur_length(DimensionName);
  virtual void scroll_forward(DimensionName);
  virtual void scroll_backward(DimensionName);
  virtual void page_forward(DimensionName);
  virtual void page_backward(DimensionName);
  virtual void scroll_to(DimensionName, Coord);
  virtual void scale_to(DimensionName, float fraction_visible);
  virtual void zoom_to(float magnification);
private:
  void Init();
  void AllocateChild(Canvas*, Allocation&);
  void ReTransform();

  Transformer* difference;
  Requisition interiorReq;
  Allocation interiorAlloc;
  Allocation alloc;
  Extension interiorExt;
  Requirement* horizontal, *vertical;
  Coord transX, transY;
  float scale;
  Canvas* canvas;
  Coord xScrollStep;
  Coord yScrollStep;
};


#endif


