// $Header: /tmp_mnt/ober/pro2ober/interviews/src/bin/gclass/RCS/list.C,v 1.1 89/06/28 13:58:20 gjb Exp $
// implements classes BaseNode and BaseList.
/*
Copyright 1991, Brown University, Providence, RI

Permission to use and modify this software and its documentation for any
purpose other than its incorporation into a commercial product is hereby
granted without fee.  Permission to copy and distribute this software and its
documentation for non-commercial use only is also granted without fee,
provided, however, that the above copyright notice appear in all copies, that
both that copyright notice and this permission notice appear in supporting
documentation, that the name of Brown University not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission, and that Brown University be notified by the distributor when
such distributions are done outside of his or her organization. Brown
University makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied warranty.
Brown University requests notification of any modifications to this software
or its documentation.

Please send the name, organization, and address (postal and electronic) of
recipients of software outside of the receiving organization to:

	Software Librarian
	Computer Science Department, Box 1910
	Brown University
	Providence, RI 02912
		or
	brusd@cs.brown.edu

All electronic notifications will be acknowledged.
*/

#include "list.h"

// BaseNode zeroes its pointers.

BaseNode::BaseNode () {
    prev = nil;
    next = nil;
    valuep = nil;
}

// ~BaseNode frees storage allocated by derived classes.

BaseNode::~BaseNode () {
    Delete();
}

// Delete frees storage allocated by this class and its ancestors.

void BaseNode::Delete () {
    // no storage allocated by base class
}

// GetValue returns the referenced member or nil if valuep is nil.

void* BaseNode::GetValue () {
    return valuep ? *valuep : nil;
}

// BaseList starts with only a header node.

BaseList::BaseList () {
    cur = head = new BaseNode;
    head->prev = head->next = head;
    size = 0;
}

// ~BaseList destroys the list.

BaseList::~BaseList () {
    DeleteAll();
    delete head;
}

// Index returns the node that would be the indexed element if the
// list was a C array (0 = first, 1 = next, ..., size-1 = last) or nil
// if the index is out of bounds.  Index sets the current node to the
// indexed node if it's in the list; otherwise, the current node
// remains the same.

BaseNode* BaseList::Index (int index) {
    BaseNode* elem = nil;
    if (index >= 0 && index < size) {
	elem = head->next;
	for (int i = 0; i < index; i++) {
	    elem = elem->next;
	}
	cur = elem;
    }
    return elem;
}

// Find returns true if the list contains a node with the same value
// as the given pointer (i.e., equal addresses) or false if there is
// no such node.  Find sets the current node to that node only if it
// finds such a node; otherwise, the current node remains the same.

boolean BaseList::Find (void* value) {
    for (BaseNode* elem = head->next; elem != head; elem = elem->next) {
	if (elem->valuep != nil && *elem->valuep == value) {
	    cur = elem;
	    return true;
	}
    }
    return false;
}

// Append inserts a node after the last node of the list.  The current
// node remains the same.

void BaseList::Append (BaseNode* appendee) {
    BaseNode* last = head->prev;
    appendee->prev = last;
    appendee->next = head;
    head->prev = appendee;
    last->next = appendee;
    ++size;
}

// Prepend inserts a node before the first node of the list.  The
// current node remains the same.

void BaseList::Prepend (BaseNode* prependee) {
    BaseNode* first = head->next;
    prependee->prev = head;
    prependee->next = first;
    first->prev = prependee;
    head->next  = prependee;
    ++size;
}

// InsertAfterCur inserts a node after the current node of the list.
// The current node remains the same.

void BaseList::InsertAfterCur (BaseNode* prependee) {
    BaseNode* first = cur->next;
    prependee->prev = cur;
    prependee->next = first;
    first->prev = prependee;
    cur->next   = prependee;
    ++size;
}

// InsertBeforeCur inserts a node before the current node of the list.
// The current node remains the same.

void BaseList::InsertBeforeCur (BaseNode* appendee) {
    BaseNode* last = cur->prev;
    appendee->prev = last;
    appendee->next = cur;
    cur->prev  = appendee;
    last->next = appendee;
    ++size;
}

// RemoveCur removes the current node of the list (if it's a node and
// not the head) and sets the current node to the following node.

void BaseList::RemoveCur () {
    if (cur != head) {
	BaseNode* before = cur->prev;
	BaseNode* after  = cur->next;
	after->prev  = before;
	before->next = after;
	cur = after;
	--size;
    }
}

// DeleteCur deletes the current node of the list (if it's a node and
// not the head) and sets the current node to the following node.

void BaseList::DeleteCur () {
    if (cur != head) {
	BaseNode* before = cur->prev;
	BaseNode* after  = cur->next;
	after->prev  = before;
	before->next = after;
	delete cur;
	cur = after;
	--size;
    }
}

// DeleteAll deletes all the nodes of the list, making the list empty
// again, and sets the current node to the list's head.

void BaseList::DeleteAll () {
    BaseNode* after = nil;
    for (BaseNode* doomed = head->next; doomed != head; doomed = after) {
	after = doomed->next;
	delete doomed;
    }
    cur = head;
    head->prev = head->next = head;
    size = 0;
}
