// $Header: /tmp_mnt/ober/pro2ober/interviews/src/bin/gclass/RCS/list.h,v 1.2 90/03/16 22:29:18 gjb Exp Locker: gjb $
// declares classes BaseNode and BaseList.
/*
Copyright 1991, Brown University, Providence, RI

Permission to use and modify this software and its documentation for any
purpose other than its incorporation into a commercial product is hereby
granted without fee.  Permission to copy and distribute this software and its
documentation for non-commercial use only is also granted without fee,
provided, however, that the above copyright notice appear in all copies, that
both that copyright notice and this permission notice appear in supporting
documentation, that the name of Brown University not be used in advertising or
publicity pertaining to distribution of the software without specific, written
prior permission, and that Brown University be notified by the distributor when
such distributions are done outside of his or her organization. Brown
University makes no representations about the suitability of this software for
any purpose.  It is provided "as is" without express or implied warranty.
Brown University requests notification of any modifications to this software
or its documentation.

Please send the name, organization, and address (postal and electronic) of
recipients of software outside of the receiving organization to:

	Software Librarian
	Computer Science Department, Box 1910
	Brown University
	Providence, RI 02912
		or
	brusd@cs.brown.edu

All electronic notifications will be acknowledged.
*/

#ifndef list_h
#define list_h

#include <InterViews/2.6/InterViews/defs.h>

// A BaseNode contains links to its previous and next BaseNodes.
// BaseLists can link BaseNodes to each other to form lists of
// BaseNodes.

class BaseNode {

    friend class BaseList;

public:

    BaseNode();
    ~BaseNode();

    void* GetValue();

protected:

    virtual void Delete();

    void** valuep;		// points to one of our member variables

private:

    BaseNode* prev;		// points to previous BaseNode in list
    BaseNode* next;		// points to next BaseNode in list

};

// A BaseList maintains and iterates through a list of BaseNodes.

class BaseList {
public:
    BaseList();
    ~BaseList();
    int Size()
    {
      return size;
    }
    boolean AtEnd()
      {
	return cur == head;
      }
    BaseNode* First()
      {
	cur = head->next;
	return cur;
      }
    BaseNode* Last()
      {
	cur = head->prev;
	return cur;
      }
    BaseNode* Prev()
      {
	cur = cur->prev;
	return cur;
      }
    BaseNode* Next()
      {
	cur = cur->next;
	return cur;
      }
    BaseNode* GetCur()
      {
	return cur;
      }
    BaseNode* Index(int);
    boolean Find(void*);
    void Append(BaseNode*);
    void Prepend(BaseNode*);
    void InsertAfterCur(BaseNode*);
    void InsertBeforeCur(BaseNode*);
    void RemoveCur();
    void DeleteCur();
    void DeleteAll();
    BaseNode* operator ++()
      {
	cur = cur->next;
	return cur;
      }
    BaseNode* operator --()
      {
	cur = cur->prev;
	return cur;
      }
private:
    BaseNode* head;		// points to dummy head of circular list
    BaseNode* cur;		// points to current node of circular list
    int size;			// stores number of non-dummy nodes in list
};

#endif
