/* simplebutton.C: Classes for basic buttons. 
 *
 * $Id$
 */

#include "simplebutton.h"

#include <InterViews/bitmap.h>
#include <InterViews/label.h>
#include <IV-look/bevel.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/world.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <IV-look/telltale.h>
#include <IV-look/choice.h>
#include <InterViews/listener.h>
#include <InterViews/sensor.h>
#include <InterViews/event.h>
#include <InterViews/margin.h>
#include <InterViews/stencil.h>

SimpleButton::SimpleButton(Action* a, 
			   Telltale* t) : Button(a, t)

{
}

void SimpleButton::choose(boolean state)

{
  if (telltale_ != nil)
    telltale_->choose(state);
}

void SimpleButton::enable(boolean state)

{
  if (telltale_ != nil)
    telltale_->enable(state);
}

LabelButton::LabelButton(Action* doIt,
			 const char* text) : SimpleButton(doIt, nil)

{
  World* theWorld = World::current();
  Color* fg = theWorld->foreground();
  Color* bg = theWorld->background();
  Label* label = new Label(text, theWorld->font(), fg);
  Margin* frame = new Margin(label, 2, 0, 2, 2, 0, 2, 2, 0, 2, 2, 0, 2);
  telltale_ = new ChoiceItem(
			     new Bevel(frame, new Outset(bg)),
			     new Bevel(frame, new Inset(bg))
			     );
  telltale_->ref();
  listener_->body(telltale_);
}

PlainButton::PlainButton(Action* doIt) : SimpleButton(doIt, nil)

{
  Color* bg = World::current()->background();
  Glyph* box = new TBBox(
			 new LRBox(
				   new HGlue(PlainButtonSize, 0.0, 0.0)
				   ),
			 new VGlue(PlainButtonSize, 0.0, 0.0)
			 );
  telltale_ = new ChoiceItem(
			     new Bevel(box, new Outset(bg)),
			     new Bevel(box, new Inset(bg)),
			     new Bevel(box, new Inset(bg))
			     );
  telltale_->ref();
  listener_->body(telltale_);
}

IconButton::IconButton(Action* doIt,
		       Bitmap* icon) : SimpleButton(doIt, nil)

{
  Color* bg = World::current()->background();
  Color* fg = World::current()->foreground();
  Stencil* theBits = new Stencil(icon, fg);
  telltale_ = new ChoiceItem(
			     new Bevel(theBits, new Outset(bg)),
			     new Bevel(theBits, new Inset(bg)),
			     new Bevel(theBits, new Inset(bg))
			     );
  telltale_->ref();
  listener_->body(telltale_);
}

