#include "ArgListOption.h" /* Automatic prepend ()*^*@# */
// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

#ifndef _ArgListOptionARRAYH
#define _ArgListOptionARRAYH

/*
 * A resizable array of ArgListOptions.
 *
 * If ArgListOption is user-defined, it must have the following operations -
 *
 * class ArgListOption ... {
 *   public:
 *     ArgListOption();
 *     ~ArgListOption();
 *     ...
 *     ArgListOption& operator = (ArgListOption&);
 * };
 *
 * If UNSAFE_ARRAY not defined when including ArgListOptionArray.h, then array bounds
 * and argument range checking will be done.
 *
 */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/28 16:23:54 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	Array.hP,v $
 * Revision 1.3  91/11/28  16:23:54  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.2  91/11/04  19:43:48  sanjay
 * Prepare for release
 * 
 * Revision 1.1  91/06/28  20:19:50  sanjay
 * Initial revision
 * 
 */

class ArgListOptionArray {
  public:

    /* Constructors */
    ArgListOptionArray();				/* Empty array */
    ArgListOptionArray(int predict);		/* Empty array with size prediction */
    ArgListOptionArray(const ArgListOption*, int);		/* Initialized with C array */
    ArgListOptionArray(const ArgListOptionArray&);		/* Initialized with another Array */
    ArgListOptionArray(ArgListOption, int);			/* Fill with n copies of element */

    /* Destructor */
    ~ArgListOptionArray();

    ArgListOptionArray& operator=(const ArgListOptionArray&);	/* Assignment operator */

    /* Array indexing */
    ArgListOption& operator[](int index) const;
    ArgListOption& Slot(int index) const;

    /* Other Array operations */
    int  Size() const;			/* Return size; */
    ArgListOption& High() const;			/* Return last element */
    ArgListOption* AsPointer() const;		/* Return as pointer to base */
    void Append(ArgListOption v);			/* Append an element */
    void Append(ArgListOption, int n);		/* Append n copies of element */
    void Concat(const ArgListOption*, int);	/* Concatenate C array */
    void Concat(const ArgListOptionArray&);	/* Concatenate another Array */
    void Fill(ArgListOption,int start,int count);	/* Fill specified region with ArgListOption */
    ArgListOption  Remove();			/* Remove and return last element */
    void Remove(int num);		/* Remove last num elements */
    void Clear();			/* Remove all elements */

    /* Storage stuff */
    void Reclaim();			/* Reclaim all unused space */
  protected:
    /* Error handlers */
    void BoundError(int) const;
    void RangeError(int) const;

    void BoundCheck(int) const;
    void RangeCheck(int) const;
    void SizeCheck (int) const;

  private:
    ArgListOption*	store;			/* Actual storage */
    int		alloc;			/* Size of allocated storage */
    int		size;			/* Size of used storage */

    /* Storage enlargers */
    void EnlargeAllocationTo(int s);	/* Enlarge storage to s */
    void EnlargeTo(int s);		/* Enlarge storage to s if necessary */
};

inline void ArgListOptionArray::BoundCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if ((val < 0) || (val >= size)) BoundError(val);
#endif
}

inline void ArgListOptionArray::RangeCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if (val < 0) RangeError(val);
#endif
}

inline void ArgListOptionArray::SizeCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if ((val < 0) || (val > size)) BoundError(val);
#endif
}

inline ArgListOptionArray::ArgListOptionArray() {
    alloc = 0;
    size  = 0;
}

inline int ArgListOptionArray::Size() const {
    return size;
}

inline ArgListOption& ArgListOptionArray::operator[](int index) const {
    BoundCheck(index);
    return store[index];
}

inline ArgListOption& ArgListOptionArray::Slot(int index) const {
    BoundCheck(index);
    return store[index];
}

inline ArgListOption& ArgListOptionArray::High() const {
    return store[size-1];
}

inline ArgListOption* ArgListOptionArray::AsPointer() const {
    return store;
}

inline void ArgListOptionArray::Append(ArgListOption v) {
    if (size >= alloc)
	EnlargeAllocationTo(size+1);
    store[size++] = v;
}

inline ArgListOption ArgListOptionArray::Remove() {
    if (size > 0) size--;
    return store[size];
}

inline void ArgListOptionArray::Remove(int num) {
    SizeCheck(num);
    size -= num;
}

inline void ArgListOptionArray::Clear() {
    size = 0;
}

#endif /*_ArgListOptionARRAYH */
