// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/28 16:23:43 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	Array.ccP,v $
 * Revision 1.3  91/11/28  16:23:43  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.2  91/11/04  19:43:45  sanjay
 * Prepare for release
 * 
 * Revision 1.1  91/06/28  20:19:50  sanjay
 * Initial revision
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include "Date.Array.h"
#include "config.h"

DateArray::DateArray(int predict) {
    RangeCheck(predict);
    alloc = 0;
    size = 0;
    EnlargeTo(predict);
    size = 0;
}

DateArray::~DateArray() {
    if (alloc > 0) deleteArray(alloc) store;
}

DateArray::DateArray(const Date* src, int s) {
    RangeCheck(s);
    alloc = 0;
    size  = 0;
    EnlargeTo(s);
    for (int i = 0; i < s; i++)
	store[i] = src[i];
}

DateArray::DateArray(const DateArray& d) {
    alloc = 0;
    size  = 0;
    EnlargeTo(d.size);
    for (int i = 0; i < size; i++)
	store[i] = d.store[i];
}

DateArray::DateArray(Date element, int numberOfCopies) {
    RangeCheck(numberOfCopies);
    alloc = 0;
    size = 0;
    EnlargeTo(numberOfCopies);
    for (int i = 0; i < size; i++)
	store[i] = element;
}

DateArray& DateArray::operator=(const DateArray& d) {
    size = 0;
    EnlargeTo(d.size);
    for (int i = 0; i < size; i++)
	store[i] = d.store[i];
    return (*this);
}

void DateArray::Append(Date element, int n) {
    RangeCheck(n);
    int oldsize = size;
    EnlargeTo(size + n);
    for (int i = 0; i < n; i++)
	store[i + oldsize] = element;
}

void DateArray::Concat(const DateArray& d) {
    int oldsize = size;
    EnlargeTo(size + d.size);
    for (int i = 0; i < d.size; i++)
	store[i+oldsize] = d.store[i];
}

void DateArray::Concat(const Date* src, int s) {
    RangeCheck(s);
    int oldsize = size;
    EnlargeTo(s + size);
    for (int i = 0; i < s; i++)
	store[i+oldsize] = src[i];
}

void DateArray::Fill(Date element, int start, int count) {
    BoundCheck(start);
    RangeCheck(count);
    BoundCheck(start + count - 1);
    for (int i = start; i < start + count; i++)
	store[i] = element;
}

void DateArray::EnlargeTo(int newsize) {
    if (newsize > alloc)
	EnlargeAllocationTo(newsize);
   size = newsize;
}

void DateArray::EnlargeAllocationTo(int newsize) {
    int newalloc = alloc * 2;
    if (newsize > newalloc) newalloc = newsize;

    Date* oldstore = store;
    store = new Date[newalloc];

    for (int i = 0; i < size; i++)
	store[i] = oldstore[i];

    if (alloc > 0) deleteArray(alloc) oldstore;
    alloc = newalloc;
}

void DateArray::Reclaim() {
    if (alloc > size) {
	/* Some free entries that can be reclaimed */
	if (size > 0) {
	    /* Array not empty - create new store */
	    Date* newstore = new Date[size];
	    for (int i = 0; i < size; i++)
		newstore[i] = store[i];
	    deleteArray(alloc) store;
	    alloc = size;
	    store = newstore;
	}
	else {
	    /* Array empty - delete old store */
	    if (alloc > 0) {
		deleteArray(alloc) store;
		alloc = 0;
	    }
	}
    }
}

void DateArray::BoundError(int b) const {
    fprintf(stderr, "Array(0x%x): %d out of bounds\n", (long) this, b);
    abort();
}

void DateArray::RangeError(int r) const {
    fprintf(stderr, "Array(0x%x): %d out of range\n", (long) this, r);
    abort();
}
