/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.2 $
 * $Date: 91/11/04 19:44:00 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	Time.cc,v $
 * Revision 1.2  91/11/04  19:44:00  sanjay
 * Prepare for release
 * 
 * Revision 1.1  91/06/28  20:07:10  sanjay
 * Initial revision
 * 
 */

#include <time.h>
#include <sys/time.h>
#include <math.h>
#include "Month.h"
#include "WeekDay.h"
#include "Time.h"

/*
 * Want to get high resolution for region of time that will be most
 * heavily used.  This region will probably be around now. Therefore,
 * rep will be number of seconds elapsed since we construct the first
 * Time value for this run of the program.
 */

int    Time::initialized = 0;	/* Initialized yet? */
double Time::offset = 0.0;	/* Offset for time values */

void Time::Initialize() {
    struct timeval buf;
    gettimeofday(&buf, 0);

    offset = buf.tv_sec;
    initialized = 1;
}

Time Time::Now() {
    struct timeval buf;
    gettimeofday(&buf, 0);
    return Time(buf);
}

void Time::BreakDown(int& mday, WeekDay& wday, Month& month, int& year,
		     int& hour, int& min, int& sec, int& milli)
{
    if (! initialized) Initialize();

    long clock = (long) floor(rep + offset);
    struct tm* t = localtime(&clock);

    mday  = t->tm_mday;				/* tm_mday in 1..31 */
    wday  = WeekDay::Sunday() + t->tm_wday;	/* tm_wday in 0..6.  Sun = 0 */
    month = Month::January() + t->tm_mon;	/* tm_mon  in 0..11. Jan = 0 */
    year  = t->tm_year + 1900;
    hour  = t->tm_hour;
    min   = t->tm_min;
    sec   = t->tm_sec;
    milli = (int) rint((rep + offset - clock) * 1000.0);
}

void Time::BreakDownDate(int& mday, WeekDay& wday, Month& month, int& year) {
    BreakDown(mday, wday, month, year);
}

void Time::BreakDownClock(int& hour, int& min, int& sec, int& milli) {
    WeekDay junkWDay;
    Month   junkMonth;

    BreakDown(junkInt, junkWDay, junkMonth, junkInt, hour, min, sec, milli);
}

Time::Time(const struct timeval& tv) {
    if (! initialized) Initialize();
    rep = (tv.tv_sec - offset) + ((double) tv.tv_usec) / 1000000.0;
}

void Time::Convert(struct timeval& tv) {
    if (! initialized) Initialize();

    tv.tv_sec  = (long) floor(rep + offset);
    tv.tv_usec = (long) rint((rep + offset - tv.tv_sec) * 1000000.0);
}

Duration::Duration(const struct timeval& tv) {
    rep = tv.tv_sec + ((double) tv.tv_usec) / 1000000.0;
}

void Duration::Convert(struct timeval& tv) {
    tv.tv_sec  = (long) floor(rep);
    tv.tv_usec = (long) rint((rep - tv.tv_sec) / 1000000.0);
}
