/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/04 19:44:02 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	Timer.cc,v $
 * Revision 1.4  91/11/04  19:44:02  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/04/16  15:22:30  sanjay
 * Made closures reference counted
 * 
 * Revision 1.2  91/03/28  11:51:39  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:49  sanjay
 * Initial deposit
 *
 */

#include "TimerActionP.Array.h"
#include "Timer.h"
#include "closure.h"

TimerAction::TimerAction(Time t, Closure* c) {
    time = t;
    closure = c;
    closure->Reference();
}

TimerAction::~TimerAction() {
    Unref(closure);
}

Timer::Timer() : queue(*(new TimerActionPArray)) { }

Timer::~Timer() {
    for (int i = 0; i < queue.Size(); i++)
	delete queue[i];
    delete &queue;
}

int Timer::Pending() {
    return (queue.Size() != 0);
}

Time Timer::NextFire() {
    if (queue.Size() > 0) {
	Time min = queue[0]->GetTime();
	for (int i = 0; i < queue.Size(); i++) {
	    if (min > queue[i]->GetTime())
		min = queue[i]->GetTime();
	}
	return min;
    }
    else {
	return Time::Now() + Duration::Week();
    }
}

TimerAction* Timer::FireAt(Time t, Closure* c) {
    TimerAction* e = new TimerAction(t, c);
    queue.Append(e);
    return e;
}

void Timer::Remove(TimerAction* e) {
    for (int i = 0; i < queue.Size(); i++) {
	if (queue[i] == e) {
	    delete e;
	    queue[i] = queue.High();
	    queue.Remove();
	    return;
	}
    }
}

int Timer::Contains(TimerAction* e) {
    for (int i = 0; i < queue.Size(); i++) {
	if (queue[i] == e) {
	    return 1;
	}
    }
    return 0;
}

/*
 * First move all ready elements to another list so that
 * the callbacks from the closure work properly.
 */
void Timer::Tick() {
    TimerActionPArray temp;
    Time now = Time::Now();

    /* Move ready events to temp list */
    temp.Clear();
    int i = 0;
    while (i < queue.Size()) {
	if (queue[i]->GetTime() <= now) {
	    temp.Append(queue[i]);
	    queue[i] = queue.High();
	    queue.Remove();
	}
	else {
	    i++;
	}
    }

    for (i = 0; i < temp.Size(); i++) {
	temp[i]->GetClosure()->Invoke();
	delete temp[i];
    }
}
