typedef class TimerAction* TimerActionP; /* Automatic prepend ()*^*@# */
// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

#ifndef _TimerActionPARRAYH
#define _TimerActionPARRAYH

/*
 * A resizable array of TimerActionPs.
 *
 * If TimerActionP is user-defined, it must have the following operations -
 *
 * class TimerActionP ... {
 *   public:
 *     TimerActionP();
 *     ~TimerActionP();
 *     ...
 *     TimerActionP& operator = (TimerActionP&);
 * };
 *
 * If UNSAFE_ARRAY not defined when including TimerActionPArray.h, then array bounds
 * and argument range checking will be done.
 *
 */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/28 16:23:54 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	Array.hP,v $
 * Revision 1.3  91/11/28  16:23:54  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.2  91/11/04  19:43:48  sanjay
 * Prepare for release
 * 
 * Revision 1.1  91/06/28  20:19:50  sanjay
 * Initial revision
 * 
 */

class TimerActionPArray {
  public:

    /* Constructors */
    TimerActionPArray();				/* Empty array */
    TimerActionPArray(int predict);		/* Empty array with size prediction */
    TimerActionPArray(const TimerActionP*, int);		/* Initialized with C array */
    TimerActionPArray(const TimerActionPArray&);		/* Initialized with another Array */
    TimerActionPArray(TimerActionP, int);			/* Fill with n copies of element */

    /* Destructor */
    ~TimerActionPArray();

    TimerActionPArray& operator=(const TimerActionPArray&);	/* Assignment operator */

    /* Array indexing */
    TimerActionP& operator[](int index) const;
    TimerActionP& Slot(int index) const;

    /* Other Array operations */
    int  Size() const;			/* Return size; */
    TimerActionP& High() const;			/* Return last element */
    TimerActionP* AsPointer() const;		/* Return as pointer to base */
    void Append(TimerActionP v);			/* Append an element */
    void Append(TimerActionP, int n);		/* Append n copies of element */
    void Concat(const TimerActionP*, int);	/* Concatenate C array */
    void Concat(const TimerActionPArray&);	/* Concatenate another Array */
    void Fill(TimerActionP,int start,int count);	/* Fill specified region with TimerActionP */
    TimerActionP  Remove();			/* Remove and return last element */
    void Remove(int num);		/* Remove last num elements */
    void Clear();			/* Remove all elements */

    /* Storage stuff */
    void Reclaim();			/* Reclaim all unused space */
  protected:
    /* Error handlers */
    void BoundError(int) const;
    void RangeError(int) const;

    void BoundCheck(int) const;
    void RangeCheck(int) const;
    void SizeCheck (int) const;

  private:
    TimerActionP*	store;			/* Actual storage */
    int		alloc;			/* Size of allocated storage */
    int		size;			/* Size of used storage */

    /* Storage enlargers */
    void EnlargeAllocationTo(int s);	/* Enlarge storage to s */
    void EnlargeTo(int s);		/* Enlarge storage to s if necessary */
};

inline void TimerActionPArray::BoundCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if ((val < 0) || (val >= size)) BoundError(val);
#endif
}

inline void TimerActionPArray::RangeCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if (val < 0) RangeError(val);
#endif
}

inline void TimerActionPArray::SizeCheck(int val) const {
#ifndef UNSAFE_ARRAY
    if ((val < 0) || (val > size)) BoundError(val);
#endif
}

inline TimerActionPArray::TimerActionPArray() {
    alloc = 0;
    size  = 0;
}

inline int TimerActionPArray::Size() const {
    return size;
}

inline TimerActionP& TimerActionPArray::operator[](int index) const {
    BoundCheck(index);
    return store[index];
}

inline TimerActionP& TimerActionPArray::Slot(int index) const {
    BoundCheck(index);
    return store[index];
}

inline TimerActionP& TimerActionPArray::High() const {
    return store[size-1];
}

inline TimerActionP* TimerActionPArray::AsPointer() const {
    return store;
}

inline void TimerActionPArray::Append(TimerActionP v) {
    if (size >= alloc)
	EnlargeAllocationTo(size+1);
    store[size++] = v;
}

inline TimerActionP TimerActionPArray::Remove() {
    if (size > 0) size--;
    return store[size];
}

inline void TimerActionPArray::Remove(int num) {
    SizeCheck(num);
    size -= num;
}

inline void TimerActionPArray::Clear() {
    size = 0;
}

#endif /*_TimerActionPARRAYH */
