/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.13 $
 * $Date: 91/11/28 14:48:13 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	alarmer.cc,v $
 * Revision 1.13  91/11/28  14:48:13  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.12  91/11/04  19:44:10  sanjay
 * Prepare for release
 * 
 * Revision 1.11  91/09/11  20:20:19  sanjay
 * Parse attributes for alarm times.
 * 
 * Revision 1.10  91/08/06  09:44:44  sanjay
 * Get item list from application instead of calendar.
 * 
 * Revision 1.9  91/06/28  20:09:30  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.8  91/04/22  19:47:44  sanjay
 * Removed Date::Midnight() usage
 * 
 * Revision 1.7  91/04/18  19:07:44  sanjay
 * Fixed reminder verboseness
 * 
 * Revision 1.6  91/04/16  15:22:31  sanjay
 * Made closures reference counted
 * 
 * Revision 1.5  91/04/05  10:52:22  sanjay
 * Was computing wrong time range for compute interval.
 * Therefore appointments around two hour boundaries would
 * end up missing certain reminders.
 * 
 * Revision 1.4  91/04/01  11:56:56  sanjay
 * Added include file capability
 * 
 * Revision 1.3  91/03/29  10:32:34  sanjay
 * Put arrays in library
 * 
 * Revision 1.2  91/03/28  11:51:40  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:50  sanjay
 * Initial deposit
 *
 */

#include <stdio.h>
#include <ctype.h>

#include "Date.h"
#include "Time.h"

#include "alarmer.h"
#include "application.h"
#include "closure.h"
#include "item.h"
#include "reminder.h"
#include "remindmgr.h"
#include "xworld.h"
#include "Timer.h"

#include "int.Array.h"
#include "Pointer.Array.h"

declare(Closure,Alarmer);
implement(Closure,Alarmer);

Alarmer::Alarmer(Application* a)
     : (a), pending(*(new PointerArray)), alarms(*(new intArray))
{
    /*
     * Read upto a maximum of 100 alarm values.
     */
    int i = 1;
    while (i <= 100) {
	const char* attr;
	char property[10];

	sprintf(property, "alarm%d", i);
	attr = a->World()->GetAttribute(property);
	if (attr == nil) {
	    /* Stop at missing attribute value */
	    break;
	}

	int value;

	/* Parse value and check range - 0 to 20 hours */
	if ((sscanf(attr, "%d", &value) != 1) ||
	    (value < 0) ||
	    (value >= 20 * 60)) {
	    /* Could not parse specification */
	    fprintf(stderr, "%s: illegal value <%s>\n", property, attr);
	}
	else {
	    alarms.Append(value);
	}

	i++;
    }

    if (alarms.Size() == 0) {
	/* Use default values */
	alarms.Append(0);
	alarms.Append(5);
	alarms.Append(10);
	alarms.Append(15);
    }

    /*
     * Compute min/max.
     */
    minAlarm = alarms[0];
    maxAlarm = alarms[0];
    for (i = 1; i < alarms.Size(); i++) {
	if (alarms[i] < minAlarm) minAlarm = alarms[i];
	if (alarms[i] > maxAlarm) maxAlarm = alarms[i];
    }

    ticker = new Closure1(Alarmer)(this, Alarmer::Tick);
    ticker->Reference();

    recomputer = new Closure1(Alarmer)(this, Alarmer::Recompute);
    recomputer->Reference();

    timer = a->GetTimer();
    tickAction = nil;
    recomputeAction = nil;
    Recompute();
}

Alarmer::~Alarmer() {
    if (tickAction != nil) {
	timer->Remove(tickAction);
    }
    if (recomputeAction != nil) {
	timer->Remove(recomputeAction);
    }
    for (int i = 0; i < pending.Size(); i++) {
	delete ((Alarmer_Notice*) (pending[i]));
    }
    delete &pending;
    delete &alarms;

    Unref(ticker);
    Unref(recomputer);
}

void Alarmer::ItemMsg(ApptViewItemMsg t, Item* item) {
    Appointment* a = item->AsAppointment();
    if (a == nil) return;

    switch (t) {
      case ApptView::Change:
	RemoveAppt(a);
	AddAppt(a);
	break;
      case ApptView::Delete:
	RemoveAppt(a);
	break;
      case ApptView::Add:
	AddAppt(a);
	break;
    }

    if (tickAction != nil) {
	timer->Remove(tickAction);
	tickAction = nil;
    }
    Tick();
}

void Alarmer::Msg(ApptViewMsg) {}

void Alarmer::Tick() {
    tickAction = nil;

    Time now = Time::Now();
    Time nextTick = now + Duration::Second() * 1000000.0;
    int i = 0;
    while (i < pending.Size()) {
	Alarmer_Notice* n = (Alarmer_Notice*) (pending[i]);
	if (n->expiration > now) {
	    if (n->expiration < nextTick) {
		nextTick = n->expiration;
	    }
	    i++;
	    continue;
	}

	/* Put up reminder */
	application->GetRemindMgr()->AddReminder(n->appointment, n->apptStart);

	/* Delete pending[i] */
	delete n;
	if (i == (pending.Size() - 1)) {
	    pending.Remove();
	}
	else {
	    pending[i] = pending.Remove();
	}
    }

    tickAction = timer->FireAt(nextTick, ticker);
}

void Alarmer::Recompute() {
    recomputeAction = nil;
    StartRecompute();

    for (int i = 0; i < pending.Size(); i++) {
	delete ((Alarmer_Notice*) (pending[i]));
    }
    pending.Clear();

    int calSize = application->NumItems();
    for (i = 0; i < calSize; i++) {
	Appointment* appt = application->GetItem(i)->AsAppointment();
	if (appt != nil) {
	    AddAppt(appt);
	}
    }

    FinishRecompute();
}

void Alarmer::StartRecompute() {
    if (recomputeAction != nil) {
	timer->Remove(recomputeAction);
	recomputeAction = nil;
    }

    /*
     * Compute reminders for next 6 hours.
     */
    finishTime = Time::Now() + Duration::Hour() * 6;
}

void Alarmer::FinishRecompute() {
    recomputeAction = timer->FireAt(finishTime, recomputer);

    if (tickAction != nil) {
	timer->Remove(tickAction);
	tickAction = nil;
    }
    Tick();
}

void Alarmer::AddAppt(Appointment* a) {
    /* Compute appt range */
    Time now = Time::Now();

    int hour, min, sec, msec;
    now.BreakDownClock(hour, min, sec, msec);
    Duration nowOffset = (Duration::Hour()	  * hour +
			  Duration::Minute()	  * min	 +
			  Duration::Second()	  * sec	 +
			  Duration::MilliSecond() * msec);

    /*
     * Get everything today and tomorrow.
     */
    Date today = Date(now);
    Time todayStart = now - nowOffset;

    for (int i = 0; i <= 1; i++) {
	if (! a->IsOn(today + i)) continue;

	Time apptStart = todayStart + (Duration::Day() * i +
				       Duration::Minute() * a->GetStart());

	/* Add all alarms in range */
	for (int j = 0; j < alarms.Size(); j++) {
	    Time alarmTime = apptStart - Duration::Minute() * alarms[j];
	    if (alarmTime < now)  continue;
	    if (alarmTime > finishTime) continue;

	    Alarmer_Notice* n = new Alarmer_Notice;
	    n->expiration  = alarmTime;
	    n->apptStart   = apptStart;
	    n->appointment = a;

	    pending.Append((Pointer) n);
	}
    }
}

void Alarmer::RemoveAppt(Appointment* a) {
    application->GetRemindMgr()->RemoveRemindersFor(a);

    int i = 0;
    while (i < pending.Size()) {
	Alarmer_Notice* n = (Alarmer_Notice*) (pending[i]);
	if (n->appointment != a) {
	    i++;
	    continue;
	}

	/* Delete pending[i] */
	delete n;
	if (i == (pending.Size() - 1)) {
	    pending.Remove();
	}
	else {
	    pending[i] = pending.Remove();
	}
    }
}
