// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:44:11 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	alarmer.h,v $
 * Revision 1.3  91/11/04  19:44:11  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/04/16  15:22:32  sanjay
 * Made closures reference counted
 * 
 * Revision 1.1  91/03/28  11:41:06  sanjay
 * Initial deposit
 * 
 */

#ifndef _ALARMERH
#define _ALARMERH

#include "apptview.h"
#include "Time.h"

/*
 * Generates reminders at appropriate times.
 */

class Closure;
class Reminder;
class Appointment;
class Timer;
class TimerAction;
class PointerArray;
class intArray;

/*
 * Information kept per notification supposed to fire soon.
 */
struct Alarmer_Notice {
    Time expiration;	/* Time at which to post reminder */
    Time apptStart;	/* Time at which appt starts */
    Appointment* appointment;
};

class Alarmer : public ApptView {
  public:
    Alarmer(Application*);
    virtual ~Alarmer();

    /* Callbacks from ApptView */
    virtual void ItemMsg(ApptViewItemMsg, Item*);
    virtual void Msg(ApptViewMsg);
  protected:
    Timer*	  timer;	/* Timer used for firing notifications */
    PointerArray& pending;	/* Pending notifications */
    intArray&     alarms;	/* Times for different alarms */

    void Tick();			/* Called when some appt ready */
    TimerAction*	tickAction;	/* Hook into timer entry for self */
    Closure*		ticker;

    void Recompute();	/* Called when time to recompute pending list */
    TimerAction* recomputeAction;
    Closure*	 recomputer;

    void StartRecompute();
    void FinishRecompute();
    void AddAppt(Appointment*);
    void RemoveAppt(Appointment*);

    /* Information used during recomputation. */

    /* Time till which reminders are being computed */
    Time finishTime;

    /* Min/Max alarm times */
    int minAlarm;
    int maxAlarm;
};

#endif /* _ALARMERH */
