/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.33 $
 * $Date: 91/11/28 16:24:06 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	application.cc,v $
 * Revision 1.33  91/11/28  16:24:06  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.32  91/11/28  14:48:30  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.31  91/11/06  17:59:55  sanjay
 * Do not maintain auto-save files for shared calendars. Auto-save for
 * shared calendars writes directly to the shared files.
 * 
 * Cleaned up other IO handling.
 * 
 * Revision 1.30  91/11/04  19:44:16  sanjay
 * Prepare for release
 * 
 * Revision 1.29  91/11/04  11:50:24  sanjay
 * Print appropriate error message when file cannot be read.
 * 
 * Revision 1.28  91/09/18  20:33:37  sanjay
 * Use OccurCache for listing items on certain dates.
 * 
 * Revision 1.27  91/09/06  12:05:48  sanjay
 * Added more default resources.
 * 
 * Revision 1.26  91/08/13  16:30:34  sanjay
 * Removed bug which caused any InterViews command line options
 * to result in a usage error.
 * 
 * Revision 1.25  91/08/13  09:55:56  sanjay
 * Add main calendar entries at end.
 * 
 * Revision 1.24  91/08/08  16:20:12  sanjay
 * Removed unnecessary status line property.
 * 
 * Revision 1.23  91/08/07  10:59:01  sanjay
 * Added re-read.
 * 
 * Revision 1.22  91/08/06  13:55:00  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.21  91/08/06  09:45:24  sanjay
 * Added editing support for included files.
 * 
 * Revision 1.20  91/07/08  11:28:31  sanjay
 * Changed fonts to Times-Roman.
 * 
 * Revision 1.19  91/06/28  20:09:32  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.18  91/04/30  18:20:50  sanjay
 * Add comments
 * 
 * Revision 1.17  91/04/23  15:37:30  sanjay
 * Call SetFocus(nil) at various places to save pending changes into
 * the volatile calendar structure.
 * 
 * Revision 1.16  91/04/16  15:22:34  sanjay
 * Made closures reference counted
 * 
 * Revision 1.15  91/04/11  13:30:43  sanjay
 * Fixed window popup
 * 
 * Revision 1.14  91/04/11  10:26:35  sanjay
 * Encapsulated calendar file handling
 * 
 * Revision 1.13  91/04/10  11:33:22  sanjay
 * Added printing
 * 
 * Revision 1.12  91/04/09  15:25:06  sanjay
 * Minor default font fixes.
 * 
 * Revision 1.11  91/04/03  21:36:30  sanjay
 * Added range listing
 * 
 * Revision 1.10  91/04/03  20:58:14  sanjay
 * Layout improvements
 * 
 * Revision 1.9  91/04/03  20:39:37  sanjay
 * Put reminders in window instead of piping
 * 
 * Revision 1.8  91/04/02  12:32:59  sanjay
 * Improved auto-save.
 * 
 * Revision 1.7  91/04/01  20:30:31  sanjay
 * Better IO error handling
 * 
 * Revision 1.6  91/04/01  17:26:26  sanjay
 * Fixed spurious usage error.
 * 
 * Revision 1.5  91/04/01  11:56:58  sanjay
 * Added include file capability
 * 
 * Revision 1.4  91/03/29  10:32:40  sanjay
 * Put arrays in library
 * 
 * Revision 1.3  91/03/28  11:51:42  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:52  sanjay
 * Initial deposit
 *
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <InterViews/box.h>
#include <InterViews/message.h>

#include "Date.h"
#include "Time.h"

#include "alarmer.h"
#include "application.h"
#include "apptview.h"
#include "arglist.h"
#include "calargs.h"
#include "calfile.h"
#include "calendar.h"
#include "closure.h"
#include "confirm.h"
#include "dayview.h"
#include "editrepeat.h"
#include "filemap.h"
#include "iremind.h"
#include "item.h"
#include "lexer.h"
#include "occurcache.h"
#include "popup.h"
#include "printer.h"
#include "remindmgr.h"
#include "textpane.h"
#include "vertscroller.h"
#include "xworld.h"

#include "Timer.h"

#include "Pointer.Array.h"
#include "char.Array.h"

declare(Closure,Application);
implement(Closure,Application);

static OptionDesc options[] = {
    { nil }
};

static PropertyData properties[] = {
    /* Miscellaneous resources */
    { "*monthName.minWidth",	"September",				nil },
    { "*MonthDisplay.hpad",	"10",					nil },
    { "*MonthDisplay.vpad",	"4",					nil },
    { "*NoteList.displayLines",	"10",					nil },
    { "*Reminder.geometry",	"+300+0",				nil },

    /* Font resources */
    { "*MonthDisplay.font",	"-*-times-medium-r-*-*-*-140-*",	nil },
    { "*monthName.font",	"-*-times-medium-r-*-*-*-240-*",	nil },
    { "*yearNumber.font",	"-*-times-medium-r-*-*-*-240-*",	nil },
    { "*Reminder*font",		"-*-times-medium-r-*-*-*-180-*",	nil },

    /* Color resources */
    { "*background",		"thistle",				nil },
    { "*foreground",		"black",				nil },
    { "*ApptFrame*background",	"blue",					nil },
    { "*ApptFrame*foreground",	"yellow",				nil },
    { "*Reminder*background",	"blue",					nil },
    { "*Reminder*foreground",	"yellow",				nil },

    { nil,			nil,					nil }
};

static const char* defaultFileName = ".calendar";

/* Utility routines. */
static char const* none = "";
static char* Copy(char const*, char const* = none, char const* = none);

/*
 * Constructor.
 */
Application::Application(int& argc, char* argv[])
     : views(*(new PointerArray)),
       editRepeats(*(new PointerArray))
{
    const char* a;
    const char* fileName;

    /*
     * Parse command line arguments.
     */
    CalArgs args(argc, argv);
    args.Parser()->Parse();

    /*
     * Get calendar name.
     *	1. Check on command line.
     *  2. Check in environment.
     *  3. Use ~/.calendar.
     */
    if (args.saveFile != nil) {
	fileName = Copy(args.saveFile);
    }
    else if ((a = getenv("CALENDAR")) != nil) {
	fileName = Copy(a);
    }
    else {
	const char* home = getenv("HOME");
	if (home == nil) home = ".";
	fileName = Copy(home, "/", defaultFileName);
    }

    /*
     * Open the display.
     */
    if (! args.remindOnly) {
	world = new XWorld("Calendar", properties, options,
			   args.Argc(), args.Argv());
    }

    if (args.Argc() > 1) {
	/* Unknown argument */
	Usage();
	exit(1);
    }

    /* Create cache for item occurrences */
    occurCache = new OccurCache(this);

    /* Create top-level calendar and add to calendarMap */
    calendarMap = new FileMap;
    calFile = MakeCalendar(fileName, args.remindOnly);
    calendarMap->Add(calFile);

    /* Create and add included calendars to calendarMap */
    for (int i = 0; i < calFile->GetCalendar()->NumIncludes(); i++) {
	char const* name = calFile->GetCalendar()->GetInclude(i);

	calendarMap->Add(MakeCalendar(name, args.remindOnly));
    }

    /*
     * Check if display needs to be opened.
     */
    if (args.remindOnly) {
	/* Dont open the display */

	/*
	 * Generate listing for today.
	 */
	fflush(stdout);
	FPrinter fp(fileno(stdout));

	Date today = Date::Today();
	IRemind::Remind(&fp, this, today, today);

	fp.Flush();

	exit(0);
    }

    /*
     * Get all customizable attributes.
     */

    /* Construct print command */
    printCommand = Copy("lpr");

    /* Auto save level */
    autoSaveLevel = 10;
    if ((a = world->GetAttribute("autoSaveLevel")) != nil) {
	sscanf(a, "%d", &autoSaveLevel);
	if (autoSaveLevel < 1) autoSaveLevel = 1;
    }

    autoSaveDelay = 30;		/* Delay 30 seconds by default */
    if ((a = world->GetAttribute("autoSaveDelay")) != nil) {
	sscanf(a, "%d", &autoSaveDelay);
	if (autoSaveDelay < 15) autoSaveDelay = 15;
    }

    /*
     * Initialize auto-save mechanism.
     */
    autoSaver = new Closure1(Application)(this, Application::AutoSaveTimeout);
    autoSaver->Reference();
    autoSaveHook = nil;
    modCount = 0;

    /*
     * Miscellaneous initializations.
     */
    openViews = 0;
    clipboard = nil;
    focus = nil;

    /* Set last selected directory to junk value */
    lastSelectedDir = Copy("/");
    SetSelectedFile(fileName);

    /*
     * Initialize helper views (alarmer and reminder manager)/
     */
    remindMgr = new RemindMgr(this);
    alarmer = new Alarmer(this);

    /*
     * Create user-interface window.
     */
    world->InsertApplication(new DayView(this, Date::Today()));

    /*
     * Accept user input.
     */
    Run();
}

/*
 * Destructor - just reclaims memory.
 */
Application::~Application() {
    delete (char*) lastSelectedDir;
    delete (char*) printCommand;
    delete alarmer;
    delete remindMgr;

    if (autoSaveHook != nil) {
	world->GetTimer()->Remove(autoSaveHook);
	autoSaveHook = nil;
    }
    Unref(autoSaver);

    for (int i = 0; i < editRepeats.Size(); i++) {
	EditRepeat* editor = ((EditRepeat*) editRepeats[i]);
	delete editor;
    }
    delete &editRepeats;

    delete &views;
    delete occurCache;

    for (i = 0; i < calendarMap->Size(); i++) {
	delete calendarMap->Get(i);
    }
    delete calendarMap;

    delete world;
    if (clipboard != nil) delete clipboard;
}

Calendar* Application::GetMainCalendar() {
    return calFile->GetCalendar();
}

ApptView* Application::Focus() {
    return focus;
}

void Application::NewEditRepeat(Item* i) {
    SetFocus(nil);

    EditRepeat* er;

    if (editRepeats.Size() > 0) {
	er = (EditRepeat*) editRepeats.Remove();
    }
    else {
	er = new EditRepeat(this);
    }
    World()->InsertApplication(er);
    er->Edit(i);
}

void Application::DeleteEditRepeat(EditRepeat* er) {
    World()->Remove(er);
    editRepeats.Append((Pointer) er);
}

void Application::SetFocus(ApptView* v) {
    if (focus != v) {
	ApptView* oldFocus = focus;
	focus = v;

	if (oldFocus != nil) oldFocus->Msg(ApptView::Unfocus);
	if (focus != nil) focus->Msg(ApptView::Focus);
    }
}
	

void Application::Attach(ApptView* v) {
    views.Append((Pointer) v);
}

void Application::Detach(ApptView* v) {
    for (int i = 0; i < views.Size(); i++) {
	if (views[i] == ((Pointer) v)) {
	    views[i] = views.High();
	    views.Remove();
	    break;
	}
    }
}

void Application::Add(Item* item, Calendar* cal) {
    CalFile* file = calendarMap->Lookup(cal);
    if (file == nil) {
	Error("Internal error: calendar not found in map");
	file = calFile;
    }

    file->GetCalendar()->Add(item);
    occurCache->Added(item);
    for (int i = 0; i < views.Size(); i++) {
	((ApptView*) views[i])->ItemMsg(ApptView::Add, item);
    }
    Modified(file);
}

void Application::Delete(Item* item) {
    item->MarkDeleted();
    occurCache->Removed(item);
    for (int i = 0; i < views.Size(); i++) {
	((ApptView*) views[i])->ItemMsg(ApptView::Delete, item);
    }

    Calendar* cal = item->GetCalendar();
    cal->Remove(item);

    Modified(calendarMap->Lookup(cal));
}

int Application::NumItems() {
    int sum = 0;

    for (int i = 0; i < calendarMap->Size(); i++) {
	sum += calendarMap->Get(i)->GetCalendar()->Size();
    }

    return sum;
}

Item* Application::GetItem(int index) {
    /*
     * Look in included calendars first [1..size-1]
     */
    for (int i = 1; i < calendarMap->Size(); i++) {
	Calendar* cal = calendarMap->Get(i)->GetCalendar();

	if (index < cal->Size()) {
	    /* Desired item is in cal */
	    return cal->Get(index);
	}

	index -= cal->Size();
    }

    /*
     * Return item from main calendar
     */
    return calFile->GetCalendar()->Get(index);
}

void Application::Change(Item* item) {
    if (! item->Deleted()) {
	occurCache->Changed(item);
	for (int i = 0; i < views.Size(); i++) {
	    ((ApptView*) views[i])->ItemMsg(ApptView::Change, item);
	}
	Modified(calendarMap->Lookup(item->GetCalendar()));
    }
}

void Application::Run() {
    world->Run();
}

void Application::Flush() {
    world->Flush();
}

void Application::Sync() {
    world->Sync();
}

void Application::AddCalendar(char const* name) {
    CalFile* newFile = new CalFile(this, name);

    Calendar* ac = newFile->GetAutoSaveIfNewer();
    if (ac != nil) {
	Confirm3* cf = new Confirm3("Warning",
				    newFile->GetName(),
				    "Auto save file is newer. Read it?",
				    "Yes",
				    "No",
				    "Cancel");
	char c = cf->Popup(world);
	delete cf;
	switch (c) {
	  case 'y':
	    newFile->SetCalendar(ac);
	    newFile->Modified();
	    break;
	  case 'n':
	    delete ac;
	    if (! newFile->Read()) {
		delete newFile;
		FileError(name, Lexer::LastError());
		return;
	    }
	    break;
	  case 'c':
	    delete ac;
	    delete newFile;
	    return;
	}
    }
    else if (! newFile->Read()) {
	delete newFile;
	FileError(name, Lexer::LastError());
	return;
    }

    SetFocus(nil);

    /* Add included calendar name to top-level calendar */
    calFile->GetCalendar()->Include(name);
    calendarMap->Add(newFile);
    Modified(calFile);

    /* Notify all views about new items */
    AddCalendarItems(newFile->GetCalendar());
}

void Application::RemoveCalendar(char const* name) {
    /* Find calendar */
    CalFile* file = calendarMap->Lookup(name);
    if (file == 0) {
	FileError(name, "Could not find calendar");
	return;
    }

    SetFocus(nil);
    if (file->IsModified()) {
	Confirm3* cf = new Confirm3
	    ("Warning",
	     file->GetName(),
	     "Calendar has been modified. Save?",
	     "Yes",
	     "No",
	     "Cancel remove");
	char result = cf->Popup(World());
	delete cf;
	switch (result) {
	  case 'y':
	    /* Write file */
	    if (file->FileHasChanged()) {
		Confirm2* cf = new Confirm2
		    ("Warning",
		     file->GetName(),
		     "Changed since last read. Write anyway?",
		     "Yes",
		     "Cancel remove");
		boolean result = cf->Popup(World());
		delete cf;
		if (! result) {
		    return;
		}
	    }

	    if (! file->Write()) {
		FileError(file->GetName(), "Error writing calendar");
		return;
	    }
	    break;
	  case 'n':
	    /* Throw away modifications */
	    break;
	  case 'c':
	    /* Cancel calendar removal */
	    return;
	}
    }

    Calendar* cal = file->GetCalendar();

    /* Remove all calendar items from views */
    RemoveCalendarItems(cal);

    /* Remove calendar name from top-level calendar */
    calFile->GetCalendar()->Exclude(name);

    /* Remove entry from calendarMap */
    calendarMap->Remove(file);

    Modified(calFile);
}

char const* Application::GetName(Calendar* cal) {
    CalFile* file = calendarMap->Lookup(cal);
    if (file == nil) {
	return "";
    }
    else {
	return file->GetName();
    }
}

char const* Application::GetShortName(Calendar* cal) {
    CalFile* file = calendarMap->Lookup(cal);
    if (file == nil) {
	return "";
    }
    else if (file == calFile) {
	return "Main calendar";
    }
    else {
	char const* name = file->GetName();
	int lastSlash = -1;

	for (int i = 0; name[i] != '\0'; i++) {
	    if (name[i] == '/')
		lastSlash = i;
	}

	if (lastSlash < 0)
	    return name;
	else
	    return name + lastSlash + 1;
    }
}

Calendar* Application::GetCalendarByName(char const* name) {
    CalFile* file = calendarMap->Lookup(name);
    if (file == nil) {
	return nil;
    }
    else {
	return file->GetCalendar();
    }
}

boolean Application::IsModified() {
    /* Save any pending changes */
    if (focus != nil) focus->Msg(ApptView::Save);

    for (int i = calendarMap->Size() - 1; i >= 0; i--) {
	if (calendarMap->Get(i)->IsModified()) return true;
    }
    return false;
}

boolean Application::Save() {
    SetFocus(nil);

    boolean success = true;

    for (int i = calendarMap->Size() - 1; i >= 0; i--) {
	CalFile* file = calendarMap->Get(i);

	if (! file->IsModified()) continue;

	if (file->FileHasChanged()) {
	    Confirm2* cf = new Confirm2
		("Warning",
		 file->GetName(),
		 "Changed since last read. Write anyway?",
		 "Yes",
		 "No");
	    boolean result = cf->Popup(World());
	    delete cf;
	    if (! result) {
		success = false;
		continue;
	    }
	}


	if (! file->Write()) {
	    FileError(file->GetName(), "Error writing calendar.");
	    success = false;
	}
    }

    modCount = 0;
    return success;
}

boolean Application::ReRead() {
    int i;

    SetFocus(nil);

    /* Make new calendar map */
    FileMap* newMap = new FileMap;

    /* Flag for re-read success */
    boolean success;

    /* Read top-level calendar and construct includes */
    CalFile* newTop = new CalFile(this, calFile->GetName());
    newMap->Add(newTop);

    if (newTop->Read()) {
	success = true;

	/* Read included calendars */
	for (i = 0; i < newTop->GetCalendar()->NumIncludes(); i++) {
	    char const* name = newTop->GetCalendar()->GetInclude(i);
	    CalFile* newFile = new CalFile(this, name);
	    newMap->Add(newFile);
	    if (! newFile->Read()) {
		success = false;
		FileError(name, "Error re-reading calendar");
		break;
	    }
	}
    }
    else {
	success = false;
	FileError(calFile->GetName(), Lexer::LastError());
    }

    FileMap* toBeDeleted;
    if (success) {
	/* Send ApptView::Delete messages for all old items */
	for (i = calendarMap->Size() - 1; i >= 0; i--) {
	    RemoveCalendarItems(calendarMap->Get(i)->GetCalendar());
	}

	toBeDeleted = calendarMap;
	calendarMap = newMap;
	calFile = newTop;

	/* Add new calendar items - first from included calendars */
	for (i = 1; i < calendarMap->Size(); i++) {
	    AddCalendarItems(calendarMap->Get(i)->GetCalendar());
	}
	AddCalendarItems(calFile->GetCalendar());
    }
    else {
	toBeDeleted = newMap;
    }
    
    /* Delete extra storage */
    for (i = toBeDeleted->Size() - 1; i >= 0; i--) {
	delete toBeDeleted->Get(i);
    }
    delete toBeDeleted;

    modCount = 0;
    return success;
}

void Application::Quit() {
    exit(0);
}

Timer* Application::GetTimer() {
    return world->GetTimer();
}

void Application::Modified(CalFile* file) {
    if (file == nil) {
	Error("Internal error: called Application::Modified(nil)");
	return;
    }

    file->Modified();

    modCount++;

    if (modCount >= autoSaveLevel) {
	/* Auto-save right now */

	/* Remove any pending auto-save timeout */
	if (autoSaveHook != nil) {
	    world->GetTimer()->Remove(autoSaveHook);
	    autoSaveHook = nil;
	}

	AutoSave();
    }
    else {
	/* Set-up timeout if necessary */
	if (autoSaveHook == nil) {
	    Time fireTime = Time::Now() + Duration::Second() * autoSaveDelay;
	    autoSaveHook = world->GetTimer()->FireAt(fireTime, autoSaver);
	}
    }
}

void Application::AutoSave() {
    /* Copy editor changes into volatile calendar */

    if (focus != nil) focus->Msg(ApptView::Save);

    /* Auto save main calendar, and actually save other calendars */
    for (int i = calendarMap->Size() - 1; i >= 0; i--) {
	CalFile* file = calendarMap->Get(i);

	if (file == calFile) {
	    /* Auto-save main calendar */
	    if (! file->IsModified()) continue;

	    if (! file->AutoSave()) {
		FileError(file->GetName(), "Auto save failed");
	    }
	}
	else {
	    /* Actually save shared calendars */
	    if (! file->IsModified()) continue;

	    boolean write = true;

	    while (file->FileHasChanged()) {
		/* Version on disk is newer */
		Confirm3* cf = new Confirm3
		    ("Warning",
		     file->GetName(),
		     "Conflicting changes to shared calendar",
		     "Overwrite changes",
		     "Cancel save",
		     "Read from disk");

		char c = cf->Popup(world);
		delete cf;
		switch (c) {
		  case 'y':
		    /* Overwrite */
		    write = true;
		    break;
		  case 'n':
		    /* Cancel the save */
		    write = false;
		    break;
		  case 'c':
		    /* Revert from disk */
		    Calendar* old = file->ReRead();
		    if (old != nil) {
			/* Successful read from disk */
			RemoveCalendarItems(old);
			AddCalendarItems(file->GetCalendar());
			delete old;
			write = false;
			break;
		    }
		    else {
			/* Read failed - repeat */
			continue;
		    }
		    break;
		}

		/* Get here when finishing loop */
		break;
	    }

	    if (write) {
		if (! file->Write()) {
		    FileError(file->GetName(), "Error writing calendar");
		}
	    }
	}
    }

    modCount = 0;
}

void Application::AutoSaveTimeout() {
    autoSaveHook = nil;

    /* Save any pending changes */
    if (focus != nil) focus->Msg(ApptView::Save);

    AutoSave();
}

void Application::Error(const char* msg) {
    Popup* popup = new Popup(msg);
    popup->SetName("Error");
    world->Popup(popup);
}

void Application::FileError(char const* file, char const* msg) {
    VBox* contents = new VBox
	(new HBox(new Message("File: "), new Message(file)),
	 new Message(msg));
    contents->Align(Left);

    Popup* popup = new Popup(contents);

    popup->SetName("Error");
    world->Popup(popup);
}

Item* Application::GetClipboard() {
    return clipboard;
}

void Application::SetClipboard(Item* item) {
    if (clipboard != nil) delete clipboard;
    clipboard = ((item == nil) ? nil : item->Clone());
}

void Application::GenerateReminders(Date d1, Date d2) {
    SetFocus(nil);

    charArray memory;

    CAPrinter printer(memory);
    IRemind::Remind(&printer, this, d1, d2);
    printer.Flush();

    Interactor* display = new VertScroller
	(new TextPane(memory.AsPointer(), memory.Size()),
	 round(6 * inches)
	 );

    Popup* popup = new Popup(display);
    popup->SetName("Listing");
    world->InsertApplication(popup);
}

void Application::Usage() {
    fprintf(stderr, "Usage: ical [-calendar fileName]     ; Calendar file\n");
    fprintf(stderr, "            [-remindOnly]            ; List imminent items and exit\n");
    fprintf(stderr, "            [-list]                  ; Same as -remindOnly\n");
    fprintf(stderr, "       The following options make sense only if -remindOnly is not specified\n");
    fprintf(stderr, "            [X options]\n");
}

static char* Copy(char const* s1, char const* s2, char const* s3) {
    int len1 = strlen(s1);
    int len2 = strlen(s2);
    int len3 = strlen(s3);

    char* buf = new char[len1 + len2 + len3 + 1];

    strcpy(buf, s1);
    strcpy(buf+len1, s2);
    strcpy(buf+len1+len2, s3);

    return buf;
}

void Application::PrintMonth(Date d) {
    SetFocus(nil);

    FILE* stream = popen(printCommand, "w");
    if (stream == nil) {
	Error("Could not execute printCommand");
	return;
    }

    fflush(stream);
    FPrinter fp(fileno(stream));

    IRemind::PSPrintMonth(&fp, this, d);
    fp.Flush();

    int status = pclose(stream);
    if (status != 0) {
	Error("printCommand failed!");
    }
}

void Application::PrintDays(Date d, int num, int cols, boolean landscape) {
    SetFocus(nil);

    FILE* stream = popen(printCommand, "w");
    if (stream == nil) {
	Error("Could not execute printCommand");
	return;
    }

    fflush(stream);
    FPrinter fp(fileno(stream));

    IRemind::PSPrintDays(&fp, this, d, num, cols, landscape);
    fp.Flush();

    int status = pclose(stream);
    if (status != 0) {
	Error("printCommand failed!");
    }
}

const char* Application::GetPrintCommand() {
    return printCommand;
}

void Application::SetPrintCommand(const char* pc) {
    if (strcmp(pc, printCommand) != 0) {
	delete (char*) printCommand;
	printCommand = Copy(pc);
    }
}

void Application::SetSelectedFile(char const* name) {
    delete (char*) lastSelectedDir;
    lastSelectedDir = Copy(name);
}

char const* Application::GetLastSelectedDir() {
    return lastSelectedDir;
}

CalFile* Application::MakeCalendar(char const* name, boolean onTTY) {
    CalFile* file = new CalFile(this, name);

    if (onTTY) {
	/* Do a plain read */
	Calendar* cal = CalFile::ReadFrom(name);
	if (cal == nil) {
	    fprintf(stderr, "%s: %s\n", name, Lexer::LastError());
	    exit(1);
	}
	file->SetCalendar(cal);
    }
    else {
	/* Do sophisticated read with auto-save detection and error handling */
	Calendar* ac = file->GetAutoSaveIfNewer();
	if (ac != nil) {
	    Confirm3* cf = new Confirm3("Warning",
					file->GetName(),
					"Auto save file is newer. Read it?",
					"Yes",
					"No",
					"Exit");
	    char c = cf->Popup(world);
	    delete cf;
	    switch (c) {
	      case 'y':
		file->SetCalendar(ac);
		file->Modified();
		break;
	      case 'n':
		delete ac;
		if (! file->Read()) {
		    fprintf(stderr, "%s: %s\n", name, Lexer::LastError());
		    exit(1);
		}
		break;
	      case 'c':
		exit(1);
		break;
	    }
	}
	else if (! file->Read()) {
	    fprintf(stderr, "%s: %s\n", name, Lexer::LastError());
	    exit(1);
	}
    }

    return file;
}

void Application::AddCalendarItems(Calendar* cal) {
    for (int i = 0; i < cal->Size(); i++) {
	Item* item = cal->Get(i);

	occurCache->Added(item);
	for (int j = 0; j < views.Size(); j++) {
	    ((ApptView*) views[j])->ItemMsg(ApptView::Add, item);
	}
    }
}

void Application::RemoveCalendarItems(Calendar* cal) {
    for (int i = 0; i < cal->Size(); i++) {
	Item* item = cal->Get(i);

	item->MarkDeleted();
	occurCache->Removed(item);
	for (int j = 0; j < views.Size(); j++) {
	    ((ApptView*) views[j])->ItemMsg(ApptView::Delete, item);
	}
    }
}
