/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.11 $
 * $Date: 91/11/28 16:24:12 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	apptlist.cc,v $
 * Revision 1.11  91/11/28  16:24:12  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.10  91/11/04  19:44:21  sanjay
 * Prepare for release
 * 
 * Revision 1.9  91/09/18  20:33:56  sanjay
 * Use OccurCache for listing items on certain dates.
 * 
 * Revision 1.8  91/08/06  13:55:02  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.7  91/08/06  09:46:35  sanjay
 * Notify DayView on selection change.
 * Changed error messages for read-only items.
 * 
 * Revision 1.6  91/04/11  13:30:44  sanjay
 * Fixed window popup
 * 
 * Revision 1.5  91/04/02  19:44:53  sanjay
 * Fixed time display
 * 
 * Revision 1.4  91/04/01  11:57:00  sanjay
 * Added include file capability
 * 
 * Revision 1.3  91/03/30  15:06:29  sanjay
 * Removed funky item creation
 * 
 * Revision 1.2  91/03/28  11:51:43  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:54  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/brush.h>
#include <InterViews/event.h>
#include <stdio.h>
#include "application.h"
#include "apptlist.h"
#include "calendar.h"
#include "dayview.h"
#include "item.h"
#include "noteinfo.h"
#include "occurcache.h"
#include "popup.h"
#include "xworld.h"

#include "NoteInfoP.List.h"

#define ForChildren(l,c) for(c = (l)->First(); c; c = c->Next())
#define ForRevChildren(l,c) for(c = (l)->Last(); c; c = c->Prev())

ApptList::ApptList(Application* a, DayView* dv, Date d)
     : DisplayList(a)
{
    SetClassName("ApptList");

    dayview = dv;

    lightBrush = new Brush(0xaaaa);
    lightBrush->Reference();

    startTime = DisplayList_defaultStartTime;
    overlapPad = round(0.2 * inch);

    numLines = ComputeNumLines();
    date = d - 1;
    SetDate(d);
}

ApptList::~ApptList() {
    Unref(lightBrush);
}

void ApptList::Edit(NoteInfo* note) {
    DisplayList::Edit(note);
    if (focusChild != nil) {
	RaiseChild(focusChild);
	dayview->SetFocus(focusChild->Get());
    }
}

void ApptList::FinishEdit() {
    DisplayList::FinishEdit();
    dayview->SetFocus(nil);
}

boolean ApptList::Contains(Item* i) {
    return (i->AsAppointment() &&
	    App()->GetOccurCache()->ItemIsOn(i, date));
}

int ApptList::ComputeNumLines() {
    return (DisplayList_daySize / lineUnit);
}

void ApptList::GetAttributes() {
    DisplayList::GetAttributes();

    const char* a;
    int value;

    if ((a = GetAttribute("overlapPad")) != nil) {
	sscanf(a, "%d", &value);
	if (value >= 2) overlapPad = value;
    }

    if ((a = GetAttribute("startTime")) != nil) {
	int hourPart, minutePart;

	if (sscanf(a, "%d:%d", &hourPart, &minutePart) == 2) {
	    if ((hourPart >= 0) && (hourPart < 24) &&
		(minutePart >= 0) && (minutePart < DisplayList_hourSize) &&
		(((hourPart * DisplayList_hourSize + minutePart) % lineUnit) == 0)) {
		    startTime = hourPart * DisplayList_hourSize + minutePart;
		}
	}
    }

    /* Make sure we do not start displaying too late */
    if ((startTime / lineUnit + displayLines) > numLines) {
	startTime = (numLines - displayLines) * lineUnit;
    }

    separatorPos = output->GetFont()->Width("00:00am") + 2 * pad;
    totalWidth = separatorPos + round(inches * textWidth);
}

void ApptList::Redraw(Coord l, Coord b, Coord r, Coord t) {
    output->ClearRect(canvas, l, b, r, t);
    output->Clip(canvas, l, b, r, t);

    /* Draw separator */
    output->Line(canvas, separatorPos, 0, separatorPos, ymax);

    /* Draw each line */
    int time = 0;

    for (int i = 0; i < numLines; i++) {
	/* Draw the ith line - but only if it needs to be */
	Coord pos = Position(time + lineUnit);
	if ((pos <= t) && ((pos + lineHeight) >= b)) {
	    if ((time % DisplayList_hourSize) == 0) {
		/* Draw a dark line */
		output->Line(canvas, 0, pos+lineHeight, xmax, pos+lineHeight);

		/* Print text */
		int hour;
		char meridian;
		char buffer[10];

		hour = time / DisplayList_hourSize;
		if (hour >= 12) {
		    meridian = 'p';
		    hour -= 12;
		}
		else {
		    meridian = 'a';
		}
		if (hour == 0) hour = 12;

		sprintf(buffer, "%d:00%cm", hour, meridian);
		output->Text(canvas,
			     buffer,
			     separatorPos
			     - pad
			     - output->GetFont()->Width(buffer),
			     pos + pad);
	    }
	    else {
		/* Draw a light line */
		Brush* orig = output->GetBrush();

		orig->Reference();
		output->SetBrush(lightBrush);
		output->Line(canvas, separatorPos, pos+lineHeight,
			     xmax, pos+lineHeight);
		output->SetBrush(orig);
		Unref(orig);
	    }
	}

	time += lineUnit;
    }

    output->NoClip();
}

void ApptList::Handle(Event& e) {
    NoteInfo* i;

    if (scrolling) {
	HandleScrollEvent(e);
	return;
    }

    if (moving) {
	HandleMoveEvent(e);
	return;
    }

    if (resizing) {
	HandleResizeEvent(e);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	switch (e.button) {
	  case LEFTMOUSE:
	    DisplayList::Handle(e);
	    break;
	  case MIDDLEMOUSE:
	    i = FindChild(e);
	    if (i == nil) {
		HandleScrollEvent(e);
	    }
	    else {
		Edit(i);
		HandleMoveEvent(e);
	    }
	    break;
	  case RIGHTMOUSE:
	    i = FindChild(e);
	    if (i != nil) {
		Edit(i);
		HandleResizeEvent(e);
	    }
	    break;
	}
	break;
      case KeyEvent:
	if ((focusChild != nil) && (! focusChild->Get()->ReadOnly())) {
	    focusChild->Handle(e);
	}
	break;
      default:
	break;
    }
}

void ApptList::HandleMoveEvent(Event& e) {
    if ((focusChild != nil) && (focusChild->Get()->ReadOnly())) {
	Popup* popup = new Popup("Permission denied.");
	popup->SetName("Warning");
	App()->World()->Popup(popup);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	if (focusChild != nil) {
	    moving = true;
	    CatchMotion();
	    if (e.target != this) e.target->GetRelative(e.x, e.y, this);
	    moveOffsetInChild = Position(focusChild->GetStart()) - e.y;
	}
	break;
      case UpEvent:
	moving = false;
	IgnoreMotion();
	if (focusChild != nil) {
	    /* Update the calendar */
	    focusChild->Save();
	    focusChild->SetModified(false);
	    App()->Change(focusChild->Get());
	}
	break;
      case MotionEvent:
	if (focusChild != nil) {
	    if (e.target != this) e.target->GetRelative(e.x, e.y, this);
	    int newStart = Time(e.y + moveOffsetInChild);
	    newStart = (newStart / itemUnit) * itemUnit;
	    focusChild->SetStart(newStart);
	    ScrollChild(focusChild);
	}
	break;
      default:
	break;
    }
}

void ApptList::HandleResizeEvent(Event& e) {
    if ((focusChild != nil) && (focusChild->Get()->ReadOnly())) {
	Popup* popup = new Popup("Permission denied.");
	popup->SetName("Warning");
	App()->World()->Popup(popup);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	if (focusChild != nil) {
	    CatchMotion();
	    resizing = true;
	    trackTop = false;
	    trackBottom = false;
	}
	break;
      case UpEvent:
	resizing = false;
	IgnoreMotion();
	if (focusChild != nil) {
	    /* Update the calendar */
	    focusChild->Save();
	    focusChild->SetModified(false);
	    App()->Change(focusChild->Get());
	}
	break;
      case MotionEvent:
	if (focusChild != nil) {
	    if (e.target != this) e.target->GetRelative(e.x, e.y, this);
	    int newStart, newLength;
	    boolean changed = false;

	    if (Time(e.y) < focusChild->GetStart()) {
		/* Resize top */
		newStart = Time(e.y);
		newStart = (newStart / itemUnit) * itemUnit;
		newLength = focusChild->GetFinish() - newStart + 1;
		trackTop = true;
		trackBottom = false;
		changed = true;
	    }
	    else if (Time(e.y) > (focusChild->GetFinish())) {
		/* Resize bottom */
		newStart = focusChild->GetStart();
		newLength = Time(e.y) - newStart;
		newLength = (newLength / itemUnit) * itemUnit;
		trackTop = false;
		trackBottom = true;
		changed = true;
	    }
	    else {
		/* Have not left window yet */
		if (trackTop) {
		    /* Can shrink top */
		    if (Time(e.y) < focusChild->GetFinish()) {
			newStart = Time(e.y);
			newStart = (newStart / itemUnit) * itemUnit;
			newLength = focusChild->GetFinish() - newStart + 1;
			changed = true;
		    }
		}
		else if (trackBottom) {
		    /* Can shrink bottom */
		    if (Time(e.y) > focusChild->GetStart()) {
			newStart = focusChild->GetStart();
			newLength = Time(e.y) - focusChild->GetStart();
			newLength = (newLength / itemUnit) * itemUnit;
			changed = true;
		    }
		}
	    }
	    if (changed) {
		/* Minimum length is one line */
		if (newLength >= lineUnit) {
		    focusChild->SetStart(newStart);
		    focusChild->SetLength(newLength);
		    PlaceChild(focusChild);
		}
	    }
	}
	break;
      default:
	break;
    }
}

void ApptList::FixLayout() {
    /*
     * Allocate offset array for local computation.
     */
    int offset[DisplayList_daySize];

    /* Unit of overlap checking must be at least 1 */
    int unit = itemUnit;
    if (unit < 1) unit = 1;

    /* Compute size of day in term of units */
    int dayUnits = DisplayList_daySize / unit;

    for (int i = 0; i < dayUnits; i++)
	offset[i] = 0;

    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	NoteInfo* note =cell->Element();
	int start = note->GetStart() / unit;
	int finish = note->GetFinish() / unit;

	/*
	 * Compute maximum adjustment required by a slot.
	 */
	int adjust = 0;
	for (i = start; i <= finish; i++) {
	    if (adjust < offset[i])
		adjust = offset[i];
	}

	/*
	 * Update adjustment required by overlapping slots.
	 */
	for (i = start; i <= finish; i++)
	    offset[i] = adjust + 1;

	if (note->GetAdjust() != adjust) {
	    note->SetAdjust(adjust);
	    PlaceChild(note);
	}
    }
}


void ApptList::PlaceChild(NoteInfo* child) {
    if (canvas == nil) return;

    /* Place the child */
    Place(child->Visual(),
	  separatorPos + pad + child->GetAdjust() * overlapPad,
	  Bottom(child) - yOffset,
	  xmax - pad,
	  Top(child) - yOffset
	  );
}

void ApptList::ScrollChild(NoteInfo* child) {
    if (canvas == nil) return;

    Move(child->Visual(),
	 separatorPos + pad + child->GetAdjust() * overlapPad,
	 Bottom(child) - yOffset
	 );
}

Item* ApptList::MakeItem(Event& e) {
    int time = Time(e.y);

    if ((time < 0) || (time >= DisplayList_daySize))
	return nil;

    Calendar* cal = App()->GetMainCalendar();
    if (cal->ReadOnly()) {
	Popup* popup = new Popup("Permission denied.");
	popup->SetName("Warning");
	App()->World()->Popup(popup);
	return nil;
    }

    Appointment* appt = new Appointment(cal);
    appt->SetText("");
    appt->SetStart(time - (time % lineUnit));
    appt->SetLength(lineUnit);
    appt->SetDate(date);

    return appt;
}

NoteInfo* ApptList::MakeChild(Item* item) {
    NoteInfo* child = new NoteInfo(item, pad);
    child->Revert();
    child->SetModified(false);
    return child;
}

void ApptList::DefaultPosition() {
    int start  = startTime;
    int length = (canvas == nil)
	? (displayLines * lineUnit)
	: ((ymax * lineUnit) / lineHeight);

    /* Compute min visible and max visible time slot */
    int maxTime = 0;
    int minTime = DisplayList_daySize - 1;
    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	if (cell->Element()->GetStart() < minTime)
	    minTime = cell->Element()->GetStart();
	if (cell->Element()->GetFinish() > maxTime)
	    maxTime = cell->Element()->GetFinish();
    }
    if (minTime < 0) minTime = 0;
    if (maxTime >= DisplayList_daySize) maxTime = DisplayList_daySize - 1;
    int childLength = maxTime - minTime + 1;

    /* Compute start and end times */
    if (minTime < start) {
	/* First appt starts too early */
	start = minTime;
    }
    if (maxTime > (start + length - 1)) {
	/* Last appt ends too late */
	if (childLength < length) {
	    /* All children can fit in */
	    start = minTime + childLength - length;
	}
	else {
	    /* Not all children can fit in - just put first child at top */
	    start = minTime;
	}
    }

    /* Adjust start to lineUnit */
    start = (start / lineUnit) * lineUnit;

    ScrollTo(((DisplayList_daySize - (start+length)) / lineUnit) * lineHeight);
}

int ApptList::Time(Coord y) {
    Coord pos = (numLines * lineHeight) - (y + yOffset);

    return (((pos * lineUnit) / lineHeight) / DisplayList_gridSize)
	* DisplayList_gridSize;
}

Coord ApptList::Position(int time) {
    Coord pos = (time * lineHeight) / lineUnit;

    return (numLines * lineHeight) - pos - yOffset;
}

boolean ApptList::Contains(NoteInfo* child, Coord x, Coord y) {
    return ((Top(child) >= y) &&
	    (Bottom(child) <= y) &&
	    (x >= (separatorPos + pad + child->GetAdjust() * overlapPad)));
}

void ApptList::RaiseChild(NoteInfo* child) {
    if (children->Last()->Element() == child) {
	/* Already last */
	FixLayout();
	return;
    }

    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	if (cell->Element() == child) {
	    children->Remove(cell);
	    children->Append(child);
	    Raise(child->Visual());
	    FixLayout();
	    break;
	}
    }
}

int ApptList::Top(NoteInfo* child) {
    return Position(child->GetStart()) + yOffset;
}

int ApptList::Bottom(NoteInfo* child) {
    return Position(child->GetFinish()) + yOffset;
}
