// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:44:22 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	apptlist.h,v $
 * Revision 1.3  91/11/04  19:44:22  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/08/06  09:46:07  sanjay
 * Notify DayView on selection change.
 * 
 * Revision 1.1  91/03/28  11:41:09  sanjay
 * Initial deposit
 * 
 */

#ifndef _APPTLISTH
#define _APPTLISTH

#include <InterViews/scene.h>
#include "displaylist.h"

class Brush;
class Date;
class DayView;

/*
 * Display appointments for a given day.
 *
 * Class Name:
 *	ApptList
 *
 * Resources:
 *	startTime	Time to put at top initially: format hh:mm.
 *	overlapPad	Horizontal offset for overlapping appointment.
 */
class ApptList : public DisplayList {
  public:
    ApptList(Application*, DayView*, Date);
    virtual ~ApptList();

    virtual void Edit(NoteInfo*);
    virtual void FinishEdit();
  protected:
    virtual void GetAttributes();
    virtual void Redraw(Coord, Coord, Coord, Coord);
    virtual void Handle(Event&);
    virtual void HandleMoveEvent(Event&);
    virtual void HandleResizeEvent(Event&);
    virtual void DefaultPosition();
    virtual void FixLayout();
    virtual int  Top(NoteInfo*);
    virtual int  Bottom(NoteInfo*);
    virtual int  ComputeNumLines();

    virtual boolean   Contains(Item*);
    virtual Item*     MakeItem(Event&);
    virtual NoteInfo* MakeChild(Item*);

    virtual boolean   Contains(NoteInfo*, Coord, Coord);
    virtual void      PlaceChild(NoteInfo*);
    virtual void      ScrollChild(NoteInfo*);
    virtual void      RaiseChild(NoteInfo*);

    Brush* lightBrush;
    int    startTime;
    int    overlapPad;		/* Offset for  overlapping appt */ 
    Coord  separatorPos;	/* X coordinate for separator bar */

    /*
     * Resize information.
     */
    boolean resizing;
    boolean trackTop, trackBottom;

    /*
     * Moving information.
     * Distance between top of child and click Y coordinate.
     */
    Coord moveOffsetInChild;
    boolean moving;

    /*
     * Time/Coordinate conversion.
     * These conversions take the current scroll offset into account.
     */
    int   Time(Coord y);	/* Time for given coordinate */
    Coord Position(int time);	/* Coordinate for given time */

    /*
     * Top-level window.
     */
    DayView* dayview;
};

#endif /* _APPTLISTH */
