/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:44:25 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	arglist.cc,v $
 * Revision 1.5  91/11/04  19:44:25  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/06/28  14:32:27  sanjay
 * Make array deletions portable.
 * 
 * Revision 1.3  91/04/16  15:22:35  sanjay
 * Made closures reference counted
 * 
 * Revision 1.2  91/03/28  11:51:45  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:56  sanjay
 * Initial deposit
 *
 */

#include <string.h> 
#include "config.h"
#include "ArgListOption.Array.h"
#include "arglist.h"
#include "closure.h"

ArgList::ArgList(int count, char* list[], Closure* def)
     : options(*(new ArgListOptionArray))
{
    argc = count;
    args = new char*[argc];
    for (int i = 0; i < argc; i++)
	args[i] = list[i];

    index = 1;
    defaultClosure = def;
    defaultClosure->Reference();
}

ArgList::~ArgList() {
    for (int i = 0; i < options.Size(); i++) {
	Unref(options[i].closure);
    }
    delete &options;

    deleteArray(argc) args;
    Unref(defaultClosure);
}

void ArgList::AddOption(const char* s, Closure* c) {
    ArgListOption option;

    option.str = s;
    option.prefix_match = 0;
    option.closure = c;
    option.closure->Reference();

    options.Append(option);
}

void ArgList::AddPrefixOption(const char* s, Closure* c) {
    ArgListOption option;

    option.str = s;
    option.prefix_match = 1;
    option.closure = c;
    option.closure->Reference();

    options.Append(option);
}

void ArgList::Parse() {
    index = 1;
    while (index < argc) {
	/* Find closure */
	char* arg = args[index];
	Closure* closure = defaultClosure;

	for (int i = options.Size() - 1; i >= 0; i--) {
	    if (options[i].prefix_match) {
		/* Check for prefix */
		int len1 = strlen(arg);
		int len2 = strlen(options[i].str);

		if (len1 < len2) continue;
		if (strncmp(arg, options[i].str, len2) != 0) continue;
	    }
	    else {
		/* Check for exact match */
		if (strcmp(arg, options[i].str) != 0) continue;
	    }

	    /* Matched */
	    closure = options[i].closure;
	    break;
	}

	pending_advance = 1;
	closure->Invoke();
	index += pending_advance;
    }
}

char* ArgList::Fetch(int distance) {
    return args[index + distance];
}

void ArgList::Advance(int k) {
    pending_advance += k;
}

int ArgList::Remaining() {
    return (argc - index - 1);
}
