/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.7 $
 * $Date: 91/11/06 18:01:38 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	confirm.cc,v $
 * Revision 1.7  91/11/06  18:01:38  sanjay
 * Do not maintain auto-save files for shared calendars. Auto-save for
 * shared calendars writes directly to the shared files.
 * 
 * Cleaned up other IO handling.
 * 
 * Revision 1.6  91/11/04  19:44:39  sanjay
 * Prepare for release
 * 
 * Revision 1.5  91/08/06  13:55:07  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.4  91/05/06  11:17:17  sanjay
 * Allow keyboard shortcuts to dismiss dialogs
 * 
 * Revision 1.3  91/04/11  13:30:45  sanjay
 * Fixed window popup
 * 
 * Revision 1.2  91/04/02  12:33:01  sanjay
 * Improved auto-save.
 * 
 * Revision 1.1  91/04/01  20:30:33  sanjay
 * Better IO error handling
 * 
 *
 */

#include <InterViews/button.h>
#include <InterViews/message.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/sensor.h>
#include <InterViews/event.h>
#include "xworld.h"
#include "confirm.h"

static const float space = 0.3;         /* Inches */

Confirm3::Confirm3(char const* title,
		   char const* msg,
		   char const* yesMsg,
		   char const* noMsg,
		   char const* cancelMsg)
     : (new ButtonState(0), nil)
{
    Init(title, new Message(msg, Left), yesMsg, noMsg, cancelMsg);
}

Confirm3::Confirm3(char const* title,
		   char const* file,
		   char const* msg,
		   char const* yesMsg,
		   char const* noMsg,
		   char const* cancelMsg)
     : (new ButtonState(0), nil)
{
    VBox* contents = new VBox
	(new HBox(new Message("File: "), new Message(file)),
	 new Message(msg));
    contents->Align(Left);

    Init(title, contents, yesMsg, noMsg, cancelMsg);
}

Confirm3::Confirm3(char const* title,
		   Interactor* msg,
		   char const* yesMsg,
		   char const* noMsg,
		   char const* cancelMsg)
     : (new ButtonState(0), nil)
{
    Init(title, msg, yesMsg, noMsg, cancelMsg);
}

void Confirm3::Init(char const* title,
		    Interactor* msg,
		    char const* yesMsg,
		    char const* noMsg,
		    char const* cancelMsg)
{
    SetClassName("Confirm3");
    SetInstance(title);
    SetName(title);

    input = new Sensor;
    input->Catch(KeyEvent);
    Listen(input);

    Insert(
        new HBox(
            new HGlue(round(space * inches)),
            new VBox(
                new VGlue(round(space * inches)),
		msg,
                new VGlue(round(space * inches)),
                new HBox(
                    new PushButton(cancelMsg, state, (int) 'c'),
                    new HGlue(round(space * inches)),
                    new PushButton(noMsg, state, (int) 'n'),
                    new HGlue(round(space * inches)),
                    new PushButton(yesMsg, state, (int) 'y')
                    ),
                new VGlue(round(space * inches))
                ),
            new HGlue(round(space * inches))
            )
        );

    /* Unref state so that is deleted when all attached interactors go away */
    Unref(state);
}

char Confirm3::Popup(XWorld* w) {
    w->Popup(this);

    int v;
    state->SetValue(0);
    v = 0;

    /* Only pass events to self */
    Event e;
    do {
	Read(e);

	/* See if event occurred in this window */
	Coord x, y;
	x = e.x;
	y = e.y;
	e.target->GetRelative(x, y, this);

	if ((x >= 0) && (x <= xmax) && (y >= 0) && (y <= ymax)) {
	    e.target->Handle(e);
	    state->GetValue(v);
	}
    } while ((v == 0) && (e.target != nil));

    switch (v) {
      case 'y':
	return 'y';
      case 'n':
	return 'n';
      default:
	return 'c';
    }
}

void Confirm3::Handle(Event& e) {
    if (e.eventType == KeyEvent) {
	switch (e.keystring[0]) {
	  case '\n':
	  case '\r':
	  case 'y':
	  case 'Y':
	    /* Okay */
	    state->SetValue('y');
	    break;

	  case '\007': /* Control-G */
	  case 'c':
	  case 'C':
	    /* Cancel */
	    state->SetValue('c');
	    break;

	  case 'n':
	  case 'N':
	    /* No */
	    state->SetValue('n');
	    break;
	}
    }
}

Confirm2::Confirm2(char const* title,
		   char const* msg,
		   char const* yesMsg,
		   char const* noMsg)
     : (new ButtonState(0), nil)
{
    Init(title, new Message(msg, Left), yesMsg, noMsg);
}

Confirm2::Confirm2(char const* title,
		   char const* file,
		   char const* msg,
		   char const* yesMsg,
		   char const* noMsg)
     : (new ButtonState(0), nil)
{
    VBox* contents = new VBox
	(new HBox(new Message("File: "), new Message(file)),
	 new Message(msg));
    contents->Align(Left);

    Init(title, contents, yesMsg, noMsg);
}

Confirm2::Confirm2(char const* title,
		   Interactor* msg,
		   char const* yesMsg,
		   char const* noMsg)
     : (new ButtonState(0), nil)
{
    Init(title, msg, yesMsg, noMsg);
}

void Confirm2::Init(char const* title,
		    Interactor* msg,
		    char const* yesMsg,
		    char const* noMsg)
{
    SetClassName("Confirm");
    SetInstance(title);
    SetName(title);

    input = new Sensor;
    input->Catch(KeyEvent);
    Listen(input);

    Insert(
        new HBox(
            new HGlue(round(space * inches)),
            new VBox(
                new VGlue(round(space * inches)),
                msg,
                new VGlue(round(space * inches)),
                new HBox(
                    new PushButton(noMsg, state, 2),
                    new HGlue(round(space * inches)),
                    new PushButton(yesMsg, state, 1)
                    ),
                new VGlue(round(space * inches))
                ),
            new HGlue(round(space * inches))
            )
        );

    /* Unref state so that is deleted when all attached interactors go away */
    Unref(state);
}

boolean Confirm2::Popup(XWorld* w) {
    w->Popup(this);

    int v;
    state->SetValue(0);
    v = 0;

    /* Only pass events to self */
    Event e;
    do {
	Read(e);

	/* See if event occurred in this window */
	Coord x, y;
	x = e.x;
	y = e.y;
	e.target->GetRelative(x, y, this);

	if ((x >= 0) && (x <= xmax) && (y >= 0) && (y <= ymax)) {
	    e.target->Handle(e);
	    state->GetValue(v);
	}
    } while ((v == 0) && (e.target != nil));

    return (v == 1);
}

void Confirm2::Handle(Event& e) {
    if (e.eventType == KeyEvent) {
	switch (e.keystring[0]) {
	  case '\n':
	  case '\r':
	  case 'y':
	  case 'Y':
	    /* Okay */
	    state->SetValue(1);
	    break;

	  case '\007': /* Control-G */
	  case 'n':
	  case 'N':
	    state->SetValue(2);
	    break;
	}
    }
}
