// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

#ifndef _CONFIRMH
#define _CONFIRMH

/*
 * $Revision: 1.7 $
 * $Date: 91/11/06 18:01:39 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	confirm.h,v $
 * Revision 1.7  91/11/06  18:01:39  sanjay
 * Do not maintain auto-save files for shared calendars. Auto-save for
 * shared calendars writes directly to the shared files.
 * 
 * Cleaned up other IO handling.
 * 
 * Revision 1.6  91/11/04  19:44:40  sanjay
 * Prepare for release
 * 
 * Revision 1.5  91/08/06  13:55:06  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.4  91/05/06  11:17:18  sanjay
 * Allow keyboard shortcuts to dismiss dialogs
 * 
 * Revision 1.3  91/04/11  13:30:46  sanjay
 * Fixed window popup
 * 
 * Revision 1.2  91/04/02  12:33:02  sanjay
 * Improved auto-save.
 * 
 * Revision 1.1  91/04/01  20:30:34  sanjay
 * Better IO error handling
 * 
 *
 */

/*
 * Dialog for presenting a three-way choice
 *	Yes		'y'
 *	No		'n'
 *	Cancel		'c'
 *
 */

#include <InterViews/dialog.h>

class Confirm3 : public Dialog {
  public:
    Confirm3(const char* title,
	     const char* message,
	     const char* yesMessage,
	     const char* noMessage,
	     const char* cancelMessage);
    Confirm3(const char* title,
	     const char* file,
	     const char* message,
	     const char* yesMessage,
	     const char* noMessage,
	     const char* cancelMessage);
    Confirm3(const char* title,
	     Interactor* message,
	     const char* yesMessage,
	     const char* noMessage,
	     const char* cancelMessage);

    /* Popup dialog and return choice - oneof['y', 'n', 'c'] */
    char Popup(class XWorld*);

    virtual void Handle(Event&);
  private:
    void Init(const char*, Interactor*, const char*, const char*, const char*);
};

/*
 * Two way version of Confirm.
 * Return tue on yes, false on no.
 */

class Confirm2 : public Dialog {
  public:
    Confirm2(const char* title,
	     const char* message,
	     const char* yesMessage,
	     const char* noMessage);
    Confirm2(const char* title,
	     const char* file,
	     const char* message,
	     const char* yesMessage,
	     const char* noMessage);
    Confirm2(const char* title,
	     Interactor* message,
	     const char* yesMessage,
	     const char* noMessage);

    boolean Popup(class XWorld*);

    virtual void Handle(Event&);
  private:
    void Init(const char*, Interactor*, const char*, const char*);
};

#endif /* _CONFIRMH */
