/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/04 19:44:41 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	dateeditor.cc,v $
 * Revision 1.4  91/11/04  19:44:41  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/09/18  14:10:45  sanjay
 * Pass Application to monthdisplay and dateeditor to allow monthdisplay
 * appearence to change with number of items on a certain day of the
 * month. The appearence changing has not been implemented yet.
 * 
 * Revision 1.2  91/03/28  11:51:47  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:41:59  sanjay
 * Initial deposit
 *
 */

#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/adjuster.h>
#include "Month.h"
#include "WeekDay.h"
#include "Year.h"
#include "dateeditor.h"
#include "datestate.h"
#include "field.h"
#include "inteditor.h"
#include "monthdisplay.h"

DateEditor::DateEditor(Application* a, DateState* xd) {
    SetClassName("DateEditor");

    date = xd;

    int d, y;
    Month m;
    WeekDay wd;
    date->GetDate().BreakDown(d, wd, m, y);

    day = new RangeState(d, 1, 31);
    day->Attach(this);
    year = new RangeState(y, Year::First(), Year::Last());
    year->Attach(this);
    month = new MonthState(m.Index(), year);
    month->Attach(this);

    /* Displayers */
    MonthDisplay* md = new MonthDisplay(a, day, month, year);
    IntEditor* monthName  = new IntEditor("monthName",  month, Center);
    IntEditor* yearNumber = new IntEditor("yearNumber", year, CenterRight);

    /* Adjusters */
    Adjuster* monthDown = new LeftMover(monthName, 5);
    Adjuster* monthUp   = new RightMover(monthName, 5);
    Adjuster* yearDown  = new LeftMover(yearNumber, 5);
    Adjuster* yearUp    = new RightMover(yearNumber, 5);

    /* Layout */
    HBox* top = new HBox(monthDown, monthName, monthUp,
			 new HGlue(round(0.25 * inch)),
			 yearDown, yearNumber, yearUp);
    top->Align(Center);

    VBox* entire = new VBox(top, md);
    entire->Align(Center);
    Insert(entire);
}

DateEditor::~DateEditor() {
    day->Detach(this);
    month->Detach(this);
    year->Detach(this);

    if (component != nil) {
	delete component;
	component = nil;
    }

    delete day;
    delete month;
    delete year;
}

void DateEditor::SetDate(Date dt) {
    int d, y;
    Month m;
    WeekDay wd;
    dt.BreakDown(d, wd, m, y);

    day->Set(d);
    month->Set(m.Index());
    year->Set(y);
}

void DateEditor::Update() {
    date->SetDate(Date(day->Get(), (Month::First() + (month->Get() - 1)),
		       year->Get()));
}
