/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.24 $
 * $Date: 91/11/28 14:48:36 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	dayview.cc,v $
 * Revision 1.24  91/11/28  14:48:36  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.23  91/11/04  19:44:44  sanjay
 * Prepare for release
 * 
 * Revision 1.22  91/11/01  14:48:23  sanjay
 * Fixed up menu item display.
 * 
 * Revision 1.21  91/10/28  13:25:25  sanjay
 * Menu items support enable/disable and check marks.
 * 
 * Revision 1.20  91/10/24  16:58:15  sanjay
 * Added check mark on SetRemind menu entry.
 * 
 * Revision 1.19  91/09/18  14:17:10  sanjay
 * Pass Application to monthdisplay and dateeditor to allow monthdisplay
 * appearence to change with number of items on a certain day of the
 * month. The appearence changing has not been implemented yet.
 * 
 * Revision 1.18  91/08/08  16:22:01  sanjay
 * - Added up/down movers to scrollbars.
 * - Fixed up status line layout.
 * - Added command for saving and exiting without
 *   bothering the user with too many questions.
 * 
 * Revision 1.17  91/08/07  10:59:17  sanjay
 * Added re-read.
 * 
 * Revision 1.16  91/08/06  13:55:08  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.15  91/08/06  09:50:44  sanjay
 * Added <Move Item> command.
 * Show calendar for selected item.
 * 
 * Revision 1.14  91/05/06  11:17:19  sanjay
 * Allow keyboard shortcuts to dismiss dialogs
 * 
 * Revision 1.13  91/04/22  07:41:07  sanjay
 * Fixed repeat problems
 * 
 * Revision 1.12  91/04/16  16:29:39  sanjay
 * Added Make Unique op
 * 
 * Revision 1.11  91/04/16  15:22:41  sanjay
 * Made closures reference counted
 * 
 * Revision 1.10  91/04/14  19:07:28  sanjay
 * Allow deletion of specific occurrences of repeating items
 * 
 * Revision 1.9  91/04/11  13:30:46  sanjay
 * Fixed window popup
 * 
 * Revision 1.8  91/04/10  11:33:23  sanjay
 * Added printing
 * 
 * Revision 1.7  91/04/03  21:36:32  sanjay
 * Added range listing
 * 
 * Revision 1.6  91/04/02  12:33:03  sanjay
 * Improved auto-save.
 * 
 * Revision 1.5  91/04/01  20:30:33  sanjay
 * Better IO error handling
 * 
 * Revision 1.4  91/04/01  11:57:03  sanjay
 * Added include file capability
 * 
 * Revision 1.3  91/03/29  16:22:04  sanjay
 * Popup message on usage error
 * 
 * Revision 1.2  91/03/28  11:51:47  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:02  sanjay
 * Initial deposit
 *
 */

#include <InterViews/glue.h>
#include <InterViews/menu.h>
#include <InterViews/box.h>
#include <InterViews/border.h>
#include <InterViews/button.h>
#include <InterViews/filechooser.h>
#include <InterViews/adjuster.h>
#include <InterViews/scroller.h>
#include <string.h>

#include "Month.h"
#include "WeekDay.h"
#include "Year.h"
#include "Date.h"

#include "xworld.h"
#include "popup.h"
#include "item.h"
#include "application.h"
#include "calendar.h"
#include "dayview.h"
#include "allview.h"
#include "apptlist.h"
#include "notelist.h"
#include "dateeditor.h"
#include "datestate.h"
#include "selectinclude.h"
#include "field.h"
#include "fieldshower.h"
#include "confirm.h"
#include "getstring.h"
#include "menuclosure.h"
#include "closure.h"

declare(Closure,DayView);
implement(Closure,DayView);

DayView::DayView(Application* a, Date d) {
    SetClassName("DayView");
    SetName("Calendar");

    application = a;
    application->AddView();

    date = new DateState(d);
    date->Attach(this);

    status1 = new StringState("Selected Calendar: ");
    status2 = new StringState("");

    currentRemindLevel = new ButtonState(-1);
    currentRemindLevel->Reference();

    itemSelected = new ButtonState(false);
    itemSelected->Reference();

    Layout();
}

DayView::~DayView() {
    application->RemoveView();

    delete component;
    component = nil;

    date->Detach(this);
    delete date;

    delete status1;
    delete status2;
    Unref(currentRemindLevel);
    Unref(itemSelected);
}

void DayView::Layout() {
    MenuBar* menuBar = new MenuBar;

    PulldownMenu* calMenu = new PulldownMenu("Calendar");
    calMenu->Include(ViewItem("Save",		DayView::Save));
    calMenu->Include(ViewItem("Re-Read",	DayView::ReRead));
    calMenu->Include(new MenuItem(new HBorder));
    calMenu->Include(ViewItem("Exit",		DayView::SaveExit));
    calMenu->Include(new MenuItem(new HBorder));
    calMenu->Include(ViewItem("Abort",		DayView::AbortExit));

    PulldownMenu* printMenu = new PulldownMenu("Print");
    printMenu->Include(ViewItem("One Day",	DayView::PrintOne));
    printMenu->Include(ViewItem("Two Days",	DayView::PrintTwo));
    printMenu->Include(ViewItem("Four Days",	DayView::PrintFour));
    printMenu->Include(ViewItem("Eight Days",	DayView::PrintEight));
    printMenu->Include(ViewItem("Ten Days",	DayView::PrintTen));
    printMenu->Include(ViewItem("Month",	DayView::PrintMonth));

    PulldownMenu* viewMenu = new PulldownMenu("View");
    viewMenu->Include(ViewItem("Today",		DayView::Today));
    viewMenu->Include(new MenuItem(new HBorder));
    viewMenu->Include(ViewItem("New View",	DayView::NewDay));
    viewMenu->Include(ViewItem("List All",	DayView::NewAll));
    viewMenu->Include(ViewItem("Close",		DayView::Close));

    PulldownMenu* incMenu = new PulldownMenu("Include");
    incMenu->Include(new SelectInclude("Move Item to ...",
				       application,
				       true,
				       this,
				       DayView::MoveToCal,
				       itemSelected,
				       true));
    incMenu->Include(new MenuItem(new HBorder));
    incMenu->Include(ViewItem("Add Include",	DayView::AddCal));
    incMenu->Include(new SelectInclude("Remove Include ...",
				       application,
				       false,
				       this,
				       DayView::RemoveCal));

    PulldownMenu* editMenu = new PulldownMenu("Edit");
    editMenu->Include(ItemItem("Cut",		DayView::Cut));
    editMenu->Include(ItemItem("Copy",		DayView::Copy));
    editMenu->Include(ViewItem("Paste",		DayView::Paste));
    editMenu->Include(new MenuItem(new HBorder));
    editMenu->Include(ItemItem("Make Unique",	DayView::MakeUnique));

    PulldownMenu* repMenu = new PulldownMenu("Repeat");
    repMenu->Include(ItemItem("Don't Repeat",	DayView::NoRepeat));
    repMenu->Include(ItemItem("Daily",		DayView::Daily));
    repMenu->Include(ItemItem("Weekdays Only",	DayView::Weekdays));
    repMenu->Include(ItemItem("Weekly",		DayView::Weekly));
    repMenu->Include(ItemItem("Monthly",	DayView::Monthly));
    repMenu->Include(ItemItem("Annual",		DayView::Annual));
    repMenu->Include(new MenuItem(new HBorder));
    repMenu->Include(ItemItem("Edit ...",	DayView::EditRep));

    PulldownMenu* listMenu = new PulldownMenu("List");
    listMenu->Include(ViewItem("One Day",	DayView::OneList));
    listMenu->Include(ViewItem("Seven Days",	DayView::SevenList));
    listMenu->Include(ViewItem("Ten Days",	DayView::TenList));
    listMenu->Include(ViewItem("Thirty Days",	DayView::ThirtyList));
    listMenu->Include(ViewItem("Week",		DayView::WeekList));
    listMenu->Include(ViewItem("Month",		DayView::MonthList));

    PulldownMenu* rMenu = new PulldownMenu("Remind");
    rMenu->Include(RemindItem(0,  "On Occurrence"));
    rMenu->Include(RemindItem(1,  "A Day Early"));
    rMenu->Include(RemindItem(2,  "Two Days Early"));
    rMenu->Include(RemindItem(5,  "Five Days Early"));
    rMenu->Include(RemindItem(10, "Ten Days Early"));
    rMenu->Include(RemindItem(15, "Fifteen Days Early"));

    menuBar->Include(calMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(printMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(viewMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(incMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(editMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(repMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(listMenu);
    menuBar->Include(new Control(new HGlue(round(0.2 * inch), 0)));
    menuBar->Include(rMenu);

    Box* menuBox = new HBox(menuBar, new HGlue);

    dateEditor = new DateEditor(application, date);
    appts = new ApptList(application, this, date->GetDate());
    notes = new NoteList(application, this, date->GetDate());

    VBox* lhs = new VBox(dateEditor,
			 new HBorder,
			 new HBox(new VBox(new UpMover(notes, 5),
					   new VScroller(notes),
					   new DownMover(notes, 5)),
				  notes));
    lhs->Align(Left);

    HBox* contents = new HBox(lhs,
			      new VBorder,
			      appts,
			      new VBox(new UpMover(appts, 5),
				       new VScroller(appts),
				       new DownMover(appts, 5)));
    contents->Align(Top);

    HBox* statusLine = new HBox
	(new FieldShower("Status", status1, Left),
	 new FieldShower("Status", status2, Left),
	 new HGlue);
    statusLine->Align(Center);
    statusLine->Propagate(false);

    VBox* box = new VBox(menuBox,new HBorder,contents,new HBorder,statusLine);
    box->Align(Left);
    Insert(box);
}

void DayView::SetFocus(Item* item) {
    if (item == nil) {
	status2->Set("");
	currentRemindLevel->SetValue(-1);
	itemSelected->SetValue(false);
    }
    else {
	status2->Set(application->GetShortName(item->GetCalendar()));
	currentRemindLevel->SetValue(item->GetRemindStart());
	itemSelected->SetValue(true);
    }
}

void DayView::Update() {
    appts->SetDate(date->GetDate());
    notes->SetDate(date->GetDate());
}

Item* DayView::Selection() {
    Item* i;

    i = appts->Selection();
    if (i == nil) i = notes->Selection();

    if (i == nil) {
	Warn("No item selected.");
    }
    return i;
}

Item* DayView::ModSelection() {
    Item* i = Selection();
    if ((i != nil) && i->ReadOnly()) {
	Warn("Permission denied.");
	i = nil;
    }
    return i;
}

void DayView::Save() {
    if (application->IsModified()) {
	application->Save();
    }
    else {
	Warn("Calendar has not been modified.");
    }
}

void DayView::ReRead() {
    if (application->IsModified()) {
	Confirm2* cf = new Confirm2
	    ("Warning",
	     "Calendar has been modified. Re-reading will lose changes.",
	     "Re-read",
	     "Cancel");

	boolean result = cf->Popup(application->World());
	delete cf;
	if (! result) {
	    return;
	}
    }
    application->ReRead();
}

void DayView::SaveExit() {
    if (application->IsModified() && (! application->Save())) {
	return;
    }
    application->Quit();
}

void DayView::AbortExit() {
    if (! application->IsModified()) {
	application->Quit();
    }
    else {
	Confirm3* cf = new Confirm3("Warning",
				    "Calendar has been modified.  Save?",
				    "Yes",
				    "No",
				    "Cancel");
	char result = cf->Popup(application->World());
	delete cf;

	switch (result) {
	  case 'y':
	    if (application->Save()) {
		application->Quit();
	    }
	    break;
	  case 'n':
	    application->Quit();
	    break;
	  default:
	    break;
	}
    }
}

boolean DayView::SetPrintCommand() {
    GetString* getter = new GetString("Calendar", "Enter print command",
				      application->GetPrintCommand());
    application->World()->Popup(getter);
    boolean result = getter->Accept();
    if (result) {
	application->SetPrintCommand(getter->Text());
    }
    delete getter;
    return result;
}

void DayView::PrintMonth() {
    if (SetPrintCommand()) {
	application->PrintMonth(date->GetDate());
    }
}

void DayView::PrintOne() {
    if (SetPrintCommand()) {
	application->PrintDays(date->GetDate(), 1, 1, false);
    }
}

void DayView::PrintTwo() {
    if (SetPrintCommand()) {
	application->PrintDays(date->GetDate(), 2, 2, true);
    }
}

void DayView::PrintFour() {
    if (SetPrintCommand()) {
	application->PrintDays(date->GetDate(), 4, 2, false);
    }
}

void DayView::PrintEight() {
    if (SetPrintCommand()) {
	application->PrintDays(date->GetDate(), 8, 4, true);
    }
}

void DayView::PrintTen() {
    if (SetPrintCommand()) {
	application->PrintDays(date->GetDate(), 10, 5, true);
    }
}

void DayView::Today() {
    dateEditor->SetDate(Date::Today());
}

void DayView::NewDay() {
    application->World()->InsertApplication
	(new DayView(application, date->GetDate()));
}

void DayView::NewAll() {
    application->World()->InsertApplication(new AllView(application));
}

void DayView::Close() {
    if (application->OpenViews() > 1) {
	/* There are other open views.  OK to kill this one. */
	delete this;
    }
    else {
	Warn("No other views are open.");
    }
}

void DayView::AddCal() {
    if (application->GetMainCalendar()->ReadOnly()) {
	Warn("Permission denied");
	return;
    }

    char const* lastDir = application->GetLastSelectedDir();

    FileChooser* chooser = new FileChooser
	("Include calendar ...",	/* Title */
	 "",				/* Sub-title */
	 lastDir,			/* Initial directory */
	 10, 24,			/* Dimensions */
	 "Include",			/* Button label */
	 Center				/* Alignment */
	 );

    application->World()->Popup(chooser);

    if (chooser->Accept()) {
	application->AddCalendar(chooser->Choice());
	application->SetSelectedFile(chooser->Choice());
    }

    delete chooser;
}

void DayView::RemoveCal(char const* name) {
    if (application->GetMainCalendar()->ReadOnly()) {
	Warn("Permission denied");
	return;
    }

    application->RemoveCalendar(name);
}

void DayView::Cut() {
    Item* i = ModSelection();
    if (i == nil) return;

    if (i->Repeats()) {
	/* Item repeats - ask if all occurrences should be cut */
	Confirm3* cf = new Confirm3("Warning",
				    "This item repeats. Cut all occurrences?",
				    "Yes",
				    "Just this one",
				    "Cancel");
	char result = cf->Popup(application->World());
	delete cf;

	switch (result) {
	  case 'y':
	    /* Cut all occurrences */
	    application->SetClipboard(i);
	    application->Delete(i);
	    break;
	  case 'n':
	    /* Cut this occurrence */
	    application->SetClipboard(i);
	    i->DeleteOn(date->GetDate());
	    application->Change(i);
	    break;
	  case 'c':
	    /* Cancel */
	    break;
	}
    }
    else {
	/* Delete all occurrences */
	application->SetClipboard(i);
	application->Delete(i);
    }
}

void DayView::Copy() {
    Item* i = Selection();
    if (i != nil) {
	application->SetClipboard(i);
    }
}

void DayView::Paste() {
    if (application->GetMainCalendar()->ReadOnly()) {
	Warn("Permission denied");
	return;
    }

    Item* i = application->GetClipboard();
    if (i != nil) {
	Item* newItem = i->Clone();
	newItem->SetDate(date->GetDate());
	application->Add(newItem, application->GetMainCalendar());
    }
    else {
	Warn("No item in clipboard.");
    }
}

void DayView::MakeUnique() {
    Item* i = ModSelection();
    if (i == nil) return;

    if (! i->Repeats()) {
	Warn("Item does not repeat.");
	return;
    }

    /*
     * Make new item that occurs on just this date.
     * Suppress old item on this date.
     */
    Item* x = i->Clone();
    x->SetDate(date->GetDate());
    i->DeleteOn(date->GetDate());

    application->Change(i);
    application->Add(x, i->GetCalendar());
}

void DayView::MoveToCal(char const* name) {
    Calendar* cal = application->GetCalendarByName(name);
    if (cal == nil) {
	Warn("Internal error: calendar not found");
	return;
    }

    if (cal->ReadOnly()) {
	Warn("Permission denied");
	return;
    }

    Item* i = ModSelection();
    if (i == nil) return;

    /* Make new copy */
    Item* x = i->Clone();
    x->SetCalendar(cal);

    application->Add(x, cal);
    application->Delete(i);
}

void DayView::EditRep() {
    Item* i = ModSelection();
    if (i != nil) {
	application->NewEditRepeat(i);
    }
}

void DayView::NoRepeat() {
    Item* i = ModSelection();
    if (i != nil) {
	i->SetDate(date->GetDate());
	application->Change(i);
    }
}

void DayView::Daily() {
    Item* i = ModSelection();
    if (i != nil) {
	SmallIntSet set;

	/* Create set containing all possible days of the month */
	set.Clear();
	for (int index = 1; index <= 31; index++)
	    set.Insert(index);
	i->SetMonthDays(set);

	/* Create set containing all possible months */
	set.Clear();
	for (index = 1; index <= 12; index++)
	    set.Insert(index);
	i->SetMonths(set);

	i->SetEveryYear();

	application->Change(i);
    }
}

void DayView::Weekdays() {
    Item* i = ModSelection();
    if (i != nil) {
	SmallIntSet set;

	/* Create set containing all weekdays */
	set.Clear();
	for (int index = 1; index <= 7; index++) {
	    set.Insert(index);
	}
	set.Remove(WeekDay::Sunday().Index());
	set.Remove(WeekDay::Saturday().Index());
	i->SetWeekDays(set);

	/* Create set containing all possible months */
	set.Clear();
	for (index = 1; index <= 12; index++)
	    set.Insert(index);
	i->SetMonths(set);

	i->SetEveryYear();

	application->Change(i);
    }
}

void DayView::Weekly() {
    Item* i = ModSelection();
    if (i != nil) {
	SmallIntSet set;

	/* Create set containing this weekday */
	WeekDay wDay = date->GetDate().GetWDay();
	set.Clear();
	set.Insert(wDay.Index());
	i->SetWeekDays(set);

	/* Create set containing all possible months */
	set.Clear();
	for (int index = 1; index <= 12; index++)
	    set.Insert(index);
	i->SetMonths(set);

	i->SetEveryYear();

	application->Change(i);
    }
}

void DayView::Monthly() {
    Item* i = ModSelection();
    if (i != nil) {
	SmallIntSet set;

	/* Create set containing just this day of the month */
	set.Clear();
	set.Insert(date->GetDate().GetMDay());
	i->SetMonthDays(set);

	/* Create set containing all possible months */
	set.Clear();
	for (int index = 1; index <= 12; index++)
	    set.Insert(index);
	i->SetMonths(set);

	i->SetEveryYear();

	application->Change(i);
    }
}

void DayView::Annual() {
    Item* i = ModSelection();
    if (i != nil) {
	SmallIntSet set;

	/* Create set containing just this day of the month */
	set.Clear();
	set.Insert(date->GetDate().GetMDay());
	i->SetMonthDays(set);

	/* Create set containing just this month */
	set.Clear();
	set.Insert(date->GetDate().GetMonth().Index());
	i->SetMonths(set);

	i->SetEveryYear();

	application->Change(i);
    }
}

void DayView::OneList() {
    Date d = date->GetDate();
    application->GenerateReminders(d, d);
}

void DayView::SevenList() {
    Date d = date->GetDate();
    application->GenerateReminders(d, d + 6);
}

void DayView::TenList() {
    Date d = date->GetDate();
    application->GenerateReminders(d, d + 9);
}

void DayView::ThirtyList() {
    Date d = date->GetDate();
    application->GenerateReminders(d, d + 29);
}

void DayView::WeekList() {
    Date d = date->GetDate();
    int diff = d.GetWDay() - WeekDay::First();
    application->GenerateReminders(d - diff, d - diff + 6);
}

void DayView::MonthList() {
    Date d = date->GetDate();
    int mday, year;
    Month month;
    WeekDay wday;
    d.BreakDown(mday, wday, month, year);

    Date firstOfMonth = d - mday + 1;
    Date lastOfMonth  = firstOfMonth + month.Size(year) - 1;
    application->GenerateReminders(firstOfMonth, lastOfMonth);
}

void DayView::SetRemind(int days) {
    Item* i = ModSelection();
    if (i != nil) {
	i->SetRemindStart(days);
	currentRemindLevel->SetValue(days);
	application->Change(i);
    }
}

void DayView::Warn(char const* msg) {
    Popup* popup = new Popup(msg);
    popup->SetName("Warning");
    application->World()->Popup(popup);
}

MenuItem* DayView::ViewItem(const char* str, void (DayView::*method)(),
			    ButtonState* enableState, int enableValue)
{
    return new MenuClosure(str, new Closure1(DayView)(this, method),
			   enableState, enableValue);
}

MenuItem* DayView::ItemItem(const char* str, void (DayView::*method)()) {
    return new MenuClosure(str, new Closure1(DayView)(this, method),
			   itemSelected, true);
}

class DVSetRemindClosure : public Closure {
  public:
    DVSetRemindClosure(DayView* v, int d) {
	view = v;
	days = d;
    }
    virtual void Invoke();
  private:
    DayView* view;
    int      days;
};

void DVSetRemindClosure::Invoke() {
    view->SetRemind(days);
}

MenuItem* DayView::RemindItem(int d, const char* t) {
    return new MenuClosure(t, new DVSetRemindClosure(this, d),
			   itemSelected, true,
			   currentRemindLevel, d);
}
