// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.12 $
 * $Date: 91/11/04 19:44:46 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	dayview.h,v $
 * Revision 1.12  91/11/04  19:44:46  sanjay
 * Prepare for release
 * 
 * Revision 1.11  91/11/01  14:48:25  sanjay
 * Fixed up menu item display.
 * 
 * Revision 1.10  91/10/24  16:58:13  sanjay
 * Added check mark on SetRemind menu entry.
 * 
 * Revision 1.9  91/08/08  16:20:50  sanjay
 * Renamed some member functions.
 * Added StringState for status title.
 * 
 * Revision 1.8  91/08/07  10:59:19  sanjay
 * Added re-read.
 * 
 * Revision 1.7  91/08/06  09:49:39  sanjay
 * Added <Move Item> command.
 * Show calendar for selected item.
 * 
 * Revision 1.6  91/04/16  16:29:41  sanjay
 * Added Make Unique op
 * 
 * Revision 1.5  91/04/16  15:22:43  sanjay
 * Made closures reference counted
 * 
 * Revision 1.4  91/04/10  11:33:25  sanjay
 * Added printing
 * 
 * Revision 1.3  91/04/03  21:36:31  sanjay
 * Added range listing
 * 
 * Revision 1.2  91/04/01  11:57:04  sanjay
 * Added include file capability
 * 
 * Revision 1.1  91/03/28  11:41:16  sanjay
 * Initial deposit
 * 
 */

#ifndef _DAYVIEWH
#define _DAYVIEWH

#include <InterViews/scene.h>
#include "Date.h"

/*
 * View a day.
 */

class Application;
class ApptList;
class ButtonState;
class DateState;
class DateEditor;
class DayView;
class Item;
class MenuItem;
class NoteList;
class StringState;

class DayView : public MonoScene {
  public:
    DayView(Application*, Date);
    virtual ~DayView();

    virtual void Update();

    /* Set focus */
    void SetFocus(Item*);

    /* Public to allow calling from closure */
    void SetRemind(int);
  protected:
    Application* application;
    ApptList*    appts;
    NoteList*	 notes;
    DateState*   date;
    DateEditor*  dateEditor;
    StringState* status1;
    StringState* status2;
    ButtonState* currentRemindLevel;
    ButtonState* itemSelected;

    void Layout();

    /* Obtain Selection for read-only operation */
    Item* Selection();

    /* Item selection for mutator operation */
    Item* ModSelection();

    /* Calendar operations. */
    void Save();
    void ReRead();
    void SaveExit();
    void AbortExit();

    boolean SetPrintCommand();
    void PrintMonth();
    void PrintOne();
    void PrintTwo();
    void PrintFour();
    void PrintEight();
    void PrintTen();

    void Today();
    void NewDay();
    void NewAll();
    void Close();

    void AddCal();
    void RemoveCal(char const*);
    void ListCal(char const*);

    void Cut();
    void Copy();
    void Paste();
    void MoveToCal(char const*);
    void MakeUnique();

    /* Repetition operations. */
    void NoRepeat();
    void Daily();
    void Weekdays();
    void Weekly();
    void Monthly();
    void Annual();
    void EditRep();

    /* Listing operations */
    void OneList();
    void SevenList();
    void TenList();
    void ThirtyList();
    void WeekList();
    void MonthList();

    void Warn(char const*);

    /* Make a general menu item */
    MenuItem* ViewItem(const char*, void (DayView::*method)(),
		       ButtonState* enable = nil, int value = 0);

    /* Make a menu item that operates only when an item is selected */
    MenuItem* ItemItem(const char*, void (DayView::*method)());

    /* Make a menu item for changing remind levels */
    MenuItem* RemindItem(int, const char*);
};

#endif /* _DAYVIEWH */
