/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.8 $
 * $Date: 91/11/28 16:24:28 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	displaylist.cc,v $
 * Revision 1.8  91/11/28  16:24:28  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.7  91/11/04  19:44:49  sanjay
 * Prepare for release
 * 
 * Revision 1.6  91/08/06  09:53:45  sanjay
 * Get item list from application instead of calendar.
 * 
 * Revision 1.5  91/04/10  18:00:23  sanjay
 * Fixed redraw problems
 * 
 * Revision 1.4  91/04/01  11:57:05  sanjay
 * Added include file capability
 * 
 * Revision 1.3  91/03/30  15:06:30  sanjay
 * Removed funky item creation
 * 
 * Revision 1.2  91/03/28  11:51:48  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:03  sanjay
 * Initial deposit
 *
 */

#include <InterViews/shape.h>
#include <InterViews/font.h>
#include <InterViews/painter.h>
#include <InterViews/perspective.h>
#include <InterViews/event.h>
#include <InterViews/canvas.h>
#include <InterViews/sensor.h>
#include <stdio.h>
#include "displaylist.h"
#include "item.h"
#include "application.h"
#include "noteinfo.h"
#include "basic.h"
#include "NoteInfoP.List.h"

/*
 * Macro for traversing children.
 */
#define ForChildren(l,c) for(c = (l)->First(); c; c = c->Next())
#define ForRevChildren(l,c) for(c = (l)->Last(); c; c = c->Prev())

declare(AVForwarder,DisplayList)
implement(AVForwarder,DisplayList)

DisplayList::DisplayList(Application* a) {
    Init(a);
}

void DisplayList::Init(Application* a) {
    SetClassName("DisplayList");

    application = a;

    view = new AVForwarder(DisplayList)(a, this);

    yOffset = 0;
    scrolling = false;

    lineUnit = 30;
    itemUnit = 15;
    pad = 2;
    lineHeight = 20;
    textWidth = DisplayList_defaultTextWidth;
    totalWidth = round(textWidth * inches);
    displayLines = DisplayList_defaultDisplayLines;

    children = new NoteInfoPList;
    focusChild = nil;

    catchingMotion = false;

    perspective = new Perspective;

    input = new Sensor;
    input->Catch(KeyEvent);
    input->Catch(UpEvent);
    input->Catch(DownEvent);
    Listen(input);

    numLines = ComputeNumLines();
}

DisplayList::~DisplayList() {
    FinishEdit();

    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	NoteInfo* info = cell->Element();
	delete info;
    }

    delete children;
    delete view;
    Unref(perspective);
    Unref(input);
}

Application* DisplayList::App() {
    return application;
}

void DisplayList::Edit(NoteInfo* note) {
    if ((focusChild != nil) && (focusChild != note)) {
	FinishEdit();
    }
    if (focusChild != note) {
	focusChild = note;
	note->Focussed();
	App()->SetFocus(view);
    }
}

void DisplayList::Msg(ApptViewMsg t) {
    switch (t) {
      case ApptView::Save:
	Save();
	break;
      case ApptView::Focus:
	Focus();
	break;
      case ApptView::Unfocus:
	Unfocus();
	break;
    }
}

void DisplayList::Save() {
    if (focusChild != nil) {
	NoteInfo* note = focusChild;

	if (note->Modified()) {
	    /* Save note */
	    note->Save();
	    note->SetModified(false);
	    App()->Change(note->Get());
	}
    }
}

void DisplayList::FinishEdit() {
    IgnoreMotion();

    if (focusChild != nil) {
	NoteInfo* note = focusChild;

	focusChild = nil;
	note->Unfocussed();
	if (App()->Focus() == view)
	    App()->SetFocus(nil);

	if (note->Modified()) {
	    /* Save note */
	    note->Save();
	    note->SetModified(false);
	    App()->Change(note->Get());
	}
    }
}

boolean DisplayList::Contains(Item* a) {
    return true;
}

boolean DisplayList::Selected() {
    return (focusChild != nil);
}

Item* DisplayList::Selection() {
    if (focusChild != nil) {
	Save();
	return focusChild->Get();
    }
    else {
	return nil;
    }
}

void DisplayList::SetDate(Date d) {
    int i;

    if (date == d)
	return;

    FinishEdit();

    date = d;

    /* Add all items for new day */
    int calSize = App()->NumItems();
    NoteInfoPList* oldList = children;
    NoteInfoPList* insertList = new NoteInfoPList;
    children = new NoteInfoPList;
    for (i = 0; i < calSize; i++) {
	Item* item = App()->GetItem(i);

	if (Contains(item)) {
	    NoteInfo* child;

	    if (oldList->Empty()) {
		/* Have to create new editor */
		child = MakeChild(item);
		children->Append(child);
		insertList->Append(child);
	    }
	    else {
		/* Reuse old editor */
		child = oldList->First()->Element();
		oldList->Remove(oldList->First());
		child->Set(item);
		child->Revert();
		child->SetModified(false);
		children->Append(child);
	    }
	}
    }

    /*
     * Delete old unused children.
     */
    NoteInfoPCell* cell;
    ForChildren(oldList, cell) {
	NoteInfo* info = cell->Element();
	delete info;
    }
    delete oldList;

    /*
     * Insert newly inserted children.
     */
    ForChildren(insertList, cell) {
	Insert(cell->Element()->Visual());
    }
    delete insertList;

    /*
     * Place all children.
     */
    ForChildren(children, cell) {
	PlaceChild(cell->Element());
    }

    FixLayout();
    DefaultPosition();
}

void DisplayList::ItemMsg(ApptViewItemMsg t, Item* item) {
    boolean isMember = (t != ApptView::Delete) && Contains(item);

    if ((focusChild != nil) && (focusChild->Get() == item) && (! isMember)) {
	/*
	 * The focus child is about to disappear from this date.
	 */
	FinishEdit();
    }

    /* Handle add/delete if item is a new or old member for day */
    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	NoteInfo* i = cell->Element();
	if (i->Get() == item) {
	    /* Check to see if item still exists for today */
	    if (! isMember) {
		children->Remove(cell);
		delete i;
		FixLayout();
		return;
	    }

	    i->Revert();
	    i->SetModified(false);
	    PlaceChild(i);
	    FixLayout();
	    return;
	}
    }

    /* Check if we need to add the item */
    if (isMember) {
	NoteInfo* child = MakeChild(item);
	children->Append(child);
	Insert(child->Visual());
	PlaceChild(child);
	FixLayout();
    }
}

void DisplayList::Focus() { }

void DisplayList::Unfocus() {
    FinishEdit();
}

void DisplayList::Draw() {
    Redraw(0, 0, xmax, ymax);

    NoteInfoPCell* cell;
    ForChildren(children, cell)
	cell->Element()->Visual()->Draw();
}

void DisplayList::Resize() {
    if (canvas != nil) {
	canvas->SetBackground(output->GetBgColor());
	numLines = ComputeNumLines();
	perspective->height = numLines * lineHeight;
	perspective->width = xmax + 1;
	perspective->curwidth = xmax + 1;
	perspective->curheight = ymax + 1;

	NoteInfoPCell* cell;
	ForChildren(children, cell)
	    PlaceChild(cell->Element());

	Draw();

	ScrollTo(perspective->cury);
    }
}

void DisplayList::Adjust(Perspective& np) {
    float scale = float(perspective->height) / float(np.height);
    ScrollTo(perspective->y0 + int((np.cury - np.y0) * scale));
    np = *perspective;
}

void DisplayList::Reconfig() {
    GetAttributes();
    SetShape();
    SetPerspective();
    DefaultPosition();
}

void DisplayList::GetAttributes() {
    lineHeight = output->GetFont()->Height() + 4 * pad;

    /* Get resources */
    const char* a;
    int value;
    float fValue;

    if ((a = GetAttribute("itemUnit")) != nil) {
	sscanf(a, "%d", &value);

	/* Make a multiple of gridSize */
	if ((value % DisplayList_gridSize) != 0) {
	    value = (value / DisplayList_gridSize) * DisplayList_gridSize;
	}

	/* Should be > 0 and should divide day */
	if ((value > 0) && ((DisplayList_daySize % value) == 0))
	    itemUnit = value;
    }

    if ((a = GetAttribute("lineUnit")) != nil) {
	sscanf(a, "%d", &value);

	/* Make a multiple of gridSize */
	if ((value % DisplayList_gridSize) != 0)
	    value = (value / DisplayList_gridSize) * DisplayList_gridSize;

	/* Should be > 0 and should divide day */
	if ((value > 0) && ((DisplayList_daySize % value) == 0))
	    lineUnit = value;
    }

    if ((a = GetAttribute("pad")) != nil) {
	sscanf(a, "%d", &value);
	if (value >= 2) pad = value;
    }

    if ((a = GetAttribute("textWidth")) != nil) {
	sscanf(a, "%f", &fValue);

	if (fValue > 0.0)
	    textWidth = fValue;
    }

    if ((a = GetAttribute("displayLines")) != nil) {
	sscanf(a, "%d", &value);

	if (value > 0)
	    displayLines = value;
    }
    numLines = ComputeNumLines();
    if (displayLines > numLines)
	displayLines = numLines;

    totalWidth = round(textWidth * inches);
}

void DisplayList::SetShape() {
    shape->vunits = lineHeight;
    shape->Rect(totalWidth, lineHeight * displayLines);
    shape->Rigid(0, hfil,
		 lineHeight * (displayLines - 1),
		 lineHeight * (numLines - displayLines));
}

void DisplayList::SetPerspective() {
    /* Setup perspective */
    yOffset = 0;

    perspective->Init(0, 0, totalWidth, numLines * lineHeight);
    perspective->curx = 0;
    perspective->cury = 0;
    perspective->curwidth = totalWidth;
    perspective->curheight = displayLines * lineHeight;
    perspective->sx = 0;
    perspective->sy = lineHeight;
    perspective->lx = 0;
    perspective->ly = lineHeight;
}

void DisplayList::Redraw(Coord l, Coord b, Coord r, Coord t) {
    output->ClearRect(canvas, l, b, r, t);
}

int DisplayList::ComputeNumLines() {
    int length = 0;

    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	length += cell->Element()->GetLength();
    }

    /*
     * Make length at least as big as window height,
     * and allow at least two extra lines.
     */
    int newLength = length;
    if (canvas != nil) {
	if (newLength < (ymax * lineUnit) / lineHeight)
	    newLength = (ymax * lineUnit) / lineHeight;
    }
    if (newLength < (length + 2 * lineUnit))
	newLength = length + 2 * lineUnit;

    return (newLength + lineUnit - 1) / lineUnit;
}

void DisplayList::Handle(Event& e) {
    NoteInfo* i;

    if (scrolling) {
	HandleScrollEvent(e);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	switch (e.button) {
	  case LEFTMOUSE:
	    i = FindChild(e);
	    if ((i == nil) && (e.target == this)) {
		Item* item = MakeItem(e);
		if (item != nil) {
		    App()->Add(item, App()->GetMainCalendar());

		    /* Now find resulting note and Edit it */
		    NoteInfoPCell* cell;
		    ForChildren(children, cell) {
			if (cell->Element()->Get() == item) {
			    Edit(cell->Element());
			    break;
			}
		    }
		}
	    }
	    else if (i != nil) {
		/* Focus on the selected Item */
		Edit(i);
		if (focusChild != nil) focusChild->Handle(e);
	    }
	    break;
	  case MIDDLEMOUSE:
	    HandleScrollEvent(e);
	    break;
	  default:
	    break;
	}
	break;
      default:
	break;
    }
}

void DisplayList::HandleScrollEvent(Event& e) {
    switch (e.eventType) {
      case DownEvent:
	scrolling = true;
	CatchMotion();
	if (e.target != this) e.target->GetRelative(e.x, e.y, this);
	scrollInfo = e.y;		/* Last mouse position */
	break;
      case UpEvent:
	scrolling = false;
	IgnoreMotion();
	break;
      case MotionEvent:
	if (e.target != this) e.target->GetRelative(e.x, e.y, this);
	int oldOffset = perspective->cury;
	ScrollBy(scrollInfo - e.y);
	scrollInfo = scrollInfo - (perspective->cury - oldOffset);
	break;
      default:
	break;
    }
}

void DisplayList::FixLayout() {
    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	PlaceChild(cell->Element());
    }
    int lines = ComputeNumLines();
    if (numLines != lines) {
	numLines = lines;
	if (canvas != nil) Resize();
    }
}

void DisplayList::PlaceChild(NoteInfo* child) {
    if (canvas == nil) return;

    /* Place the child */
    Place(child->Visual(), 0, Bottom(child)-yOffset, xmax, Top(child)-yOffset);
}

void DisplayList::ScrollChild(NoteInfo* child) {
    if (canvas == nil) return;

    Move(child->Visual(), 0, Bottom(child) - yOffset);
}

Item* DisplayList::MakeItem(Event& e) {
    return nil;
}

NoteInfo* DisplayList::MakeChild(Item* item) {
    NoteInfo* child = new NoteInfo(item, pad);
    child->Revert();
    child->SetModified(false);
    return child;
}

void DisplayList::GetComponents(Interactor** list, int listSize,
			    Interactor**& result, int& resultSize)
{
    NoteInfoPCell* cell;
    int numChildren;

    numChildren = 0;
    
    ForChildren(children, cell)
	numChildren++;

    resultSize = numChildren;
    result = (listSize < numChildren) ? (new Interactor*[numChildren]) : list;

    int index = 0;
    ForChildren(children, cell) {
	result[index] = cell->Element()->Visual();
	index++;
    }
}

void DisplayList::ScrollTo(int y) {
    /* Get first line number */
    int pos = (numLines * lineHeight) - (y + perspective->curheight);
    int firstLine = ((pos * lineUnit) / lineHeight) / lineUnit;

    /* Make sure first is not too large */
    int size = (perspective->curheight + lineHeight - 1) / lineHeight;

    if (firstLine + size > numLines) {
	firstLine = numLines - size;
    }

    /* Make sure first is not too small */
    if (firstLine < 0) {
	firstLine = 0;
    }

    /* Compute y offset and scroll */
    perspective->cury = (perspective->height - (firstLine * lineHeight)
			 - perspective->curheight);
    perspective->Update();

    if (yOffset != perspective->cury) {
	/* Scroll region */
	Coord change = perspective->cury - yOffset;
	yOffset = perspective->cury;

	if (canvas != nil) {
	    NoteInfoPCell* cell;
	    ForChildren(children, cell) {
		ScrollChild(cell->Element());
	    }

	    MoveBackground(change);
	}
    }
}

void DisplayList::MoveBackground(Coord) {
    if (canvas != nil) {
	Redraw(0, 0, xmax, ymax);
    }
}

void DisplayList::ScrollBy(int dy) {
    if (dy != 0) {
	ScrollTo(perspective->cury + dy);
    }
}

void DisplayList::DefaultPosition() {
    ScrollTo((numLines - displayLines) * lineHeight);
}

NoteInfo* DisplayList::FindChild(Event& e) {
    NoteInfoPCell* cell;

    int pos = e.y + yOffset;

    /*
     * Return matching child.
     */
    ForRevChildren(children, cell) {
	NoteInfo* i = cell->Element();

	if (Contains(i, e.x, pos)) return i;
    }

    return nil;
}

boolean DisplayList::Contains(NoteInfo* child, Coord x, Coord y) {
    return (Top(child) >= y) && (Bottom(child) <= y);
}

NoteInfoPCell* DisplayList::FindCell(NoteInfo* child) {
    NoteInfoPCell* cell;

    ForChildren(children, cell) {
	if (cell->Element() == child)
	    return cell;
    }
    return nil;
}

int DisplayList::Top(NoteInfo* child) {
    int length = 0;
    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	if (cell->Element() == child) {
	    return (numLines * lineHeight) - (length * lineHeight) / lineUnit;
	}
	length += cell->Element()->GetLength();
    }
    return (numLines * lineHeight);
}

int DisplayList::Bottom(NoteInfo* child) {
    int length = 0;
    NoteInfoPCell* cell;
    ForChildren(children, cell) {
	length += cell->Element()->GetLength();
	if (cell->Element() == child) {
	    return (numLines * lineHeight) - (length * lineHeight) / lineUnit;
	}
    }
    return 0;
}

void DisplayList::CatchMotion() {
    if (! catchingMotion) {
	catchingMotion = true;
	input->Catch(MotionEvent);
	Listen(input);
    }
}

void DisplayList::IgnoreMotion() {
    if (catchingMotion) {
	catchingMotion = false;
	input->Ignore(MotionEvent);
	Listen(input);
    }
}
