// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.5 $
 * $Date: 91/11/28 16:24:31 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	displaylist.h,v $
 * Revision 1.5  91/11/28  16:24:31  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.4  91/11/04  19:44:50  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/04/10  18:00:24  sanjay
 * Fixed redraw problems
 * 
 * Revision 1.2  91/03/30  15:06:30  sanjay
 * Removed funky item creation
 * 
 * Revision 1.1  91/03/28  11:41:17  sanjay
 * Initial deposit
 * 
 */
#ifndef _DISPLAYLISTH
#define _DISPLAYLISTH

#include <InterViews/scene.h>
#include "apptview.h"
#include "Date.h"

class Item;
class Application;

class NoteInfo;
class NoteInfoPList;
class NoteInfoPCell;

/*
 * Display Items for a given day.
 *
 * Class Name:
 *	DisplayList
 *
 * Resources Used:
 *	lineUnit	Time per line in minutes.
 *	itemUnit	Items must be multiple of this.
 *	pad		Padding.
 *	textWidth	Width of text entry portion (in inches).
 *	displayLines	Number of lines to display.
 */
class DisplayList : public Scene {
  public:
    DisplayList(Application*);
    virtual ~DisplayList();

    /*
     * Date.
     */
    void	SetDate(Date);
    Date	GetDate()		{ return date; }

    Application* App();

    /*
     * Is a child of this list selected?
     */
    virtual boolean Selected();

    /*
     * Get selected appointment, if any.
     */
    virtual Item* Selection();

    /*
     * Start editing specified child.
     */
    virtual void Edit(NoteInfo*);

    /*
     * Finish editing focusChild, if any.
     */
    virtual void FinishEdit();

    /* ApptView callbacks */
    virtual void ItemMsg(ApptViewItemMsg, Item*);
    virtual void Msg(ApptViewMsg);

    /*
     * Shape/Contents
     */
    virtual void Draw();
    virtual void Resize();
    virtual void Adjust(Perspective&);
  protected:
    void Init(Application*);

    /*
     * To be overriden by derived classes.
     */
    virtual void Reconfig();
    virtual void Redraw(Coord, Coord, Coord, Coord);
    virtual void Handle(Event&);

    /*
     * Various notifications.
     */
    void Save();
    void Focus();
    void Unfocus();

    /*
     * Routines called in order by default Reconfig.
     */
    virtual void GetAttributes();
    virtual void SetShape();
    virtual void SetPerspective();

    virtual boolean Contains(Item*);		/* Should item be displayed */
    virtual int     ComputeNumLines();		/* Compute total size */

    /*
     * Layout routines.
     * By default, children are laid out contiguosly from top to bottom.
     */
    virtual void    FixLayout();		/* Re-layout everybody */
    virtual int     Top(NoteInfo*);		/* Compute child top coord */
    virtual int	    Bottom(NoteInfo*);		/* Compute child bot coord */
    virtual boolean Contains(NoteInfo*, Coord, Coord);	/* Child contains pt */

    /*
     * Create a new item.
     * Can return nil to indicate that no child editor
     * should be created.
     */
    virtual Item* MakeItem(Event&);

    /*
     * Create a new child - and initialize from item.
     */
    virtual NoteInfo* MakeChild(Item*);

    /*
     * Scroll to default position.
     */
    virtual void DefaultPosition();

    /*
     * Move window background in response to a scroll.
     * Change is coordinate amount by which background
     * should be moved down (negative amounts mean move
     * up).
     */
    virtual void MoveBackground(Coord change);

    /*
     * Children:
     */
    virtual NoteInfoPCell*	FindCell(NoteInfo*);
    virtual NoteInfo*		FindChild(Event&);

    virtual void		PlaceChild(NoteInfo*);
    virtual void		ScrollChild(NoteInfo*);

    /*
     * Scrolling.
     */
    boolean scrolling;		/* Are we scrolling? */
    Coord   yOffset;		/* Real Y coord of bottom point displayed */
    Coord   scrollInfo;		/* Screen Y coord of last scroll event */

    virtual void HandleScrollEvent(Event&);
    void	 ScrollTo(int);
    void	 ScrollBy(int);

    /*
     * Date.
     */
    Date date;

    /*
     * Attributes.
     */
    int lineUnit;
    int itemUnit;
    int pad;
    int displayLines;
    int numLines;
    int lineHeight;
    int totalWidth;
    float textWidth;

    /*
     * Children.
     */
    NoteInfoPList* children;
    NoteInfo* focusChild;

    virtual void GetComponents(Interactor**, int, Interactor**&, int&);

    /*
     * Event handling.
     */
    boolean catchingMotion;

    void CatchMotion();
    void IgnoreMotion();

    /*
     * ApptView that forwards to this.
     */
    ApptView* view;

    Application* application;

    /*
     * Constants.
     */
};

#define DisplayList_hourSize		(60)
#define DisplayList_daySize		(24 * 60)
#define DisplayList_defaultTextWidth	(3.5)
#define DisplayList_defaultDisplayLines	(20)
#define DisplayList_gridSize		(5)
#define DisplayList_defaultStartTime	(8 * DisplayList_hourSize)
#define DisplayList_dayGrids		(DisplayList_daySize / DisplayList_gridSize)

#endif /* _DISPLAYLISTH */

