/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:44:51 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	dmenu.cc,v $
 * Revision 1.3  91/11/04  19:44:51  sanjay
 * Prepare for release
 * 
 *
 */

/*
 * Dynamic menu implementation.
 */

#include <InterViews/box.h>
#include <InterViews/frame.h>
#include <InterViews/shape.h>
#include <InterViews/world.h>
#include "dmenu.h"

DMenu::DMenu(const char* text, ButtonState* b1,int v1, ButtonState* b2,int v2)
     : (text, nil, b1, v1, b2, v2)
{
    Init();
}

void DMenu::Init() {
    SetClassName("DMenu");
    state = new ControlState;
    state->Reference();
    scene = nil;
    body = nil;
    depth = 3;
    opened = false;

    /* Align this->BottomLeft to body->TopLeft */
    this_x = 0.0;
    this_y = 0.0;
    body_x = 0.0;
    body_y = 1.0;
}

DMenu::~DMenu() {
    if (body != nil) delete body;
    Unref(state);
}

void DMenu::Align(float tx, float ty, float bx, float by) {
    this_x = tx;
    this_y = ty;
    body_x = bx;
    body_y = by;
}

void DMenu::MakePullDown() {
    Align(0.0, 0.0, 0.0, 1.0);
}

void DMenu::MakePullRight() {
    Align(1.0, 1.0, 0.0, 1.0);
}

void DMenu::SetDepth(int d) {
    depth = d;
}

int DMenu::GetDepth() {
    return depth;
}

boolean DMenu::Changed() {
    return false;
}

Scene* DMenu::MakeBody() {
    return new VBox;
}

void DMenu::FillBody() {
}

void DMenu::Include(Control* c) {
    scene->Insert(c);
    c->SetState(state);
}

void DMenu::Open() {
    if (!Enabled()) {
	/* Do not insert body */
	return;
    }

    if ((body != nil) && Changed()) {
	delete body;
	body = nil;
    }

    if (body == nil) {
	scene = MakeBody();
	FillBody();

	body = new ShadowFrame(scene, depth, depth);
	body->Config(GetWorld());
	Shape* s = body->GetShape();
	body_width  = s->width;
	body_height = s->height;
    }

    opened = true;

    Coord x, y;

    x = round(((xmax + 1) * this_x) - (body_width * body_x));
    y = round(((ymax + 1) * this_y) - (body_height * body_y));
    GetRelative(x, y);
    GetWorld()->InsertPopup(body, x, y);
    State()->Push(state);
    state->Activate();
    GetWorld()->Flush();
}

void DMenu::Leave() { }

void DMenu::Close() {
    if (opened) {
	opened = false;

	state->Pop();
	GetWorld()->Remove(body);
	GetWorld()->Flush();
    }
}
