// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:44:52 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	dmenu.h,v $
 * Revision 1.3  91/11/04  19:44:52  sanjay
 * Prepare for release
 * 
 *
 */

#ifndef _DMENUH
#define _DMENUH

/*
 * Dynamic menus.  Entries are recomputed each time menu is about to be
 * posted. Derives from MenuClosure to allow the specification of
 * enable/disable, and check marks.
 */

#include "menuclosure.h"

class DMenu : public MenuClosure {
  public:
    DMenu(const char*,
	  ButtonState* enable = nil, int enableValue = 0,
	  ButtonState* check  = nil, int checkValue  = 0);
    virtual ~DMenu();

    /* Specify alignment points as fraction of interactor dimensions */
    void Align(float this_x, float this_y, float body_x, float body_y);

    /* Special alignments */
    void MakePullDown();
    void MakePullRight();

    /* Set/Get shadow depth */
    void SetDepth(int);
    int  GetDepth();

    /* Add item - should only be called from Changed() or FillBody(). */
    virtual void Include(Control*);
  protected:
    /*
     * Abstract methods.
     * - MakeBody is called to create the menu body.
     * - FillBody is called to fill in the menu body.
     * - Changed is called before popup, and if it returns true the
     *   body is deleted, and a new one is created by calling
     *   MakeBody and then FillBody.
     */

    /* Default - return false */
    virtual boolean Changed();

    /* Default - Create an empty vbox */
    virtual Scene*  MakeBody();

    /* Default - noop */
    virtual void FillBody();

    virtual void Open();
    virtual void Close();
    virtual void Leave();
  private:
    void Init();

    Scene* scene;		/* Scene for menu contents */
    Scene* body;		/* Surrounding shadow frame */
    ControlState* state;
    int depth;
    boolean opened;		/* True iff body has been inserted in world */

    /* Alignment */
    float this_x, this_y, body_x, body_y;

    /* Body dimensions */
    Coord body_width, body_height;
};

#endif /* _DMENUH */
