/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.6 $
 * $Date: 91/11/28 16:24:33 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	editrepeat.cc,v $
 * Revision 1.6  91/11/28  16:24:33  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.5  91/11/28  14:48:39  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.4  91/11/04  19:44:53  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/04/16  15:22:44  sanjay
 * Made closures reference counted
 * 
 * Revision 1.2  91/03/28  11:51:49  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:04  sanjay
 * Initial deposit
 *
 */

#include <InterViews/box.h>
#include <InterViews/border.h>
#include <InterViews/deck.h>
#include <InterViews/button.h>
#include <InterViews/glue.h>
#include <InterViews/message.h>
#include <InterViews/tray.h>
#include "WeekDay.h"
#include "Month.h"
#include "Date.h"
#include "editrepeat.h"
#include "mdayset.h"
#include "wdayset.h"
#include "mdaychoose.h"
#include "wdaychoose.h"
#include "monthset.h"
#include "monthchoose.h"
#include "yearselector.h"
#include "yearrange.h"
#include "item.h"
#include "buttonclosure.h"
#include "closure.h"
#include "appteditor.h"
#include "application.h"

declare(AVForwarder,EditRepeat);
implement(AVForwarder,EditRepeat);

declare(Closure,EditRepeat);
implement(Closure,EditRepeat);

EditRepeat::EditRepeat(Application* a) {
    SetClassName("EditRepeat");
    SetName("Edit Item Repetition");
    view = new AVForwarder(EditRepeat)(a, this);
    currentItem = nil;
    Layout();
}

EditRepeat::~EditRepeat() {
    FinishEdit();

    dayControl->Detach(this);
    monthControl->Detach(this);
    yearControl->Detach(this);

    delete view;
}

void EditRepeat::FinishEdit() {
    if (currentItem != nil) {
	Item* item = currentItem;
	currentItem = nil;

	textDisplay->Edit("");

	/* Remove application focus from this view */
	if (view->App()->Focus() == view)
	    view->App()->SetFocus(nil);

	CopyTo(item);

	view->App()->Change(item);
    }
}

void EditRepeat::Edit(Item* i) {
    if (currentItem != i) {
	FinishEdit();
    }

    if (currentItem != i) {
	CopyFrom(i);
	currentItem = i;
    }
}

void EditRepeat::ItemMsg(ApptViewItemMsg t, Item* i) {
    if (currentItem == i) {
	switch (t) {
	  case ApptView::Add:
	    /* Ignore new item additions */
	    break;
	  case ApptView::Delete:
	    Cancel();
	    break;
	  case ApptView::Change:
	    CopyFrom(currentItem);
	    break;
	}
    }
}

void EditRepeat::Msg(ApptViewMsg t) {
    switch (t) {
      case ApptView::Save:
	Apply();
	break;
      case ApptView::Focus:
	break;
      case ApptView::Unfocus:
	break;
    }
}

void EditRepeat::CopyFrom(Item* item) {
    int	        todayDay;
    WeekDay	todayWDay;
    Month	todayMonth;
    int         todayYear;
    SmallIntSet s;

    Date::Today().BreakDown(todayDay, todayWDay, todayMonth, todayYear);

    /*
     * Initialize all fields to today.
     */
    s.Clear();
    s.Insert(todayDay);
    mdaySet->Edit(s);
    s.Clear();
    s.Insert(todayWDay.Index());
    wdaySet->Edit(s);
    mdayChoose->Edit(todayDay);
    wdayChoose->Edit(todayWDay.Index());

    s.Clear();
    s.Insert(todayMonth.Index());
    monthSet->Edit(s);
    monthChoose->Edit(todayMonth.Index());

    yearSelector->Edit(todayYear);
    yearRange->Edit(todayYear, todayYear);

    /*
     * Set day spec based on item parameters.
     */
    if (item->IsWeekly()) {
	s = item->GetWeekDays();
	if (s.Size() == 1) {
	    dayControl->SetValue((int) Weekly);
	    for (int i = 1; i <= 7; i++) {
		if (s.Member(i)) {
		    wdayChoose->Edit(i);
		    break;
		}
	    }
	}
	else if (s.Size() == 7) {
	    dayControl->SetValue((int) Daily);
	}
	else {
	    dayControl->SetValue((int) WDays);
	}
	wdaySet->Edit(s);
    }
    else {
	s = item->GetMonthDays();
	if (s.Size() == 1) {
	    dayControl->SetValue((int) Monthly);
	    for (int i = 1; i <= 31; i++) {
		if (s.Member(i)) {
		    mdayChoose->Edit(i);
		    break;
		}
	    }
	}
	else if (s.Size() == 31) {
	    dayControl->SetValue((int) Daily);
	}
	else {
	    dayControl->SetValue((int) MDays);
	}
	mdaySet->Edit(s);
    }

    s = item->GetMonths();
    if (s.Size() == 1) {
	monthControl->SetValue((int) OneMonth);
	for (int i = 1; i <= 12; i++) {
	    if (s.Member(i)) {
		monthChoose->Edit(i);
		break;
	    }
	}
    }
    else if (s.Size() == 12) {
	monthControl->SetValue((int) AllMonths);
    }
    else {
	monthControl->SetValue((int) RangeOfMonths);
	monthSet->Edit(s);
    }

    if (item->EveryYear()) {
	yearControl->SetValue((int) AllYears);
    }
    else if (item->FirstYear() == item->LastYear()) {
	yearControl->SetValue((int) OneYear);
	yearSelector->Edit(item->FirstYear());
    }
    else {
	yearControl->SetValue((int) RangeOfYears);
	yearRange->Edit(item->FirstYear(), item->LastYear());
    }

    textDisplay->Edit(item->GetText());
}

void EditRepeat::CopyTo(Item* item) {
    SmallIntSet s;
    int v, i;

    dayControl->GetValue(v);
    switch ((RepeatDayType) v) {
      case Daily:
	s.Clear();
	for (i = 1; i <= 7; i++)
	    s.Insert(i);
	item->SetWeekDays(s);
	break;
      case Weekly:
	s.Clear();
	s.Insert(wdayChoose->Get());
	item->SetWeekDays(s);
	break;
      case Monthly:
	s.Clear();
	s.Insert(mdayChoose->Get());
	item->SetMonthDays(s);
	break;
      case WDays:
	item->SetWeekDays(wdaySet->Get());
	break;
      case MDays:
	item->SetMonthDays(mdaySet->Get());
	break;
    }

    monthControl->GetValue(v);
    switch ((RepeatMonthType) v) {
      case AllMonths:
	s.Clear();
	for (i = 1; i <= 12; i++)
	    s.Insert(i);
	item->SetMonths(s);
	break;
      case OneMonth:
	s.Clear();
	s.Insert(monthChoose->Get());
	item->SetMonths(s);
	break;
      case RangeOfMonths:
	item->SetMonths(monthSet->Get());
	break;
    }

    yearControl->GetValue(v);
    switch ((RepeatYearType) v) {
      case AllYears:
	item->SetEveryYear();
	break;
      case OneYear:
	item->SetYears(yearSelector->Get(), yearSelector->Get());
	break;
      case RangeOfYears:
	item->SetYears(yearRange->GetFirst(), yearRange->GetLast());
	break;
    }
}

void EditRepeat::Update() {
    int v;

    if (canvas == nil) return;

    dayControl->GetValue(v);
    switch ((RepeatDayType) v) {
      case Daily:
	dayDeck->FlipTo(1);
	break;
      case Weekly:
	dayDeck->FlipTo(5);
	break;
      case Monthly:
	dayDeck->FlipTo(4);
	break;
      case WDays:
	dayDeck->FlipTo(3);
	break;
      case MDays:
	dayDeck->FlipTo(2);
	break;
      default:
	break;
    }

    monthControl->GetValue(v);
    switch ((RepeatMonthType) v) {
      case AllMonths:
	monthDeck->FlipTo(1);
	break;
      case OneMonth:
	monthDeck->FlipTo(2);
	break;
      case RangeOfMonths:
	monthDeck->FlipTo(3);
	break;
      default:
	break;
    }

    yearControl->GetValue(v);
    switch ((RepeatYearType) v) {
      case AllYears:
	yearDeck->FlipTo(1);
	break;
      case OneYear:
	yearDeck->FlipTo(2);
	break;
      case RangeOfYears:
	yearDeck->FlipTo(3);
	break;
      default:
	break;
    }
}

void EditRepeat::Layout() {
    Coord smallSkip = round(0.1 * inches);
    Coord medSkip = round(0.1 * inches);

    dayControl	= new ButtonState((int) Monthly);
    dayDeck	= new Deck;
    mdaySet	= new MDaySet;
    wdaySet	= new WDaySet;
    mdayChoose	= new MDayChoose;
    wdayChoose	= new WDayChoose;

    dayDeck->Insert(new Message(""));
    dayDeck->Insert(mdaySet);
    dayDeck->Insert(wdaySet);
    dayDeck->Insert(mdayChoose);
    dayDeck->Insert(wdayChoose);

    monthControl= new ButtonState((int) AllMonths);
    monthDeck	= new Deck;
    monthSet	= new MonthSet;
    monthChoose	= new MonthChoose;

    monthDeck->Insert(new Message(""));
    monthDeck->Insert(monthChoose);
    monthDeck->Insert(monthSet);

    yearControl	 = new ButtonState((int) AllYears);
    yearDeck	 = new Deck;
    yearSelector = new YearSelector;
    yearRange	 = new YearRange;

    yearDeck->Insert(new Message(""));
    yearDeck->Insert(yearSelector);
    yearDeck->Insert(yearRange);

    VBox* b1 = new VBox;
    b1->Insert(new VGlue(smallSkip));
    b1->Insert(new RadioButton("Daily",		dayControl, (int) Daily));
    b1->Insert(new VGlue(smallSkip));
    b1->Insert(new RadioButton("Weekly",	dayControl, (int) Weekly));
    b1->Insert(new VGlue(smallSkip));
    b1->Insert(new RadioButton("Monthly",	dayControl, (int) Monthly));
    b1->Insert(new VGlue(smallSkip));
    b1->Insert(new RadioButton("Days of the week",  dayControl, (int) WDays));
    b1->Insert(new VGlue(smallSkip));
    b1->Insert(new RadioButton("Days of the month", dayControl, (int) MDays));
    b1->Insert(new VGlue(smallSkip));

    VBox* b2 = new VBox;
    b2->Insert(new VGlue(smallSkip));
    b2->Insert(new RadioButton("Every Month",	monthControl, (int)AllMonths));
    b2->Insert(new VGlue(smallSkip));
    b2->Insert(new RadioButton("One Month",	monthControl, (int)OneMonth));
    b2->Insert(new VGlue(smallSkip));
    b2->Insert(new RadioButton("Set of Months", monthControl,(int)RangeOfMonths));
    b2->Insert(new VGlue(smallSkip));
    b2->Align(Left);

    VBox* b3 = new VBox;
    b3->Insert(new VGlue(smallSkip));
    b3->Insert(new RadioButton("Every Year",	yearControl, (int)AllYears));
    b3->Insert(new VGlue(smallSkip));
    b3->Insert(new RadioButton("One Year",	yearControl, (int)OneYear));
    b3->Insert(new VGlue(smallSkip));
    b3->Insert(new RadioButton("Year in Range",yearControl,(int)RangeOfYears));
    b3->Insert(new VGlue(smallSkip));
    b3->Align(Left);

    Border* vb = new VBorder;
    Border* xb = new HBorder;
    Border* hb = new HBorder;

    Tray* tray = new Tray;

    tray->HBox(tray, hb, tray);
    tray->HBox(tray, xb, tray);
    tray->VBox(tray, vb, tray);

    /* Align topleft box */
    tray->Align(TopLeft, b1, new TGlue(medSkip, medSkip));
    tray->Align(Right,   b1, Left, vb, new TGlue(medSkip, 0));
    tray->Align(Bottom,  b1, Top,  hb, new TGlue(0, medSkip));

    /* Align midleft box */
    tray->Align(Left,   b2, new TGlue(medSkip, medSkip));
    tray->Align(Right,  b2, Left,   vb, new TGlue(medSkip, 0));
    tray->Align(Bottom, b2, Top,    xb, new TGlue(0, medSkip));
    tray->Align(Top,    b2, Bottom, hb, new TGlue(0, medSkip));

    /* Align bottomleft box */
    tray->Align(BottomLeft, b3, new TGlue(medSkip, medSkip));
    tray->Align(Right,      b3, Left,   vb, new TGlue(medSkip, 0));
    tray->Align(Top,        b3, Bottom, xb, new TGlue(0, medSkip));

    /* Align topright box */
    tray->Align(TopRight, dayDeck, new TGlue(medSkip, medSkip));
    tray->Align(Left,     dayDeck, Right, vb, new TGlue(medSkip, 0));
    tray->Align(Bottom,   dayDeck, Top,   hb, new TGlue(0, medSkip));

    /* Align midright box */
    tray->Align(Right,  monthDeck, new TGlue(medSkip, medSkip));
    tray->Align(Left,   monthDeck, Right,  vb, new TGlue(medSkip, 0));
    tray->Align(Bottom, monthDeck, Top,    xb, new TGlue(0, medSkip));
    tray->Align(Top,    monthDeck, Bottom, hb, new TGlue(0, medSkip));

    /* Align bottomright box */
    tray->Align(BottomRight, yearDeck, new TGlue(medSkip, medSkip));
    tray->Align(Left,        yearDeck, Right,  vb, new TGlue(medSkip, 0));
    tray->Align(Top,         yearDeck, Bottom, xb, new TGlue(0, medSkip));

    textDisplay = new ApptEditor;

    Closure* applyClosure  = new Closure1(EditRepeat)(this,EditRepeat::Apply);
    Closure* cancelClosure = new Closure1(EditRepeat)(this,EditRepeat::Cancel);
    Closure* doneClosure   = new Closure1(EditRepeat)(this,EditRepeat::Done);

    HBox* buttons = new HBox
	(new ButtonClosure("Apply",	applyClosure),
	 new HGlue,
	 new ButtonClosure("Cancel",	cancelClosure),
	 new ButtonClosure("Done",	doneClosure)
	 );

    Insert(new VBox(textDisplay,
		    new HBorder,
		    tray,
		    new HBorder,
		    buttons));

    dayControl->Attach(this);
    monthControl->Attach(this);
    yearControl->Attach(this);
    Unref(dayControl);
    Unref(monthControl);
    Unref(yearControl);
}

void EditRepeat::Apply() {
    if (currentItem != nil) {
	CopyTo(currentItem);
	view->App()->Change(currentItem);
    }
}

void EditRepeat::Cancel() {
    if (currentItem != nil) {
	currentItem = nil;
	textDisplay->Edit("");

	/* Remove application focus from this view */
	if (view->App()->Focus() == view)
	    view->App()->SetFocus(nil);
    }
    view->App()->DeleteEditRepeat(this);
}

void EditRepeat::Done() {
    Apply();
    Cancel();
}

void EditRepeat::Resize() {
    MonoScene::Resize();
    Update();
}
