// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:44:54 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	editrepeat.h,v $
 * Revision 1.3  91/11/04  19:44:54  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/04/16  15:22:45  sanjay
 * Made closures reference counted
 * 
 * Revision 1.1  91/03/28  11:41:18  sanjay
 * Initial deposit
 * 
 */

#ifndef _EDITREPEATH
#define _EDITREPEATH

#include <InterViews/scene.h>
#include "apptview.h"
#include "smallintset.h"

class Application;

class Deck;
class ButtonState;
class Item;

class MDaySet;
class WDaySet;
class MDayChoose;
class WDayChoose;

class MonthSet;
class MonthChoose;

class YearSelector;
class YearRange;

class ApptEditor;

class EditRepeat;

class EditRepeat : public MonoScene {
  public:
    EditRepeat(Application*);
    virtual ~EditRepeat();

    virtual void Edit(Item*);

    /* ApptView callbacks */
    virtual void ItemMsg(ApptViewItemMsg, Item*);
    virtual void Msg(ApptViewMsg);
  protected:
    void FinishEdit();

    void CopyFrom(Item*);
    void CopyTo(Item*);

    virtual void Update();
    void Layout();

    /*
     * Day control.
     */
    enum RepeatDayType {
	Daily,
	Weekly,
	Monthly,
	WDays,
	MDays
	};

    ButtonState* dayControl;
    Deck*        dayDeck;
    MDaySet*	 mdaySet;
    WDaySet*	 wdaySet;
    MDayChoose*	 mdayChoose;
    WDayChoose*	 wdayChoose;

    /*
     * Interactor			Index in deck.
     * -				1
     * monthDaySet			2
     * weekDaySet			3
     * monthDayChooser			4
     * weekDayChooser			5
     */

    /*
     * Month control
     */
    enum RepeatMonthType {
	AllMonths,
	OneMonth,
	RangeOfMonths
	};

    ButtonState* monthControl;
    Deck*	 monthDeck;
    MonthSet*	 monthSet;
    MonthChoose* monthChoose;

    /*
     * Interactor			Index in deck
     * -				1
     * monthSelector			2
     * monthSet				3
     */

    /*
     * Year control
     */
    enum RepeatYearType {
	AllYears,
	OneYear,
	RangeOfYears
	};

    ButtonState*  yearControl;
    Deck*	  yearDeck;
    YearSelector* yearSelector;
    YearRange*	  yearRange;

    /*
     * Interactor			Index in deck
     * -				1
     * yearSelector			2
     * yearRangeSelector		3
     */

    /*
     * ApptView that forwards application messages here.
     */
    ApptView* view;

    /*
     * Current item.
     */
    Item* currentItem;

    /*
     * Text displayer.
     */
    ApptEditor* textDisplay;

    /* Closure routines */
    void Apply();
    void Cancel();
    void Done();

    virtual void Resize();
};

#endif /* _EDITREPEATH */
