/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/04 19:44:58 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	field.cc,v $
 * Revision 1.4  91/11/04  19:44:58  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/06/28  14:32:45  sanjay
 * Make array deletions portable.
 * 
 * Revision 1.2  91/03/28  11:51:51  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:08  sanjay
 * Initial deposit
 *
 */

#include <stdio.h>
#include <string.h>
#include "Month.h"
#include "config.h"
#include "field.h"

const char* IntState::Unparse () const {
    static char buffer[1000];
    sprintf(buffer, "%d", value);
    return buffer;
}

StringState::StringState(const char* init) {
    len = strlen(init);
    value = new char[len+1];
    strcpy(value, init);
}

StringState::~StringState() {
    deleteArray(len+1) value;
}

void StringState::Set(const char* val) {
    if (strcmp(val, value) != 0) {
	deleteArray(len+1) value;
	len = strlen(val);
	value = new char[len+1];
	strcpy(value, val);
	Notify();
    }
}

EnumState::EnumState(int init, int len, const int* vals, const char** names) {
    listlength = len;
    valuelist = vals;
    namelist = names;
    index = Find(init);
}

void EnumState::Set(int val) {
    int newindex =  Find(val);
    if (newindex != index) {
	index = newindex;
	Notify();
    }
}

int EnumState::Find(int val) const {
    for (int i = 0; i < listlength; i++) {
	if (valuelist[i] == val) {
	    return i;
	}
    }
    /* lib_error_handler("EnumState", "illegal value"); */
    return 0;
}

void RangeState::SetMin(int m) {
    minValue = m;
    if (value < minValue) Notify();
}

void RangeState::SetMax(int m) {
    maxValue = m;
    if (value > maxValue) Notify();
}

const char* MonthState::Unparse() const {
    return (Month::First() + value - 1).Name();
}

void MonthState::Notify() {
    /* On wrap-around -- modify the year */
    int newYear;

    newYear = year->Get();
    if (year != nil) {
	if (value > 12) {
	    newYear = year->Get() + (value - 1) / 12;
	}
	else if (value < 1) {
	    newYear = year->Get() - (12 - value) / 12;
	}
    }
    while ((value - 1) < 0)
	value += 12;
    value = (value - 1) % 12 + 1;
    IntState::Notify();
    if (year != nil)
	year->Set(newYear);
}

int AmPmState::values[2] = { (int) AmPmState::Am, (int) AmPmState::Pm };

const char* AmPmState::names[2] = { "am", "pm" };

const char* HourState::Unparse() const {
    static char buffer[3];

    int val = value % 12;
    if (val == 0) val = 12;
    sprintf(buffer, "%d", val);
    return buffer;
}
