/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:01 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	fieldshower.cc,v $
 * Revision 1.3  91/11/04  19:45:01  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:51:52  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:09  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/shape.h>
#include <InterViews/scene.h>
#include <stdlib.h>
#include "fieldshower.h"
#include "field.h"

FieldShower::FieldShower(FieldState* state, Alignment a)
{
    Init(state, a);
}

FieldShower::FieldShower(const char* name, FieldState* state, Alignment a)
{
    SetInstance(name);
    Init(state, a);
}

FieldShower::~FieldShower() {
    if (subject != nil) {
	subject->Detach(this);
    }
}

void FieldShower::Init(FieldState* state, Alignment a) {
    SetClassName("FieldShower");
    subject = state;
    minWidth = 1;
    pad = 2;

    switch (a) {
      case Bottom:
	align = BottomCenter;
	break;
      case Top:
	align = TopCenter;
	break;
      case VertCenter:
	align = Center;
	break;
      case Left:
	align = CenterLeft;
	break;
      case Right:
	align = CenterRight;
	break;
      case HorizCenter:
	align = Center;
	break;
      default:
	align = a;
	break;
    }

    state->Attach(this);
    Update();
}

void FieldShower::Reconfig() {
    /* Get attributes */
    const char* a;

    if ((a = GetAttribute("minWidth")) != nil) {
	minWidth = output->GetFont()->Width(a) + 2 * pad;
    }

    if ((a = GetAttribute("pad")) != nil) {
	Coord p = atoi(a);

	if ((p >= 0) && (p < round(10 *inch))) pad = p;
    }

    /* Get text width */
    Coord textWidth = output->GetFont()->Width(subject->Unparse()) + 2 * pad;

    shape->width = max(textWidth, minWidth);
    shape->height = output->GetFont()->Height() + 2 * pad;
    shape->Rigid(shape->width - textWidth + 2 * pad, 0, 2 * pad, 0);
}

void FieldShower::Redraw(Coord x1, Coord y1, Coord x2, Coord y2) {
    output->ClearRect(canvas, x1, y1, x2, y2);
    Refresh();
}

void FieldShower::Refresh() {
    Coord x, y;
    Font* f = output->GetFont();

    const char* text = subject->Unparse();
    x = 0;
    y = 0;
    Align(align, f->Width(text) + 2 * pad, f->Height() + 2 * pad, x, y);
    output->Text(canvas, text, x + pad, y + pad);
}

void FieldShower::Update() {
    if (canvas != nil) {
	int oldwidth = shape->width;
	Reconfig();
	if ((shape->width != oldwidth) && (Parent() != nil)) {
	    Parent()->Change(this);
	}
	output->ClearRect(canvas, 0, 0, xmax, ymax);
	Refresh();
    }
}
