/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/04 19:45:07 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	getstring.cc,v $
 * Revision 1.4  91/11/04  19:45:07  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/05/06  11:17:20  sanjay
 * Allow keyboard shortcuts to dismiss dialogs
 * 
 * Revision 1.2  91/04/11  13:30:47  sanjay
 * Fixed window popup
 * 
 * Revision 1.1  91/04/10  11:33:27  sanjay
 * Added printing
 * 
 *
 */

#include <InterViews/frame.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/message.h>
#include <InterViews/button.h>
#include <InterViews/streditor.h>
#include <InterViews/sensor.h>
#include <InterViews/event.h>
#include "getstring.h"

static const float space = 0.3;         /* Inches */

GetString::GetString(const char* title, const char* initial)
     : (new ButtonState(0), nil)
{
    Layout("GetString", title, initial);
}

GetString::GetString(const char* name, const char* title, const char* initial)
     : (new ButtonState(0), nil)
{
    Layout(name, title, initial);
}

boolean GetString::Accept() {
    Dialog::Accept();

    int v;
    state->GetValue(v);

    switch (v) {
      case 1:
      case 'y':
      case '\n':
      case '\r':
	return true;
      default:
	return false;
    }
}

const char* GetString::Text() {
    return editor->Text();
}

void GetString::Layout(const char* name, const char* title, const char* init) {
    input = new Sensor;
    input->Catch(KeyEvent);
    Listen(input);

    SetClassName("GetString");
    SetInstance(name);
    SetName(name);

    editor = new StringEditor(state, "XXXXXXXXXXXXXXXXXXXXXXXXX", "\n\r\007");
    editor->Message(init);

    PushButton* ok = new PushButton("Okay", state, 'y');
    PushButton* cancel = new PushButton("Cancel", state, 'n');

    Insert(
	new MarginFrame(
	    new VBox(
		new Message(title, Left, 0, hfil, 0),
		new VGlue(round(space * inches), 0),
		new Frame(editor),
		new VGlue(round(space * inches), 0),
		new HBox(cancel, new HGlue(round(space * inches)), ok)
		),
	    round(space * inches)
	    )
	);

    /* Unref state so that is deleted when all attached interactors go away */
    Unref(state);
}

void GetString::Handle(Event& e) {
    if (e.eventType == KeyEvent) {
	switch (e.keystring[0]) {
	  case '\n':
	  case '\r':
	    /* Okay */
	    state->SetValue('y');
	    break;

	  case '\007': /* Control-G */
	    state->SetValue('n');
	    break;
	}
    }
}
