/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:10 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	inteditor.cc,v $
 * Revision 1.3  91/11/04  19:45:10  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:51:53  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:11  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/perspective.h>
#include <InterViews/shape.h>
#include <stdio.h>
#include <stdlib.h>
#include "inteditor.h"
#include "field.h"

IntEditor::IntEditor(IntState* v, Alignment a) : (v, a) {
    Init(v);
}

IntEditor::IntEditor(const char* name, IntState* v, Alignment a) : (v, a) {
    Init(v);
    SetInstance(name);
}

void IntEditor::Init(IntState* v) {
    SetClassName("IntEditor");
    state = v;
    perspective = new Perspective;
    perspective->sx = 1;
    perspective->lx = 100;
    perspective->sy = 1;
    perspective->ly = 100;
}

IntEditor::~IntEditor() {
    Unref(perspective);
}

void IntEditor::Adjust(Perspective& p) {
    boolean legal;	/* Is this a legal adjustment */
    boolean large;	/* Was a large movement made */
    boolean increased;	/* Was the state increased */

    legal = false;
    if ((p.curx != 0) && (p.cury == 0)) {
	/* Horizontal mover */
	increased = true;
	if (p.curx < 0) {
	    increased = false;
	    p.curx = - p.curx;
	}
	if (p.curx == perspective->sx) {
	    large = false;
	    legal = true;
	}
	else if (p.curx == perspective->lx) {
	    large = true;
	    legal = true;
	}
    }
    if ((p.cury != 0) && (p.curx == 0)) {
	/* Vertical mover */
	increased = true;
	if (p.cury < 0) {
	    increased = false;
	    p.cury = - p.cury;
	}
	if (p.cury == perspective->sy) {
	    large = false;
	    legal = true;
	}
	else if (p.cury == perspective->ly) {
	    large = true;
	    legal = true;
	}
    }
    p = *perspective;

    if (legal) {
	switch (increased) {
	  case true:
	    switch (large) {
	      case false:
		Increase();
		break;
	      case true:
		IncreaseLarge();
		break;
	    }
	    break;
	  case false:
	    switch (large) {
	      case false:
		Decrease();
		break;
	      case true:
		DecreaseLarge();
		break;
	    }
	}
    }
}

void IntEditor::Increase() {
    state->Set(state->Get() + 1);
}

void IntEditor::Decrease() {
    state->Set(state->Get() - 1);
}

void IntEditor::IncreaseLarge() {
    state->Set(state->Get() + 10);
}

void IntEditor::DecreaseLarge() {
    state->Set(state->Get() - 10);
}
