/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.12 $
 * $Date: 91/11/28 16:25:32 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	iremind.cc,v $
 * Revision 1.12  91/11/28  16:25:32  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.11  91/11/04  19:45:12  sanjay
 * Prepare for release
 * 
 * Revision 1.10  91/08/06  09:54:30  sanjay
 * Get item list from application instead of calendar.
 * 
 * Revision 1.9  91/06/28  20:09:35  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.8  91/04/22  19:47:46  sanjay
 * Removed Date::Midnight() usage
 * 
 * Revision 1.7  91/04/22  07:41:26  sanjay
 * Use WeekDay::Index
 * 
 * Revision 1.6  91/04/10  11:33:25  sanjay
 * Added printing
 * 
 * Revision 1.5  91/04/03  21:36:34  sanjay
 * Added range listing
 * 
 * Revision 1.4  91/04/03  20:39:38  sanjay
 * Put reminders in window instead of piping
 * 
 * Revision 1.3  91/03/29  10:33:07  sanjay
 * Put arrays in library
 * 
 * Revision 1.2  91/03/28  11:51:53  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:11  sanjay
 * Initial deposit
 *
 */

/*
 * Generate notices and reminders for a particular date.
 */

#include "Time.h"
#include "Date.h"
#include "Month.h"
#include "WeekDay.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "iremind.h"
#include "application.h"
#include "item.h"
#include "printer.h"

#include "Pointer.Array.h"

/*
 * Collect and print reminders.
 */
void IRemind::Remind(Printer* printer, Application* appl, Date d1, Date d2) {
    PointerArray reminders;	/* Items with reminders today */

    CollectReminders(appl, d1, d2, reminders);
    PrintReminders(printer, reminders);
}

void IRemind::PSPrint(Printer* printer, Application* appl, Date d1, Date d2,
		      int columns, int firstOffset)
{
    PointerArray reminders;	/* Items with reminders today */

    CollectReminders(appl, d1, d2, reminders);
    if (reminders.Size() == 0) return;

    /*
     * Compute number of rows.
     */
    Date firstDate = ((Remind_Info*) reminders[0])->date;
    Date lastDate = ((Remind_Info*) reminders.High())->date;
    int numEntries = (lastDate - firstDate) + firstOffset + 1;
    int numRows = (numEntries + columns - 1) / columns;

    Date lastPrinted = firstDate - 1;

    for (int i = 0; i < reminders.Size(); i++) {
	Remind_Info* info = ((Remind_Info*) reminders[i]);

	if (lastPrinted != info->date) {
	    /* Finish last day if necessary */
	    if (lastPrinted >= firstDate) {
		printer->Put("] ShowDay\n");
	    }

	    /* Print date header */
	    int pos = (info->date - firstDate) + firstOffset;
	    if (pos >= numEntries) continue;

	    int col = pos % columns;
	    int row = pos / columns;

	    char buffer[100];
	    sprintf(buffer, "%d %d () ()\n [", row, col);
	    printer->Put(buffer);

	    lastPrinted = info->date;
	}

	/* Print this reminder */
	printer->Put('[');

	/* Print appt header */
	Appointment* appt = info->item->AsAppointment();
	if (appt != nil) {
	    printer->Put("(");
	    PrintTime(printer, appt->GetStart());
	    printer->Put(" to ");
	    PrintTime(printer, appt->GetFinish());
	    printer->Put(")\n");
	}

	/* Print all the lines */

	/* Finish reminder */
	printer->Put(']');

	delete info;
    }

    if (lastPrinted >= firstDate) {
	printer->Put("] ShowDay\n");
    }
}

void IRemind::PSPrintDay(Printer* printer, Application* appl, Date d,
			 PointerArray& reminders)
{
    printer->Put('[');

    for (int i = 0; i < reminders.Size(); i++) {
	Remind_Info* info = ((Remind_Info*) reminders[i]);
	if (info->date != d) {
	    continue;
	}

	Item* item = info->item;

	/* Print this reminder */
	printer->Put('[');

	/* Print appt header */
	Appointment* appt = item->AsAppointment();
	if (appt != nil) {
	    printer->Put("(");
	    PrintTime(printer, appt->GetStart());
	    printer->Put(" to ");
	    PrintTime(printer, appt->GetFinish());
	    printer->Put(")\n");
	}

	/* Print all the lines */

	char const* text = item->GetText();
	int len = strlen(text);

	/* Strip trailing whitespace */
	while (len > 0) {
	    char c = text[len - 1];

	    if (isascii(c) && isspace(c)) {
		len--;
		continue;
	    }
	    else {
		break;
	    }
	}

	int bol = 1;		/* At beginning of line? */
	for (int j = 0; j < len; j++) {
	    if (bol) {
		printer->Put('(');
		bol = 0;
	    }
	    char c = text[j];
	    switch (c) {
	      case '(':
	      case ')':
	      case '\\':
		/* Quote with a backslash */
		printer->Put('\\');
		printer->Put(c);
		break;
	      case '\n':
		printer->Put(')');
		printer->Put('\n');
		bol = 1;
		break;
	      default:
		printer->Put(c);
		break;
	    }
	}
	if (! bol) {
	    /* Finish the line */
	    printer->Put(")\n");
	}

	/* Finish reminder */
	printer->Put("]\n");
    }

    printer->Put("]\n");
}

void IRemind::PSPrintMonth(Printer* printer, Application* appl, Date d) {
    char buffer[100];

    int day, year;
    Month month;
    WeekDay wday;

    d.BreakDown(day, wday, month, year);
    Date firstDate(1, month, year);
    Date lastDate(month.Size(year), month, year);

    PSPrintHeader(printer);
    printer->Put("                SetLandScape\n");
    printer->Put("7               SetGridWidth\n");
    printer->Put("5               SetGridHeight\n");
    printer->Put("0.5 inch        SetBorderWidth\n");
    printer->Put("0.3 inch        SetBorderHeight\n");
    printer->Put("0.5 inch        SetHeaderHeight\n");
    printer->Put("0.1 inch        SetHeaderSep\n");
    printer->Put("10              SetMinLines\n");
    printer->Put("15              SetMaxLines\n");
    printer->Put("0.3 inch        SetColumnHeight\n");
    printer->Put("0.1 inch        SetColumnSep\n");
    printer->Put("0.25            SetTitleFraction\n");

    sprintf(buffer, "(%s, %d) SetHeaderLeft\n", month.Name(), year);
    printer->Put(buffer);

    PSPrintPrintingTime(printer);

    printer->Put("ComputeLayout\n");
    printer->Put("0	()	(Sun)	()	ColumnHead\n");
    printer->Put("1	()	(Mon)	()	ColumnHead\n");
    printer->Put("2	()	(Tue)	()	ColumnHead\n");
    printer->Put("3	()	(Wed)	()	ColumnHead\n");
    printer->Put("4	()	(Thu)	()	ColumnHead\n");
    printer->Put("5	()	(Fri)	()	ColumnHead\n");
    printer->Put("6	()	(Sat)	()	ColumnHead\n");

    int col = firstDate.GetWDay().Index() - 1;
    int row = 0;

    int num = 1;

    PointerArray reminders;
    CollectReminders(appl, firstDate, lastDate, reminders);

    Date xd = firstDate;
    while (xd <= lastDate) {
	sprintf(buffer, "%d %d () (%2d)\n", row, col, num);
	printer->Put(buffer);

	PSPrintDay(printer, appl, xd, reminders);

	printer->Put("ShowDay\n");

	xd += 1;
	num++;
	col++;
	if (col == 7) {
	    col = 0;
	    row++;
	    if (row == 5) {
		row = 0;
	    }
	}
    }

    printer->Put("\nshowpage\n");

    for (int i = 0; i < reminders.Size(); i++) {
	delete ((Remind_Info*) reminders[i]);
    }
}

void IRemind::PSPrintDays(Printer* printer, Application* appl, Date d,
			  int num, int cols, boolean landscape)
{
    char buffer[100];

    Date firstDate(d);
    Date lastDate(d + num - 1);

    int year1;
    int year2;
    year1 = firstDate.GetYear();
    year2 = lastDate.GetYear();

    int rows = (num + cols - 1) / cols;

    PSPrintHeader(printer);

    if (landscape) {
	printer->Put("SetLandScape\n");
    }
    else {
	printer->Put("SetPortrait\n");
    }

    sprintf(buffer, "%d SetGridWidth\n", cols);
    printer->Put(buffer);

    sprintf(buffer, "%d SetGridHeight\n", rows);
    printer->Put(buffer);

    printer->Put("0.5 inch        SetBorderWidth\n");
    printer->Put("0.3 inch        SetBorderHeight\n");
    printer->Put("0.5 inch        SetHeaderHeight\n");
    printer->Put("0.1 inch        SetHeaderSep\n");
    printer->Put("18              SetMinLines\n");
    printer->Put("18              SetMaxLines\n");
    printer->Put("0.1             SetTitleFraction\n");

    if (year1 != year2) {
	sprintf(buffer, "(%d - %d) SetHeaderLeft\n", year1, year2);
    }
    else {
	sprintf(buffer, "(%d) SetHeaderLeft\n", year1);
    }
    printer->Put(buffer);

    PSPrintPrintingTime(printer);

    printer->Put("ComputeLayout\n");

    int col = 0;
    int row = 0;

    PointerArray reminders;
    CollectReminders(appl, firstDate, lastDate, reminders);

    Date xd = firstDate;
    while (xd <= lastDate) {
	int day, year;
	Month month;
	WeekDay wday;
	xd.BreakDown(day, wday, month, year);

	sprintf(buffer, "%d %d (%s) (%s %d)\n",
		row, col,
		wday.ShortName(),
		month.ShortName(),
		day);
	printer->Put(buffer);

	PSPrintDay(printer, appl, xd, reminders);

	printer->Put("ShowDay\n");

	xd += 1;
	col++;
	if (col == cols) {
	    col = 0;
	    row++;
	}
    }

    printer->Put("\nshowpage\n");

    for (int i = 0; i < reminders.Size(); i++) {
	delete ((Remind_Info*) reminders[i]);
    }
}

void IRemind::CollectReminders(Application* appl, Date d1, Date d2,
			       PointerArray& reminders)
{
    /*
     * Collect items and reminders.
     */
    int calSize = appl->NumItems();
    for (int i = 0; i < calSize; i++) {
	Item* item = appl->GetItem(i);

	Date first = item->FirstSince(d1);

	while ((first <= d2) || ((first - d2) <= item->GetRemindStart())) {
	    Remind_Info* info = new Remind_Info;

	    info->date = first;
	    info->item = item;
	    reminders.Append((Pointer) info);

	    first = item->FirstSince(first + 1);
	}
    }

    /*
     * Sort reminders.
     */

    /* Invariant - reminders[0..i-1] is sorted list of min i elements */
    i = 0;
    while (i < reminders.Size()) {
	/* Find min element in reminders[i..] */

	int minIndex = i;
	Remind_Info* minInfo = (Remind_Info*) reminders[i];

	for (int j = i+1; j < reminders.Size(); j++) {
	    Remind_Info* r = (Remind_Info*) reminders[j];

	    if (Precedes(r, minInfo)) {
		minIndex = j;
		minInfo = r;
	    }
	}

	/* Append to sorted list */

	reminders[minIndex] = reminders[i];
	reminders[i] = (Pointer) minInfo;

	i++;
    }
}

void IRemind::PrintReminders(Printer* printer, PointerArray& reminders) {
    /*
     * Print reminders (delete them when printed).
     */
    if (reminders.Size() == 0) {
	printer->Put("No reminders\n");
    }
    else {
	Date lastDate = ((Remind_Info*) reminders[0])->date;
	PrintDate(printer, lastDate);
	printer->Put('\n');

	for (int i = 0; i < reminders.Size(); i++) {
	    Remind_Info* info = (Remind_Info*) reminders[i];

	    if (info->date != lastDate) {
		lastDate = info->date;
		printer->Put('\n');
		PrintDate(printer, lastDate);
		printer->Put('\n');
	    }

	    PrintItem(printer, info->item);
	    delete info;
	}
    }
}

/*
 * Order on Remind_Infos.
 */
int IRemind::Precedes(Remind_Info* r1, Remind_Info* r2) {
    if (r1->date < r2->date) return true;
    if (r1->date > r2->date) return false;

    Appointment* a1 = r1->item->AsAppointment();
    Appointment* a2 = r2->item->AsAppointment();

    /* Compute start times. */

    int s1 = (a1 == nil) ? -1 : a1->GetStart();
    int s2 = (a2 == nil) ? -1 : a2->GetStart();

    return s1 < s2;
}

/*
 * Print time.
 */
void IRemind::PrintTime(Printer* printer, int time) {
    int hour;
    int minute;
    char meridian;

    hour = time / 60;
    if (hour >= 12) {
	meridian = 'p';
	hour -= 12;
    }
    else {
	meridian = 'a';
    }
    if (hour == 0) hour = 12;

    minute = time % 60;

    char buffer[20];
    sprintf(buffer, "%d:%02d%cm", hour, minute, meridian);
    printer->Put(buffer);
}

/*
 * Print date.
 */
void IRemind::PrintDate(Printer* printer, Date date) {
    int day;
    WeekDay wday;
    Month month;
    int year;

    date.BreakDown(day, wday, month, year);

    printer->Put(wday.Name());
    printer->Put(' ');
    printer->Put(month.Name());
    printer->Put(' ');
    printer->Put(day);
    printer->Put(", ");
    printer->Put(year);
}

/*
 * Print item.
 */
void IRemind::PrintItem(Printer* printer, Item* item) {
    printer->Put(" * ");

    Appointment* a = item->AsAppointment();
    if (a != nil) {
	PrintTime(printer, a->GetStart());
	printer->Put(" to ");
	PrintTime(printer, a->GetFinish());
	printer->Put("\n   ");
    }

    /* Print text indented properly */

    char const* text = item->GetText();
    int len = strlen(text);

    /* Strip trailing whitespace */
    while (len > 0) {
	char c = text[len - 1];

	if (isascii(c) && isspace(c)) {
	    len--;
	    continue;
	}
	else {
	    break;
	}
    }

    for (int i = 0; i < len; i++) {
	printer->Put(text[i]);
	if (text[i] == '\n') {
	    /* Indent new line */
	    printer->Put("   ");
	}
    }

    printer->Put('\n');
}

void IRemind::PSPrintPrintingTime(Printer* printer) {
    Time now(Time::Now());
    Date today(now);

    int hour, min;
    now.BreakDownClock(hour, min);
    min += hour * 60;

    int mday;
    WeekDay wday;
    Month month;
    int year;
    today.BreakDown(mday, wday, month, year);

    char buffer[80];

    sprintf(buffer, "%d/%d/%d, ", month.Index(), mday, year);

    /*
     * Fake entries to put printing time on top right.
     * Save old values on PS stack, and restore after
     * printing the time.
     */

    /* Save old parameters */
    printer->Put("BorderHeight\n");
    printer->Put("HeaderHeight\n");
    printer->Put("HeaderLeft\n");
    printer->Put("HeaderRight\n");

    /* Print heading */
    printer->Put("HeaderHeight 2 div	 SetHeaderHeight\n");
    printer->Put("()			 SetHeaderLeft\n");

    printer->Put("(Printed ");
    printer->Put(buffer);
    PrintTime(printer, min);
    printer->Put(") SetHeaderRight\n");

    printer->Put("gsave ComputeLayout grestore\n");

    /* Restore saved parameters */
    printer->Put("SetHeaderRight\n");
    printer->Put("SetHeaderLeft\n");
    printer->Put("SetHeaderHeight\n");
    printer->Put("SetBorderHeight\n");
}

#ifdef __STDC__
#define L(x) #x,
#else
#define L(x) "x",
#endif

static const char* PSHeaderLines[] = {
L(%!PS-Adobe-)
L(%%DocumentFonts: Times-Roman)
L(%%Title: Calendar listing)
L(%%Creator: ical)
L(%%Pages: (atend))
L(%%EndComments)

L(% Simple defs)
L(/inch { 72 mul } def)
L(/RealWidth 8.5 inch def)
L(/RealHeight 11 inch def)
L(/FontName /Times-Roman def)

L(% User-defined parameters (with defaults))
L(/LandScape      true            def)
L(/GridWidth      4               def)
L(/GridHeight     2               def)
L(/BorderWidth    0.5 inch        def)
L(/BorderHeight   0.3 inch        def)
L(/HeaderHeight   0.5 inch        def)
L(/HeaderSep      0.2 inch        def)
L(/HeaderLeft     ()              def)
L(/HeaderCenter   ()              def)
L(/HeaderRight    ()              def)
L(/ColumnHeight   0               def)
L(/ColumnSep      0               def)
L(/TitleFraction  0.16            def)
L(/MinLines       1               def)
L(/MaxLines       100             def)

L(% User invoked commands)
L(/SetLandScape           { /LandScape true def }         def)
L(/SetPortrait            { /LandScape false def }        def)
L(/SetGridWidth           { /GridWidth exch def }         def)
L(/SetGridHeight          { /GridHeight exch def }        def)
L(/SetBorderWidth         { /BorderWidth exch def }       def)
L(/SetBorderHeight        { /BorderHeight exch def }      def)
L(/SetHeaderHeight        { /HeaderHeight exch def }      def)
L(/SetHeaderLeft          { /HeaderLeft exch def }        def)
L(/SetHeaderCenter        { /HeaderCenter exch def }      def)
L(/SetHeaderRight         { /HeaderRight exch def }       def)
L(/SetHeaderSep           { /HeaderSep exch def }         def)
L(/SetTitleFraction       { /TitleFraction exch def }     def)
L(/SetMinLines            { /MinLines exch def }          def)
L(/SetMaxLines            { /MaxLines exch def }          def)
L(/SetColumnHeight        { /ColumnHeight exch def }      def)
L(/SetColumnSep           { /ColumnSep exch def } def)

L(% Compute layout)
L(/ComputeLayout {)
L(    % Adjust for landscape)
L(    LandScape {)
L(        /PageWidth RealHeight def)
L(        /PageHeight RealWidth def)
L(        RealWidth 0 translate)
L(        90 rotate)
L(    })
L(    {)
L(        /PageWidth RealWidth def)
L(        /PageHeight RealHeight def)
L(    } ifelse)

L(    /DayWidth PageWidth BorderWidth 2 mul sub GridWidth div def)
L(    /DayHeight PageHeight HeaderHeight sub HeaderSep sub)
L(               ColumnHeight sub ColumnSep sub)
L(               BorderHeight 2 mul sub GridHeight div def)
L(    /DayTitleHeight DayHeight TitleFraction mul def)

L(    % Layout the header)
L(    newpath)
L(        /HeaderY PageHeight BorderHeight sub HeaderHeight sub def)
L(        FontName findfont HeaderHeight 0.8 mul scalefont setfont)

L(        % Place left string)
L(        BorderWidth HeaderY moveto)
L(        HeaderLeft show)

L(        % Place center string)
L(        PageWidth HeaderCenter stringwidth pop sub 2 div HeaderY moveto)
L(        HeaderCenter show)

L(        % Place right string)
L(        PageWidth BorderWidth sub HeaderRight stringwidth pop sub)
L(        HeaderY moveto)
L(        HeaderRight show)
L(   stroke)

L(   % Translate to grid origin)
L(   BorderWidth BorderHeight translate)
L(} def)

L(% Draw the day boundary)
L(/DayBoundary {)
L(    0 0 moveto DayWidth DayHeight DrawBox)
L(} def)

L(% Draw the title boundary.)
L(/DayTitleBoundary {)
L(    DayLeft () eq {)
L(        % Left is empty. Title is just right justified box)

L(        % Compute right string width)
L(        FontName findfont DayTitleHeight 0.8 mul scalefont findfont)
L(        DayRight stringwidth pop)
L(        /DayRightWidth exch def)

L(        % Draw box)
L(        DayWidth DayRightWidth sub DayHeight DayTitleHeight sub moveto)
L(        DayRightWidth DayTitleHeight DrawBox)
L(    })
L(    {)
L(        % Title extends for entire DayWidth)
L(        0 DayHeight DayTitleHeight sub moveto)
L(        DayWidth DayTitleHeight DrawBox)
L(    })
L(    ifelse)
L(} def)

L(% Fill in the titles)
L(/DayTitle {)
L(    FontName findfont DayTitleHeight 0.8 mul scalefont setfont)

L(    % Show DayLeft)
L(    0 DayHeight DayTitleHeight sub moveto)
L(    DayTitleHeight 0.2 mul dup rmoveto)
L(    DayLeft show)

L(    % Show DayRight)
L(    0 DayHeight DayTitleHeight sub moveto)
L(    DayWidth DayTitleHeight 0.2 mul sub DayTitleHeight 0.2 mul rmoveto)
L(    DayRight stringwidth pop neg 0 rmoveto)
L(    DayRight show)
L(} def)

L(% Draw the box for a particular day at the current position)
L(% Input:)
L(%       left:   string)
L(%       right:  string)
L(%       list:   array[array[string]])

L(/DayDraw {)
L(    /DayList  exch def)
L(    /DayRight exch def)
L(    /DayLeft  exch def)

L(    % Fill title box)
L(    currentpoint translate)

L(    gsave newpath DayTitleBoundary 0.9 setgray    fill   grestore )
L(    gsave newpath DayTitleBoundary 1 setlinewidth stroke grestore )
L(    gsave newpath DayTitle                        fill   grestore)
L(    gsave newpath DayBoundary      3 setlinewidth stroke grestore)

L(    % Set clipping for text)
L(    newpath DayTitleBoundary DayBoundary eoclip)

L(    % Compute ThisTitleHeight - if just right title then 0 else DayTitleHeight)
L(    /ThisTitleHeight DayLeft () eq { 0 } { DayTitleHeight } ifelse def)

L(    % Count number of lines to get line height)
L(    /RealLines 0 DayList { length add } forall def)
L(    RealLines)
L(    dup MinLines lt { pop MinLines } if)
L(    dup MaxLines gt { pop MaxLines } if)
L(    /NumLines exch def)

L(    DayHeight ThisTitleHeight sub NumLines div /DayLineHeight exch def)
L(    /DayBorder DayLineHeight 0.1 mul def)

L(    % Put item lines in title portion only if space is tight)
L(    DayHeight DayTitleHeight sub DayLineHeight RealLines mul ge {)
L(        % More than enough space available)
L(        /ThisTitleHeight DayTitleHeight def)
L(    } if)

L(    % Set font)
L(    FontName findfont DayLineHeight 0.9 mul scalefont setfont)

L(    % Get top line position)
L(    /DayPosition DayHeight ThisTitleHeight sub DayLineHeight 0.9 mul sub def)

L(    DayList {)
L(        % Display an item (list of strings).)
L(        {)
L(            newpath)
L(                DayBorder 2 mul DayPosition moveto)
L(                show)
L(            closepath fill)
L(            /DayPosition DayPosition DayLineHeight sub def)
L(        } forall)

L(        % Draw line at end)
L(        newpath)
L(            0 DayPosition DayLineHeight 0.8 mul add moveto)
L(            DayWidth 0 rlineto)
L(        closepath 0 setlinewidth stroke)
L(    } forall)
L(} def)

L(% Draw box)
L(% Input:)
L(%       width height)
L(/DrawBox {)
L(    exch dup 0 rlineto)
L(    exch dup 0 exch rlineto)
L(    exch neg 0 rlineto)
L(    neg 0 exch rlineto)
L(    closepath)
L(} def)

L(% Show day)
L(% Input:)
L(%       row)
L(%       column)
L(%       left text)
L(%       right text)
L(%       items)
L(/ShowDay {)
L(    5 3 roll)
L(    DayWidth mul exch GridHeight exch sub 1 sub DayHeight mul moveto)

L(    gsave DayDraw grestore)
L(} def)

L(% Show Column heading)
L(% Input:)
L(%       column)
L(%       ltext)
L(%       ctext)
L(%       rtext)
L(/ColumnHead {)
L(    /ColRight  exch def)
L(    /ColCenter exch def)
L(    /ColLeft   exch def)
L(    /ColNum    exch def)

L(    gsave)
L(        DayWidth ColNum mul GridHeight DayHeight mul ColumnSep add translate)
L(        FontName findfont ColumnHeight 0.8 mul scalefont setfont)

L(        % Show left)
L(        newpath 0 0 moveto ColLeft show stroke)

L(        % Show center)
L(        newpath DayWidth ColCenter stringwidth pop sub 2 div 0 moveto)
L(                ColCenter show stroke)

L(        % Show right)
L(        newpath DayWidth ColRight stringwidth pop sub 0 moveto)
L(                ColRight show stroke)
L(    grestore)
L(} def)

L(%%EndProlog)
0
};

static const char** PSHeader = PSHeaderLines;

void IRemind::PSPrintHeader(Printer* printer) {
    int i = 0;

    while (PSHeader[i]) {
	printer->Put(PSHeader[i]);
	printer->Put('\n');
	i++;
    }
}
