/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:17 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	labelgrid.cc,v $
 * Revision 1.3  91/11/04  19:45:17  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:51:55  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:13  sanjay
 * Initial deposit
 *
 */

#include <InterViews/shape.h>
#include <InterViews/painter.h>
#include <stdlib.h>
#include "labelgrid.h"

LabelGrid::LabelGrid(int c, int r) {
    SetClassName("LabelGrid");
    columns	= c;
    rows	= r;
    hpad	= 2;
    vpad	= 2;
    maxWidth	= 0;
    maxHeight	= 0;
};

void LabelGrid::Reconfig() {
    int r, c;

    /*
     * Get inter-label spacing.
     */
    const char* a;
    if ((a = GetAttribute("hpad")) != nil) hpad = atoi(a);
    if ((a = GetAttribute("vpad")) != nil) vpad = atoi(a);

    /*
     * Compute maxWidth and maxHeight.
     */
    maxWidth = 0;
    maxHeight = 0;
    for (r = 0; r < rows; r++) {
	for (c = 0; c < columns; c++) {
	    Coord w, h;
	    GetExtent(c, r, w, h);
	    if (w > maxWidth)  maxWidth  = w;
	    if (h > maxHeight) maxHeight = h;
	}
    }

    /*
     * Compute the shape.
     */
    shape->width  = maxWidth  * columns + hpad * (columns + 1);
    shape->height = maxHeight * rows    + vpad * (rows    + 1);
    shape->Rigid();
};

void LabelGrid::Redraw(Coord l, Coord b, Coord r, Coord t) {
    if (canvas != nil) {
	output->ClearRect(canvas, l, b, r, t);

	/*
	 * Convert l, b, r, t into column and row numbers.
	 */
	l = FindColumn(l);
	r = FindColumn(r);
	b = FindRow(b);
	t = FindRow(t);

	for (int col = l; col <= r; col++) {
	    Coord leftEdge  = FindLeft(col);
	    Coord rightEdge = FindRight(col);

	    for (int row = t; row <= b; row++) {
		Paint(leftEdge, FindBottom(row), rightEdge, FindTop(row),
		      col, row);
	    }
	}
    }
}

int LabelGrid::FindRow(Coord y) {
    int row = (ymax - (y + vpad / 2)) / (maxHeight + vpad);
    if (row < 0) row = 0;
    if (row >= rows) row = rows - 1;
    return row;
}

int LabelGrid::FindColumn(Coord x) {
    int col = (x - hpad / 2) / (maxWidth + hpad);
    if (col < 0) col = 0;
    if (col >= columns) col = columns - 1;
    return col;
}

Coord LabelGrid::FindLeft(int col) {
    return col * (maxWidth + hpad) + hpad;
}

Coord LabelGrid::FindRight(int col) {
    return (col + 1) * (maxWidth + hpad);
}

Coord LabelGrid::FindBottom(int row) {
    return ymax - (row + 1) * (maxHeight + vpad);
}

Coord LabelGrid::FindTop(int row) {
    return ymax - (row * (maxHeight + vpad) + vpad);
}
