/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.6 $
 * $Date: 91/11/04 19:45:19 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	lexer.cc,v $
 * Revision 1.6  91/11/04  19:45:19  sanjay
 * Prepare for release
 * 
 * Revision 1.5  91/11/04  11:50:42  sanjay
 * Print appropriate error message when file cannot be read.
 * 
 * Revision 1.4  91/06/28  20:09:36  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.3  91/03/29  10:33:10  sanjay
 * Put arrays in library
 * 
 * Revision 1.2  91/03/28  11:51:56  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:14  sanjay
 * Initial deposit
 *
 */

#include <values.h>
#include <ctype.h>

#include "lexer.h"
#include "char.Array.h"

char const* Lexer::lastError = "";

Lexer::Lexer(char const* file) {
    tempBuffer = new charArray;
    input = fopen(file, "r");
    if (input == 0) {
	SetError("could not open file");
	return;
    }

    status = Valid;
    NextChar();
}

Lexer::~Lexer() {
    if (input) fclose(input);
    delete tempBuffer;
}

int Lexer::SkipWS() {
    char c;

    while (Peek(c)) {
	if (isascii(c) && isspace(c)) {
	    NextChar();
	    continue;
	}
	else {
	    return 1;
	}
    }

    return 0;
}

int Lexer::GetId(char const*& x) {
    char c;

    if (! Next(c)) return 0;

    if ((! isascii(c)) || ((c != '_') && (! isalpha(c)))) {
	SetError("illegal character when expecting id");
	return 0;
    }

    tempBuffer->Clear();
    tempBuffer->Append(c);
    while (Peek(c)) {
	if (isascii(c) && ((c == '_') || (isalpha(c)))) {
	    tempBuffer->Append(c);
	    NextChar();
	    continue;
	}

	/* Hit end of id */
	tempBuffer->Append('\0');
	x = tempBuffer->AsPointer();
	return 1;
    }

    return 0;
}

int Lexer::GetUntil(char terminator, char const*& x) {
    tempBuffer->Clear();

    char c;
    while (Peek(c) && (c != terminator)) {
	tempBuffer->Append(c);
	NextChar();
    }

    switch (Status()) {
      case Valid:
	x = tempBuffer->AsPointer();
	return 1;
      case Eof:
	SetError("unexpected end of file");
	return 0;
      case Error:
	return 0;
    }
}
 
int Lexer::GetNumber(int& x) {
    char c;

    if (! Peek(c)) return 0;

    if ((! isascii(c)) || (! isdigit(c))) {
	SetError("non-digit while reading number");
	return 0;
    }

    int num = 0;
    while (Peek(c)) {
	if (isascii(c) && isdigit(c)) {
	    int digit = c - '0';

	    if (((MAXINT - digit) / 10) < num) {
		/* Would overflow - skip all remaining digits */
		while (Peek(c)) {
		    if (isascii(c) && isdigit(c)) {
			NextChar();
		    }
		    break;
		}
		SetError("integer overflow");
		return 0;
	    }

	    num = num * 10 + digit;
	    NextChar();
	    continue;
	}

	x = num;
	return 1;
    }

    return 0;
}

int Lexer::GetText(char* buf, int len) {
    for (int i = 0; i < len; i++) {
	char c;
	if (! Next(c)) return 0;
	buf[i] = c;
    }
    return 1;
}

int Lexer::Skip(char const* str) {
    while (*str) {
	char c;

	if (! Peek(c)) return 0;
	if (c != *str) {
	    SetError("unexpected character");
	    return 0;
	}

	str++;
	NextChar();
    }
    return 1;
}

void Lexer::NextChar() {
    int c = getc(input);

    if (c == EOF) {
	if (feof(input)) status = Eof;
	if (ferror(input)) {
	    SetError("IO error");
	}
    }
    else {
	lookahead = (char) c;
    }
}
