/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:23 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	mdays.cc,v $
 * Revision 1.3  91/11/04  19:45:23  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:51:57  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:17  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <stdio.h>
#include "mdays.h"

static const int numRows = 5;
static const int numCols = 7;

MDays::MDays() : (numCols, numRows) {
    input = updownEvents;
    input->Reference();
    Listen(input);
}

MDays::~MDays() {
    Unref(input);
}

void MDays::Reconfig() {
    cellWidth = output->GetFont()->Width("88");
    cellHeight = output->GetFont()->Height();
    LabelGrid::Reconfig();
}

void MDays::GetExtent(int, int, Coord& w, Coord& h) {
    w = cellWidth;
    h = cellHeight;
}

void MDays::Paint(Coord l, Coord b, Coord r, Coord t, int col, int row){
    int day = row * numCols + col + 1;

    if ((day >= 1) && (day <= 31)) {
	char str[10];

	sprintf(str, "%d", day);

	if (Selected(day)) {
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, r - output->GetFont()->Width(str), b);
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	}
	else {
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, r - output->GetFont()->Width(str), b);
	}
    }
}

void MDays::Changed(int day) {
    if (canvas != nil) {
	int c = (day - 1) % numCols;
	int r = (day - 1) / numCols;

	Paint(FindLeft(c), FindBottom(r), FindRight(c), FindTop(r), c, r);
    }
}

void MDays::Handle(Event& e) {
    switch (e.eventType) {
      case DownEvent:
	int r = FindRow(e.y);
	int c = FindColumn(e.x);

	int day = r * numCols + c + 1;

	if ((day >= 1) && (day <= 31)) {
	    Hit(day);
	}
	break;
      default:
	break;
    }
}
