/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.6 $
 * $Date: 91/11/04 19:45:27 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	menuclosure.cc,v $
 * Revision 1.6  91/11/04  19:45:27  sanjay
 * Prepare for release
 * 
 * Revision 1.5  91/11/01  14:48:28  sanjay
 * Fixed up menu item display.
 * 
 * Revision 1.4  91/10/28  13:25:29  sanjay
 * Menu items support enable/disable and check marks.
 * 
 * Revision 1.3  91/04/16  15:22:47  sanjay
 * Made closures reference counted
 * 
 * Revision 1.2  91/03/28  11:51:59  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:19  sanjay
 * Initial deposit
 *
 */

#include <InterViews/bitmap.h>
#include <InterViews/button.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/painter.h>
#include <InterViews/shape.h>
#include <stdlib.h>
#include "menuclosure.h"
#include "checkmark.h"
#include "closure.h"

Bitmap* MenuClosureItem::checkBitmap = nil;

MenuClosure::MenuClosure(const char* s, Closure* c,
			 ButtonState* bs1, int v1,
			 ButtonState* bs2, int v2)
     : ((Interactor*) 0)
{
    closure = c;

    enableState = bs1;
    checkState  = bs2;
    enableValue = v1;
    checkValue  = v2;

    if (closure != nil)     closure->Reference();
    if (enableState != nil) enableState->Reference();
    if (checkState != nil)  checkState->Reference();

    item = new MenuClosureItem(s, bs1, v1, bs2, v2);
    Insert(item);
}

MenuClosure::~MenuClosure() {
    Unref(closure);
    Unref(enableState);
    Unref(checkState);
}

void
MenuClosure::Do() {
    if (Enabled() && (closure != nil)) {
	closure->Invoke();
    }
}

boolean
MenuClosure::Enabled() {
    if (enableState != nil) {
	int value;
	enableState->GetValue(value);
	return (value == enableValue);
    }
    else {
	return true;
    }
}

boolean
MenuClosure::Checked() {
    if (checkState != nil) {
	int value;
	checkState->GetValue(value);
	return (value == checkValue);
    }
    else {
	return false;
    }
}

MenuClosureItem::MenuClosureItem(const char* t,
				 ButtonState* bs1, int v1,
				 ButtonState* bs2, int v2)
{
    enableState = bs1;
    checkState  = bs2;
    enableValue = v1;
    checkValue  = v2;

    text        = t;
    pad         = 2;
    highlighted = false;
}

void MenuClosureItem::Highlight(boolean h) {
    /* Do not highlight disabled items */
    if (! Enabled()) {
	h = false;
    }

    if (highlighted != h) {
	highlighted = h;
	Refresh();
    }
}

void MenuClosureItem::Reconfig() {
    Coord check_width, check_height;

    if (checkState != nil) {
	if (checkBitmap == nil) {
	    checkBitmap = new Bitmap(checkmark_bits,
				     checkmark_width,
				     checkmark_height);
	}
	check_width = checkBitmap->Width();
	check_height = checkBitmap->Height();
    }
    else {
	check_width = 0;
	check_height = 0;
    }

    const char* padding = GetAttribute("pad");
    if (padding != nil) {
	pad = atoi(padding);
    }

    Coord height = pad + max(output->GetFont()->Height(), check_height) + pad;
    Coord width  = pad + output->GetFont()->Width(text) + check_width   + pad;

    shape->Rect(width, height);
    shape->Rigid(0, hfil);
}

void MenuClosureItem::Redraw(Coord l, Coord b, Coord r, Coord t) {
    Color*	origFg  = output->GetFgColor(); origFg->Reference();
    Color*	origBg  = output->GetBgColor(); origBg->Reference();

    if (Enabled() && highlighted) {
	/* Reverse if enabled and highlighted */
	output->SetColors(origBg, origFg);
    }

    output->ClearRect(canvas, l, b, r, t);

    if (checkState != nil) {
	if (Checked()) output->Stencil(canvas, pad, pad, checkBitmap);
	output->Text(canvas, text, checkBitmap->Width() + pad, pad);
    }
    else {
	output->Text(canvas, text, pad, pad);
    }

    /* Cross out the contents if disabled */
    if (!Enabled()) {
	output->Line(canvas, 0, ymax / 2, xmax, ymax / 2);
    }

    /* Restore previous drawing state */
    output->SetColors(origFg, origBg);

    Unref(origFg);
    Unref(origBg);
}

void MenuClosureItem::Refresh() {
    if (canvas != nil) {
	Redraw(0, 0, xmax, ymax);
    }
}

boolean
MenuClosureItem::Enabled() {
    if (enableState != nil) {
	int value;
	enableState->GetValue(value);
	return (value == enableValue);
    }
    else {
	return true;
    }
}

boolean
MenuClosureItem::Checked() {
    if (checkState != nil) {
	int value;
	checkState->GetValue(value);
	return (value == checkValue);
    }
    else {
	return false;
    }
}
