// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:45:28 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	menuclosure.h,v $
 * Revision 1.5  91/11/04  19:45:28  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/11/01  14:48:29  sanjay
 * Fixed up menu item display.
 * 
 * Revision 1.3  91/10/28  13:25:30  sanjay
 * Menu items support enable/disable and check marks.
 * 
 * Revision 1.2  91/04/16  15:22:48  sanjay
 * Made closures reference counted
 * 
 * Revision 1.1  91/03/28  11:41:32  sanjay
 * Initial deposit
 * 
 */

#ifndef _MENUCLOSUREH
#define _MENUCLOSUREH

#include <InterViews/menu.h>

class ButtonState;
class Closure;
class MenuClosureItem;

/*
 * A MenuItem that calls a closure.

 * Also takes two buttonstates and two values.
 *
 * If first button state has first value, then item is enabled.
 *
 * If second button state has second value, the item is check marked.
 *
 * If first button state is nil, then item is always enabled.
 *
 * If second button state is nil then item is never checked.
 */
class MenuClosure : public MenuItem {
  public:
    MenuClosure(const char*, Closure*,
		ButtonState* enable = nil, int enableValue = 0,
		ButtonState* check  = nil, int checkValue  = 0);
    virtual ~MenuClosure();

    virtual void Do();

    boolean Enabled();
    boolean Checked();
  protected:
    Closure* closure;
    MenuClosureItem* item;

    /* Enable/disable info */
    ButtonState* enableState;
    ButtonState* checkState;
    int          enableValue;
    int          checkValue;
};

/*
 * Visual for menu items.
 *
 * Provides check marks and enable/disable.
 */

class MenuClosureItem : public Interactor {
  public:
    MenuClosureItem(const char*,
		    ButtonState* enable = nil, int enableValue = 0,
		    ButtonState* check  = nil, int checkValue  = 0);

    /* Highlight the item */
    virtual void Highlight(boolean);
  protected:
    virtual void Reconfig();
    virtual void Redraw(Coord, Coord, Coord, Coord);

    /* Low-level operation called when visual appearance is changed */
    void Refresh();

    boolean Enabled();
    boolean Checked();
  private:
    /* Enable/disable info */
    ButtonState* enableState;
    ButtonState* checkState;
    int          enableValue;
    int          checkValue;

    /* Current visual information */
    const char*	 text;
    Coord	 pad;
    boolean	 highlighted;

    /* Global bitmap */
    static Bitmap* checkBitmap;
};

#endif /* _MENUCLOSUREH */
