// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:45:32 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	monthdisplay.h,v $
 * Revision 1.5  91/11/04  19:45:32  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/09/19  14:37:09  sanjay
 * Added resource lookup for holiday coloring.
 * 
 * Revision 1.3  91/09/18  20:34:50  sanjay
 * Show non-empty days in different colors.
 * 
 * Revision 1.2  91/09/18  14:17:14  sanjay
 * Pass Application to monthdisplay and dateeditor to allow monthdisplay
 * appearence to change with number of items on a certain day of the
 * month. The appearence changing has not been implemented yet.
 * 
 * Revision 1.1  91/03/28  11:41:34  sanjay
 * Initial deposit
 * 
 */
#ifndef MONTHDISPLAYH
#define MONTHDISPLAYH

#include "apptview.h"
#include "labelgrid.h"

class Application;
class Color;
class IntState;

class MonthDisplay : public LabelGrid {
  public:
    MonthDisplay(Application*, IntState* day, IntState* month, IntState* year);
    virtual ~MonthDisplay();

    virtual void Update();

    /* ApptView callbacks */
    virtual void ItemMsg(ApptViewItemMsg, Item*);
    virtual void Msg(ApptViewMsg);
  protected:
    Application* application;

    IntState* day;
    IntState* month;
    IntState* year;
    int	      maxWidth;
    int       displayedDay, displayedMonth, displayedYear;
    Color*    holidayColor;
    Color*    interestColor;
    Painter*  holidayPainter;
    Painter*  interestingPainter;

    /*
     * Number of blank entries in the first row of the month calendar.
     */
    int dayAdjust;
    int daysInMonth;

    /*
     * ApptView that forwards to this.
     */
    ApptView* view;

    /*
     * Count of number of items per date in current month.
     */
    int numberOfItems[32];

    virtual void GetExtent(int, int, Coord&, Coord&);
    virtual void Paint(Coord, Coord, Coord, Coord, int, int);
    virtual void Reconfig();
    virtual void Handle(Event&);
  private:
    void RightJustify(Painter*, const char*, Coord, Coord, Coord, Coord);
    void ComputeMonthInfo();
};

#endif /*MONTHDISPLAYH */
