/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:33 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	months.cc,v $
 * Revision 1.3  91/11/04  19:45:33  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:52:00  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:21  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <stdio.h>
#include "Month.h"
#include "months.h"

static const int numRows = 6;
static const int numCols = 2;

Months::Months() : (numCols, numRows) {
    input = updownEvents;
    input->Reference();
    Listen(input);
}

Months::~Months() {
    Unref(input);
}

void Months::Reconfig() {
    cellWidth = output->GetFont()->Width("September");
    cellHeight = output->GetFont()->Height();
    LabelGrid::Reconfig();
}

void Months::GetExtent(int, int, Coord& w, Coord& h) {
    w = cellWidth;
    h = cellHeight;
}

void Months::Paint(Coord l, Coord b, Coord r, Coord t, int col, int row){
    int month = row * numCols + col + 1;

    if ((month >= 1) && (month <= 12)) {
	char const* str = (Month::First() + (month - 1)).Name();

	if (Selected(month)) {
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, l, b);
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	}
	else {
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, l, b);
	}
    }
}

void Months::Changed(int month) {
    if (canvas != nil) {
	int c = (month - 1) % numCols;
	int r = (month - 1) / numCols;

	Paint(FindLeft(c), FindBottom(r), FindRight(c), FindTop(r), c, r);
    }
}

void Months::Handle(Event& e) {
    switch (e.eventType) {
      case DownEvent:
	int r = FindRow(e.y);
	int c = FindColumn(e.x);

	int month = r * numCols + c + 1;

	if ((month >= 1) && (month <= 31)) {
	    Hit(month);
	}
	break;
      default:
	break;
    }
}
