/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.6 $
 * $Date: 91/11/28 16:25:35 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	noteinfo.cc,v $
 * Revision 1.6  91/11/28  16:25:35  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.5  91/11/04  19:45:37  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/08/06  09:55:52  sanjay
 * Removed extra Item pointer.
 * Changed CopyTo and CopyFrom names to Save and Revert.
 * 
 * Revision 1.3  91/04/02  12:33:04  sanjay
 * Improved auto-save.
 * 
 * Revision 1.2  91/03/28  11:52:01  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:23  sanjay
 * Initial deposit
 *
 */

#include <InterViews/event.h>
#include <string.h>
#include "noteinfo.h"
#include "focusframe.h"
#include "appteditor.h"
#include "item.h"

NoteInfo::NoteInfo(Item* i, int padding) {
    relative = i;

    adjust = 0;

    editor = new ApptEditor;
    editor->Edit(relative->GetText());
    editor->SetModified(false);

    frame = new FocusFrame(editor, padding);
}

NoteInfo::~NoteInfo() {
    delete frame;
}

boolean NoteInfo::Modified() {
    Appointment* appt = relative->AsAppointment();

    return (editor->Modified() ||
	    ((appt != nil) && ((start != appt->GetStart()) ||
			       (length != appt->GetLength()))
	     ));
}

void NoteInfo::SetModified(boolean b) {
    editor->SetModified(b);
}

void NoteInfo::Save() {
    char* buffer = new char[editor->Length() + 1];
    editor->Copy(buffer);
    relative->SetText(buffer);
    delete buffer;

    if (relative->AsNotice()) {
	relative->AsNotice()->SetLength(length);
    }
    else if (relative->AsAppointment()) {
	relative->AsAppointment()->SetStart(start);
	relative->AsAppointment()->SetLength(length);
    }
}

void NoteInfo::Revert() {
    int dot = editor->Dot();
    int mark = editor->Mark();
    editor->Edit(relative->GetText());
    editor->Select(dot, mark);

    if (relative->AsNotice()) {
	length = relative->AsNotice()->GetLength();
    }
    else if (relative->AsAppointment()) {
	start = relative->AsAppointment()->GetStart();
	length = relative->AsAppointment()->GetLength();
    }
}

void NoteInfo::Focussed() {
    frame->Focussed();
    editor->Focussed();
}

void NoteInfo::Unfocussed() {
    frame->Unfocussed();
    editor->Unfocussed();
}

void NoteInfo::Handle(Event& e) {
    e.target->GetRelative(e.x, e.y, editor);
    e.target = editor;
    editor->Handle(e);
}

Interactor* NoteInfo::Visual() {
    return frame;
}
