// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:38 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	noteinfo.h,v $
 * Revision 1.3  91/11/04  19:45:38  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/08/06  09:55:28  sanjay
 * Removed extra Item pointer.
 * Changed CopyTo and CopyFrom names to Save and Revert.
 * 
 * Revision 1.1  91/03/28  11:41:36  sanjay
 * Initial deposit
 * 
 */

#ifndef _NOTEINFOH
#define _NOTEINFOH

#include <InterViews/defs.h>

/*
 * Maintain information about a notice.
 *
 * Also keeps track of the related item.
 * and a special adjustment factor for overlapping items.
 */

class FocusFrame;
class ApptEditor;
class Item;
class Interactor;
class Event;

class NoteInfo {
  public:
    NoteInfo(Item*, int pad);
    virtual ~NoteInfo();

    boolean Modified();
    void    SetModified(boolean);

    Item* Get();
    void  Set(Item*);

    int  GetAdjust();
    void SetAdjust(int i);

    Interactor* Visual();
    void Handle(Event&);

    void Focussed();
    void Unfocussed();

    void Revert();		/* Revert back to associated item state */
    void Save();		/* Set associated item state from self */

    int  GetStart();
    int  GetLength();
    int  GetFinish();

    void SetStart(int);
    void SetLength(int);
  private:
    ApptEditor* editor;
    FocusFrame* frame;
    Item* relative;
    int start;
    int length;

    int adjust;
};

inline Item* NoteInfo::Get() {
    return relative;
}

inline void NoteInfo::Set(Item* a) {
    relative = a;
}

inline int NoteInfo::GetAdjust() {
    return adjust;
}

inline void NoteInfo::SetAdjust(int a) {
    adjust = a;
}

inline int NoteInfo::GetStart() {
    return start;
}

inline void NoteInfo::SetStart(int s) {
    start = s;
}

inline int NoteInfo::GetLength() {
    return length;
}

inline void NoteInfo::SetLength(int l) {
    length = l;
}

inline int NoteInfo::GetFinish() {
    return start + length - 1;
}

#endif /* _NOTEINFOH */
