/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.10 $
 * $Date: 91/11/28 16:25:36 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	notelist.cc,v $
 * Revision 1.10  91/11/28  16:25:36  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.9  91/11/04  19:45:39  sanjay
 * Prepare for release
 * 
 * Revision 1.8  91/09/18  20:33:57  sanjay
 * Use OccurCache for listing items on certain dates.
 * 
 * Revision 1.7  91/08/06  13:55:10  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.6  91/08/06  09:56:14  sanjay
 * Notify DayView of selection changes.
 * 
 * Revision 1.5  91/04/11  13:30:48  sanjay
 * Fixed window popup
 * 
 * Revision 1.4  91/04/01  11:57:10  sanjay
 * Added include file capability
 * 
 * Revision 1.3  91/03/30  15:06:31  sanjay
 * Removed funky item creation
 * 
 * Revision 1.2  91/03/28  11:52:02  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:24  sanjay
 * Initial deposit
 *
 */

#include <InterViews/event.h>
#include <InterViews/shape.h>
#include "application.h"
#include "calendar.h"
#include "dayview.h"
#include "item.h"
#include "noteinfo.h"
#include "notelist.h"
#include "occurcache.h"
#include "popup.h"
#include "xworld.h"

NoteList::NoteList(Application* a, DayView* dv, Date d)
     : DisplayList(a)
{
    SetClassName("NoteList");

    dayview = dv;

    resizing = false;
    trackBottom = false;

    date = d - 1;
    SetDate(d);
}

NoteList::~NoteList() { }

void NoteList::Edit(NoteInfo* note) {
    DisplayList::Edit(note);
    if (focusChild != nil) {
	dayview->SetFocus(focusChild->Get());
    }
}

void NoteList::FinishEdit() {
    DisplayList::FinishEdit();
    dayview->SetFocus(nil);
}

boolean NoteList::Contains(Item* i) {
    return (i->AsNotice() &&
	    App()->GetOccurCache()->ItemIsOn(i, date));
}

void NoteList::SetShape() {
    DisplayList::SetShape();
    shape->vstretch = vfil;
}

void NoteList::Handle(Event& e) {

    if (scrolling) {
	HandleScrollEvent(e);
	return;
    }

    if (resizing) {
	HandleResizeEvent(e);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	switch (e.button) {
	  case LEFTMOUSE:
	    DisplayList::Handle(e);
	    break;
	  case MIDDLEMOUSE:
	    DisplayList::Handle(e);
	    break;
	  case RIGHTMOUSE:
	    NoteInfo* i = FindChild(e);
	    if (i != nil) {
		Edit(i);
		HandleResizeEvent(e);
	    }
	    break;
	}
	break;
      case KeyEvent:
	if ((focusChild != nil) && (! focusChild->Get()->ReadOnly())) {
	    focusChild->Handle(e);
	}
	break;
      default:
	break;
    }
}

void NoteList::HandleResizeEvent(Event& e) {
    if ((focusChild != nil) && (focusChild->Get()->ReadOnly())) {
	Popup* popup = new Popup("Permission denied.");
	popup->SetName("Warning");
	App()->World()->Popup(popup);
	return;
    }

    switch (e.eventType) {
      case DownEvent:
	if (focusChild != nil) {
	    CatchMotion();
	    resizing = true;
	    trackBottom = false;
	}
	break;
      case UpEvent:
	resizing = false;
	IgnoreMotion();
	if (focusChild != nil) {
	    /* Update the calendar */
	    focusChild->Save();
	    focusChild->SetModified(false);
	    App()->Change(focusChild->Get());
	}
	break;
      case MotionEvent:
	if (focusChild != nil) {
	    if (e.target != this) e.target->GetRelative(e.x, e.y, this);

	    if ((e.y < (Bottom(focusChild) - yOffset)) || trackBottom) {
		/* Resize bottom */
		int newHeight = Top(focusChild) - (e.y + yOffset);
		int newLength = (((newHeight * lineUnit) / lineHeight) /
				 itemUnit) * itemUnit;
		trackBottom = true;
		if (newLength < lineUnit)
		    newLength = lineUnit;
		if (newLength != focusChild->GetLength()) {
		    focusChild->SetLength(newLength);
		    PlaceChild(focusChild);
		}
	    }
	}
	break;
      default:
	break;
    }
}

Item* NoteList::MakeItem(Event& e) {
    Calendar* cal = App()->GetMainCalendar();
    if (cal->ReadOnly()) {
	Popup* popup = new Popup("Permission denied.");
	popup->SetName("Warning");
	App()->World()->Popup(popup);
	return nil;
    }

    Notice* note = new Notice(cal);
    note->SetText("");
    note->SetLength(lineUnit);
    note->SetDate(date);
    return note;
}

NoteInfo* NoteList::MakeChild(Item* item) {
    return DisplayList::MakeChild(item);
}
