/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/28 16:25:37 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	occurcache.cc,v $
 * Revision 1.4  91/11/28  16:25:37  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.3  91/11/04  19:45:42  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/09/20  10:11:57  sanjay
 * Added missing return statement at end of GetCacheEntry.
 * 
 * Revision 1.1  91/09/18  20:31:26  sanjay
 * Initial revision
 * 
 *
 */

#include "application.h"
#include "item.h"
#include "occurcache.h"

#include "DateItem.List.h"
#include "ItemP.List.h"

OccurCache::OccurCache(Application* a) {
    application = a;

    cache = new DateItemList;
    maxSize = 32;
    size = 0;
}

OccurCache::~OccurCache() {
    DateItemCell* cell;
    ForListElements(cell,cache) {
	DateItem& item = cell->Element();
	delete item.list;
    }

    delete cache;
}

int OccurCache::NumberOn(Date d) {
    ItemPList* list = GetCacheEntry(d)->Element().list;
    ItemPCell* cell;

    int count = 0;
    ForListElements(cell, list) {
	count++;
    }
    return count;
}

boolean OccurCache::ItemIsOn(Item* i, Date d) {
    ItemPList* list = GetCacheEntry(d)->Element().list;
    ItemPCell* cell;

    ForListElements(cell, list) {
	if (cell->Element() == i) {
	    return true;
	}
    }

    return false;
}

void OccurCache::Preload(Date start, int days) {
    int i;

    if (days < 1) return;

    /* Make sure there is enough room in cache for specified dates */
    if (maxSize < days) {
	maxSize = days;
    }

    /* Count dates that are already cached */
    int cacheHit = 0;
    for (i = 0; i < days; i++) {
	if (FindCacheEntry(start+i) != nil) cacheHit++;
    }

    /* If there were enough cache hits, preloading is probably not worth it */
    if (cacheHit > days / 2) {
	return;
    }

    /* Remove old entries */
    for (i = 0; i < days; i++) {
	DateItemCell* entry = FindCacheEntry(start + i);
	if (entry != nil) {
	    DateItem& item = entry->Element();
	    delete item.list;
	    cache->Remove(entry);
	    size--;
	}
    }

    /* Make room in cache */
    while ((size > (maxSize - days)) && (cache->First() != nil)) {
	DateItem& item = cache->First()->Element();
	delete item.list;
	cache->Remove(cache->First());
	size--;
    }

    /* Make new entries */
    DateItemCell** entries = new DateItemCell*[days];
    for (i = 0; i < days; i++) {
	DateItem element;
	element.date = start + i;
	element.list = new ItemPList;
	size++;
	entries[i] = cache->Append(element);
    }

    /* Collect items in range */

    Date last = start + (days - 1);

    int numItems = application->NumItems();
    for (i = 0; i < numItems; i++) {
	Item* item = application->GetItem(i);

	Date current = item->FirstSince(start);
	while (current <= last) {
	    /* Add item to entry for specified date */
	    entries[current - start]->Element().list->Append(item);

	    current = item->FirstSince(current + 1);
	}
    }

    delete [days] entries;
}

DateItemCell* OccurCache::FindCacheEntry(Date d) {
    DateItemCell* cell;
    ForReverseListElements(cell, cache) {
	if (cell->Element().date == d) {
	    cache->MoveToEnd(cell);
	    return cell;
	}
    }
    return nil;
}

DateItemCell* OccurCache::GetCacheEntry(Date d) {
    DateItemCell* cell = FindCacheEntry(d);
    if (cell != nil) return cell;

    if (size >= maxSize) {
        /* Reuse oldest entry from cache */
	cell = cache->First();
	cache->MoveToEnd(cell);

	DateItem& item = cell->Element();
	delete item.list;
    }
    else {
	/* Add new entry */
	DateItem element;
	cell = cache->Append(element);
	size++;
    }

    cell->Element().date = d;
    cell->Element().list = new ItemPList;

    /* Get items for entry */
    int numItems = application->NumItems();
    for (int i = 0; i < numItems; i++) {
	Item* item = application->GetItem(i);

	if (item->IsOn(d)) {
	    cell->Element().list->Append(item);
	}
    }

    return cell;
}

void OccurCache::Changed(Item* item) {
    Removed(item);
    Added(item);
}

void OccurCache::Removed(Item* item) {
    /* Remove from all lists in cache */
    DateItemCell* dateinfo;
    ForListElements(dateinfo, cache) {
	ItemPCell* iteminfo;
	ForListElements(iteminfo, dateinfo->Element().list) {
	    if (iteminfo->Element() == item) {
		dateinfo->Element().list->Remove(iteminfo);
		break;
	    }
	}
    }
}

void OccurCache::Added(Item* item) {
    /* Add to all appropriate lists in cache */
    DateItemCell* dateinfo;
    ForListElements(dateinfo, cache) {
	if (item->IsOn(dateinfo->Element().date)) {
	    /* Add to item list for date */
	    dateinfo->Element().list->Append(item);
	}
    }
}
