/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.9 $
 * $Date: 91/11/28 14:48:43 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	popup.cc,v $
 * Revision 1.9  91/11/28  14:48:43  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.8  91/11/04  19:45:44  sanjay
 * Prepare for release
 * 
 * Revision 1.7  91/05/06  11:17:21  sanjay
 * Allow keyboard shortcuts to dismiss dialogs
 * 
 * Revision 1.6  91/04/16  15:22:49  sanjay
 * Made closures reference counted
 * 
 * Revision 1.5  91/04/11  13:30:49  sanjay
 * Fixed window popup
 * 
 * Revision 1.4  91/04/03  20:58:16  sanjay
 * Layout improvements
 * 
 * Revision 1.3  91/03/29  16:11:48  sanjay
 * Fixed popup layout.
 * 
 * Revision 1.2  91/03/28  11:52:02  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:25  sanjay
 * Initial deposit
 *
 */

#include <InterViews/message.h>
#include <InterViews/tray.h>
#include <InterViews/sensor.h>
#include <InterViews/event.h>
#include "popup.h"
#include "closure.h"
#include "buttonclosure.h"

declare(Closure,Popup);
implement(Closure,Popup);

Popup::Popup(char const* msg) {
    SetClassName("Popup");
    Layout(new Message(msg));
}

Popup::Popup(Interactor* i) {
    SetClassName("Popup");
    Layout(i);
}

Popup::Popup() {
    SetClassName("Popup");
}

static TGlue* MakeSpace(Coord x, Coord y) {
    return new TGlue(x, y, 0, 0);
}

void Popup::Layout(Interactor* i) {
    input = new Sensor;
    input->Catch(KeyEvent);
    Listen(input);

    Tray* tray = new Tray;

    Closure* exitClosure = new Closure1(Popup)(this, Popup::Exit);
    Button* button = new ButtonClosure("Okay", exitClosure);

    /* Align i */
    tray->Align(TopLeft, i, MakeSpace(round(0.2*inches), round(0.2*inches)));
    tray->Align(Right,   i, MakeSpace(round(0.2*inches), 0));
    tray->Align(Bottom, i, Top, button, MakeSpace(0, round(0.2*inches)));

    /* Align button */
    tray->Align(Right, i, button);
    tray->Align(BottomLeft, button, new TGlue(round(0.2*inches),
					      round(0.2*inches),
					      hfil, 0));
    Insert(tray);
}

void Popup::Handle(Event& e) {
    if (e.eventType == KeyEvent) {
	char c = e.keystring[0];

	if ((c == '\n') || (c == '\r')) {
	    Exit();
	}
    }
}

void Popup::Exit() {
    delete this;
}
