/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:45:46 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	printer.cc,v $
 * Revision 1.5  91/11/04  19:45:46  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/06/28  20:09:37  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.3  91/04/05  10:53:41  sanjay
 * Fixed overflow check in Printer::Put(char).
 * 
 * Revision 1.2  91/04/03  20:44:25  sanjay
 * Further reminder listing fixes
 * 
 *
 */

#include <stdio.h>
#include <string.h>
#include <os/fs.h>
#include "printer.h"

#include "char.Array.h"

Printer::Printer(int l) {
    buffer = new char[l];
    len = l;
    pos = 0;
}

Printer::~Printer() {
    delete buffer;
}

void Printer::Put(int i) {
    char text[20];

    sprintf(text, "%d", i);
    Put(text);
}

void Printer::Put(const char* str) {
    Put(str, strlen(str));
}

void Printer::Put(const char* str, int strl) {
    /* Flush if necessary */
    if (len < (pos + strl)) {
	/* Not enough room in buffer */
	if (pos > 0) {
	    /* Something to flush */
	    Output(buffer, pos);
	    pos = 0;
	}
    }

    if (len < (pos + strl)) {
	/* Still not enough room */
	Output(str, strl);
    }
    else {
	for (int i = 0; i < strl; i++) {
	    buffer[pos++] = str[i];
	}
    }
}

void Printer::Flush() {
    if (pos > 0) {
	Output(buffer, pos);
	pos = 0;
    }
}

FPrinter::FPrinter(int fd) : (1024) {
    file = fd;
}

void FPrinter::Output(const char* str, int len) {
    write(file, str, len);
}

CAPrinter::CAPrinter(charArray& ca) : (1024), memory(ca) {
}

void CAPrinter::Output(const char* str, int strl) {
    memory.Concat(str, strl);
}
