// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

#ifndef _PRINTERH
#define _PRINTERH

/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:45:47 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	printer.h,v $
 * Revision 1.3  91/11/04  19:45:47  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/04/05  10:53:56  sanjay
 * Fixed overflow check in Printer::Put(char).
 * 
 * Revision 1.1  91/04/03  20:40:12  sanjay
 * Put reminders in window instead of piping
 * 
 *
 */

/*
 * Abstract printer class.
 * Simple stop-gap until 2.0 stream libraries become widely
 * available - for example with GNU C++.
 */
class Printer {
  public:
    Printer(int buflen);
    virtual ~Printer();

    void Put(char);
    void Put(int);
    void Put(const char*);
    void Put(const char*, int len);

    void Flush();
  protected:
    /*
     * Called to flush buffer when more space is needed.
     */
    virtual void Output(const char*, int len) = 0;
  private:
    char* buffer;		/* Output buffer */
    int   len;			/* Buffer length */
    int   pos;			/* Pointer to next empty spot. pos <= len. */
};

/*
 * Print to a file.
 */
class FPrinter : public Printer {
  public:
    FPrinter(int fd);
  protected:
    virtual void Output(const char* str, int len);

    int file;
};

/*
 * Print to character array.
 */
class CAPrinter : public Printer {
  public:
    CAPrinter(class charArray&);
  protected:
    virtual void Output(const char*, int);

    class charArray& memory;
};

inline void Printer::Put(char c) {
    /* Flush if necessary */
    if (pos == len) {
	/* Not enough room in buffer */
	if (pos > 0) {
	    /* Something to flush */
	    Output(buffer, pos);
	    pos = 0;
	}
    }

    if (pos == len) {
	/* Still not enough room */
	Output(&c, 1);
    }
    else {
	buffer[pos++] = c;
    }
}

#endif /* _PRINTERH */
