/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.9 $
 * $Date: 91/11/28 16:25:38 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	reminder.cc,v $
 * Revision 1.9  91/11/28  16:25:38  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.8  91/11/28  14:48:45  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.7  91/11/04  19:45:48  sanjay
 * Prepare for release
 * 
 * Revision 1.6  91/09/11  20:21:08  sanjay
 * Added button to suppress all reminders for appt.
 * 
 * Revision 1.5  91/08/13  16:31:51  sanjay
 * Print 1 minute instead of 1 minutes.
 * 
 * Revision 1.4  91/04/18  19:07:46  sanjay
 * Fixed reminder verboseness
 * 
 * Revision 1.3  91/04/16  15:22:51  sanjay
 * Made closures reference counted
 * 
 * Revision 1.2  91/03/28  11:52:03  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:26  sanjay
 * Initial deposit
 *
 */

#include <InterViews/texteditor.h>
#include <InterViews/textbuffer.h>
#include <InterViews/textdisplay.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <InterViews/frame.h>
#include <stdio.h>
#include <string.h>
#include "Time.h"
#include "reminder.h"
#include "item.h"
#include "remindmgr.h"
#include "closure.h"
#include "buttonclosure.h"

declare(Closure,Reminder);
implement(Closure,Reminder);

/*
 * A TextEditor for displaying reminders.
 */
class Reminder_TE : public TextEditor {
  public:
    Reminder_TE(int r, int c, int t, int h) : (r, c, t, h) { }
    TextDisplay* Display();
};

TextDisplay* Reminder_TE::Display() {
    return display;
}

Reminder::Reminder(RemindMgr* m, Appointment* a, Time startTime, int ml) {
    SetClassName("Reminder");
    SetInstance("Reminder");
    SetName("Reminder");

    mgr = m;
    minutesLeft = ml;

    Layout(a);
    RecomputeRemaining();
}

Reminder::~Reminder() {
    delete textBuffer;
    delete region;
}

void Reminder::Tick(int ml) {
    minutesLeft = ml;
    RecomputeRemaining();
}

void Reminder::Layout(Appointment* a) {
    char const* text = a->GetText();
    int length = strlen(text);

    int newlines = 0;
    for (int i = 0; i < length; i++) {
	if (text[i] == '\n') newlines++;
    }
    if (text[length-1] != '\n') newlines++;

    remainingLine = 4 + newlines;

    region     = new char[length + 200];
    textBuffer = new TextBuffer(region, 0, length + 200);

    display = new Reminder_TE(5 + newlines, 40, 8, Plain);
    display->Edit(textBuffer);
    display->Display()->CaretStyle(NoCaret);

    /* Add appointment time - two lines */
    InsertText("Appointment from ");
    InsertTime(a->GetStart());
    InsertText(" to ");
    InsertTime(a->GetFinish());
    InsertText("\n\n");

    /* Add appointment info - "newlines" lines */
    InsertText(text);
    if (text[length-1] != '\n') InsertText("\n");

    /* Add separator line */
    InsertText("\n");

    /* Add dummy "remaining minutes" line */
    InsertText("???\n");

    Closure* snoozeClosure = new Closure1(Reminder)(this, Reminder::Snooze);
    Closure* shutUpClosure = new Closure1(Reminder)(this, Reminder::ShutUp);

    VBox* box = new VBox
	(display,
	 new HBox(new ButtonClosure("Snooze",  snoozeClosure),
		  new HGlue,
		  new ButtonClosure("Shut up", shutUpClosure)));
    box->Align(Right);

    Insert(new MarginFrame(box, round(0.2 * inches)));
}

void Reminder::RecomputeRemaining() {
    /* Go to correct line */
    display->BeginningOfText();
    display->ForwardLine(remainingLine - 1);

    /* Get current text and delete it */
    int start = display->Dot();
    display->EndOfLine();
    display->DeleteText(start - display->Dot());

    /* Insert new selection */
    if (minutesLeft == 0) {
	InsertText("Appointment is now.");
    }
    else if (minutesLeft < 0) {
	InsertText("Late for appointment.");
    }
    else {
	char buffer[100];
	const char* unit;

	unit = (minutesLeft == 1) ? "minute" : "minutes";
	sprintf(buffer, "Appointment is in %d %s.", minutesLeft, unit);
	InsertText(buffer);
    }
}

void Reminder::InsertText(char const * text) {
    display->InsertText(text, strlen(text));
}

void Reminder::InsertTime(int timeOfDay) {
    char meridian;
    int hour;
    int minute;

    hour = timeOfDay / 60;
    if (hour >= 12) {
	meridian = 'p';
	hour -= 12;
    }
    else {
	meridian = 'a';
    }
    if (hour == 0) hour = 12;

    minute = timeOfDay % 60;

    char buffer[20];
    sprintf(buffer, "%d:%02d%cm", hour, minute, meridian);

    InsertText(buffer);
}

void Reminder::Snooze() {
    mgr->RemoveReminder(this);
}

void Reminder::ShutUp() {
    mgr->DisableReminder(this);
}
