/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.9 $
 * $Date: 91/11/28 14:48:47 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	remindmgr.cc,v $
 * Revision 1.9  91/11/28  14:48:47  sanjay
 * Renamed Closure(T) to Closure1(T)
 * 
 * Revision 1.8  91/11/04  19:45:50  sanjay
 * Prepare for release
 * 
 * Revision 1.7  91/09/11  20:21:59  sanjay
 * Keep track of appts for which reminders have been suppressed.
 * 
 * Revision 1.6  91/06/28  20:09:38  sanjay
 * Moved Array and Time classes into hierarchy.
 * 
 * Revision 1.5  91/04/18  19:07:46  sanjay
 * Fixed reminder verboseness
 * 
 * Revision 1.4  91/04/16  15:22:52  sanjay
 * Made closures reference counted
 * 
 * Revision 1.3  91/03/29  10:33:30  sanjay
 * Put arrays in library
 * 
 * Revision 1.2  91/03/28  11:52:03  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:27  sanjay
 * Initial deposit
 *
 */

#include "Time.h"
#include "application.h"
#include "closure.h"
#include "reminder.h"
#include "remindmgr.h"
#include "xworld.h"
#include "Timer.h"

#include "Pointer.Array.h"

declare(Closure,RemindMgr);
implement(Closure,RemindMgr);

/*
 * Info per active/disabled reminder.
 */
struct RemindMgr_Info {
    Appointment* appt;
    Reminder*	 reminder;
    Time	 startTime;
};

RemindMgr::RemindMgr(Application* a)
     : active(*(new PointerArray)), disabled(*(new PointerArray))
{
    application = a;

    ticker = new Closure1(RemindMgr)(this, RemindMgr::Tick);
    ticker->Reference();

    timer = a->GetTimer();
    hook = nil;
    SetTimer();
}

RemindMgr::~RemindMgr() {
   if (hook != nil) {
	timer->Remove(hook);
    }

    int i;

    for (i = 0; i < active.Size(); i++) {
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);

	delete info->reminder;
	delete info;
    }

    for (i = 0; i < disabled.Size(); i++) {
	RemindMgr_Info* info = (RemindMgr_Info*) (disabled[i]);

	/* Disabled info has no active reminders */
	delete info;
    }

    delete &active;
    delete &disabled;

    Unref(ticker);
}

void RemindMgr::AddReminder(Appointment* a, Time st) {
    int i;

    /* Check if reminders are disabled */
    for (i = 0; i < disabled.Size(); i++) {
	if (Matches((RemindMgr_Info*) (disabled[i]), a, st)) {
	    /* Reminder is disabled */
	    return;
	}
    }

    /* Remove any active reminder for this appt */
    i = 0;
    while (i < active.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);
	if (Matches(info, a, st)) {
	    /* Remove it */
	    delete info->reminder;
	    delete info;
	    active[i] = active.High();
	    active.Remove();
	}
	else {
	    i++;
	}
    }

    /* Add new reminder */
    Reminder* r = new Reminder(this, a, st, round((st-Time::Now()).Minutes()));
    application->World()->InsertApplication(r);
    application->World()->RingBell(30);

    RemindMgr_Info* info = new RemindMgr_Info;
    active.Append((Pointer) info);
    info->appt = a;
    info->startTime = st;
    info->reminder = r;

    SetTimer();
}

void RemindMgr::RemoveReminder(Reminder* r) {
    int i = 0;
    while (i < active.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);
	if (info->reminder == r) {
	    delete info;
	    active[i] = active.High();
	    active.Remove();
	}
	else {
	    i++;
	}
    }

    delete r;
    SetTimer();
}

void RemindMgr::RemoveRemindersFor(Appointment* a) {
    int i = 0;
    while (i < active.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);
	if (info->appt == a) {
	    delete info->reminder;
	    delete info;
	    active[i] = active.High();
	    active.Remove();
	}
	else {
	    i++;
	}
    }

    i = 0;
    while (i < disabled.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (disabled[i]);
	if (info->appt == a) {
	    delete info;
	    disabled[i] = disabled.High();
	    disabled.Remove();
	}
	else {
	    i++;
	}
    }

    SetTimer();
}

void RemindMgr::DisableReminder(Reminder* r) {
    int i = 0;
    while (i < active.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);
	if (info->reminder == r) {
	    /*
	     * Switch entry to disabled list -
	     * Note - reminder field does not matter in disabled list.
	     */
	    disabled.Append((Pointer) info);

	    /* Remove from active list */
	    active[i] = active.High();
	    active.Remove();
	}
	else {
	    i++;
	}
    }
    delete r;

    SetTimer();
}

void RemindMgr::Tick() {
    hook = nil;

    int i;
    Time now = Time::Now();

    /* Update active reminders */
    for (i = 0; i < active.Size(); i++) {
	/*
	 * Update minutes-left field for active[i].
	 */
	RemindMgr_Info* info = (RemindMgr_Info*) (active[i]);
	info->reminder->Tick(round((info->startTime - now).Minutes()));
    }

    /* Remove old disable entries */
    i = 0;
    while (i < disabled.Size()) {
	RemindMgr_Info* info = (RemindMgr_Info*) (disabled[i]);
	if (info->startTime < now) {
	    /* Appointment instance has started */
	    delete info;
	    disabled[i] = disabled.High();
	    disabled.Remove();
	}
	else {
	    i++;
	}
    }

    SetTimer();
}

/*
 * Set timer.
 *
 * If there are any reminders (active or disabled, set timer at minute
 * boundaries, else set no timer.
 */
void RemindMgr::SetTimer() {
    /*
     * If a timer is already set, then no need to do anything.
     * It will go off at the next minute boundary.
     */
    if (hook != nil) {
	return;
    }

    if ((active.Size() > 0) || (disabled.Size() > 0)) {
	/* Fire at next minute boundary */
	Time now = Time::Now();
	int hour, min, sec, msec;
	now.BreakDownClock(hour, min, sec, msec);

	hook = timer->FireAt(now - Duration::Second() * sec +
			     Duration::Minute(),
			     ticker);
    }
}

boolean RemindMgr::Matches(RemindMgr_Info* info, Appointment* a, Time st) {
    if (info->appt != a) return false;

    /* Time fields should be within a second of each other */
    if (((info->startTime - st) <= Duration::Second()) ||
	((st - info->startTime) <= Duration::Second())) {
	return true;
    }

    return false;
}
