// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:45:51 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	remindmgr.h,v $
 * Revision 1.5  91/11/04  19:45:51  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/09/11  20:21:42  sanjay
 * Keep track of appts for which reminders have been suppressed.
 * 
 * Revision 1.3  91/04/18  19:07:47  sanjay
 * Fixed reminder verboseness
 * 
 * Revision 1.2  91/04/16  15:22:53  sanjay
 * Made closures reference counted
 * 
 * Revision 1.1  91/03/28  11:41:40  sanjay
 * Initial deposit
 * 
 */

#ifndef _REMINDMGRH
#define _REMINDMGRH

/*
 * Manage reminders - update <time left to appt> field in each reminder
 * periodically.
 *
 * Abstractly maintains two lists -
 *	Active reminders
 *	Disabled reminders
 */

#include "apptview.h"
#include "Time.h"

class Application;
class Appointment;
class Closure;
class PointerArray;
class Reminder;
class RemindMgr_Info;
class Timer;
class TimerAction;

class RemindMgr {
  public:
    RemindMgr(Application*);
    virtual ~RemindMgr();

    /*
     * Add a reminder.
     * Removes old reminders for specified <appointment,Time> instance.
     */
    void AddReminder(Appointment*, Time);

    /*
     * Remove a reminder.
     * Called on user input.
     */
    void RemoveReminder(Reminder*);

    /*
     * Remove all reminders for specified appointment.
     */
    void RemoveRemindersFor(Appointment*);

    /*
     * Disable a reminder.
     */
    void DisableReminder(Reminder*);
  protected:
    void Tick();		/* Called when timer expires */
    void SetTimer();		/* Set timer as appropriate */

    Application* application;

    Timer* timer;		/* Global timer used for notifications */
    PointerArray& active;	/* List of active reminders */
    PointerArray& disabled;	/* List of disabled reminders */
    TimerAction* hook;		/* Hook to manipulate timer entry */

    Closure* ticker;

    /* Compare appt and start times */
    static boolean Matches(RemindMgr_Info*, Appointment*, Time);
};

#endif /* _REMINDMGRH */
