/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.4 $
 * $Date: 91/11/04 19:45:53 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	selectinclude.cc,v $
 * Revision 1.4  91/11/04  19:45:53  sanjay
 * Prepare for release
 * 
 * Revision 1.3  91/11/01  14:48:31  sanjay
 * Fixed up menu item display.
 * 
 * Revision 1.2  91/08/06  09:57:01  sanjay
 * Display calendar base names.
 * 
 * Revision 1.1  91/08/05  17:14:07  sanjay
 * Initial revision
 * 
 *
 */

#include <InterViews/border.h>
#include <InterViews/menu.h>
#include "application.h"
#include "calendar.h"
#include "dayview.h"
#include "selectinclude.h"

/*
 * Menu items for selectingincluded calendars.
 */
class SIncludeItem : public MenuItem {
  public:
    SIncludeItem(char const* name,
		 char const* displayed,
		 DayView*,
		 void (DayView::*method)(char const*));

    virtual void Do();
  private:
    char const* name;
    DayView* view;
    void (DayView::*method)(char const*);
};

SIncludeItem::SIncludeItem(char const* calName,
			   char const* displayedName,
			   DayView* v,
			   void (DayView::*m)(char const*))
     : (displayedName)
{
    name = calName;
    view = v;
    method = m;
}

void SIncludeItem::Do() {
    (view->*method)(name);
}

SelectInclude::SelectInclude(char const* title, Application* a,
			     boolean showTop,
			     DayView* v, void (DayView::*m)(char const*),
			     ButtonState* enable, int enableValue)
     : (title, enable, enableValue)
{
    showTopLevel = showTop;
    application = a;
    view = v;
    method = m;
    MakePullRight();
}

boolean SelectInclude::Changed() {
    return true;
}

void SelectInclude::FillBody() {
    Calendar* cal = application->GetMainCalendar();

    if (showTopLevel) {
	Include(new SIncludeItem(application->GetName(cal),
				 application->GetShortName(cal),
				 view, method));
	Include(new MenuItem(new HBorder));
    }

    for (int i = 0; i < cal->NumIncludes(); i++) {
	Calendar* c = application->GetCalendarByName(cal->GetInclude(i));
	Include(new SIncludeItem(application->GetName(c),
				 application->GetShortName(c),
				 view, method));
    }
}
