// This may look like C code, but it is really -*- C++ -*-
/* Copyright (c) 1991 by Sanjay Ghemawat */

/*
 * $Revision: 1.3 $
 * $Date: 91/11/28 16:25:41 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	smallintset.h,v $
 * Revision 1.3  91/11/28  16:25:41  sanjay
 * CFRONT compatibility
 * 
 * Revision 1.2  91/11/04  19:45:58  sanjay
 * Prepare for release
 * 
 * Revision 1.1  91/03/28  11:41:42  sanjay
 * Initial deposit
 * 
 */
#ifndef SMALLINTSETH
#define SMALLINTSETH

#include <InterViews/defs.h>
#include <stdio.h>

class Lexer;

static const int SISetLargestMember = 31;

/*
 * A Set of small integers (range 0..SISetLargestMember)
 */
class SmallIntSet {
  public:
    SmallIntSet();
    SmallIntSet(const SmallIntSet&);
    ~SmallIntSet();

    SmallIntSet& operator= (const SmallIntSet&);

    /*
     * Set operations.
     */
    void Clear();
    void Insert(unsigned int);
    void Remove(unsigned int);
    boolean Member(unsigned int) const;

    /*
     * Comparison.
     */
    boolean operator == (SmallIntSet);
    boolean operator != (SmallIntSet);

    /*
     * Set union.
     */
    friend SmallIntSet operator+(const SmallIntSet, const SmallIntSet);

    /*
     * Set intersection.
     */
    friend SmallIntSet operator*(const SmallIntSet, const SmallIntSet);

    /*
     * Set subtraction.
     */
    friend SmallIntSet operator-(const SmallIntSet, const SmallIntSet);

    /*
     * Set size.
     */
    unsigned int Size() const;

    /*
     * I/O.
     */
    boolean Read(Lexer*);
    void Write(FILE*) const;
  protected:
    unsigned long bits;

    SmallIntSet(unsigned long rep);
};

inline SmallIntSet::SmallIntSet()			{ bits = 0; }
inline SmallIntSet::SmallIntSet(const SmallIntSet& s)	{ bits = s.bits; }
inline SmallIntSet::SmallIntSet(unsigned long rep)	{ bits = rep; }
inline SmallIntSet::~SmallIntSet()			{ }
inline SmallIntSet& SmallIntSet::operator=(const SmallIntSet& s) {
    bits = s.bits;
    return *this;
}

inline void SmallIntSet::Clear() {
    bits = 0;
}

inline void SmallIntSet::Insert(unsigned int x) {
    if (x <= SISetLargestMember)
	bits |= (1 << x);
}

inline void SmallIntSet::Remove(unsigned int x) {
    bits &= ~(1 << x);
}

inline boolean SmallIntSet::Member(unsigned int x) const {
    return (bits & (1 << x));
}

inline boolean SmallIntSet::operator == (SmallIntSet s) {
    return bits == s.bits;
}

inline boolean SmallIntSet::operator != (SmallIntSet s) {
    return bits != s.bits;
}

inline SmallIntSet operator+(const SmallIntSet a, const SmallIntSet b) {
    return SmallIntSet(a.bits | b.bits);
}

inline SmallIntSet operator*(const SmallIntSet a, const SmallIntSet b) {
    return SmallIntSet(a.bits & b.bits);
}

inline SmallIntSet operator-(const SmallIntSet a, const SmallIntSet b) {
    return SmallIntSet(a.bits & (a.bits ^ b.bits));
}

#endif /*SMALLINTSETH */
