/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.3 $
 * $Date: 91/11/04 19:46:07 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	wdays.cc,v $
 * Revision 1.3  91/11/04  19:46:07  sanjay
 * Prepare for release
 * 
 * Revision 1.2  91/03/28  11:52:07  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:36  sanjay
 * Initial deposit
 *
 */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <stdio.h>
#include "WeekDay.h"
#include "wdays.h"

static const int numRows = 7;

static inline const char* name(int i) {
    return (WeekDay::First() + (i - 1)).Name();
}

WDays::WDays() : (1, numRows) {
    input = updownEvents;
    input->Reference();
    Listen(input);
}

WDays::~WDays() {
    Unref(input);
}

void WDays::Reconfig() {
    cellWidth = output->GetFont()->Width("Wednesday");
    cellHeight = output->GetFont()->Height();
    LabelGrid::Reconfig();
}

void WDays::GetExtent(int, int, Coord& w, Coord& h) {
    w = cellWidth;
    h = cellHeight;
}

void WDays::Paint(Coord l, Coord b, Coord r, Coord t, int col, int row){
    if (col != 0) return;

    int day = row + 1;

    if ((day >= 1) && (day <= 7)) {
	char const* str = name(day);
	if (Selected(day)) {
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, l, b);
	    output->SetColors(output->GetBgColor(), output->GetFgColor());
	}
	else {
	    output->ClearRect(canvas, l, b, r, t);
	    output->Text(canvas, str, l, b);
	}
    }
}

void WDays::Changed(int day) {
    if (canvas != nil) {
	int c = 0;
	int r = day - 1;

	Paint(FindLeft(c), FindBottom(r), FindRight(c), FindTop(r), c, r);
    }
}

void WDays::Handle(Event& e) {
    switch (e.eventType) {
      case DownEvent:
	int r = FindRow(e.y);
	int c = FindColumn(e.x);

	if (c != 0) break;
	int day = r + 1;

	if ((day >= 1) && (day <= 7)) {
	    Hit(day);
	}
	break;
      default:
	break;
    }
}
