/* Copyright (c) 1991 by Sanjay Ghemawat */
/*
 * $Revision: 1.5 $
 * $Date: 91/11/04 19:46:11 $
 * $Author: sanjay $
 * $State: Exp $
 *
 * $Log:	xworld.cc,v $
 * Revision 1.5  91/11/04  19:46:11  sanjay
 * Prepare for release
 * 
 * Revision 1.4  91/08/06  13:55:11  sanjay
 * Error popups are now centered on the mouse.
 * 
 * Revision 1.3  91/04/11  13:30:50  sanjay
 * Fixed window popup
 * 
 * Revision 1.2  91/03/28  11:52:08  sanjay
 * Fixed log
 * 
 * Revision 1.1  91/03/28  11:42:38  sanjay
 * Initial deposit
 *
 */

#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <stdio.h>
#include <sys/time.h>
#include "Timer.h"
#include "xworld.h"

XWorld::XWorld(const char* title, PropertyData* prop, OptionDesc* opt,
	       int& argc, char* argv[])
     : (title, prop, opt, argc, argv)
{
    timer = new Timer;
    input = new Sensor;
    Listen(input);
}

XWorld::~XWorld() {
    delete timer;
    Unref(input);
}

void XWorld::Run() {
    Event e;

    Next(e);
    while (e.target != nil) {
	e.target->Handle(e);
	Next(e);
    }
}

void XWorld::Next(Event& e) {

    if (timer->Pending()) {
	Duration remaining = timer->NextFire() - Time::Now();
	struct timeval tv;

	Duration zero = Duration::Second() * 0;

	if (remaining < zero) remaining = zero;
	remaining.Convert(tv);
	input->CatchTimer(tv.tv_sec, tv.tv_usec);
    }
    Read(e);
}

void XWorld::Handle(Event& e) {
    switch (e.eventType) {
      case TimerEvent:
	timer->Tick();
	break;
      default:
	break;
    }
}

void XWorld::Popup(Interactor* i, Interactor* bg) {
    if (bg == nil) bg = this;

    Coord x, y;
    bg->Align(Center, 0, 0, x, y);
    bg->GetRelative(x, y, this);
    InsertTransient(i, bg, x, y, Center);
}

void XWorld::Popup(Interactor* i) {
    Event e;

    Poll(e);
    InsertTransient(i, this, e.x, e.y, Center);
}
