
/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <InterViews/button.h>
#include <InterViews/menu.h>
#include <InterViews/frame.h>
#include <InterViews/banner.h>
#include <InterViews/filechooser.h>
#include <InterViews/textbuffer.h>
#include <InterViews/scene.h>
#include <InterViews/message.h>
#include <InterViews/world.h>
#include <InterViews/window.h>
#include <InterViews/event.h>
#include <InterViews/sensor.h>
#include <InterViews/dialog.h>
#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <Unidraw/dialogs.h>

#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include "key4code.h"

#include "SearchDialog.h"
#include "GotoDialog.h"
#include "Iedit.h"
#include "filemgr.h"

#if 0
#include <stream.h>
#endif

#include <InterViews/2.6/_enter.h>

int abs(int i){ if(i >= 0) return(i); else return(-i); }

static const int MAXWINDOWS = 50;	/* Prety large number of widnows */

static const char* Reading = "Reading.  Please stand by....";
static const char* helpInstance = "help";

boolean ForceReadOnly = false;
boolean iconic = false;
boolean IsWrap = false;
boolean IsIndent = false;
int tabsize = 8;
static Iedit* helpExist = nil;
Iedit* KillThis = nil;

static void beep(){
	World* w = World::current();
	w->flush();
	w->RingBell(1);
}

static Iedit* ieditList[MAXWINDOWS];

static void PutIeditList(Iedit* iedit){
	for(int i=0; i<MAXWINDOWS; i++){
		if(ieditList[i] == nil){
			ieditList[i] = iedit;
			return;
		}
	}
	beep();
}

static void NotifyIeditList(){
	for(int i=0; i<MAXWINDOWS; i++){
		if(ieditList[i]){
			ieditList[i]->UpdateStatus(false);
		}
	}
}

static void DelIeditList(Iedit* iedit){
	for(int i=0; i<MAXWINDOWS; i++){
		if(ieditList[i] == iedit){
			ieditList[i] = nil;
			return;
		}
	}
}


IeditWindow :: IeditWindow(Iedit* iedit) : ApplicationWindow(iedit){}

IeditWindow :: ~IeditWindow() {}

void IeditWindow :: configure(){
	ApplicationWindow :: configure();
	iconic(::iconic);
}

Iedit::Iedit(const char* name) : Iedit_core(name) {
	Init(getwd(new char[128]));
}

Iedit::Iedit(const char* name, const char* startdir) : Iedit_core(name) {
	Init(startdir);
}

void Iedit::MapWindow(){
	window->map();
}

void Iedit::Init(const char* startdir){
	SetClassName("Iedit");
	_mgr = new FileManager();
	fileDialog = new  FileChooser(
		"Choose a file for Open:","",
		startdir, 24, 32, "OK", Center
	);
	searchDialog = new SearchDialog("searchDialog");
	gotoDialog = new GotoDialog("gotoDialog");
	tabDialog = new GotoDialog("tabDialog");
	tabDialog->ChangeTitle("Tab Size:");
	quote = false;
	doubleClick = dialogActive = false;
	lastClick = 0;
	views++;
	messageLine = new Banner("messageLine",state,"","");
	position = new Banner("position","","",pos);
	messageFrame->Remove(dummyMessage);
	messageFrame->Insert(new HBox(messageLine,position));
	window = new IeditWindow(this);
	Open("");
	PutIeditList(this);
	NotifyIeditList();
}

int Iedit::views = 0;

Iedit::~Iedit(){
	const char* name = GetInstance();
	if(!strcmp(name,helpInstance)) helpExist = nil;
	delete _mgr;
	delete fileDialog;
	delete searchDialog;
	delete gotoDialog;
	delete messageLine;
	delete position;
	views--;
	DelIeditList(this);
	NotifyIeditList();
	if(window) {
		window->unmap();
		delete window;
	}
}

void Iedit::Handle(Event& e){
	if(e.target != this) 
		{ e.target->Handle(e); return; }
	else if(e.eventType == KeyEvent) {
		Keyevent(e);
	} else if(e.eventType == DownEvent) {
		static const long click = 300;	// 300 milliseconds
		switch(e.button){
		case LEFTMOUSE:
			GetRelative(e.x, e.y, editor);
			if(e.control_is_down()){
				editor->SelectMore(editor->Locate(e.x,e.y));
				doubleClick = false;
			} else {
				editor->Select(editor->Locate(e.x, e.y));
				do {
					editor->ScrollToView(e.x, e.y);
					editor->SelectMore(editor->Locate(e.x,e.y));
					Poll(e);
					GetRelative(e.x, e.y, editor);
				} while(e.leftmouse);
				if(e.time() - lastClick <= click){
					if(doubleClick){
						SelectLine();
						doubleClick = false;
					} else {
						SelectWord();
						doubleClick = true;
					}
				} else doubleClick = false;
				lastClick = e.time();
			}
			editor->ScrollToSelection();
			UpdateStatus(false);
			break;
		case MIDDLEMOUSE:
			if(e.control_is_down()){ Paste();UpdateStatus(true); } 
			else editor->GrabScroll(e); 
			break;
		case RIGHTMOUSE:
			if(e.control_is_down()){ 
				Copy();
				editor->Select(editor->Dot());
			} else 	editor->RateScroll(e); 
			break;
		}
	}
}

void Iedit::Keyevent(Event&e){
	boolean modified = false;
	if( e.len != 0){
		char c = e.keystring[0];
		if(quote) {
			InsertChar(c); quote = false;
		} else 
		switch(c){
		case '\b':	// Backward Delete
		case '\177': 
			if(e.control_is_down())
				KillToBeginningOfLine();
			else
				editor->DeleteText(-1);
			modified = true;
			break;
		case Ctrl('B'):
			MatchBracket(); break;
		case Ctrl('D'):	// Forward Delete
			editor->DeleteText(1);
			modified = true;
	 		break;
		case Ctrl('G'): GotoLine();	break;
		case Ctrl('F'): Include(); return;
		case Ctrl('K'):	// Kill line
			KillRestOfLine();
			modified = true;
		 	break;
		case Ctrl('N'): New(); 		return;
		case Ctrl('O'): Open(); 	return;
		case Ctrl('P'): 
			Paste();
		 	break;
		case Ctrl('Q'): Quit();		return;
		case Ctrl('R'):	Search(); 	return;
		case Ctrl('S'):	Save(); 	return;
		case Ctrl('T'): GoToSelection(); break;
		case Ctrl('V'):	quote = true;	return;
		case Ctrl('W'): SelectWord();	break;
		case Ctrl('L'): SelectLine();	break;
		case Ctrl('Y'):	// Yank
			Copy();	
			editor->Select(editor->Dot());
			break;
		case '\015':
			InsertChar('\n');
			modified = true;
			break;
		default:
			if(!iscntrl(c) || c == '\t') {
				InsertChar(c);
				modified = true;
				break;
			}
			return;
		}
	} else {
		TextBuffer* text = _mgr->GetTextBuffer();
		int lineOffset = text->LineOffset(editor->Dot());
		switch(e.keycode()){
		case KeyUpArraw:
			editor->BackwardLine(1); 
			GotoLineOffset(lineOffset);
			break;
		case KeyDownArraw: 
			editor->ForwardLine(1);
			GotoLineOffset(lineOffset);
			break;
		case KeyLeftArraw: 
			if(e.control_is_down())
				editor->BackwardWord(1);
			else 
				editor->BackwardCharacter(1);
			break;
		case KeyRightArraw: 
			if(e.control_is_down())
				editor->ForwardWord(1);
			else 
				editor->ForwardCharacter(1);
			break;
		case KeyCopy:
			Copy();	editor->Select(editor->Dot());
			break;
		case KeyCut:
			Cut();
			break;
		case KeyPaste:
			Paste();
			break;
		case KeyFind:
			Search(); return;
		case KeyOpen:
			Open(); return;
		case KeyHome:
			editor->Select(text->BeginningOfLine(editor->Dot()));
			break;
		case KeyEnd:
			editor->Select(text->EndOfLine(editor->Dot()));
			break;
		case KeyF1:
			GotoLine(); break;
		case KeyF2:
			Clear(); return;
		case KeyF3:
			MatchBracket(); break;
		case KeyF4:
			GoToSelection(); break;
		case KeyPageUp:
			editor->BackwardPage(1);
			break;
		case KeyPageDown:
			editor->ForwardPage(1);
			break;
		case KeyIns:
		case KeyFront:
			New(); return;
		case KeyHelp:
			HelpMe(); return;
		default: return;
		}
	}
	editor->ScrollToSelection();
	UpdateStatus(modified);
}

int Iedit::Selection(){
	return abs(editor->Mark() - editor->Dot());
}

void Iedit::SelectAll(){ editor->SelectAll(); }

void Iedit::SelectWord(){
	int dot = editor->Dot();
	TextBuffer* text = _mgr->GetTextBuffer();
	if(text->IsBeginningOfLine(dot)) dot++;
	editor->Select(text->BeginningOfWord(dot)),
	dot = editor->Dot();
	editor->Select(
		text->BeginningOfWord(dot),
		text->EndOfWord(dot)
	);
}

void Iedit::SelectLine(){
	int dot = editor->Dot();
	TextBuffer* text = _mgr->GetTextBuffer();
	editor->Select(
		text->BeginningOfLine(dot),
		text->EndOfLine(dot)+1
	);

}

void Iedit::KillRestOfLine(){
	int dot = editor->Dot();
	TextBuffer* text = _mgr->GetTextBuffer();
	if(text->IsEndOfLine(dot)){
		editor->DeleteText(1);
	} else {
		editor->Select(dot,text->EndOfLine(dot));
		Cut();
	}
}

void Iedit::KillToBeginningOfLine(){
	int dot = editor->Dot();
	TextBuffer* text = _mgr->GetTextBuffer();
	int beg = text->BeginningOfLine(dot);
	int numb = dot - beg;
	if(numb > 0){
		editor->Select(beg, dot);
		Cut();
	} else editor->DeleteText(-1);
}

void Iedit::GotoLineOffset(int off){
	int dot = editor->Dot();
	TextBuffer* text = _mgr->GetTextBuffer();
	if(dot+off >= text->EndOfLine(dot))
		editor->Select(text->EndOfLine(dot));
	else  editor->ForwardCharacter(off);
}

void Iedit::GoToSelection(){
	editor->ScrollToSelection();
}

void Iedit::InsertChar(char c){
	editor->DeleteSelection();
	editor->ScrollToSelection();
	_mgr->CheckBuffer(editor, 1);
	editor->InsertText(&c,1);
}

void Iedit::Cut() {
	editor->Cut();
	UpdateStatus(true);
}

void Iedit::Copy() {
	editor->Copy();
}

void Iedit::Paste(){
	const char* cbuf = editor->CutBuffer();
	if(! *cbuf) return;
	_mgr->CheckBuffer(editor, strlen(cbuf));
	editor->Paste(cbuf);
	UpdateStatus(true);
}

void Iedit::New(){
	char* curpath = strdup(fileDialog->Choice());
	char* tail = strrchr(curpath, '/');
	if(tail) *(tail+1) = '\0';
	Iedit* iedit = new Iedit("iedit", curpath);
	iedit->	MapWindow();
	delete curpath;
	UpdateStatus(false);
}

static boolean IsHelpWindow(Iedit* iedit){
	if(!helpExist || helpExist != iedit) return false;
	iedit->Report("This is the help window","Do not try to do editing here");
	return true;
}

void Iedit::HelpMe(){ HelpWindow(this);}

void Iedit::HelpWindow(Iedit* iedit){
	if(access(HelpFile, R_OK) || access(HelpFile, F_OK)){
		char buf[128]; char* sorry = "Sorry, no help is available";
		sprintf(buf, "%s: not accessible", HelpFile);
		if(iedit) iedit->Report(buf, sorry);
		else fprintf(stderr,"%s\n%s\n", buf, sorry);
		return;
	}
	if(helpExist){
		helpExist->window->deiconify();
		return;
	}
	Iedit* help = new Iedit(helpInstance);
	help->window->name("iedit-help");
	help->window->icon_name("iedit-help");
	boolean fr = ForceReadOnly;
	ForceReadOnly = true;
	help->Open(HelpFile);
	ForceReadOnly = fr;
	helpExist = help;
	help->MapWindow();
}

void Iedit::_Load(){
	if(IsHelpWindow(this)) return;
	if(dialogActive) { beep(); return;}
	const char* name = fileName->Text();
	if(*name == '\0'){
		Report("Sorry, no file to be load","");
		return;
	}
	Open(strdup(name));
}

void Iedit::_Save(){ Save(fileName->Text()); }

void Iedit::Clear(){
	if(IsHelpWindow(this)) return;
	if(Confirm("This operation will clear the contents of the edit buffer",
			"Do you really want to do this?") != 'y')
		return;
	if(!dialogActive) Open(""); else beep();
}

void Iedit::Open(){
	if(IsHelpWindow(this)) return;
	if(dialogActive) {beep();return;} dialogActive = true;
	if(!ConfirmSave()) {
		dialogActive = false;
		return;
	}
	fileDialog->SetTitle("Choose a file to Open:");
	if(PopUpDialog(fileDialog) == 1){
		Open(strdup(fileDialog->Choice()));
	}
	dialogActive = false;
}

static boolean autowrap(const char* file){
	const char* sufix = World::current()->property_value("autowrap");
	char* sp = (char*) sufix;
	char* fp = strrchr((char*)file, '.');
	if(fp) while(sp = strchr(sp,'.')){
		int len = strlen(fp);
		if(!strncmp(fp, sp, len) && (sp[len] == ' ' || sp[len] == '\0'))
			return true;
		sp++;
	}
	return false;
}

static boolean autoindent(const char* file){
	const char* sufix = World::current()->property_value("autoindent");
	char* sp = (char*) sufix;
	char* fp = strrchr((char*)file, '.');
	if(fp) while(sp = strchr(sp,'.')){
		int len = strlen(fp);
		if(!strncmp(fp, sp, len) && (sp[len] == ' ' || sp[len] == '\0'))
			return true;
		sp++;
	}
	return false;
}

void Iedit::Open(const char* name){
	if(IsHelpWindow(this)) return;
	if(!ConfirmSave()) return;
	char* fname = strrchr(name, '/');
	if(!fname) fname = (char*) name;
	else fname++;
	char iname[128];
	sprintf(iname, "Iedit %s",fname[1] ? fname : "");
	window->icon_name(strdup(iname));
	SetWrap(IsWrap || autowrap(name));
	SetIndent(IsIndent || autoindent(name));
	Message(Reading);
	boolean readonly = _mgr->Open(name);
	editor->TabSize(tabsize);
	editor->Edit(_mgr->GetTextBuffer());
	if(readonly) editor->ReadOnly(true);
	else editor->ReadOnly(ForceReadOnly);
	if(*name){
		fileName->Message(name);
	}
	else {
		char* name = strdup(fileDialog->Choice()), *tail;
		tail = strrchr(name, '/');
		if(tail) *++tail = '\0';
		fileName->Message(name);
		delete name;
	}
	UpdateStatus(true);
}

boolean matchBracket(char& open, char& close, int& direction){
	direction = 0;
	switch(open){
	case '{':
		close = '}'; direction = 1; break;
	case '}':
		close = '{'; direction = -1; break;
	case '(':
		close = ')'; direction = 1; break;
	case ')':
		close = '('; direction = -1; break;
	case '[':
		close = ']'; direction = 1; break;
	case ']':
		close = '['; direction = -1; break;
	}
	return direction != 0;
}

void Iedit::MatchBracket(){
	char open, close;
	int direction;
	TextBuffer* text = _mgr->GetTextBuffer();
	open = text->Char(editor->Dot());
	if(!matchBracket(open, close, direction)) return;
	int opens = 1;
	for(int dot = editor->Dot() + direction;
		dot >= 0 && dot <= text->EndOfText(); 
			dot += direction){
		char c = text->Char(dot);
		if(c == open) opens++;
		else if(c == close){
			if(--opens == 0) {
				editor->Select(dot);
				editor->ScrollToSelection();
				return;
			}
		}
	}
	beep();
}

void Iedit::Include(){
	if(IsHelpWindow(this)) return;
	if(dialogActive) {beep();return;} dialogActive = true;
	fileDialog->SetTitle("Choose a file to Include:");
	if(PopUpDialog(fileDialog) == 1){
		Include(strdup(fileDialog->Choice()));
	}
	dialogActive = false;
}

boolean Iedit::ReadOnlyReport(){
	if(editor->ReadOnly()) {
		Report("Current edit buffer is Read Only","Sorry I cannot modify it");
		return true;
	}
	return false;
}

void Iedit::Include(const char* fname){
	if(IsHelpWindow(this)) return;
	if(ReadOnlyReport()) return;
	if(_mgr->GetTextBuffer()->Length() == 0){
		char* oldname = strdup(fileName->Text());
		Open(fname);
		fileName->Message(oldname);
		delete oldname;
	} else {
		struct stat filestats;
		stat(fname, &filestats);
		int req_size = round (filestats.st_size + _mgr->GetTextBuffer()->Length() + MINFREESIZE);
		FILE* f = fopen(fname, "r");
		if(f == nil) return;
		Message(Reading);
		if(req_size <= _mgr->BufferSize()){
			int size = round(filestats.st_size) + 1;
			char* buf = new char[size];
			if(buf == nil) return;
			char* b = buf;
			while(size > 1 && fgets(b,size, f) != nil){
				int len = strlen(b);
				b += len;
				size -= len;
			}
			editor->InsertText(buf, b-buf);
			editor->ScrollToSelection();
		} else {
			if(_mgr->Include(editor, f, round(filestats.st_size)) == false)
				Report("Run out of memory","Cannot Include");
	
		}
		fclose(f);
	}
	editor->ScrollToSelection();
	editor->Dirty(true);
	UpdateStatus(true);
}

void Iedit::Save(){
	if(IsHelpWindow(this)) return;
	if(dialogActive) {beep();return;} dialogActive = true;
	fileDialog->SetTitle("Save this file to:");
	if(PopUpDialog(fileDialog) == 1){
		Save(fileDialog->Choice());
	}
	dialogActive = false;
}

boolean Iedit::Save(const char* name){
	if(IsHelpWindow(this)) return false;
	struct stat sbuf;
	boolean isDir = false;
	if(!stat(name,&sbuf) && S_ISDIR(sbuf.st_mode)){
		isDir = true;
		errno = EISDIR;
	}
		
	if(*name && !isDir && access(name,W_OK) == 0 && access(name, F_OK) == 0 &&
	    Confirm(name,"Overwrite existing file?") != 'y')
		return true;
	if(*name == '\0' || isDir || !_mgr->Save(name)){ // Save failed
		int err = errno;
		char buf[128];
		buf[0] = '\0';
		if(errno)
			sprintf(buf,"%s: %s",name, sys_errlist[errno]);
		else if(*name == '\0')
			strcpy(buf,"No file name is specified");
		Report(buf, "Sorry. I cannot save it");
		if(err == EACCES) {
			editor->ReadOnly(true);
			UpdateStatus(true);
		}
		return false;
	}
	editor->ReadOnly(false);
	editor->Dirty(false);
	UpdateStatus(true);
	return true;
}

void Iedit::Close(){
	if(ConfirmSave()){
		if(views == 1){
			char ans = Confirm(
				"This is the last window of the editor",
				"Do you really want to leave now ?"
				
			);
			if(ans != 'y') return;
		}
		RemoveDialog(fileDialog);
		RemoveDialog(gotoDialog);
		RemoveDialog(searchDialog);
		if(KillThis && KillThis != this)
			delete KillThis;
		KillThis = this;
	}
}

void Iedit::Quit(){
	Close();
}

#include "version.h"

void debug(Iedit*){
}

void Iedit::Version(){
	char buf[128];
	strcpy(buf, author);
	strcat(buf, date);
	Report(versionNumber,buf);
	debug(this);
}

boolean Iedit::ConfirmSave(){
	if(! editor->Dirty()) return true;
	char ans = Confirm(
		"The contents of the file has been changed",
		"Save the file first ?"
	);
	if(ans == 'y') return Save(_mgr->GetFileName());
	return ans == 'n';
}

char Iedit::Confirm(const char* title, const char* subtitle){
	ConfirmDialog confirm(title, subtitle);
	InsertDialog(&confirm, Center);
	World::current()->flush();
	World::current()->RingBell(30);
	char ans = confirm.Confirm();
	RemoveDialog(&confirm);
	return ans;
}

void Iedit::Report(const char* title, const char* subtitle){
	AcknowledgeDialog report( title, subtitle);
	InsertDialog(&report,Center);
	World::current()->flush();
	World::current()->RingBell(30);
	report.Acknowledge();
	RemoveDialog(&report);
}

void Iedit::UpdateStatus(boolean modified){
	TextBuffer* text = _mgr->GetTextBuffer();
	if(modified) {
		sprintf(state," %c %s  %5d Lines  %5d Byes",
			(editor->Dirty() == true) ? '$' : ' ',
			(editor->ReadOnly() == true) ? "ReadOnly" : "Read / Write",
			text->LineNumber(text->EndOfText()),
			text->Length()
		);
		messageLine->Update();
	}
	sprintf(pos,"At: [%3d,%2d] Tab(%d) Wins(%d) ",
		text->LineNumber(editor->Dot())+1,
		text->LineOffset(editor->Dot())+1,
		editor->TabSize(), views
	);
	position->Update();
}

void Iedit::Message(const char* mesg){
	pos[0] = '\0';
	strncpy(state, mesg, 119);
	state[119] = '\0';
	messageLine->Update();
	position->Update();
	if(output) Flush();
}

boolean Iedit::PopUpDialog(Dialog* dialog){
	InsertDialog(dialog,Center);
	boolean accepted = dialog->Accept();
	RemoveDialog(dialog);
	return(accepted);
}

void Iedit::InsertDialog(Dialog* dialog){
	World* world = GetWorld();
	world->Remove(dialog); // multiple insertion of the same dialog disabled
	Event e;
	world->poll(e);
	Coord x = e.x, y = e.y;
	GetRelative(x, y, this);
	world->InsertTransient(dialog, this, x, y, Center);
}

void Iedit::InsertDialog(Dialog* dialog, Alignment align){
	World* world = GetWorld();
	world->Remove(dialog); // multiple insertion of the same dialog disabled
	Coord x, y ;
	Align(align, 0, 0, x, y);
	GetRelative(x, y, world);
	world->InsertTransient(dialog, this, x, y, Center);
}

void Iedit::RemoveDialog(Dialog* dialog){
	World* world = GetWorld();
	world->Remove(dialog);
}

static char* makeicrep(char* from, char* to){
	char c; char* prev = to;

	while(c = *from++){
		if(!isalpha(c)) *to++ = c;
		else {
			*to++ = '[';
			*to++ = tolower(c); *to++ = toupper(c);
			*to++ = ']';
		}
	}
	*to = 0;
	return prev;
}

void Iedit::Search(){
	if(dialogActive) {beep();return;}
	dialogActive = true;
	InsertDialog(searchDialog);
	boolean found = false;

	while(searchDialog->Accept()){
		char* textp 
			= (char*) searchDialog->GetStringEditor()->Text();
		char* ictext = new char[strlen(textp)*4+1];
		if(searchDialog->CaseIgnored())
			textp = makeicrep(textp, ictext);
		if(found = searchDialog->ForwardSearch()){
			int beg, end;
			if(searchDialog->IsReplace()) 
				beg = editor->Dot();
			else beg = editor->Mark();
			if(_mgr->FwdSearch(textp, beg, end)){
				editor->Select(beg, end);
			}
		} else {
			int beg = editor->Dot()-1, end;
			if(found = _mgr->BwdSearch(textp, beg, end)){
				editor->Select(beg, end);
			}
		}
		delete ictext;
		if(searchDialog->IsReplace() && found  && ! ReadOnlyReport()){
			editor->Dirty(true);
			editor->DeleteSelection();
			int dot = editor->Dot();
			const char* replaceText 
				= searchDialog->GetReplaceEditor()->Text();
			editor->InsertText(replaceText,strlen(replaceText));
			if(! searchDialog->ForwardSearch()){
				editor->Select(dot);
			}
			UpdateStatus(true);
		} else UpdateStatus(false);
		editor->ScrollToSelection();
	}
	RemoveDialog(searchDialog);
	dialogActive = false;
}

void Iedit::GotoLine() {
	if(dialogActive) {beep();return;}	dialogActive = true;
	SelStrEditor* gotoSE = gotoDialog->GetStringEditor();
	const char* cuttext = editor->CutBuffer();
	char buf[128]; buf[0] = '\0';
	if(cuttext)
	for (int i=0; i< 127 && (buf[i] = cuttext[i]); i++){
		if(!isdigit(cuttext[i])) {
			buf[i] = '\0';
			break;
		}
	}
	gotoSE->Message(buf);
	InsertDialog(gotoDialog);
	gotoSE->HandleChar(Ctrl('U'));
	boolean accepted = gotoDialog->Accept();
	RemoveDialog(gotoDialog);
	if(accepted){
		int line = atoi(gotoDialog->GetStringEditor()->Text());
		TextBuffer* text = _mgr->GetTextBuffer();
		if(line > 0){
			editor->Select(text->LineIndex(line-1));
			editor->ScrollToSelection();
		}
	}
	dialogActive = false;
}

static boolean State(ButtonState* bs){
	int value;
	bs->GetValue(value);
	return value == 1;
}

void Iedit :: SetWrap(){
	editor->Wrap((State(wrapState)));
}

void Iedit :: SetIndent(){
	editor->Indent((State(indentState)));
}

void Iedit :: SetWrap(boolean w){
	editor->Wrap(w);
	wrapState->SetValue(w ? 1 : 0);
}

void Iedit :: SetIndent(boolean i){
	editor->Indent(i);
	indentState->SetValue(i ? 1 : 0);
}

void Iedit :: TabSize_4(){
	editor->TabSize(4);
	UpdateStatus(false);
}

void Iedit :: TabSize_8(){
	editor->TabSize(8);
	UpdateStatus(false);
}

void Iedit :: _YouChooseTabSize(){
	if(PopUpDialog(tabDialog)){
		int tsz =  atoi(tabDialog->GetStringEditor()->Text());
		if(tsz > 0 && tsz <= 16)
			editor->TabSize(tsz);
		else beep();
		UpdateStatus(false);
	}
}

void Iedit :: changeFont(const char* font){
	editor->SetFont(font);
}
void Iedit :: _CourierMedium12(){
	changeFont( "*-courier-medium-r-*-120-*");
}
void Iedit :: _CourierBold12(){
	changeFont("*-courier-bold-r-*-120-*");
}

void Iedit :: _CourierMedium14(){
	changeFont( "*-courier-medium-r-*-140-*");
}
void Iedit :: _CourierBold14(){
	changeFont( "*-courier-bold-r-*-140-*");
}

void Iedit :: _CourierMedium18(){
	changeFont( "*-courier-medium-r-*-180-*");
}
void Iedit :: _CourierBold18(){
	changeFont("*-courier-bold-r-*-180-*");
}
