#ifndef FileManager_H
#define FileManager_H 1
/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>

static const int MINTEXTSIZE = 20480;
static const int MINFREESIZE = 4096;

class FileManager {
private:
	TextBuffer* _text;
	char* _buffer;
	int _size;
	char* filename;
public:
	FileManager();
	~FileManager();
	class TextBuffer* GetTextBuffer();
	boolean Open(const char* filename); /* Read Only == true; */
	boolean Save(const char* filename);
	const char* GetFileName();
	int BufferSize();
	boolean FwdSearch(const char *, int&beg, int& end);
	boolean BwdSearch(const char *, int&beg, int& end);
	void CheckBuffer(class SelEditor* editor, int inc);
	boolean Include(class SelEditor* editor, FILE* f, int inc);
};

inline int FileManager::BufferSize() { return _size; }
inline TextBuffer* FileManager::GetTextBuffer(){ return _text; }
inline const char* FileManager::GetFileName() { return filename; }

#endif