/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 * Copyright (c) 1991 Yufan Hu (yufan@cs.keele.ac.uk)
 *		Dept. of Computer Science, University of Keele, UK
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS OR KEELE UNIVERSITY OR
 * Yufan Hu BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF 
 * USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, 
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE 
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/* A monster which will look after the Cut Text
 for all objects
*/

/* xselection.c */

#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>

#include <InterViews/boolean.h>
#include <InterViews/interactor.h>
#include <InterViews/iwindow.h>
#include <InterViews/scene.h>
#include <InterViews/world.h>
#include <InterViews/event.h>
#include <IV-X11/xwindow.h>
#include <IV-X11/xevent.h>
//#include <IV-X11/worldrep.h>
//#include <IV-X11/eventrep.h>
//#include <IV-X11/windowrep.h>
//#include <IV-X11/canvasrep.h>

#include "xselection.h"

static const char* PRIMARY = "PRIMARY";
//static const char* CUT_BUFFER = "CUT_BUFFER0";
static char* current_selection = nil;
static Atom selection_atom = nil;
//static Atom cut_buffer_atom = nil;

static XDisplay* xdisplay=nil;
static Interactor* owner = nil;

static void send_selection(XEvent* ptr_event);
static void own_selection(XWindow window);
static char *get_selection(XWindow, Atom);

static XWindow xWindow(Interactor* w){
	Canvas* canvas = w->GetCanvas();
	Window* window = canvas->window();
	if(!window) return nil;
	return window->rep()->xtoplevel_;

}

Xselection::Xselection(Interactor* w){
	attached = w;
	window = nil;
	if(!current_selection) current_selection = strdup("");
	Attach();
}

Xselection::~Xselection(){
	if(owner == attached) owner = nil;
}

void Xselection::Attach(){
	if(window || !attached) return;
	Canvas* canvas = attached->GetCanvas();
	if(!canvas) return;
	window = canvas->window(); 
	if(window == nil) return;
	if(!xdisplay)
		xdisplay = World::current()->display()->rep()->display_;

	if(!selection_atom)
		selection_atom = XInternAtom(xdisplay, PRIMARY,false);
//	if(!cut_buffer_atom)
//		cut_buffer_atom = XInternAtom(xdisplay, CUT_BUFFER,false);
}

void Xselection::Text(const char* text){
	Text(text, strlen(text));
}

void Xselection::Text(const char* text, int len ){
	if(len){
		Attach();
		if(!window) return;
		own_selection(xWindow(attached));
		owner = attached;
		delete current_selection;
		current_selection = new char[len+1];
		strncpy(current_selection, text, len);
		current_selection[len] = '\0';
	}
}

const char* Xselection::Text(){
	Attach();
	XWindow xwindow = xWindow(attached);
	if(!xwindow) return "";
	if(!owner) {
		delete current_selection;
		if(! get_selection(xwindow, selection_atom))
			;
//			get_seletion(xwindow, cut_buffer_atom);
	}
	return current_selection;
}

void Xselection::Handle(Event& e){
	XEvent& ev = e.rep()->xevent_;
	if(ev.type == SelectionRequest){
			send_selection(&ev);
	} else if(ev.type == SelectionClear){
			owner = nil;
	}
}

/* functions deal with X selections */
/* These functions are extracted and modified from Richard Hesketh's
 * xselection and here is the original copyright:
 */

/* 
 * Copyright 1990 Richard Hesketh / rlh2@ukc.ac.uk
 *                Computing Lab. University of Kent at Canterbury, UK
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of Richard Hesketh and The University of
 * Kent at Canterbury not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission.
 * Richard Hesketh and The University of Kent at Canterbury make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * Richard Hesketh AND THE UNIVERSITY OF KENT AT CANTERBURY DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL Richard Hesketh OR THE
 * UNIVERSITY OF KENT AT CANTERBURY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Richard Hesketh / rlh2@ukc.ac.uk, 
 *                Computing Lab. University of Kent at Canterbury, UK
 */

#include <X11/Xatom.h>


/* Send a copy of the our owned selection to the requestor.
 * We do this by changing the contents of the property given in the
 * SelectionRequest event.
 */
static void send_selection(XEvent *ptr_event)
{
	XSelectionEvent notify_event;
	XSelectionRequestEvent  *req_event;
	unsigned char *data;
	XDisplay *dpy = xdisplay;

	data = (unsigned char *) current_selection;
	XChangeProperty(ptr_event->xselectionrequest.display,
			ptr_event->xselectionrequest.requestor,
			ptr_event->xselectionrequest.property,
			ptr_event->xselectionrequest.target,
			8, PropModeReplace, data,
			(data == nil) ? 0 :
			strlen((const char*)data));

	req_event = &(ptr_event->xselectionrequest);

	notify_event.type	  = SelectionNotify;
	notify_event.display   = req_event->display;
	notify_event.requestor = req_event->requestor;
	notify_event.selection = req_event->selection;
	notify_event.target	= req_event->target;
	notify_event.time	  = req_event->time;

	if (req_event->property == None)
			notify_event.property = req_event->target;
	else
			notify_event.property = req_event->property;

	/* tell the requestor he has a copy of the selection */
	(void)XSendEvent(req_event->display, req_event->requestor,
			False, 0, (XEvent *)&notify_event);
	XFlush(dpy);
}


/* tell the server we want to own the selection */
static void
own_selection(XWindow window)
{
	XSetSelectionOwner(xdisplay, selection_atom, window, CurrentTime);
//	XSetSelectionOwner(xdisplay, cut_buffer_atom, window, CurrentTime);
	XFlush(xdisplay);
}

static char *
get_selection(XWindow window, Atom atom){
	XEvent  event;
	static struct   timeval timeout = {3, 0};
	XDisplay* dpy = xdisplay;
	int server_fd_mask;
	extern int	  errno;
	Atom	actual_type;
	int	 actual_format;
//	Atom atom = selection_atom;
	unsigned long   nitems, bytes_after;
	char* data;
	/* tell the server we want a copy of the selection */
	XConvertSelection(dpy, atom, XA_STRING, XA_STRING, window, CurrentTime);
	server_fd_mask = (1 << ConnectionNumber(dpy));
	do {
		int	 result;

		do {
			if (XPending(dpy) > 0)
				result = 1;
			else {
				result = select(32, (fd_set *)&server_fd_mask,
						(fd_set *)NULL, (fd_set *)NULL,
						&timeout);
				if (result == -1 && errno != EINTR)
					result = 0;
			}
		} while (result == -1);

		if (result == 0)
			return current_selection = strdup("");

		XNextEvent(dpy, &event);
	} while (event.type != SelectionNotify);

	if (event.xselection.property == None)
		return current_selection = strdup("");

	XGetWindowProperty(dpy,
		event.xselection.requestor,
		event.xselection.property,
		0L, 40960L,
		False,
		AnyPropertyType,
		&actual_type,
		&actual_format,
		&nitems,
		&bytes_after,
		(unsigned char **)&data);
	if(data == nil)
		return current_selection = strdup("");
	current_selection = strdup(data);
	XFree(data);
	return current_selection;
}
