// -*- c++ -*-
// iperf: performance testor for interviews.
//        dave sternlicht mit X consortium

// interviews classes
#include <IV-look/kit.h>
//#include <InterViews/debug.h>
#include <InterViews/display.h>
#include <InterViews/margin.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>

// iperf classes
#include "tester.h"
#include "rect.h"
#include "seg.h"
#include "text.h"

#include <strings.h>
#include <stdio.h>
#include <stdlib.h>

const String  PERFGEOM = "600x600+0+0";
const Coord MARGIN_SIZE = 2.0;
const char* USESTRING = "Usage: iperf [-rect10 | -seg10 | -ftext] -repeat\n";

static PropertyData props[] = {
  { "*double_buffered", "off" }, // turn off double buffering for now...
  { "*dpi",    "72"},
  { "*repeat", "5"},
  { "*rect10", "off"},
  { "*seg10",  "off"},
  { "*ftext",  "off"},
  { nil }
};

static OptionDesc options[] = {
  {"-repeat", "*repeat", OptionValueNext},
  {"-rect10", "*rect10", OptionValueImplicit, "on"},
  {"-seg10",  "*seg10",  OptionValueImplicit, "on"},
  {"-ftext",  "*ftext",  OptionValueImplicit, "on"},
  { nil }
};

void usage() {
  fprintf(stderr, USESTRING);
  exit(1);
}

/* I want override-redirect and the geometry member of ManagedWindow
 * but I can't do this because Managed and Popup Window inherit
 * Window non-virtual!!!!
 *class ManagedPopupWindow : public ManagedWindow, 
 *public PopupWindow {
 *public:
 *  ManagedPopupWindow(Glyph* g) : ManagedWindow(g), PopupWindow(g) {};
 *};
 */

int main(int argc, char** argv) {
  Tester* tester;
  Session* session = new Session("Itest", argc, argv, options, props);
  Style* style = session->style();
  String s;
  style->find_attribute("repeat", s);
  int repeat;
  s.convert(repeat);
  if (style->value_is_on("rect10"))
    tester = new RectTester(session, repeat);
  else  if (style->value_is_on("seg10"))
    tester = new SegTester(session, repeat);
  else  if (style->value_is_on("ftext"))
    tester = new TextTester(session, repeat);
  else {
    usage();
  }
  Kit* kit = Kit::instance();
//  ManagedPopupWindow* w  = new ManagedPopupWindow(
  TransientWindow* w  = new TransientWindow(
    kit->outset_frame(
      new Margin(
//      new DebugGlyph(
	  tester, 
//	  "tester",
//	  DebugGlyph::trace_request|DebugGlyph::trace_allocate
//      ),
	MARGIN_SIZE
      ), 
      style
    )
  );
  w->transient_for(w);
  w->geometry(PERFGEOM);

  return session->run_window(w);
}

