// -*- c++ -*-
// SegTester subclass
// dave sternlicht mit X consortium

#include "seg.h"

SegTester::SegTester(Session* s, int repeat) : Tester(s, repeat) {
  objects_ = 1000;
}

const int LINE_SIZE = 10;
const int LINE_ROWS = 40;

void SegTester::proc(Canvas* c, int reps) const {
  int     size = LINE_SIZE;
  int     half = 1;
  int     h, i;
  int     rows;	          /* Number of rows filled in current column     */
  int     x, y;	          /* base of square to draw in		         */
  int     x1, y1, x2, y2; /* offsets into square			 */
  int     phase;	  /* how far into 0..8*size we are		 */
  int     phaseinc;       /* how much to increment phase at each segment */
  int     size8;	  /* 8 * size					 */
  const Color* fg = black_;

  size8 = 8 * size;
  half = (size + 19) / 20;
  
  /* All this x, x1, etc. stuff is to create a pattern that
     (1) scans down the screen vertically, with each new segment going
         into a square of size^2.
	
     (2) rotates the endpoints counter-clockwise around the square
	  
     (3) rotates by ``large'' steps if we aren't going to paint enough
	 segments to get full coverage
    */


  phaseinc = size8 / objects_;
  if (phaseinc == 0) phaseinc = 1;
  
  for (h = 0; h != reps; h++) {
      x     = half;  y     = HEIGHT - half;
      rows = 0;
      phase = 0;
    for (i = 0; i != objects_; i++) {    
      switch (phase / size) {
      case 0:
	x1 = 0;
	y2 = 0;
	x2 = size;
	y1 = phase;
	break;
      case 1:
	x1 = phase % size;    
	y2 = 0;
	x2 = size;
	y1 = size;
	break;
      case 2:
	x1 = size;
	y2 = 0;
	x2 = size - phase % size;
	y1 = size;
	break;
      case 3:
	x1 = size;
	y2 = phase % size;
	x2 = 0;
	y1 = size;
	break;
      case 4:
	x1 = size;
	y2 = size;
	x2 = 0;
	y1 = size - phase % size;
	break;
      case 5:
	x1 = size - phase % size;
	y2 = size;
	x2 = 0;
	y1 = 0;
	break;
      case 6:
	x1 = 0;
	y2 = size;
	x2 = phase % size;
	y1 = 0;
	break;
      case 7:
	x1 = 0;
	y2 = size - phase % size;
	x2 = size;
	y1 = 0;
	break;
      } /* end switch */
      
      c->new_path();
      c->move_to(x + x1, y + y1);
      c->line_to(x + x2, y + y2);
      c->close_path();
      c->stroke(fg, brush_);
      /* Change square to draw segment in */
      rows++;
      y -= size;
      if (y <= size + half || rows == LINE_ROWS) {
	/* Go to next column */
	rows = 0;
	y = HEIGHT - half;
	x += size;
	if (x >= WIDTH - size - half) {
	  x = half;
	}
      }
      
      /* Increment phase */
      phase += phaseinc;
      if (phase >= size8) phase -= size8;
    }      
    if (fg == black_)
      fg = gray_;
    else 
      fg = black_;
  }
};

void SegTester::reportTimes(double time, int n, boolean average) const {
  Tester::reportTimes(time, n, average);
  printf("10-pixel line segment\n");
}


