// -*- c++ -*-
// TextTester subclass
// dave sternlicht mit X consortium

#include "text.h"

Coord ypos;

TextTester::TextTester(Session* s, int repeat) : Tester(s, repeat) {
}

void TextTester::init(Canvas* c, int reps) {
  c->fill_rect(alloc_->left(), alloc_->bottom(), alloc_->right(), 
	       alloc_->top(), gray_);  
  dpy_->flush();

  int		i, j;
  char	ch;
  objects_ = 80;
  font_ = new Font(FONT);
  ypos = XPOS;
  height_ = (font_->ascent() + font_->descent()) + 1;
  
  charsPerLine_ = CHARS_PER_LINE;
  charsPerLine_ = (charsPerLine_ + 3) & ~3;
  
  totalLines_ = '\177' - ' ' + 1; // 177 = DEL
  if (totalLines_ > reps) totalLines_ = reps; 
  
  charBuf_ = new CharStruct*[totalLines_];
  
  for (i = 0; i != totalLines_; i++) {
    charBuf_[i] = new CharStruct[charsPerLine_];
    ch = i + ' ';
    for (j = 0; j != charsPerLine_; j++) {
      charBuf_[i][j].c = ch;
      charBuf_[i][j].w = font_->width(ch);
      if (ch == '\177') ch = ' '; else ch++; 
    }
  }
}

static int line = 0;
static int startLine = 0;

void TextTester::proc(Canvas* c, int reps) const {
  int    i, j;
  Coord charx, chary;
  for (i = 0; i != reps; i++) {
    charx = 0.0; chary = 0.0;
    for (j=0; j<charsPerLine_; j++) { // character loop
      c->character(font_, charBuf_[line][j].c, charBuf_[line][j].w,
		   black_, charx, ypos);
      charx += charBuf_[line][j].w;
    }
    ypos -= height_;
    if (ypos < height_) {
      /* Wraparound to bottom of window */
      ypos = XPOS;
      startLine = (startLine + 17) % totalLines_;
      line = startLine;
    }
    line = (line + 1) % totalLines_;
  }
}

TextTester::~TextTester() {
  int i;
  for (i = 0; i != totalLines_; i++)
    delete[] charBuf_[i];
  delete[] charBuf_;
}

void TextTester::reportTimes(double time, int n, boolean average) const {
  Tester::reportTimes(time, n, average);
  printf("Char in 80-char line (6x13)\n");
}
