// This file is part of xplot.
// 
// Copyright (C) 1989 Rene' Seindal (seindal@diku.dk)
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


#include "panel.hh"

#include <InterViews/painter.h>
#include <InterViews/shape.h>
#include <InterViews/font.h>

const int pad = 3;

PanelBox::PanelBox()
{
    SetClassName("PanelBox");
    Propagate(false);
    Align(Center);
}

void PanelBox::Reconfig()
{
    VBox::Reconfig();
    shape->Rigid();
}


PanelButton::PanelButton(const char *t, ButtonState *s, void *v)
    : (t, s, v)
{
    SetClassName("PanelButton");
};
void PanelButton::Reconfig()
{
    MakeBackground();
    if (shape->Undefined()) {
	MakeShape();
	width = shape->width;
	height = shape->height;
	shape->width += 2 * pad;
	shape->height += 2 * pad;
	shape->Rigid(0, hfil, 0, 0);
    }
}

void PanelButton::Redraw(Coord, Coord, Coord, Coord)
{
    Refresh();
}

void PanelButton::Refresh()
{
    if (chosen || hit)
	output->SetColors(output->GetBgColor(), output->GetFgColor());

    output->ClearRect(canvas, 0, 0, xmax, ymax);
    output->Text(canvas, text, (xmax - width)/2, (ymax - height)/2);

    if (chosen || hit)
	output->SetColors(output->GetBgColor(), output->GetFgColor());

    if (!enabled) {
	grayout->FillRect(canvas, 0, 0, xmax, ymax);
    }
}



void ButtonReplot::Press() {
    ((PlotFrame *)value)->Replot();
}

void ButtonPrevious::Press() {
    ((PlotFrame *)value)->Previous();
}

void ButtonNext::Press() {
    ((PlotFrame *)value)->Next();
}

void ButtonNewView::Press() {
    ((PlotFrame *)value)->NewView();
}

void ButtonDeleteView::Press() {
    ((PlotFrame *)value)->DeleteView();
}

void ButtonDeleteAll::Press() {
    ((PlotFrame *)value)->DeleteAll();
}

void ButtonQuit::Press() {
    ((PlotFrame *)value)->Quit();
}
    
void ButtonNormalize::Press() {
    ((PlotFrame *)value)->Normalize();
}

void ButtonMagnify::Press() {
    ((PlotFrame *)value)->Magnify();
}

void ButtonOneToOne::Press() {
    ((PlotFrame *)value)->OneToOne();
}

