
/*
 * figure.h
 */

#ifndef figure_h
#define figure_h

#include <InterViews/monoglyph.h>
#include <InterViews/ptrhandler.h>
#include <InterViews/event.h>

class Brush;
class Color;
class Listener;
class FigureTool;

class Figure : public MonoGlyph, public PointerHandler {
public:
  virtual void press(Event&);  

  virtual void brush(const Brush* b);
  virtual void stroke(const Color* stroke);
  virtual void fill(const Color* fill);

  virtual void button(boolean, EventButton = Event::any);

protected:
    Figure(
       const Brush* brush, const Color* stroke, const Color* fill,
       boolean closed, boolean curved, int coords, 
       EventButton button = Event::none);
    virtual ~Figure();

  FigureTool* _tool;
private:
  Listener* _listener;
};

/*
 * Point
 */

class Point : public Figure {
public:
    Point(
       const Brush* brush =nil, const Color* stroke =nil, const Color*
fill =nil,
       Coord x =0.0, Coord y =0.0, EventButton button = Event::none);
protected:
    virtual ~Point();
};

/*
 * Line
 */

class Line : public Figure {
public:
    Line(
       const Brush* brush, const Color* stroke, const Color* fill,
       Coord x1, Coord y1, Coord x2, Coord y2, EventButton button =
Event::none);
protected:
    virtual ~Line ();
};

/*
 * Rectangle
 */

class Rectangle : public Figure {
public:
    Rectangle(const Brush* brush, const Color* stroke, const Color* fill,
	      Coord l, Coord b, Coord r, Coord t, EventButton button = Event::none);
protected:
    virtual ~Rectangle ();
};

/*
 * Circle
 */

class Circle : public Figure {
public:
    Circle(const Brush* brush, const Color* stroke, const Color* fill,
	   Coord x, Coord y, Coord r, EventButton button = Event::none);
protected:
    virtual ~Circle ();
};

/*
 * Ellipse
 */

class Ellipse : public Figure {
public:
    Ellipse(const Brush* brush, const Color* stroke, const Color* fill,
	    Coord x, Coord y, Coord rx, Coord ry, EventButton button = Event::none);
protected:
    virtual ~Ellipse ();
};

/*
 * Open_BSpline
 */

class Open_BSpline : public Figure {
public:
    Open_BSpline(const Brush* brush, const Color* stroke, const Color* fill,
		 Coord* x, Coord* y, int count, EventButton button = Event::none);
protected:
    virtual ~Open_BSpline ();
};

/*
 * Close_BSpline
 */

class Closed_BSpline : public Figure {
public:
    Closed_BSpline (const Brush* brush, const Color* stroke, const Color* fill,
		    Coord* x, Coord* y, int count, EventButton button = Event::none);
protected:
    virtual ~Closed_BSpline ();
};

/*
 * Polyline
 */

class Polyline : public Figure {
public:
    Polyline(const Brush* brush, const Color* stroke, const Color* fill,
	     Coord* x, Coord* y, int count, EventButton button = Event::none);
protected:
    virtual ~Polyline ();
};

/*
 * Polygon
 */

class Polygon : public Figure {
public:
    Polygon (const Brush* brush, const Color* stroke, const Color* fill,
	     Coord* x, Coord* y, int count, EventButton button = Event::none);
protected:
    virtual ~Polygon ();
};

#endif
