#include <Dispatch/rpchdr.h>
#include <Dispatch/rpcstream.h>
#include <Dispatch/rpcwriter.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////

class Printer : public RpcWriter {
public:
    Printer(boolean binary, boolean swapped);
    ~Printer();

    void send(char);
    void send(unsigned char);
    void send(short);
    void send(unsigned short);
    void send(int);
    void send(unsigned int);
    void send(long);
    void send(unsigned long);
    void send(float);
    void send(double);
    void send(char*);
    void send(unsigned char*);
    void flush();
    int good();
protected:
    enum {
	CHAR,
	U_CHAR,
	SHORT,
	U_SHORT,
	INT,
	U_INT,
	LONG,
	U_LONG,
	FLOAT,
	DOUBLE,
	STRING,
	U_STRING
    };
};

Printer::Printer(boolean binary, boolean swapped) : RpcWriter(
    "./.no-such-service.1", /* fatal = */false, /* binary = */true
) {
    server().attach(1);
    server().binary(binary);
    server().swapped(swapped);
}

Printer::~Printer() {}

void Printer::send(char c) {
    RpcHdr hdr(CHAR);
    server() << hdr << c;
}

void Printer::send(unsigned char uc) {
    RpcHdr hdr(U_CHAR);
    server() << hdr << uc;
}

void Printer::send(short s) {
    RpcHdr hdr(SHORT);
    server() << hdr << s;
}

void Printer::send(unsigned short us) {
    RpcHdr hdr(U_SHORT);
    server() << hdr << us;
}

void Printer::send(int i) {
    RpcHdr hdr(INT);
    server() << hdr << i;
}

void Printer::send(unsigned int ui) {
    RpcHdr hdr(U_INT);
    server() << hdr << ui;
}

void Printer::send(long l) {
    RpcHdr hdr(LONG);
    server() << hdr << l;
}

void Printer::send(unsigned long ul) {
    RpcHdr hdr(U_LONG);
    server() << hdr << ul;
}

void Printer::send(float f) {
    RpcHdr hdr(FLOAT);
    server() << hdr << f;
}

void Printer::send(double d) {
    RpcHdr hdr(DOUBLE);
    server() << hdr << d;
}

void Printer::send(char* p) {
    RpcHdr hdr(STRING);
    server() << hdr << p;
}

void Printer::send(unsigned char* up) {
    RpcHdr hdr(U_STRING);
    server() << hdr << up;
}

void Printer::flush() {
    server().flush();
}

int Printer::good() {
    return server().good();
}

//////////////////////////////////////////////////////////////////////

int main(int argc, char** argv) {
    boolean binary = (argc > 1 && argv[1][0] == 'b');
    boolean swapped = (argc > 2 && argv[2][0] == 's');

    Printer printer(binary, swapped);

    char cmd[6];
    while (cout << "> " && cin >> cmd) {
	if (strcmp(cmd, "c") == 0) {
	    char c;
	    cin >> c;
	    printer.send(c);
	}
	else if (strcmp(cmd, "uc") == 0) {
	    unsigned char uc;
	    cin >> uc;
	    printer.send(uc);
	}
	else if (strcmp(cmd, "s") == 0) {
	    short s;
	    cin >> s;
	    printer.send(s);
	}
	else if (strcmp(cmd, "us") == 0) {
	    unsigned short us;
	    cin >> us;
	    printer.send(us);
	}
	else if (strcmp(cmd, "i") == 0) {
	    int i;
	    cin >> i;
	    printer.send(i);
	}
	else if (strcmp(cmd, "ui") == 0) {
	    unsigned int ui;
	    cin >> ui;
	    printer.send(ui);
	}
	else if (strcmp(cmd, "l") == 0) {
	    long l;
	    cin >> l;
	    printer.send(l);
	}
	else if (strcmp(cmd, "ul") == 0) {
	    unsigned long ul;
	    cin >> ul;
	    printer.send(ul);
	}
	else if (strcmp(cmd, "f") == 0) {
	    float f;
	    cin >> f;
	    printer.send(f);
	}
	else if (strcmp(cmd, "d") == 0) {
	    double d;
	    cin >> d;
	    printer.send(d);
	}
	else if (strcmp(cmd, "p") == 0) {
	    char p[80];
	    cin.getline(p, sizeof(p), '\n');
	    printer.send(p);
	}
	else if (strcmp(cmd, "up") == 0) {
	    unsigned char up[80];
	    cin.getline(up, sizeof(up), '\n');
	    printer.send(up);
	}
	else if (strcmp(cmd, "flush") == 0) {
	    printer.flush();
	}

	if (!printer.good()) {
	    cout << "printer eof or failure" << endl;
	}
    }

    return 0;
}
