#include <Dispatch/rpchdr.h>
#include <Dispatch/rpcreader.h>
#include <Dispatch/rpcservice.h>
#include <Dispatch/rpcstream.h>
#include <osfcn.h>
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////

class Reader : public RpcReader {
public:
    Reader(int fd, RpcService* service);
    virtual ~Reader();
protected:
    virtual void connectionClosed(int);
protected:
    RpcService* _service;

    enum {
	CHAR,
	U_CHAR,
	SHORT,
	U_SHORT,
	INT,
	U_INT,
	LONG,
	U_LONG,
	FLOAT,
	DOUBLE,
	STRING,
	U_STRING,
	NFCNS
    };
    static void receive_CHAR(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_U_CHAR(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_SHORT(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_U_SHORT(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_INT(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_U_INT(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_LONG(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_U_LONG(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_FLOAT(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_DOUBLE(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_STRING(RpcReader*, RpcHdr&, rpcstream&);
    static void receive_U_STRING(RpcReader*, RpcHdr&, rpcstream&);
};

Reader::Reader(int fd, RpcService* service) : RpcReader(
    fd, NFCNS, /* binary = */true
),
    _service(service)
{
    _function[CHAR] = &Reader::receive_CHAR;
    _function[U_CHAR] = &Reader::receive_U_CHAR;
    _function[SHORT] = &Reader::receive_SHORT;
    _function[U_SHORT] = &Reader::receive_U_SHORT;
    _function[INT] = &Reader::receive_INT;
    _function[U_INT] = &Reader::receive_U_INT;
    _function[LONG] = &Reader::receive_LONG;
    _function[U_LONG] = &Reader::receive_U_LONG;
    _function[FLOAT] = &Reader::receive_FLOAT;
    _function[DOUBLE] = &Reader::receive_DOUBLE;
    _function[STRING] = &Reader::receive_STRING;
    _function[U_STRING] = &Reader::receive_U_STRING;
}

Reader::~Reader() {}

void Reader::connectionClosed(int fd) {
    _service->quitRunning();
    delete this;
    close(fd);
}

void Reader::receive_CHAR(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    char c;
    client >> c;
    cout.width(3);
    cout << hdr.ndata() << ' ' << c << endl;
}

void Reader::receive_U_CHAR(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    unsigned char uc;
    client >> uc;
    cout.width(3);
    cout << hdr.ndata() << ' ' << uc << endl;
}

void Reader::receive_SHORT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    short s;
    client >> s;
    cout.width(3);
    cout << hdr.ndata() << ' ' << s << endl;
}

void Reader::receive_U_SHORT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    unsigned short us;
    client >> us;
    cout.width(3);
    cout << hdr.ndata() << ' ' << us << endl;
}

void Reader::receive_INT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    int i;
    client >> i;
    cout.width(3);
    cout << hdr.ndata() << ' ' << i << endl;
}

void Reader::receive_U_INT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    unsigned int ui;
    client >> ui;
    cout.width(3);
    cout << hdr.ndata() << ' ' << ui << endl;
}

void Reader::receive_LONG(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    long l;
    client >> l;
    cout.width(3);
    cout << hdr.ndata() << ' ' << l << endl;
}

void Reader::receive_U_LONG(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    unsigned long ul;
    client >> ul;
    cout.width(3);
    cout << hdr.ndata() << ' ' << ul << endl;
}

void Reader::receive_FLOAT(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    float f;
    client >> f;
    cout.width(3);
    cout << hdr.ndata() << ' ' << f << endl;
}

void Reader::receive_DOUBLE(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    double d;
    client >> d;
    cout.width(3);
    cout << hdr.ndata() << ' ' << d << endl;
}

void Reader::receive_STRING(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    char p[80];
    client >> p;
    cout.width(3);
    cout << hdr.ndata() << ' ' << p << endl;
}

void Reader::receive_U_STRING(RpcReader*, RpcHdr& hdr, rpcstream& client) {
    unsigned char up[80];
    client >> up;
    cout.width(3);
    cout << hdr.ndata() << ' ' << (char*)up << endl;
}

//////////////////////////////////////////////////////////////////////

class Service : public RpcService {
public:
    Service(const char* path);
    virtual ~Service();
protected:
    virtual void createReader(int fd);
};

Service::Service(const char* path) : RpcService(path) {}

Service::~Service() {}

void Service::createReader(int fd) {
    new Reader(fd, this);
}

//////////////////////////////////////////////////////////////////////

int main(int argc, char** argv) {
    const char* path = argc > 1 ? argv[1] : "./.reader.1";

    Service service(path);
    service.run();

    return 0;
}
