/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef ruiBox_h
#define ruiBox_h

#include <InterViews/box.h>
#include <InterViews/geometry.h>

#include <rutArray.h>
#include <ruiKnob.h>

class Canvas;
class Allocation;
class Extension;
class Hit;
class Event;
class ruiTile;

rutArrayDefine(ruiBoxKnobArray,ruiKnob)
rutArrayDefine(ruiBoxKnobAllocArray,Allocation)
 
class ruiBox : public Box , public ruiKnobClient {
public:
  ruiBox(DimensionName dim, GlyphIndex size);
  ruiBox(DimensionName dim, 
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil,
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil
  );
  virtual ~ruiBox();

  virtual void allocate(Canvas*, const Allocation&, Extension&);
  virtual void draw(Canvas*, const Allocation&) const;
  virtual void pick(Canvas*, const Allocation&, int depth, Hit&);

  virtual void append(Glyph*);
  virtual void prepend(Glyph*);
  virtual void insert(GlyphIndex, Glyph*);
  virtual void remove(GlyphIndex);
  virtual void replace(GlyphIndex, Glyph*);
  virtual void change(GlyphIndex);


  void knobsOn();
  void knobsOff();
  boolean isKnobsOn();

  void pressKnob(ruiKnob* knob, Event& event);
  void dragKnob(ruiKnob* knob, Event& event);
  void commitKnob(ruiKnob* knob, Event& event);
  void abortKnob(ruiKnob* knob, Event& event);

private:
  DimensionName dim_;
  ruiTile* tile_;
  boolean knobsOn_;
  ruiBoxKnobArray knobs_;
  ruiBoxKnobAllocArray knobAllocs_;
  boolean ridgeOn_;
  Coord ridgeX_;
  Coord ridgeY_;
  ruiKnob* ridge_;
  Allocation ridgeAlloc_;

  // used when moving knobs
  int knobIndex_;
  Coord knobComponentSize_;
  Coord knobPressOriginX_;
  Coord knobPressOriginY_;
  Coord knobPressX_;
  Coord knobPressY_;
  Coord knobXmin_;
  Coord knobXmax_;
  Coord knobYmin_;
  Coord knobYmax_;

  ruiKnob* newKnob(Coord xspan, Coord yspan);
  ruiKnob* newRidge();
  void allocateRidge();

  boolean pickKnobs(Canvas* canvas, const Allocation& allocation,
    int depth, Hit& hit);
  void allocateKnobs(); 
  void drawKnobs(Canvas* canvas, const Allocation& allocation) const;
  void drawRidge() const;

  int indexOfKnob(ruiKnob* knob);
  void dragKnobXY(ruiKnob* knob, Coord x, Coord y);
  // move alloc[i-1].right and alloc[i].left to the right by d
  //   relative to their initial positions
  void changeKnobAlloc(int i, Coord d);
  void changeRidgeAllocation();

  void damageKnob(int i);
  void damageRidge();

  Canvas* canvas_;
  Allocation allocation_;
  Extension extension_;

};

class ruiHorzBox : public ruiBox {
public:
  ruiHorzBox(GlyphIndex size);
  ruiHorzBox(
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil,
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil
  );
  virtual ~ruiHorzBox();
};

class ruiVertBox : public ruiBox {
public:
  ruiVertBox(GlyphIndex size);
  ruiVertBox(
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil,
    Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil, Glyph* =nil
  );
  virtual ~ruiVertBox();
};

#endif
