/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#include <string.h>

#include <iostream.h>
#include <iomanip.h>

#include <InterViews/canvas.h>
#include <InterViews/color.h>
#include <InterViews/glyph.h>
#include <InterViews/brush.h>
#include <InterViews/coord.h>
#include <InterViews/ptrhandler.h>
#include <InterViews/listener.h>
#include <InterViews/event.h>
#include <InterViews/fixedspan.h>
#include <InterViews/glue.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <IV-look/kit.h>

#include <ruiKnob.h>

///////////
// Knob //
///////////

ruiKnob::ruiKnob(ruiKnobClient* client, 
    Coord xspan, Coord yspan) {
  client_ = client;
  button(true);
  target(this);
  knobOn_ = false;
  rx_ = xspan;
  ry_ = yspan;
  /*
  constrainX_ = constrainX;
  constrainY_ = constrainY;
  if (constrainX_) {
    rx_ = 8;
  } else {
    rx_ = 4;
  }
  if (constrainY_) {
    ry_ = 8;
  } else {
    ry_ = 4;
  }
  */
  Kit* kit = Kit::instance();
  Style* style = Session::instance()->style();
  FixedSpan* spanX = new FixedSpan(nil, Dimension_X, rx_);
  FixedSpan* spanY = new FixedSpan(spanX, Dimension_Y, ry_);
  Glyph* frame = kit->outset_frame(spanY, style);
  body(frame);
}

ruiKnob::~ruiKnob() {
}

void ruiKnob::request(Requisition& requisition) const {
  MonoGlyph::request(requisition);
}

void ruiKnob::allocate(Canvas* canvas, const Allocation& allocation,
  Extension& extension) {
  MonoGlyph::allocate(canvas,allocation,extension);
}
 
void ruiKnob::draw(
  Canvas* canvas, 
  const Allocation& allocation
) const {
  MonoGlyph::draw(canvas,allocation);
}

void ruiKnob::press(Event& event) {
  client_->pressKnob(this, event);
  PointerHandler::press(event);
};

void ruiKnob::drag(Event& event) {
  client_->dragKnob(this, event);
};

void ruiKnob::commit(Event& event) {
  client_->commitKnob(this, event);
  PointerHandler::commit(event);
};

void ruiKnob::abort(Event& event) {
  client_->abortKnob(this, event);
  PointerHandler::abort(event);
};

void ruiKnob::knobOn() { knobOn_ = true; }
void ruiKnob::knobOff() { knobOn_ = false; }
boolean ruiKnob::isOn() const { return knobOn_; }
