/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef ruiPanner_h
#define ruiPanner_h

#include <InterViews/handler.h>
#include <InterViews/listener.h>

#include <IV-look/adjuster.h>

class Style;

typedef unsigned ruiPannerDirection;

class ruiPanner : public Listener, public Adjuster, public Handler {
public:
    enum {leftToRight = 1, rightToLeft = 0, bottomToTop = 2, topToBottom = 0};

    ruiPanner(Adjustable* x, Adjustable* y, Style*, ruiPannerDirection d =
      leftToRight | bottomToTop);
    virtual ~ruiPanner();

    virtual void request(Requisition&) const;
    virtual void allocate(Canvas*, const Allocation&, Extension&);
    virtual void draw(Canvas*, const Allocation&) const;

    virtual void update(DimensionName);
    virtual void update_all();

    virtual void event(Event&);

protected:
    void damage_thumb();

    Adjustable* y_adjust_;
    Glyph* thumb_;
    Canvas* canvas_;
    Allocation allocation_;
    Allocation thumb_allocation_;
    float xscale_, yscale_;
    ruiPannerDirection direction_;
};

#endif
