/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef ruiSheet_h
#define ruiSheet_h

#include <InterViews/action.h>
#include <InterViews/handler.h>
#include <InterViews/monoglyph.h>

#include <rutArray.h>
#include <rutString.h>

class TBBox;
class Window;

class ruiSheetItem;
rutArrayDefine(ruiSheetItemArray,ruiSheetItem)

class ruiSheetBase {
public:
  virtual ~ruiSheetBase();

  virtual int addItem(rutString label, rutString value) = 0;
  virtual void getValue(unsigned index, rutString& value) = 0;

  virtual int addItem(rutString label, float value) = 0;
  virtual void getValue(unsigned index, float& value) = 0;

  virtual int addItem(rutString label, int value) = 0;
  virtual void getValue(unsigned index, int& value) = 0;

protected:
  ruiSheetBase();
};

class ruiSheetGlyph : public MonoGlyph, public ruiSheetBase {
public:
  ruiSheetGlyph();
  ~ruiSheetGlyph();

  int addItem(rutString label, rutString value);
  void getValue(unsigned index, rutString& value);

  int addItem(rutString label, float value);
  void getValue(unsigned index, float& value);

  int addItem(rutString label, int value);
  void getValue(unsigned index, int& value);

  boolean ok();

protected:
  virtual void okAction();
  virtual void cancelAction();

  TBBox* box_;
  ruiSheetItemArray item_;
  boolean ok_;
};

declare(ActionCallback, ruiSheetGlyph)

class ruiSheetDialog {
public:
  ruiSheetDialog();
  virtual ~ruiSheetDialog();

  void dialog(Window* window);

  void done(boolean done);

protected:
  boolean done_;
};

class ruiSheet : public ruiSheetBase, public ruiSheetDialog {
public:
  ruiSheet();
  ~ruiSheet();

  int addItem(rutString label, rutString value);
  void getValue(unsigned index, rutString& value);

  int addItem(rutString label, float value);
  void getValue(unsigned index, float& value);

  int addItem(rutString label, int value);
  void getValue(unsigned index, int& value);

  boolean popup();

protected:
  class ruiSheetBody* glyph_;
};

#endif
