/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef ruiTile_h
#define ruiTile_h

#include <InterViews/layout.h>
#include <InterViews/geometry.h>

#include <rutArray.h>

class ruiTileInfo {
public:
  ruiTileInfo();
  void reset();
  Coord natural;
  Coord min;
  Coord max;
  Coord current;
  float alignment;
  boolean allocated;
};

rutArrayDefine(ruiTileInfoArray,ruiTileInfo);

class ruiTile : public Layout {
public:
    ruiTile(DimensionName);
    virtual ~ruiTile();

    virtual void request(
        GlyphIndex count, const Requisition*, Requisition& result
    );
    virtual void allocate(
        const Allocation& given, GlyphIndex count, const Requisition*,
        Allocation* result
    );

    // true if border between i-1 and ith element can be moved
    boolean flexibleBorder (int i);

    // return distance from start of tiling to border 
    //   between i-1 and ith elements
    Coord getBorderOffset (int i);

    Coord getComponentSize (int i);
    // don't use this unless you are ruiBox
    void setComponentSize (int i, Coord newSize);

    void insert(int i);
    void insertLast();
    void remove(int i);
    void change(int i);

    void moveBorder(int i, Coord delta);

    boolean canGrow(int m);
    boolean canShrink(int m);

    void getSlacks(int i, 
      Coord& growBefore, Coord& growAfter,
      Coord& shrinkBefore, Coord& shrinkAfter);
 
protected:
    ruiTileInfoArray ainfo_;
    void computeInfo(GlyphIndex count, const Requisition* request);
    Coord computeCurrentSpan();
    Coord computeChangeableSize(boolean growing);
    int computeChangeableCount(boolean growing);
    void proportionalChange(Coord newSize);

    DimensionName dimension_;
    Requisition requisition_;
};

class ruiTileReversed : public ruiTile {
public:
    ruiTileReversed(DimensionName);

    virtual void allocate(
        const Allocation& given, GlyphIndex count, const Requisition*,
        Allocation* result
    );
};

#endif
