/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef rutArray_h
#define rutArray_h

#include <generic.h>

class rutArray {
protected:
  class rutArrayRef {
  public:
    rutArrayRef& operator=(void* entry);
    operator void*() const;
    void* operator->() const;
  protected:
    friend rutArray;
    rutArrayRef(rutArray& array, int index);
  private:
    rutArray& array_;
    int index_;
  };

public:
  rutArray(int limit = 0);
  rutArray(const rutArray& array);
  virtual ~rutArray();

  rutArrayRef operator[](int i);

  int size() const;
  int equal(rutArray& const array) const;
  int indexOf(void* entry) const;
  void* get(int i) const;
  void* getLast() const;

  void put(void* entry, int i);
  void insert(void* entry, int i);
  void insertLast(void* entry);
  void* remove(int i);
  void* removeLast();
  int removeEntry(void* entry);
  void reverse();
  void reset();

protected:
  void invariants() const;
  void newUpperBound(int i);
  void** buffer_;
  int limit_;
  int size_;
};

#define rutArrayDefine(name, type) \
class name : public rutArray { \
  class name2(rutArrayRef,type) { \
  public: \
    name2(rutArrayRef,type)& operator=(type* entry) { \
      array_.put(entry, index_); \
      return *this; \
    } \
    operator type*() const {return (type*) array_.get(index_);} \
    type* operator->() const {return (type*) array_.get(index_);} \
  protected: \
    friend name; \
    name2(rutArrayRef,type)(name& array, int index) : \
      array_(array), index_(index) {} \
  private: \
    name& array_; \
    int index_; \
  }; \
public: \
  name(int limit = 0) : rutArray(limit) {} \
  name(const name& array) : rutArray(array) {} \
  name2(rutArrayRef,type) operator[](int i) { \
    return name2(rutArrayRef,type)(*(name*)this, i);} \
  int equal(name& const array) const {return rutArray::equal(array);} \
  int indexOf(type* entry) const {return rutArray::indexOf(entry);} \
  type* get(int i) const {return (type*) rutArray::get(i);} \
  type* getLast() const {return (type*) rutArray::getLast();} \
  void put(type* entry, int i) {rutArray::put(entry, i);} \
  void insert(type* entry, int i) {rutArray::insert(entry, i);} \
  void insertLast(type* entry) {rutArray::insertLast(entry);} \
  type* remove(int i) {return (type*) rutArray::remove(i);} \
  type* removeLast() {return (type*) rutArray::removeLast();} \
  int removeEntry(type* entry) {return rutArray::removeEntry(entry);} \
};

#endif
