/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#include <r.h>
#include <rutBase.h>

/////////////////////
// Base Descriptor //
/////////////////////

rutBaseDescriptor* rutBaseDescriptor::head_ = 0;

rutBaseDescriptor::rutBaseDescriptor() {
  total_ = 0;
  count_ = 0;

  next_ = rutBaseDescriptor::head_;
  rutBaseDescriptor::head_ = this;
}

rutBaseDescriptor::~rutBaseDescriptor() {
  rFail("rutBaseDescriptor destroyed!");
}

int rutBaseDescriptor::total() {
  return total_;
}

int rutBaseDescriptor::count() {
  return count_;
}

void rutBaseDescriptor::increment() {
  ++total_;
  ++count_;
}

void rutBaseDescriptor::decrement() {
  --count_;
}

rutBaseDescriptor* rutBaseDescriptor::head() {
  return head_;
}

rutBaseDescriptor* rutBaseDescriptor::next() {
  return next_;
}

//////////
// Base //
//////////

rutBase::rutBase() {
}

rutBase::~rutBase() {
}

void rutBase::debugPrint(ostream& out) const {
  rutBaseDescriptor* desc = descriptor();
  out << desc->name() << " - size: " << desc->size() << ", total: " <<
    desc->total() << ", count: " << desc->count() << endl;
}

void* rutBase::downcastVirtual(const rutBaseDescriptor*) const {
  return 0;
}
