/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

%(

#include <rutLex.h>

class rutLexParseBase {
protected:
  rutLexRegExpNode* expression_;
};

%}

%base rutLexParseBase

%{

#include <iostream.h>
#include <rutLex.h>
#define yyoverflow

%}

%left eConcat
%left sConcat
%nonassoc openClosure openGroup openSet
%left eUnion
%nonassoc zeroOrMore oneOrMore zeroOrOne
%left teConcat
%token openTag closeTag closeClosure closureRange zeroOrMore
%token oneOrMore zeroOrOne closeGroup closeSet notSet
%token setRange

%union {
  rutLexRegExpNode* node;
  rutLexSet* set;
  char character;
  int integer;
}

%token <set> any
%token <character> alpha digit other

%type <node> TE E
%type <set> S
%type <character> Other
%type <integer> Number

%%

RE:
  TE
    { expression_ = $1 } |
  E
    { expression_ = $1 } ;

/* ignore tags for now */
TE:
  openTag E closeTag
    { $$ = $2 } |
  TE E %prec teConcat
    { $$ = new rutLexConcatNode($1, $2) } |
  E TE %prec teConcat
    { $$ = new rutLexConcatNode($1, $2) } ;

E:
  E openClosure Number closeClosure
    { $$ = new rutLexExactClosureNode($1, $3) } |
  E openClosure Number closureRange closeClosure
    { $$ = new rutLexOpenClosureNode($1, $3) } |
  E openClosure Number closureRange Number closeClosure
    { $$ = new rutLexRangeClosureNode($1, $3, $5) } |
  E zeroOrMore
    { $$ = new rutLexOpenClosureNode($1, 0) } |
  E oneOrMore
    { $$ = new rutLexOpenClosureNode($1, 1) } |
  E zeroOrOne
    { $$ = new rutLexRangeClosureNode($1, 0, 1) } |
  E E %prec eConcat
    { $$ = new rutLexConcatNode($1, $2) } |
  E eUnion E
    { $$ = new rutLexUnionNode($1, $3) }|
  openGroup E closeGroup
    { $$ = $2 } |
  openSet S closeSet
    { $$ = new rutLexCharClassNode(*$2) } |
  openSet notSet S closeSet
    { $3->invert(); $$ = new rutLexCharClassNode(*$3) } |
  any
    { rutLexSet set; set.invert(); set.remove('\n');
      $$ = new rutLexCharClassNode(set); } |
  Other
    { $$ = new rutLexCharNode($1) } ;

S:
  S S %prec sConcat
    { $$ = $1; $1->add($2); delete $2 } |
  alpha setRange alpha
    { $$ = new rutLexSet; $$->add($1, $3) } |
  digit setRange digit
    { $$ = new rutLexSet; $$->add($1, $3) } |
  Other
    { $$ = new rutLexSet($1) } ;

Other:
  other |
  alpha |
  digit ;

Number:
  Number digit
    { $$ = ($1 * 10) + ($2 - '0') } |
  digit
    { $$ = $1 - '0' } ;

%%

void rutLexParse::yyerror(char* error) {
  cout << error << endl;
}

int rutLexParse::yylex() {
  return 0;
}
