/*
 * Copyright (c) 1991 Redwood Design Automation
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Redwood Design Automation may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission of
 * Redwood Design Automation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL REDWOOD DESIGN AUTOMATION BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifndef rutText_h
#define rutText_h

#include <rutString.h>

class ostream;
class rutStringArray;
class rutTextLocation;
class rutTextRegion;

typedef unsigned int rutTextKind;

class rutTextIterator {
public:
  rutTextIterator();
  virtual ~rutTextIterator();

  int next(char& ch);
  int prev();

  unsigned line() const;
  unsigned column() const;

protected:
  friend rutText;
  rutTextIterator(rutText* text, unsigned line, unsigned column);

  rutText* text_;
  unsigned line_;
  unsigned column_;
};

class rutText {
public:
  enum {linear, rectangular};

  rutText();
  virtual ~rutText();

  unsigned lines() const;
  const rutString& getNth(unsigned line) const;

  rutTextIterator iterator(const rutTextLocation& location) const;
  rutTextRegion search(const rutTextLocation& location, const rutLexRegExp&
    regExp) const;

  void clear();

  void write(const rutString& string);
  void write(char* buffer, unsigned count);

  rutText* cut(const rutTextRegion& region);
  rutText* copy(const rutTextRegion& region);
  void paste(const rutTextLocation& location, rutText* textPaste);

  friend ostream& operator<<(ostream& out, rutText& text);

protected:
  friend class rutTextIterator;
  friend class rutTextOperatorPaste;
  friend class rutTextOperatorCut;
  friend class rutTextOperatorApply;
  friend class rutTextOperatorCopy;

  rutString& get(unsigned line) const;
  rutString& getExpand(unsigned line);
  rutString getSubstring(unsigned line, unsigned column1, unsigned column2);
  rutString getSubstringEnd(unsigned line, unsigned column);

  virtual void add(rutString string);
  virtual void addNth(rutString string, unsigned line);

  virtual rutString remove();
  virtual rutString removeNth(unsigned line);

protected:
  rutStringArray* lines_;
};

class rutTextLocation {
public:
  rutTextLocation(unsigned line = 0, unsigned column = 0, rutTextKind kind =
    rutText::linear);
  virtual ~rutTextLocation();

  void kind(rutTextKind kind);
  rutTextKind kind() const;

  void line(unsigned line);
  unsigned line() const;

  void column(unsigned column);
  unsigned column() const;

protected:
  friend rutText;

  rutTextKind kind_;
  unsigned line_;
  unsigned column_;
};

class rutTextRegion {
public:
  rutTextRegion(unsigned line1 = 0, unsigned column1 = 0, unsigned line2 = 0,
    unsigned column2 = 0, rutTextKind kind = rutText::linear);
  virtual ~rutTextRegion();

  void kind(rutTextKind kind);
  rutTextKind kind() const;

  void line1(unsigned line);
  unsigned line1() const;

  void column1(unsigned column);
  unsigned column1() const;

  void line2(unsigned line);
  unsigned line2() const;

  void column2(unsigned column);
  unsigned column2() const;

  operator int();

protected:
  friend rutText;

  rutTextKind kind_;
  unsigned line1_;
  unsigned column1_;
  unsigned line2_;
  unsigned column2_;
};

class rutTextStreamBuffer : public rutText, public streambuf {
public:
  rutTextStreamBuffer();
  ~rutTextStreamBuffer();

  int overflow(int);
};

#endif
