/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Schematic capture system-specific commands.
 */

#ifndef scmcmds_h
#define scmcmds_h

#include <Unidraw/Commands/command.h>

class FileChooser;
class NetlistDialog;
class UChooser;

class AliasCmd : public Command {
public:
    AliasCmd(ControlInfo*, GraphicComp* = nil);
    AliasCmd(Editor* = nil, GraphicComp* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);

    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* AliasCmd::GetGraphicComp () { return _comp; }
inline void AliasCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

class NewToolCmd : public Command {
public:
    NewToolCmd(ControlInfo*);
    NewToolCmd(Editor* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void InitNewTool(const char*, boolean);
protected:
    FileChooser* _dialog;
};

class ToolsCmd : public Command {
public:
    ToolsCmd(ControlInfo*);
    ToolsCmd(Editor* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void InitDialog();
private:
    UChooser* _dialog;
};

class NewViewCmd : public Command {
public:
    NewViewCmd(ControlInfo*, GraphicComp* = nil);
    NewViewCmd(Editor* = nil, GraphicComp* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);

    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* NewViewCmd::GetGraphicComp () { return _comp; }
inline void NewViewCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

class DefinitionCmd : public Command {
public:
    DefinitionCmd(ControlInfo*, GraphicComp* = nil);
    DefinitionCmd(Editor* = nil, GraphicComp* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);

    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* DefinitionCmd::GetGraphicComp () { return _comp; }
inline void DefinitionCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

class ToggleDwgPaletteCmd : public Command {
public:
    ToggleDwgPaletteCmd(ControlInfo*, boolean visible = false);
    ToggleDwgPaletteCmd(Editor* = nil, boolean = false);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    boolean _visible;
};

class NetlistCmd : public Command {
public:
    NetlistCmd(ControlInfo*);
    NetlistCmd(Editor* = nil);
    virtual ~NetlistCmd();

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    NetlistDialog* _dialog;
};

class InfoCmd : public Command {
public:
    InfoCmd(ControlInfo*, GraphicComp* comp = nil);
    InfoCmd(Editor* = nil, GraphicComp* comp = nil);

    virtual void Execute();
    virtual boolean Reversible();

    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);

    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void Modify(GraphicComp*);
private:
    GraphicComp* _comp;
};

inline GraphicComp* InfoCmd::GetGraphicComp () { return _comp; }
inline void InfoCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

class TruthTableCmd : public Command {
public:
    TruthTableCmd(ControlInfo*, GraphicComp* comp = nil);
    TruthTableCmd(Editor* = nil, GraphicComp* comp = nil);

    virtual void Execute();
    virtual boolean Reversible();

    GraphicComp* GetGraphicComp();
    void SetGraphicComp(GraphicComp*);

    virtual Command* Copy();
    virtual void Read(istream&);
    virtual void Write(ostream&);
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    GraphicComp* _comp;
};

inline GraphicComp* TruthTableCmd::GetGraphicComp () { return _comp; }
inline void TruthTableCmd::SetGraphicComp (GraphicComp* comp) { _comp = comp; }

#endif
