/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * SchemConstruct implementation.
 */

#include "scmclasses.h"
#include "scmcmds.h"
#include "scmcomps.h"
#include "scmcreator.h"
#include "scmnet.h"
#include "scmtools.h"
#include "scmtransfns.h"
#include "scmvars.h"

#include <Unidraw/catalog.h>

/*****************************************************************************/

SchemCreator::SchemCreator () { }

void* SchemCreator::Create (
    ClassId id, istream& in, ObjectMap* objmap, int objid
) {
    switch (id) {
        case ALIAS_CMD:            CREATE(AliasCmd, in, objmap, objid);
        case ALIAS_COMP:           CREATE(AliasComp, in, objmap, objid);
        case BULB_COMP:            CREATE(BulbComp, in, objmap, objid);
        case DEFINITION_CMD:       CREATE(DefinitionCmd, in, objmap, objid);
        case ELEMENT_COMP:         CREATE(ElementComp, in, objmap, objid);
        case ELEMENT_VAR:          CREATE(ElementVar, in, objmap, objid);
        case INFO_CMD:             CREATE(InfoCmd, in, objmap, objid);
        case INSTANCE_COMP:        CREATE(InstanceComp, in, objmap, objid);
        case INSTANCE_TOOL:        CREATE(InstanceTool, in, objmap, objid);
        case EXAMINE_TOOL:         CREATE(ExamineTool, in, objmap, objid);
        case MANIPULATE_TOOL:      CREATE(ManipulateTool, in, objmap, objid);
        case NETLIST_CMD:          CREATE(NetlistCmd, in, objmap, objid);
        case NEWTOOL_CMD:          CREATE(NewToolCmd, in, objmap, objid);
        case NEWVIEW_CMD:          CREATE(NewViewCmd, in, objmap, objid);
        case NODE_COMP:            CREATE(NodeComp, in, objmap, objid);
        case NODE_VAR:             CREATE(NodeVar, in, objmap, objid);
        case SWITCH_COMP:          CREATE(SwitchComp, in, objmap, objid);
        case TF_TRUTHTABLE:        CREATE(TF_TruthTable, in, objmap, objid);
        case TOGGLEDWGPALETTE_CMD: CREATE(ToggleDwgPaletteCmd,in,objmap,objid);
        case TOOLS_CMD:            CREATE(ToolsCmd, in, objmap, objid);
        case TRUTHTABLE_CMD:       CREATE(TruthTableCmd, in, objmap, objid);
        case WIRE_COMP:            CREATE(WireComp, in, objmap, objid);

        default:    return Creator::Create(id, in, objmap, objid);
    }
}

void* SchemCreator::Create (ClassId id) {
    if (id == ALIAS_VIEW)          return new AliasView;
    if (id == BULB_VIEW)           return new BulbView;
    if (id == ELEMENT_VIEW)        return new ElementView;
    if (id == INSTANCE_VIEW)       return new ElementView;
    if (id == NL_ALIAS)            return new NLAlias;
    if (id == NL_ELEMENT)          return new NLElement;
    if (id == NL_GRAPHIC_COMPS)    return new NLGraphicComps;
    if (id == NL_NODE)             return new NLNode;
    if (id == NODE_VIEW)           return new NodeView;
    if (id == SWITCH_VIEW)         return new SwitchView;
    if (id == WIRE_VIEW)           return new WireView;

    return Creator::Create(id);
}
