/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Schematic capture system-specific dialog boxes.
 */

#ifndef scmdialogs_h
#define scmdialogs_h

#include <Unidraw/dialogs.h>
#include <InterViews/filechooser.h>

class ElementComp;
class ElementVarView;
class NodeVarView;
class TruthTable;
class TruthTableEditor;

class InfoDialog : public BasicDialog {
public:
    InfoDialog(Component*);
    virtual ~InfoDialog();

    virtual boolean Accept();
    virtual void Handle(Event&);
    void ApplyChanges();
private:
    void Init(Component*);
    Interactor* Interior();
    Interactor* VarViews(int);
    Interactor* ElemPart(int);
    Interactor* NodePart(int);
private:
    ElementVarView* _elemVarView;
    NodeVarView** _nodeVarView;
    int _nodes;
};

class NewToolDialog : public FileChooser {
public:
    NewToolDialog(const char* dir = "~", boolean instantiate = false);

    boolean Instantiating();
    const char* Name();
private:
    Interactor* Interior();
private:
    ButtonState* _instantiate;
};

class NetlistDialog : public FileChooser {
public:
    NetlistDialog(const char* dir = "~", boolean flatten = false);

    boolean Flattening();
    const char* Name();
private:
    Interactor* Interior();
private:
    ButtonState* _flatten;
};

class TruthTableDialog : public BasicDialog {
public:
    TruthTableDialog(TruthTable*);

    TruthTable* GetTruthTable();
    virtual boolean Accept();
    virtual void Handle(Event&);
private:
    Interactor* Interior();
    Interactor* IOLabels();
    Interactor* Editor();
private:
    TruthTable* _truthTable;
    TruthTableEditor* _truthEd;
};

inline TruthTable* TruthTableDialog::GetTruthTable () { return _truthTable; }

#endif
