/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Main class for schematic editor, derived from Editor class.
 */

#ifndef scmed_h
#define scmed_h

#include <Unidraw/editor.h>

class Command;
class CompNameVar;
class DrawingPalette;
class GraphicComp;
class GravityVar;
class MagnifVar;
class ModifStatusVar;
class PulldownMenu;
class ToolPalette;
class Tray;

class SchemEditor : public Editor {
public:
    SchemEditor(ToolPalette*, DrawingPalette*, GraphicComp* = nil);
    virtual ~SchemEditor();

    GraphicComp* GetGraphicComp();

    virtual Component* GetComponent();
    virtual Viewer* GetViewer(int = 0);
    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();
    virtual StateVar* GetState(const char*);

    virtual void SetComponent(Component*);
    virtual void SetViewer(Viewer*, int = 0);
    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);

    ToolPalette* GetToolPalette();
    DrawingPalette* GetDrawingPalette();
private:
    Interactor* Interior();
    void InitStateVars();
    void InitViewer();

    Interactor* Commands();

    void Include(Command*, PulldownMenu*);

    PulldownMenu* FileMenu();
    PulldownMenu* EditMenu();
    PulldownMenu* StructureMenu();
    PulldownMenu* AlignMenu();
    PulldownMenu* ViewMenu();
private:
    GraphicComp* _comp;
    KeyMap* _keymap;
    Viewer* _viewer;
    ToolPalette* _toolPal;
    DrawingPalette* _dwgPal;
    Tray* _tray;

    CompNameVar* _name;
    ModifStatusVar* _modifStatus;
    GravityVar* _gravity;
    MagnifVar* _magnif;
};

#endif
