/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Implementation of schematic capture system-specific manipulators.
 */

#include "scmmanips.h"
#include "scmrubs.h"

#include <Unidraw/viewer.h>
#include <Unidraw/Components/connector.h>
#include <Unidraw/Graphic/graphic.h>

#include <InterViews/event.h>
#include <InterViews/menu.h>
#include <InterViews/rubverts.h>

/*****************************************************************************/

PopupManip::PopupManip (Viewer* viewer, PopupMenu* popup, Tool* tool) {
    _viewer = viewer;
    _popup = popup;
    _tool = tool;
}

PopupManip::~PopupManip () { delete _popup; }
void PopupManip::Grasp (Event& e) { _popup->Popup(e); }
void PopupManip::Effect (Event&) { _popup->Unselect(); }

boolean PopupManip::Manipulating (Event& e) {
    e.target->Handle(e);
    return e.leftmouse;
}

void PopupManip::SetViewer (Viewer* viewer) { _viewer = viewer; }
void PopupManip::SetTool (Tool* tool) { _tool = tool; }
Viewer* PopupManip::GetViewer () { return _viewer; }
Tool* PopupManip::GetTool () { return _tool; }

/*****************************************************************************/

WireManip::WireManip (
    Viewer* v, RubberWire* rv, Transformer* t, Tool* tool
) : VertexManip(v, rv, t, tool) {
    _target = nil;
}

boolean WireManip::Manipulating (Event& e) {
    Rubberband* r = GetRubberband();

    if (e.eventType == MotionEvent) {
        Constrain(e);
        r->Track(e.x, e.y);

    } else if (e.eventType == DownEvent) {
        Constrain(e);
        GetGrowingVertices()->AppendVertex(e.x, e.y);

        if (e.button == MIDDLEMOUSE) {
            return false;
        } else {
            Grasp(e);
        }
    }
    return true;
}

void WireManip::Constrain (Event& e) {
    GraphicView* view = GetViewer()->GetGraphicView();
    _target = view->ConnectorIntersecting(e.x-SLOP,e.y-SLOP,e.x+SLOP,e.y+SLOP);

    if (_target != nil) {
        float cx, cy;
        _target->GetGraphic()->GetCenter(cx, cy);
        e.x = round(cx);
        e.y = round(cy);
    }
    GetRubberWire()->Manhattan(!e.shift);
}

RubberWire* WireManip::GetRubberWire () {return (RubberWire*) GetRubberband();}
