/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * NetlistView generic netlist external representation and subclasses 
 * for schematic components.
 */

#ifndef scmnet_h
#define scmnet_h

#include <Unidraw/Components/psview.h>

class AliasComp;
class ElementComp;
class GraphicComp;
class NodeComp;
class Path;

class NetlistView : public PreorderView {
public:
    NetlistView(GraphicComp*);
    NetlistView(GraphicComp*, boolean flattening);

    virtual boolean Emit(ostream&);

    void Flatten(boolean);
    GraphicComp* GetGraphicComp();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    void BeginBlock(ostream&);
    void EndBlock(ostream&);
    void Indent(ostream&);

    NetlistView* CreateNetlistView(GraphicComp*);
protected:
    static boolean _flattening;
    static int _totalIndent;
};

inline void NetlistView::Flatten (boolean f) { _flattening = f; }

class NLElement : public NetlistView {
public:
    NLElement(ElementComp* = nil);
    virtual ~NLElement();

    virtual boolean Definition(ostream&);
    virtual void Update();
    ElementComp* GetElemComp();

    virtual ExternView* GetView(Iterator);
    virtual void SetView(ExternView*, Iterator&);

    virtual void First(Iterator&);
    virtual void Last(Iterator&);
    virtual void Next(Iterator&);
    virtual void Prev(Iterator&);
    virtual boolean Done(Iterator);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    UList* Elem(Iterator);
    NLElement* View(UList*);

    void Identification(ostream&);

    void DeleteView(Iterator&);
    void DeleteViews();
private:
    UList* _views;
};

class NLAlias : public NLElement {
public:
    NLAlias(AliasComp* = nil);

    virtual boolean Definition(ostream&);
    AliasComp* GetAliasComp();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    boolean FlatDefinition(AliasComp*, ostream&);
};

class NLNode : public NetlistView {
public:
    NLNode(NodeComp* = nil);

    virtual boolean Definition(ostream&);
    NodeComp* GetNodeComp();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void Identification(ostream&);
    NodeComp* FindCorrespDefNode(NodeComp*, ElementComp*);
    boolean Connections(NodeComp*, ostream&, Path*);

    boolean Identify(NodeComp*, ostream&, Path*);
    boolean IdentifyWireNode(NodeComp*, ostream&, Path*);
    boolean IdentifyAliasNode(NodeComp*, ostream&);
    boolean IdentifyElemNode(NodeComp*, ostream&);
};

class NLGraphicComps : public NetlistView {
public:
    NLGraphicComps(GraphicComps* = nil);
    virtual ~NLGraphicComps();

    virtual void Update();

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    UList* Elem(Iterator);
    void DeleteView(Iterator&);
    void DeleteViews();
private:
    UList* _views;
};

#endif
