/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * ToolPalette and DrawingPalette - degenerate (i.e., no viewer)
 * editors that contain commands and/or tools for creating schematic
 * and basic drawing components.
 */

#ifndef scmpal_h
#define scmpal_h

#include <Unidraw/editor.h>

class BrushVar;
class ColorVar;
class Command;
class ControlInfo;
class ControlState;
class EditorInfo;
class FontVar;
class HBox;
class PatternVar;
class PulldownMenu;
class UControl;
class VBox;

class ToolPalette : public Editor {
public:
    ToolPalette();
    virtual ~ToolPalette();

    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();

    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);

    ControlState* GetControlState();
    EditorInfo* GetEditorInfo();

    void InstallOrRemove(const char* toolName, Tool* = nil);
private:
    Interactor* Interior();
    void InitEditorInfo();

    Interactor* Tools();
    void InitTools();
    void InitCompTools();
    void ReadCompTools();

    void Include(Tool*, const char* = "");

    UControl* FindControl(const char* toolName);
private:
    KeyMap* _keymap;
    ControlState* _curCtrl;
    Selection* _selection;
    VBox* _compTools;
    VBox* _tools;

    EditorInfo* _edInfo;
    boolean _initEdInfo;
};

inline ControlState* ToolPalette::GetControlState () { return _curCtrl; }
inline EditorInfo* ToolPalette::GetEditorInfo () { return _edInfo; }

class DrawingPalette : public Editor {
public:
    DrawingPalette(ToolPalette*);
    virtual ~DrawingPalette();

    virtual Component* GetComponent();
    virtual KeyMap* GetKeyMap();
    virtual Tool* GetCurTool();
    virtual Selection* GetSelection();
    virtual StateVar* GetState(const char*);

    virtual void SetKeyMap(KeyMap*);
    virtual void SetSelection(Selection*);

    void Iconify(boolean);
protected:
    virtual boolean DependsOn(Component*);
private:
    Interactor* Interior();
    void InitStateVars();

    Interactor* Commands();
    Interactor* Tools();
    void InitTools();

    void Include(Command*, PulldownMenu*);
    void Include(Tool*);

    PulldownMenu* FontMenu();
    PulldownMenu* BrushMenu();
    PulldownMenu* PatternMenu();
    PulldownMenu* ColorMenu(const char* name, const char* attrib);
private:
    ToolPalette* _tools;
    HBox* _dwgCompTools;
    HBox* _dwgTools;
    KeyMap* _keymap;

    FontVar* _font;
    BrushVar* _brush;
    PatternVar* _pattern;
    ColorVar* _color;
};

#endif
