/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Schematic capture system-specific tools.
 */

#ifndef scmtools_h
#define scmtools_h

#include <Unidraw/Tools/grcomptool.h>

class AliasCmd;
class DefinitionCmd;
class ElementComp;
class GraphicComp;
class InfoCmd;
class NewViewCmd;
class TruthTableCmd;

class ExamineTool : public Tool {
public:
    ExamineTool(ControlInfo* = nil);
    virtual ~ExamineTool();

    virtual Manipulator* CreateManipulator(Viewer*, Event&, Transformer* =nil);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
private:
    void InitCmds(Viewer*, GraphicComp*);
    Manipulator* CreatePopupManip(Viewer*);
private:
    AliasCmd* _aliasCmd;
    DefinitionCmd* _defnCmd;
    InfoCmd* _infoCmd;
    NewViewCmd* _newViewCmd;
    TruthTableCmd* _truthTableCmd;
};

class ManipulateTool : public Tool {
public:
    ManipulateTool(ControlInfo* = nil);

    virtual Manipulator* CreateManipulator(Viewer*, Event&, Transformer* =nil);
    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

class InstanceTool : public GraphicCompTool {
public:
    InstanceTool();
    InstanceTool(ControlInfo*, ElementComp* original);

    virtual Manipulator* CreateManipulator(Viewer*, Event&, Transformer* =nil);
    virtual Command* InterpretManipulator(Manipulator*);

    ElementComp* GetOriginal();

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

inline ElementComp* InstanceTool::GetOriginal () {
    return (ElementComp*) GetPrototype();
}

#endif
