/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * TruthTable - defines a schematic component's logic function.
 * TruthTableEditor - an interface to editing a TruthTable
 */

#ifndef scmtruth_h
#define scmtruth_h

#include "scmvars.h"
#include <InterViews/texteditor.h>

static const char* TTEDone = "\r\t\007\033";

class ButtonState;

class TruthTable {
public:
    TruthTable(int inputs, int outputs);
    virtual ~TruthTable();

    void SetInputVar(NodeVar*, int inputIndex);
    void SetOutputVar(NodeVar*, int outputIndex);
    NodeVar* GetInputVar(int inputIndex);
    NodeVar* GetOutputVar(int outputIndex);
    int Inputs();
    int Outputs();
    int InputCombos();

    void SetOutput(unsigned inputs, int outputIndex, LogicLevel output);
    LogicLevel GetOutput(unsigned inputs, int outputIndex);

    TruthTable& operator = (TruthTable&);
    TruthTable* Copy();
private:
    int _inputs, _outputs;
    NodeVar** _inVar, **_outVar;
    LogicLevel** _output;
};

inline int TruthTable::Inputs () { return _inputs; }
inline int TruthTable::Outputs () { return _outputs; }

class TruthTableEditor : public TextEditor {
public:
    TruthTableEditor(ButtonState*, TruthTable*, const char* done = TTEDone);
    virtual ~TruthTableEditor();

    TruthTable* GetTruthTable();
    virtual void Handle(Event&);
private:
    boolean HandleKeyEvent(Event&);
    boolean HandleDownEvent(Event&);
    boolean HandleChar(char);

    int LimitIndex(int);
    void InsertText(char);
    void AdvanceChar(int);
    void AdvanceLine(int);

    void UpdateTextBuffer();
    void UpdateTruthTable();
private:
    ButtonState* _state;
    TruthTable* _truth;
    const char* _done;
    char* _buf;
    int _bufsize;
    int _leftMargin;
    int _lineWidth;
    int _totalLines;
};

inline TruthTable* TruthTableEditor::GetTruthTable () { return _truth; }

#endif
