/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Implementation of schematic capture system-specific state variables.
 */

#include "scmclasses.h"
#include "scmvars.h"

#include "stdio.h"
#include "stream.h"

/*****************************************************************************/

static char buf[CHARBUFSIZE];

/*****************************************************************************/

int ElementVar::_counter;

ElementVar::ElementVar (const char* name) : NameVar(name) { _id = _counter++; }

const char* ElementVar::Identification () {
    sprintf(buf, "%s_%d", GetName(), _id);
    return buf;
}

StateVar& ElementVar::operator = (StateVar& var) {
    NameVar::operator=(var);
    return *this;
}

StateVar* ElementVar::Copy () { return new ElementVar(GetName()); }

ClassId ElementVar::GetClassId () { return ELEMENT_VAR; }

boolean ElementVar::IsA (ClassId id) {
    return ELEMENT_VAR == id || NameVar::IsA(id);
}

/*****************************************************************************/

NodeVar::NodeVar (const char* name, LogicLevel level) : NameVar(name) {
    _level = level;
}

void NodeVar::SetLogicLevel (LogicLevel level) {
    if (level != _level) {
        _level = level;
        Notify();
    }
}

StateVar& NodeVar::operator = (StateVar& var) {
    NameVar::operator=(var);

    if (var.IsA(NODE_VAR)) {
        NodeVar* nodeVar = (NodeVar*) &var;
        SetLogicLevel(nodeVar->GetLogicLevel());
    }
    return *this;
}

StateVar* NodeVar::Copy () { return new NodeVar(GetName(), _level); }

void NodeVar::Read (istream& in) {
    int level;

    NameVar::Read(in);
    in >> level;
    _level = LogicLevel(level);
}

void NodeVar::Write (ostream& out) {
    NameVar::Write(out);
    out << _level << " ";
}

ClassId NodeVar::GetClassId () { return NODE_VAR; }
boolean NodeVar::IsA (ClassId id) { return NODE_VAR == id || NameVar::IsA(id);}
