/*
 * Copyright (c) 1990 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Views of schematic capture system-specific state variables.
 */

#ifndef scmvarviews_h
#define scmvarviews_h

#include <Unidraw/stateviews.h>

class ButtonState;
class ElementVar;
class NodeVar;
class RadioButton;
class StringEditor;

class ElementVarView : public StateVarView {
public:
    ElementVarView(ElementVar*, ButtonState*);

    ElementVar* GetSubject();
    void ChangeSubject();
    
    virtual void Handle(Event&);
protected:
    virtual void Init();
    virtual boolean Stale();
private:
    StringEditor* GetStringEditor();
};

inline ElementVar* ElementVarView::GetSubject (){return (ElementVar*)_subject;}

class NodeVarView : public StateVarView {
public:
    NodeVarView(NodeVar*, ButtonState*);

    NodeVar* GetSubject();
    void ChangeSubject();

    virtual void Handle(Event&);
protected:
    virtual void Init();
    virtual boolean Stale();
private:
    StringEditor* _sedit;
    ButtonState* _iobs;

    RadioButton* _in;
    RadioButton* _out;
    RadioButton* _inout;
};

inline NodeVar* NodeVarView::GetSubject () { return (NodeVar*) _subject; }

#endif
