/* Stopwatch -- a simple stop watch emulator
 *
 * Usage:	stopwatch [title]...
 * 		where `title' is the title of the stopwatch
 *
 * The stop watch starts where you press the `start' button
 * it stops when `stop' is pressed. Press restart will restart
 * the watch without initialize it.
 *
 * This program, athough small and simple, uses many of the IV3.0 features.
 * These include Kit, Deck, Patch, LRBox, TBBox, IOCallback, Glyphs, etc.
 *
 * A very simple class, Replaceable, provides re-setable labels. It can be
 * used to display something which needs to be updated. 
 *
 * Bugs:
 *	The buttons produced by Kit looks not as what I had expected.
 *
 * Copyright: No restriction on how you can use this program. If you can keep
 * my name on all your programs dervied or use stopwatch, I will be very happy.
 * Also, do not blame me if you are not satisfied with it, or stopwatch has
 * brought some damage to you. Of cause, bug reporting and comments are
 * always welcome.
 *
 *	Yufan Hu yufan@cs.keele.ac.uk	Sun Mar 1 15:00:00 1992
 *
 */


#include <InterViews/display.h>
#include <InterViews/monoglyph.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <InterViews/center.h>
#include <InterViews/box.h>
#include <InterViews/glue.h>
#include <IV-look/button.h>
#include <IV-look/kit.h>
#include <InterViews/margin.h>
#include <InterViews/rule.h>
#include <InterViews/deck.h>
#include <InterViews/character.h>
#include <InterViews/patch.h>
#include <InterViews/font.h>
#include <Dispatch/dispatcher.h>
#include <Dispatch/iocallback.h>
#include <OS/string.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

class Replaceable : public LRBox {
public:
	Replaceable(char *, Style*);
	void set(char *,  Style* );
	Patch *patch() { return p; };
private:
	Patch *p;
};

Replaceable :: Replaceable(char *l, Style* s){
	append(Kit::instance()->label(l, s));
	p = new Patch(this);
}

void Replaceable :: set(char *l, Style* s) {	
	replace(0, Kit::instance()->label(l, s));
	p->redraw();
}

/* class definition for digits */

class Digit : public Deck {
public:
	Digit(Font *font = nil);
};

Digit :: Digit(Font *font) {
	Style *style = Session::instance()->style();
	if(!font) font = (Font *) style->font();
	for(long i = 0; i<10; i++){
		append(new Character(i + (long)'0', font, 
			style->foreground()));
	}
	flip_to(0);
}

class Digits : public LRBox {
public:
	Digits(int d = 1, Font *font = nil);
	void show(long);
	long current() { return cur; };
private:
	Digit **digits;
	int num;
	long cur;
};

Digits :: Digits(int d, Font *font) {
	if(d <= 0) d = 1;
	digits = (Digit **) malloc(sizeof(Digit *)*d);
	num = d;
	HGlue *sp = new HGlue(3.0,0,0);
	while(--d >= 0) {
		digits[d] = new Digit(font);
		append(digits[d]);
		append(sp);
	}
	cur = 0;
}

void Digits :: show(long n) {
	if(cur == n) return;
	cur = n;
	int d = 0;
	do {
		digits[d]->flip_to(n % 10);
		n /= 10;
		d++;
	} while(d < num);
}


/* Class definition for a stopwatch */

class Stopwatch {
public:
	Stopwatch(char *title = nil);
	void start();
	void restart();
	void stop();
	void update();
	void tick(long, long);
	Glyph *window() { return win; };
private:
	IOHandler* tick_tick;
	int sec, min, hour;
	int lastsec;
	boolean running;
	Glyph *watch;
	Digits *secDigits;
	Patch *secPatch;
	Digits *minDigits;
	Patch *minPatch;
	Digits *hourDigits;
	Patch *hourPatch;
	Replaceable* startfrom;
	Glyph *win;
};

ActionCallbackdeclare(Stopwatch);
ActionCallbackimplement(Stopwatch);

IOCallbackdeclare(Stopwatch);
IOCallbackimplement(Stopwatch);

Stopwatch :: Stopwatch(char *title) {
	if(!title) title = "StopWatch";
	Style *style = Session::instance()->style();
	Kit *kit = Kit::instance();
	Font* dfont = new Font("12x24");
	startfrom = nil;
	running = false;
	hourPatch = new Patch(hourDigits = new Digits(2, dfont));
	minPatch = new Patch(minDigits = new Digits(2, dfont));
	secPatch = new Patch(secDigits = new Digits(2, dfont));
	start();
	stop();
	HGlue *sp = new HGlue(1.0);
	Character *colon = new Character(':', dfont, style->foreground());
	watch = new Margin(new LRBox(sp,hourPatch,colon,minPatch,colon,secPatch,sp), 5);
	win = kit->flat_frame(
	    new Margin(
	        new TBBox(
	            new VCenter(new LRBox(
	                sp, new TBBox(
	                    kit->label(title,style), 
	                    new HRule(style->foreground(), 3.0)
	                ), sp)
	            ),
	            startfrom->patch(),
	            new HRule(style->foreground(), 1.0),
	            new VGlue(5.0),
	            kit->inset_frame(watch, style),
	            new VGlue(5.0),
	            new LRBox(
	            	new HGlue(1.0),
	            	kit->push_button(
	            		kit->outset_frame(kit->label("Start", style), style),
	            		style, 
	            		new Stopwatch_ActionCallback(this, &Stopwatch::start)
	            	),
	            	kit->push_button(
	            		kit->outset_frame(kit->label("Restartp", style),style),
	            		style, 
	            		new Stopwatch_ActionCallback(this, &Stopwatch::restart)
	                ),
	            	kit->push_button(
	            		kit->outset_frame(kit->label("Stop", style),style),
	            		style, 
	            		new Stopwatch_ActionCallback(this, &Stopwatch::stop)
	                ),
	            	kit->push_button(
	            		kit->outset_frame(kit->label("Quit", style),style),
	            		style, 
	            		kit->quit()
	            	)
	            )
	        ),
	       5
	    ),
	    style
	);
	tick_tick = new IOCallback(Stopwatch)(this, Stopwatch::tick);
}

void Stopwatch::tick(long , long ){
	Dispatcher::instance().startTimer(1, 0, tick_tick);
	if(!running) return;
	update();
}
		
void Stopwatch :: start() {
	if(running) return;
	running = true;
	time_t now = time(0);
	char *timestr = asctime(localtime(&now));
	timestr[strlen(timestr)-1] = 0;
	if(startfrom) startfrom->set(timestr, Session::instance()->style());
	else startfrom = new Replaceable(timestr, Session::instance()->style());
	sec = min = hour  = 0;
	lastsec = localtime(&now)->tm_sec;
	secDigits->show(0); minDigits->show(0); hourDigits->show(0);
	secPatch->redraw(); minPatch->redraw(); hourPatch->redraw();

	tick(0,0);
}

void Stopwatch :: restart() {
	if(running) return;
	running = true;
	time_t now = time(0);
	struct tm *tmnow = localtime(&now);
	lastsec = tmnow->tm_sec;
	tick(0,0);
}

void Stopwatch :: stop() { 
	running = false;
	Dispatcher::instance().stopTimer(tick_tick);
}

void Stopwatch :: update() {
	if(!running) return;
	time_t now = time(0);
	struct tm *tmnow = localtime(&now);
	if(tmnow->tm_sec == lastsec) return;
	lastsec = tmnow->tm_sec - lastsec;
	if(lastsec < 0) lastsec += 60;
	if((sec += lastsec) >= 60){
		sec -= 60;
		if(++min >= 60){
			min -= 60;
			hour++;
			hourDigits->show(hour);
			hourPatch->redraw();
		}
		minDigits->show(min);
		minPatch->redraw();
	}
	secDigits->show(sec);
	secPatch->redraw();
	lastsec = tmnow->tm_sec;
}

static PropertyData properties[] = {
	{"*Monochrome", "on"},
	nil
};

int main(int argc, char** argv) {
	Session* session = new Session("Stopwatch", argc, argv, 
		nil, properties);
	TBBox *box = new TBBox;
	if(argc == 1){
		box->append((new Stopwatch("No Title"))->window());
	} else {
		do { argc--; argv++;
			box->append((new Stopwatch(*argv))->window());
		} while(argc>1);
	}
	ApplicationWindow *window = new ApplicationWindow(box);
	window->map();
	session->run();
	return 0;
}
