/* This may look like C code, but it is really -*- C++ -*- */
/* Copyright (c) 1991 by Sanjay Ghemawat. */

#ifndef BOARDH
#define BOARDH

/* Assertion stuff */
#ifdef DEBUG
#   define assert(cond)	do { if (!(cond)) { exit(1); } } while (0)
#else
#   define assert(cond)
#endif

#include <InterViews/defs.h>
#include "objects.h"

#define DEF_WIDTH	10
#define DEF_HEIGHT	20

class ObjectGen;

/* A Board implements a tetris board. */

class Board {
  public:
    Board(ObjectGen* og, int ht = DEF_HEIGHT, int wd = DEF_WIDTH, int mr = 1);
	/* og:	Object Generator to use. */
	/* ht:	Height of board. */
	/* wd:	Width of board. */
	/* mr:	Number of turns allowed to move object which has hit bottom. */
    ~Board();

    void	SetGenerator(ObjectGen*);

    void	Tick();			/* One turn elapsed */
    void	RotateFalling();	/* Rotate the falling object */
    void	LeftShift();		/* Shift the falling object left */
    void	RightShift();		/* Shift the falling object right */
    void	FallAllTheWay();	/* Let object fall all the way */
    void	KillFilledRows();	/* Kill all the filled rows */

    void	Clear();		/* Clear the board */
    boolean	Filled(int);		/* Is specified row full? */
    void	Set(int row, int col, Square);	/* Set a square */
    Square	Get(int row, int col);		/* Get a square */
    Square**	Contents();		/* Return board contents. */
    int		Height();
    int		Width();
    int		Score();
    boolean	Blocked();
  private:
    ObjectGen*	generator;		/* Object generator */

    /* The following routine checks whether an object can be put at a */
    /* given location */
    boolean	LegalAt(int nx, int ny, Rotation nrot);	
    boolean	LegalPoint(int, int);

    void	DrawFalling();	/* Draw the falling object */
    void	EraseFalling();	/* Erase the falling object */

    int		height;		/* Number of rows */
    int		width;		/* Number of columns */
    Square**	square;		/* Array of squares */
    boolean*	filled;		/* Which rows are filled? */
    boolean	blocked;	/* Has board been filled up totally? */
    boolean	falling;	/* Is an object falling? */
    int		maxrest;	/* Maximum number of turns allowed at rest */
    int		score;		/* Score so far */

    /* These fields are necessary only for boards with falling objects */
    int		x, y;		/* Coordinates of falling object */
    ObjectType	ftype;		/* Falling object's type */
    Rotation	rot;		/* Current rotation of object */
    int		atrest;		/* Number of turns object has been at rest */
};

inline boolean Board::Filled(int i) {
    return filled[i];
}

inline void Board::Set(int row, int col, Square s) {
    square[row][col] = s;
}

inline Square Board::Get(int row, int col) {
    return square[row][col];
}

inline Square** Board::Contents() {
    return square;
}

inline int Board::Height() {
    return height;
}

inline int Board::Width() {
    return width;
}

inline int Board::Score() {
    return score;
}

inline boolean Board::Blocked() {
    return blocked;
}

#endif /*BOARDH */
