/* Copyright (c) 1991 by Sanjay Ghemawat */
#include "field.h"
#include <stdio.h>
#include <string.h>

FieldState::FieldState() { }

IntState::IntState(int init) {
    value = init;
}

void IntState::Set(int val) {
    if (value != val) {
	value = val;
	Notify();
    }
}

int IntState::Get() {
    return value;
}

char* IntState::Unparse() {
    static char buf[100];

    sprintf(buf, "%d", value);
    return buf;
}

StringState::StringState(char* init) {
    len = strlen(init);
    value = new char[len+1];
    strcpy(value, init);
}

void StringState::Set(char* val) {
    if (strcmp(val, value) != 0) {
	/* Value changed */
	int newlen = strlen(val);
	if (len < newlen) {
	    /* New string occupies more space - reallocate space */
	    delete value;
	    len = newlen;
	    value = new char[len+1];
	}
	strcpy(value, val);
	Notify();
    }
}

char* StringState::Get() {
    return value;
}

char* StringState::Unparse() {
    return value;
}

EnumState::EnumState(int init, int length, int* values, char** names) {
    listlength = length;
    valuelist = values;
    namelist = names;
    index = Find(init);
}

void EnumState::Set(int val) {
    int newindex =  Find(val);
    if (newindex != index) {
	index = newindex;
	Notify();
    }
}

int EnumState::Get() {
    return valuelist[index];
}

char* EnumState::Unparse() {
    return namelist[index];
}

/*
 * Return index of val in valuelist
 */
int EnumState::Find(int val) {
    for (int i = 0; i < listlength; i++) {
	if (valuelist[i] == val) {
	    return i;
	}
    }
    return 0;
}
