/* This may look like C code, but it is really -*- C++ -*- */
/* Copyright (c) 1991 by Sanjay Ghemawat. */

#ifndef FIELDH
#define FIELDH

#include <InterViews/subject.h>

/*
 * A FieldState is a base class for displaying string representations of
 * various data types.
 */
class FieldState : public Subject {
  public:
    FieldState();

    virtual char* Unparse() = 0;
};

/*
 * An IntState maintains  stuff for displaying an integer.
 */
class IntState : public FieldState {
  public:
    IntState(int);

    void		Set(int);
    int			Get();
    virtual char*	Unparse();
  protected:
    int   value;		/* Field value */
};

/*
 * A StringState maintains stuff for displaying strings.
 */
class StringState : public FieldState {
  public:
    StringState(char*);

    void		Set(char*);
    char*		Get();
    virtual char*	Unparse();
  protected:
    char*	value;
    int		len;
};

/*
 * Stuff for keeping enumerated types - for example:
 *     typedef enum { false = 0, true = 1} boolean;
 *     int vals[] = { false, true };
 *     int names[] = { "false", "true" };
 *     EnumState state(true, 2, vals, names);
 */
class EnumState : public FieldState {
  public:
    EnumState(int init, int length, int* values, char** names);

    void		Set(int);
    int			Get();
    virtual char*	Unparse();
  protected:
    int		listlength;	/* Length of namelist and valuelist */
    char**	namelist;	/* List of names for enumerated type */
    int*	valuelist;	/* List of values for enumerated type */
    int		index;		/* Index of current value */

    int		Find(int);	/* Return index of val in value list */
};

#endif /*FIELDH */
