/* This may look like C code, but it is really -*- C++ -*- */
/* Copyright (c) 1991 by Sanjay Ghemawat. */

#ifndef GAMEH
#define GAMEH

/* A Game presents a tetris board and some auxiliary stuff to */
/* the user. */

#include "objects.h"
#include <InterViews/frame.h>

class PBoard;
class Board;
class ObjectGen;
class IntState;
class EnumState;
class StringState;
class ObjectDisplay;

struct Time { long sec, usec; };

class Game : public BorderFrame {
  public:
    Game();

    virtual ~Game();
    virtual void Handle(Event&);

    virtual void Run();

    void Start		();
    void Stop		();
    void Quit		();
    void Freeze		();
    void Thaw		();
    void Harder		();
    void Easier		();
    void ToggleNext	();

    /* Select generators. */
    void UseRandom	();
    void UseRepeater	();
    void UseCycle	();
    void UseAlternator	();
  private:
    PBoard*		pboard;		/* Tetris board display */
    Board*		board;		/* Tetris board data */
    ObjectDisplay*	next;		/* Display of next object to fall */
    boolean		finished;	/* Have we quit? */
    IntState*		level;		/* Level of game */
    IntState*		score;		/* Current score */
    EnumState*		state;		/* State of game */
    StringState*	genstate;	/* Object generator's name */
    IntState*		killcount;	/* Display of number of rows filled */
    boolean		on_canvas;	/* Is mouse in window? */

    boolean		showing_next;	/* Are we showing the next object */
    boolean		sync;		/* Synchronize with display */

    ObjectGen*		generator;	/* Object generator */
    int			block_size;	/* Block size in pixels */
    int			bwidth;		/* Board width */
    int			bheight;	/* Board height */

    boolean		increase_levels;/* Game gets harder by itself */
    int			init_level;	/* Starting level of game */
    int			rows_killed;	/* Total number of rows killed */

    void		AdjustTicker();	/* Set timeout for next tick */
    void		Flash();	/* Flash filled rows before killing */
    void		SetLevel(int, boolean);	/* Set difficulty level */
    void		Turn(Square);	/* Invert filled rows */

};

#endif /*GAMEH */
