/* Copyright (c) 1991 by Sanjay Ghemawat */
#include <random.h>
#include "generator.h"

extern "C" {
    extern int getpid();
};

ObjectGen::ObjectGen() {
    name = "Generic";
    next = Thin;
    if (!random_inited) {
	srandom(getpid());
	random_inited = 1;
    }
}

ObjectType ObjectGen::get_next() {
    return Thin;
}

Alternator::Alternator() {
    name = "Altern";
    last = ZigRight;
    next = get_next();
}

ObjectType Alternator::get_next() {
    if (last == ZigLeft) {
	last = ZigRight;
    }
    else {
	last = ZigLeft;
    }
    return last;
}

CycleGen::CycleGen(ObjectType firstgen = Thin) {
    name = "Cycle";
    nextGen = firstgen;
    next = get_next();
}

ObjectType CycleGen::get_next() {
    ObjectType ret = (ObjectType) nextGen;
    nextGen = (nextGen + 1) % NumObjectTypes;
    return ret;
}

RandomGen::RandomGen() {
    name = "Random";
    next = get_next();
}

ObjectType RandomGen::get_next() {
    return (ObjectType) (random() % NumObjectTypes);
}

RepeaterGen::RepeaterGen(float chance = 0.1, ObjectType last = Thin) {
    name = "Repeat";
    lastGen = last;
    repeatChance = (int) (chance * 100.0);
    next = get_next();
}

ObjectType RepeaterGen::get_next() {
    if ((random() % 100) < repeatChance) {
	/* Repeat last value */
	return (ObjectType) lastGen;
    } else {
	/* Generate new random object */
	lastGen = random() % NumObjectTypes;
	return (ObjectType) lastGen;
    }
}
