/* This may look like C code, but it is really -*- C++ -*- */
/* Copyright (c) 1991 by Sanjay Ghemawat. */

#ifndef GENERATORH
#define GENERATORH

/* This file maintains different kinds of object generators. */

#include <random.h>
#include "objects.h"

/* Base Object Generator - only use derived classes. */
/* Call 'operator()' to get a new object. */
/* Can look ahead by examining the the member named 'next'. */
/* Subclasses should define the virtual 'get_next' to */
/* to get a new object. */

class ObjectGen {
  public:
    ObjectGen();
    ObjectType operator()();

    char* name;				/* Generator name */
    ObjectType	next;			/* Next object that will be returned */
  protected:
    /* Return the next object type. */
    virtual ObjectType get_next();

    static random_inited;		/* Has RNG been initialized */
};

/* Generator which alternates between ZigLeft and ZigRight. */
class Alternator : public ObjectGen {
  public:
    Alternator();
  protected:
    ObjectType last;
    virtual ObjectType get_next();
};

/* Generator which cycles through all objects */
class CycleGen : public ObjectGen {
  public:
    CycleGen(ObjectType firstgen = Thin);
  protected:
    int	nextGen;	/* Type of object to be generated next */
    virtual ObjectType get_next();
};

/* Random generator */
class RandomGen : public ObjectGen {
  public:
    RandomGen();
  protected:
    virtual ObjectType get_next();
};

/* Random generator, but with slight tendency to repeat */
class RepeaterGen : public ObjectGen {
  public:
    RepeaterGen(float chance = 0.1, ObjectType last = Thin);
  protected:
    int	lastGen;	/* Type of object last generated */
    int repeatChance;	/* % chance to repeat last element */
    virtual ObjectType get_next();
};

inline ObjectType ObjectGen::operator()() {
    ObjectType ret = next;	/* Store object to be returned */
    next = get_next();		/* Look ahead for next object */
    return ret;			/* Return object */
}

#endif /*GENERATORH */
