/* Copyright (c) 1991 by Sanjay Ghemawat. */

#include <InterViews/painter.h>
#include <InterViews/font.h>
#include <InterViews/shape.h>
#include "label.h"
#include "field.h"

Label::Label(FieldState* state,
	     Alignment a, char* format)
{
    Init(state, a, format);
}

Label::Label(const char* name, FieldState* state,
	     Alignment a, char* format)
{
    SetInstance(name);
    Init(state, a, format);
}

Label::Label(Painter* out, FieldState* state,
	     Alignment a, char* format) : (nil, out)
{
    Init(state, a, format);
}

Label::~Label() {
    if (subject != nil) {
	subject->Detach(this);
    }
}

void Label::SetFormat(char* format) {
    minformat = format;
    Update();
}

char*Label::GetFormat() {
    return minformat;
}

void Label::SetAlignment(Alignment a) {
    if (a != align) {
	align = a;
	Update();
    }
}

Alignment Label::GetAlignment() {
    return align;
}

void Label::Init(FieldState* state, Alignment a, char* format) {
    SetClassName("Label");
    subject = state;
    minformat = format;
    align = a;
    subject->Attach(this);
    Update();
}

void Label::Redraw(Coord x1, Coord y1, Coord x2, Coord y2) {
    output->ClearRect(canvas, x1, y1, x2, y2);
    Refresh();
}

static const int pad = 2;
void Label::Refresh() {
    int x, y;
    Font* f = output->GetFont();
    int wd = f->Width(subject->Unparse());
    int ht = f->Height();

    switch (align) {
      case TopCenter:
      case Center:
      case BottomCenter:
      case HorizCenter:
	/* Center alignment horizontally */
	x = (xmax - wd) / 2;
	break;
      case TopRight:
      case CenterRight:
      case BottomRight:
      case Right:
	/* Right alignment horizontally */
	x = xmax - pad - wd;
	break;
      default:
	/* Left alignment horizontally */
	x = pad;
	break;
    }

    switch (align) {
      case TopLeft:
      case TopCenter:
      case TopRight:
      case Top:
	/* Top alignment vertically */
	y = ymax - ht - pad;
	break;
      case CenterLeft:
      case Center:
      case CenterRight:
      case VertCenter:
	/* Center alignment vertically */
	y = (ymax - ht) / 2;
	break;
      default:
	/* Bottom alignment vertically */
	y = pad;
	break;
    }

    output->Text(canvas, subject->Unparse(), x, y);
}

void Label::Update() {
    if (canvas != nil) {
	int oldwidth = shape->width;
	Reconfig();
	if (shape->width != oldwidth) Reshape(*shape);
	output->ClearRect(canvas, 0, 0, xmax, ymax);
	Refresh();
    }
}

void Label::Reconfig() {
    char* value = subject->Unparse();
    Font* f = output->GetFont();

    minwidth = f->Width(minformat) + 2*pad;
    shape->width = f->Width(value) + 2*pad;
    if (shape->width < minwidth) shape->width = minwidth;
    shape->height= f->Height()     + 2*pad;
    shape->Rigid();
}
