/* Copyright (c) 1991 by Sanjay Ghemawat. */

#include <InterViews/world.h>
#include <InterViews/propsheet.h>
#include <stdio.h>

#include "game.h"

extern "C" {
    extern int strcasecmp(const char*, const char*);
};

static OptionDesc options[] = {
    { "-l",		".Level",	OptionValueNext },
    { "-level",		".Level",	OptionValueNext },
    { "-s",		".Size",	OptionValueNext },
    { "-size",		".Size",	OptionValueNext },
    { "-w",		".Width",	OptionValueNext },
    { "-width",		".Width",	OptionValueNext },
    { "-h",		".Height",	OptionValueNext },
    { "-height",	".Height",	OptionValueNext },
    { "-sync",		".Sync",	OptionValueImplicit, "on"	},
    { "-nosync",	".Sync",	OptionValueImplicit, "off"	},
    { "-n",		".Next",	OptionValueImplicit, "on"       },
    { "-next",		".Next",	OptionValueImplicit, "on"       },
    { "-harder",	".GetHarder",	OptionValueImplicit, "on"       },
    { "-same",		".GetHarder",	OptionValueImplicit, "off"      },
    { "-random",	".Generator",	OptionValueImplicit, "random"   },
    { "-cycle",		".Generator",	OptionValueImplicit, "cycle"    },
    { "-repeater",	".Generator",	OptionValueImplicit, "repeater" },
    { "-alt",		".Generator",	OptionValueImplicit, "alt"	},
    { "-help",		".Help",	OptionValueImplicit, "on"	},
    { nil,		nil,		nil }
};

static void
printl(char* line) {
    fputs(line, stderr);
}

void print_help() {
    printl("Usage: tetris [options]\n");
    printl("   Options are -\n");
    printl("        -l[evel]       number      ; set level \n");
    printl("        -s[ize]        number      ; set block size in pixels\n");
    printl("        -w[idth]       number      ; set width in blocks\n");
    printl("        -h[eight]      number      ; set height in blocks \n");
    printl("        -sync                      ; synchronized drawing\n");
    printl("        -nosync                    ; unsynchronized drawing\n");
    printl("        -n[ext]                    ; show next object to fall\n");
    printl("        -harder                    ; game keeps getting harder\n");
    printl("        -same                      ; game does not get harder\n");
    printl("        -random                    ; use random generator\n");
    printl("        -cycle                     ; use cycling generator\n");
    printl("        -repeater                  ; use repeating generator\n");
    printl("        -alt                       ; use alternating generator\n");
    printl("        -help                      ; print help message\n");
};

int main(int argc, char** argv)
{
    World* world = new World("tetris", options, argc, argv);

    const char* help = world->GetAttribute("Help");
    if ((argc > 1) || ((help != nil) && (strcasecmp(help, "on") == 0))) {
	print_help();
    }
    else {
	Game* game = new Game();
	world->InsertApplication(game);
	game->Run();
    }
    delete world;
    return 0;
}
