/* Copyright (c) 1991 by Sanjay Ghemawat. */

#include "pboard.h"
#include <InterViews/shape.h>
#include <InterViews/painter.h>
#include <InterViews/canvas.h>

PBoard::PBoard(Square** src, int ht, int wd, int size) {
    source = src;
    width = wd;
    height = ht;
    block_size = size;
    backing_store = new Square*[height];
    for (int i = 0; i < height; i++) {
	backing_store[i] = new Square[width];
	for (int x = 0; x < width; x++)
	    backing_store[i][x] = Empty;
    }
}

PBoard::~PBoard() {
    for (int i = 0; i < height; i++) {
	delete backing_store[i];
    }
    delete backing_store;
}

void PBoard::Reconfig() {
    shape->Rect(width*block_size, height*block_size);
}

void PBoard::Clear()
{
    if (canvas != nil) output->ClearRect(canvas, 0, 0, xmax, ymax);
    for (int i = 0; i < height; i++) {
	for (int j = 0; j < width; j++) {
	    backing_store[i][j] = Empty;
	}
    }
}

void PBoard::Refresh() {
    Square** src = source;
    Square** dst = backing_store;

    for (int i = 0; i < height; i++) {
	Square* srow = src[i];
	Square* drow = dst[i];
	for (int j = 0; j < width; j++) {
	    if (srow[j] != drow[j]) {
		drow[j] = srow[j];
                int x = j * block_size;
                int y = (height - 1 - i) * block_size;
                if (drow[j] == Empty) {
                    output->ClearRect(canvas, x+1,y+1,
                                      x+block_size-2, y+block_size-2);
                }
                else {
                    output->FillRect(canvas, x+1,y+1,
                                     x+block_size-2, y+block_size-2);
                }
	    }
	}
    }
}

void PBoard::Redraw(Coord, Coord, Coord, Coord) {
    Clear();
    Refresh();
}
