/*
 * Copyright (c) 1991, 1992, Jonathan P. Leech
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice and this permission notice appear in
 * all copies of the software and related documentation.
 *
 */

#include <InterViews/background.h>
#include <InterViews/box.h>
#include <InterViews/border.h>
#include <InterViews/glue.h>
#include <InterViews/color.h>
#include <InterViews/rule.h>
#include <InterViews/display.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>
#include <InterViews/label.h>
#include <InterViews/canvas.h>
#include <InterViews/handler.h>

#include <IV-look/kit.h>

#include <multibutton.h>

#include <stream.h>

// Selector should be subclassed to take some action when the toggle button
//  is pressed (aside from changing the visual state of the button, which
//  is handled automatically).
class EchoSelector : public Selector {
public:
    EchoSelector(Selectable *s) : Selector(s) { }
    // Selector overrides
    virtual void update(Observable *);
};

void EchoSelector::update(Observable *) {
    cout << "Multibutton state is " << selectable()->cur_selection() << endl;
}

int main(int argc, char** argv){
    Session *session = new Session("MultiToggle", argc, argv);
    Style *style = session->default_display()->style();
    WidgetKit *kit = WidgetKit::instance();
    const LayoutKit &layout = *LayoutKit::instance();

    Glyph *vspace = layout.vglue(18.0);
    Glyph *hspace = layout.hglue(36.0);
    Glyph *quit = kit->push_button("Quit", kit->quit());

    // A small integer selectable. We should say Selectable(4)
    //	to allow all the possible states of the MultiButtons created
    //	below, but will let the MultiButton constructor do it instead.
    Selectable *selectable = new Selectable;

    // A subclassed Selector which takes some action depending on the state
    //	of the Selectable it's given. In this case, it just prints the state.
    Selector *selector = new EchoSelector(selectable);

    Glyph
	*button1 = MultiButton::multi_button("Multi 1", selectable, 1),
	*button2 = MultiButton::multi_button("Multi 2", selectable, 2),
	*button3 = MultiButton::multi_button("Multi 3", selectable, 3),
	*button3redux =
	    MultiButton::multi_button("Multi 3 Again", selectable, 3);

    Glyph *world =
	layout.hbox(
	    layout.hglue(),
	    layout.vbox(
		layout.vcenter(layout.vglue(), 1.0),
		quit,
		kit->outset_frame(button1),
		kit->outset_frame(button2),
		kit->outset_frame(button3),
		kit->outset_frame(button3redux),
	    )
	);

    Window *w =
	new ApplicationWindow(
	    new Background(
		world,
		style->background()
	    )
	);

    session->run_window(w);
}
