/*
 * Copyright (c) 1991, 1992, Jonathan P. Leech
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice and this permission notice appear in
 * all copies of the software and related documentation.
 *
 */

#include <stdio.h>
#include <stream.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <IV-look/kit.h>
#include "multibutton.h"

#define DEBUG(x) x

MultiButton::MultiButton(
    Glyph *g, Style *s, TelltaleState *t,
    Selectable *selectable, unsigned index)
    : Button(g,s,t,nil), selectable_(selectable), index_(index)
{
    // Make sure Selectable includes this index in its range.
    if (index >= selectable->selections()) {
	////cout << "Increasing range of Selectable to " << index+1 << endl;
	selectable->set_selections(index+1);
    }

    selectable_->attach(this);
}

// Button toggles only on mouse button release. Wrap superclass release()
//  method to determine both on and off transitions.
void MultiButton::release(const Event &e) {
    TelltaleState *t = state();

    int before = t->test(TelltaleState::is_chosen);
    Button::release(e);
    int after = t->test(TelltaleState::is_chosen);

    if (before != after) {
	if (after) {
	    // Button goes on
	    selectable_->select(index_);
	} else {
	    // Button goes off
	    selectable_->select(0);
	}
    }
}

void MultiButton::update(Observable *o) {
    int
	selstate = (selectable_->cur_selection() == index_),
	telstate = state()->test(TelltaleState::is_chosen);

    if (selstate != telstate) {
	if (o == selectable_) {
	    DEBUG(printf("\tupdated from selectable; changing button -> %d\n",
		selstate));
	    state()->set(TelltaleState::is_chosen, selstate);
	}
    }

    Button::update(o);
}

MultiButton *MultiButton::simple_multi_button(
    const char *str, Selectable *s, unsigned index) const
{
    return MultiButton::multi_button(str,s,index);
}

MultiButton *MultiButton::multi_button(
    const char *str, Selectable *s, unsigned index)
{
    WidgetKit &kit = *WidgetKit::instance();
    return multi_button(kit.label(str), s, index);
}

MultiButton *MultiButton::multi_button(
    const String &str, Selectable *s, unsigned index)
{
    WidgetKit &kit = *WidgetKit::instance();
    return multi_button(kit.label(str), s, index);
}

MultiButton *MultiButton::multi_button(
    Glyph *g, Selectable *s, unsigned index)
{
    WidgetKit &kit = *WidgetKit::instance();
    TelltaleState *state = new TelltaleState(
	TelltaleState::is_enabled |
	TelltaleState::is_choosable |
	TelltaleState::is_toggle);

    return new MultiButton(
	kit.palette_button_look(g, state),
	kit.style(), state, s, index);
}
