/*
 * Copyright (c) 1991, 1992, Jonathan P. Leech
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice and this permission notice appear in
 * all copies of the software and related documentation.
 *
 */

#include <select.h>

Selectable::Selectable(unsigned num_selections) {
    num_selections_ = num_selections;
    cur_selection_ = 0;
}

// Should be overridden by derived classes
void Selectable::select(unsigned selection) {
    if (inrange(selection)) {
	cur_selection_ = selection;
	notify();
    }
}

// Should be overridden by derived classes
unsigned Selectable::cur_selection() const {
    return cur_selection_;
}

// Would it be more consistent to name this constrain() instead?
boolean Selectable::inrange(unsigned selection) {
    return (selection < num_selections_);
}
unsigned Selectable::selections() const {
    return num_selections_;
}

void Selectable::set_selections(unsigned num_selections) {
    // If out of range, change state to 0.
    if (cur_selection() >= num_selections) {
	select(0);
    }
    num_selections_ = num_selections;
}

Selector::Selector(Selectable *s_) : Observer(), s(s_)
{
    if (s)
	s->attach(this);
}

Selectable *Selector::selectable() const {
    return s;
}

void Selector::update(Observable *) { }
