/*
 * Copyright (c) 1991, 1992, Jonathan P. Leech
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice and this permission notice appear in
 * all copies of the software and related documentation.
 *
 */

#ifndef iv_select_h
#define iv_select_h

#include <generic.h>

#include <InterViews/resource.h>
#include <InterViews/observe.h>

class Selectable : public Observable {
public:
    Selectable(unsigned num_selections = 2);

    virtual void select(unsigned selection);// unused in derived classes
    virtual unsigned cur_selection() const; // unused in derived classes

    unsigned selections() const;
    void     set_selections(unsigned num_selections);

protected:
    boolean inrange(unsigned selection);

private:
    unsigned	num_selections_;
    unsigned	cur_selection_;		    // unused in derived classes
};

// A template selectable class using two callback methods
#define CallbackSelectable(T) name2(ivSelectable,T)

#define CallbackSelectableDeclare(T) \
class CallbackSelectable(T) : public Selectable {		\
    CallbackSelectable(T)(					\
	T *,							\
	int (T::*)(),						\
	void (T::*)(int),					\
	unsigned num_selections = 2);				\
								\
    virtual void select(unsigned selection);	/* OVERRIDE */	\
    virtual unsigned cur_selection() const;	/* OVERRIDE */	\
								\
private:							\
    T *object_;							\
    int (T::*getfunc_)();					\
    void (T::*setfunc_)(int);					\
};

#define CallbackSelectableImplement(T) \
CallbackSelectable(T)::CallbackSelectable(T)(			\
	T *object,						\
	int (T::*getfunc)(),					\
	void (T::*setfunc)(int),				\
	unsigned num_selections) :				\
    Selectable(num_selections),					\
    object_(object),						\
    getfunc_(getfunc),						\
    setfunc_(setfunc)						\
{ }								\
								\
void CallbackSelectable(T)::select(unsigned selection) {	\
    if (inrange(selection)) {					\
	(object_->*(setfunc_))(selection);			\
	notify();						\
    }								\
}								\
								\
unsigned CallbackSelectable(T)::cur_selection() {		\
    return (object_->*(getfunc_))();				\
}

class Selector : public Observer {
public:
    Selector(Selectable * = nil);
    virtual Selectable *selectable() const;
    virtual void update(Observable *);
private:
    Selectable *s;
};

#endif /*iv_select_h*/

