/*
 * Copyright (c) 1991, 1992, Jonathan P. Leech
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice and this permission notice appear in
 * all copies of the software and related documentation.
 *
 */

#include <stdio.h>
#include <InterViews/action.h>
#include <InterViews/event.h>
#include <IV-look/kit.h>
#include "togglebutton.h"

#define DEBUG(x) x

ToggleButton::ToggleButton(
    Glyph *g, Style *s, TelltaleState *t, Selectable *selectable)
    : Button(g,s,t,nil), selectable_(selectable)
{
    selectable_->attach(this);
}

void ToggleButton::update(Observable *o) {
    int
	selstate = selectable_->cur_selection() != 0,
	telstate = state()->test(TelltaleState::is_chosen);

    if (selstate != telstate) {
	if (o == selectable_) {
	    DEBUG(printf("\tupdated from selectable; changing chosen -> %d\n",
		selstate));
	    state()->set(TelltaleState::is_chosen, selstate);
	}
    }

    Button::update(o);
}

void ToggleButton::release(const Event &e) {
    TelltaleState *t = state();

    int before = t->test(TelltaleState::is_chosen);
    Button::release(e);
    int after = t->test(TelltaleState::is_chosen);

    if (before != after) {
	// Button was toggled
	selectable_->select(after != 0);
    }
}

Selectable *ToggleButton::selectable() {
    return selectable_;
}

ToggleButton *ToggleButton::toggle_button(const char *str, Selectable *s)
{
    WidgetKit &kit = *WidgetKit::instance();
    return toggle_button(kit.label(str), s);
}

ToggleButton *ToggleButton::toggle_button(const String &str, Selectable *s)
{
    WidgetKit &kit = *WidgetKit::instance();
    return toggle_button(kit.label(str), s);
}

ToggleButton *ToggleButton::toggle_button(Glyph *g, Selectable *s)
{
    WidgetKit &kit = *WidgetKit::instance();
    TelltaleState *state = new TelltaleState(
	TelltaleState::is_enabled |
	TelltaleState::is_choosable |
	TelltaleState::is_toggle);

    return new ToggleButton(
	kit.palette_button_look(g, state),
	kit.style(),
	state,
	s);
}
