// This file is part of xplot.
// 
// Copyright (C) 1989 Rene' Seindal (seindal@diku.dk)
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

#include "xplot.hh"
#include "util.hh"

#include <InterViews/defs.h>

static char *RCS_Author = "$Author: seindal $";
static char *RCS_Date = "$Date: 89/06/08 15:10:23 $";
static char *RCS_Revision = "$Revision: 2.3 $";
static char *RCS_State = "$State: Exp $";

static char *months[] = {
    "January",	"Febuary",	"March",	
    "April",	"May",		"June",	
    "July",	"August",	"Sepember",	
    "October",	"November",	"December",
};

Version::Version()
{
    date = nil;
    revision = nil;
    author = nil;
    state = nil;
    pretty = nil;
}

Version::~Version()
{
    delete date;
    delete revision;
    delete author;
    delete state;
    delete pretty;
}

const char *Version::Duplicate(const char *s, int len)
{
    return strncpy(new char[len+1], s, len);
}

const char *Version::Date()
{
    if (date != nil)
	return date;

    char buf[128];
    char junk[128];
    int year, month, day;

    strcpy(junk, RCS_Date + 7);
    if (sscanf(junk, "%02d/%02d/%02d", &year, &month, &day) == 3) {
	sprintf(buf, "%s %d, %d", months[month-1], day, 1900 + year);
	date = Duplicate(buf, strlen(buf));
    } else {
	date = nil;
    }
    return date;
}    


const char *Version::GetText(const char *src)
{
    const char *cp;

    if ((cp = index(src, ' ')) != nil || (cp = index(src, '$')) != nil)
	src = Duplicate(src, cp-src);
    else 
	src = nil;
    return src;
}

const char *Version::Author()
{
    return author == nil ? (author = GetText(RCS_Author + 9)) : author;
}

const char *Version::State()
{
    return state == nil ? (state = GetText(RCS_State + 8)) : state;
}    

const char *Version::Revision()
{
    return revision == nil ? (revision = GetText(RCS_Revision + 11)) : revision;
}    

const char *Version::PrettyVersion()
{
    if (pretty == nil) {
	static char buf[128];
	
	sprintf(buf, "xplot, version %s (%s)", Revision(), State());
	pretty = StringDup(buf);
    }
    return pretty;
}
