// This file is part of xplot.
// 
// Copyright (C) 1989 Rene' Seindal (seindal@diku.dk)
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


#include "view.hh"
#include "canvas.hh"

#include <InterViews/perspective.h>

#define P perspective			// lazy shorthand.

inline float roundoff(float x) {
    return round(x * 1000)/1000.0;
}

inline int abs(int x) {
    return x > 0 ? x : -x;
}

void PlotView::Normalize()
{
    P->curx = P->cury = P->x0 = P->y0 = 0;
    P->curwidth = P->width = xmax;
    P->curheight = P->height = ymax;
    
    Adjust(*P);
}

void PlotView::OneToOne()
{
    float scale = xmax > ymax ? YMag() : XMag();

    ZoomTo(scale, scale);
}

void PlotView::Magnify(PlotCanvas *pv)
{
    Coord x1, y1, x2, y2;
    
    pv->SelectRect(x1, y1, x2, y2);
    
    if (x1 == x2 || y1 == y2)
	return;

    AdjustTo(roundoff((x1 + x2)/2.0/P->width),
	     roundoff((y1 + y2)/2.0/P->height),
	     roundoff((float)P->width/abs(x2 - x1)),
	     roundoff((float)P->height/abs(y2 - y1)));
}








