// This file is part of xplot.
// 
// Copyright (C) 1989 Rene' Seindal (seindal@diku.dk)
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


#include "xplot.hh"
#include "bytesex.hh"
#include "util.hh"
#include "plot.hh"
#include "frame.hh"

#include <InterViews/world.h>

static PropertyData app_prop[] = {
    { "*plotGeometry",		"512x512" },
    { ".lineSolid",		"0xffff" },
    { ".lineDotted",		"0xaaaa" },
    { ".lineLongdashed",	"0xf8f8" },
    { ".lineShortdashed",	"0xeeee" },
    { ".lineDotdashed",		"0xdede" },
#ifdef BIG_ENDIAN
    { ".bigEndian",		"on" },
#else
    { ".bigEndian",		"off" },
#endif
    { "*syncScroll",		"off" },
    { nil }
};

static OptionDesc options[] = {
    { "-plotgeometry",	"*plotGeometry",	OptionValueNext,	"512x512" },
    { "-pg",		"*plotGeometry",	OptionValueNext,	"512x512" },
    { "-labelfont",	"*labelFont",		OptionValueNext,	"8x13" },
    { "-lf",		"*labelFont",		OptionValueNext,	"8x13" },
    { "-copygeometry",	"*copyGeometry",	OptionValueImplicit,	"on" },
    { "-cg",		"*copyGeometry",	OptionValueImplicit,	"on" },
    { "-leftpanel",	"*leftPanel",		OptionValueImplicit,	"on" },
    { "-lp",		"*leftPanel",		OptionValueImplicit,	"on" },

    { "-bigendian",	".bigEndian",		OptionValueImplicit,	"on" },
    { "-big",		".bigEndian",		OptionValueImplicit,	"on" },
    { "-littleendian",	".bigEndian",		OptionValueImplicit,	"off" },
    { "-little",	".bigEndian",		OptionValueImplicit,	"off" },
    { "-force",		".forceOutput",		OptionValueImplicit,	"on" },

    { "-backingtore",	".backingStore",	OptionValueImplicit,	"on" },
    { "-bs",		".backingStore",	OptionValueImplicit,	"on" },
    { "-syncscroll",	"*syncScroll",		OptionValueImplicit,	"on" },
    { "-cache",		"*TransformTextHint",	OptionValueImplicit,	"cache" },
    { nil }
};


typedef void (*new_handler_t)();
extern new_handler_t set_new_handler(new_handler_t);
void out_of_memory();

// Globals.
Version version;
World *world;

int main(int ac, char **av)
{
    char *name;
    FILE *infile;
    int nframes;
    boolean stdin_seen;
    
    set_new_handler(out_of_memory);
    
    if ((name = rindex(*av, '/')) == nil)
	name = *av;
    else
	name++;

    world = new World(name, app_prop, options, ac, av);
    nframes = 0;
    stdin_seen = false;
        
    for (--ac, av++; ac; --ac, av++) {
	if ((*av)[0] == '-' && (*av)[1] == '\0') {
	    if (stdin_seen)
		continue;
	    infile = stdin;
	    *av = "Standard Input";
	    stdin_seen = true;
	} else {
	    if ((infile = fopen(*av, "r")) == nil) {
		fprintf(stderr, "%s: %s: cannot open: %s\n", name, *av, syserror());
		continue;
	    }
	}
	
	PlotGraphSet *graphset = new PlotGraphSet(infile);
	fclose(infile);

	if (graphset->Current() != nil) {
	    world->InsertToplevel(new PlotFrame(graphset, name, *av), nil);
	    nframes++;
	} else {
	    delete graphset;
	}
    }
    if (nframes > 0)
	world->Run();

    return 0;
}

    
void out_of_memory()
{
    fprintf(stderr, "xplot: Out of memory.\n");
    exit(1);
}
